# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import shelve, os, os.path, shutil, json              # ,dircache  Modifie 07.02.20019, supprime dircache qui est obsolete dans Python 3. 
import Utile.constantesDonnees, Utile.exceptionsDonnees
import Controleurs.FInfoManager # Modifiee 07.02.2019, ajoute: "Controleurs."
from Utile.exceptionsDonnees import *

class DGManager(object):
    def getDirAuteur(self,nomAuteur):
        """ Renvoie le dossier associé à un auteur 
        pre: isinstance(nomAuteur,str) """
        # print("1. Module: Controleurs.DGManager, Class: DGManager, Methode: getDirAuteur, Ligne 28 ---> os.path.join(Utile.constantesDonnees.DG_REP, nomAuteur)=[", os.path.join(Utile.constantesDonnees.DG_REP, nomAuteur), "] ")
        return os.path.join(Utile.constantesDonnees.DG_REP, nomAuteur)
    def getFileAuteur(self,nomAuteur):
        """ Renvoie le nom du fichier stockant l'objet auteur 
        pre: isinstance(nomAuteur,str) """
        # print("2. Module: Controleurs.DGManager, Class: DGManager, Methode: getFileAuteur, Ligne 33 ---> os.path.join(self.getDirAuteur(nomAuteur),Utile.constantesDonnees.DG_AUTEUR_FILE)=[", os.path.join(self.getDirAuteur(nomAuteur),Utile.constantesDonnees.DG_AUTEUR_FILE), "] ")
        return os.path.join(self.getDirAuteur(nomAuteur),Utile.constantesDonnees.DG_AUTEUR_FILE)        
    def getDG(self,nomAuteur,nomOeuvre):
        """ Renvoie le DG associé à un auteur et une oeuvre
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str)"""
        # print("3. Module: Controleurs.DGManager, Class: DGManager, Methode: getDG, Ligne 38 ---> os.path.join(self.getDirAuteur(nomAuteur),nomOeuvre)=[", os.path.join(self.getDirAuteur(nomAuteur),nomOeuvre), "] ")
        return os.path.join(self.getDirAuteur(nomAuteur),nomOeuvre) 
    def existeAuteur(self,nomAuteur):
        """ Vrai si l'auteur existe
        pre: isinstance(nomAuteur,str)
        post: isinstance(__return__,bool) """
        lAuteurs = os.listdir(Utile.constantesDonnees.DG_REP)
        # print("4. Module: Controleurs.DGManager, Class: DGManager, Methode: existeAuteur, Ligne 45 ---> lAuteurs=[", lAuteurs, "], nomAuteur=[", nomAuteur, "] ")
        if nomAuteur in lAuteurs: return True
        else: return False
    def addAuteur(self,auteur):
        """ Crée un dossier nomAuteur 
        pre: isinstance(auteur,Auteur) """
        if (self.existeAuteur(auteur.getNom())):
            raise AuteurDejaExistant('L\'auteur "'+auteur.getNom()+'" existe deja.',auteur)
        dirAuteur = self.getDirAuteur(auteur.getNom())
        print("5. Module: Controleurs.DGManager, Class: DGManager, Methode: addAuteur, Ligne 54 ---> auteur=[", auteur, "], dirAuteur=[", dirAuteur, "] ")
        if not os.path.isdir(dirAuteur):
            os.mkdir(dirAuteur)
        fileAuteur = self.getFileAuteur(auteur.getNom())
        # print("6. Module: Controleurs.DGManager, Class: DGManager, Methode: addAuteur, Ligne 58 ---> fileAuteur=[", fileAuteur, "] ")
        # stocke l'objet auteur dans 1 fichier
        auteur_dict = {Utile.constantesDonnees.DG_AUTEUR_CLE: {"nom": auteur.getNom(), "prenom": auteur.getPrenom(), "naissance": auteur.getNaissance(), "deces": auteur.getDeces()}}       # Ajoute 12.09.2019
        file_json_auteur = open(fileAuteur, "w")                         # Modifie 12.09.2019, ancienne:  dicAuteur = shelve.open(fileAuteur) 
        json.dump(auteur_dict, file_json_auteur)                        # Modifie 12.09.2019, ancienne:  dicAuteur[Utile.constantesDonnees.DG_AUTEUR_CLE] = auteur 
        # print("7. Module: Controleurs.DGManager, Class: DGManager, Methode: addAuteur, Ligne 62 ---> dicAuteur[Utile.constantesDonnees.DG_AUTEUR_CLE]=", auteur_dict[Utile.constantesDonnees.DG_AUTEUR_CLE], " ")
        file_json_auteur.close()                                        # Modifie 12.09.2019, ancienne:  dicAuteur.close()
    def delAuteur(self,nomAuteur):
        """ Supprime un dossier nomAuteur 
        pre: isinstance(nomAuteur,str) """
        dirAuteur = self.getDirAuteur(nomAuteur)
        # print("8. Module: Controleurs.DGManager, Class: DGManager, Methode: delAuteur, Ligne 68 ---> nomAuteur=[", nomAuteur, "], dirAuteur=[", dirAuteur, "] ")
        if os.path.isdir(dirAuteur):
            self.recDelFolder(dirAuteur)  
    def getAuteur(self,nomAuteur):
        """ Renvoie l'objet Auteur associé à nomAuteur ou en crée si il n'existe pas 
        pre: isinstance(nomAuteur,str) 
        post: isinstance(__return__,Auteur)"""
        fileAuteur = self.getFileAuteur(nomAuteur)
        if os.path.isfile(fileAuteur):
            file_json_auteur = open(fileAuteur)                         # Ajoute 13.09.2019
            auteur_dict = json.load(file_json_auteur)                   # Modifie 13.09.2019, ancienne:  dicAuteur = shelve.open(fileAuteur)
            auteur = Controleurs.DGManager.Auteur(auteur_dict[Utile.constantesDonnees.DG_AUTEUR_CLE]["nom"], auteur_dict[Utile.constantesDonnees.DG_AUTEUR_CLE]["prenom"], auteur_dict[Utile.constantesDonnees.DG_AUTEUR_CLE]["naissance"], auteur_dict[Utile.constantesDonnees.DG_AUTEUR_CLE]["deces"])       # Modifie 13.09.2019, ancienne:  auteur = dicAuteur[Utile.constantesDonnees.DG_AUTEUR_CLE]
            file_json_auteur.close()                                    # Modifie 13.09.2019, ancienne:  dicAuteur.close()
        else: auteur = Auteur(nomAuteur)
        # print("9. Module: Controleurs.DGManager, Class: DGManager, Methode: getAuteur, Ligne 81 ---> nomAuteur=[", nomAuteur, "], fileAuteur=[", fileAuteur, "], auteur=[", auteur, "] ")
        return auteur
    def getListeAuteurs(self):
        """ Renvoie une liste des noms des dossiers auteurs 
        post: isinstance(__return__,list) """
        dirAuteurs = self.getDirAuteur("")
        ld = os.listdir(dirAuteurs)  # Modifiee 07.02.2019, ancienne ligne:  ld = dircache.listdir(dirAuteurs)
        print("10. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeAuteurs, Ligne 88 ---> dirAuteurs=[", dirAuteurs, "], ld=[", ld, "] ")
        ld = list(filter(lambda x: x != "CVS" and x!= Utile.constantesDonnees.DG_AUTEUR_FILE, ld))            # Modifiee 18.06.2019, ancienne ligne: ld = filter(lambda x: x != "CVS" and x!= Utile.constantesDonnees.DG_AUTEUR_FILE, ld)
        # print("11. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeAuteurs, Ligne 90 ---> dirAuteurs=[", dirAuteurs, "], ld=[", ld, "] ")
        return ld
        
    def addOeuvre(self,nomAuteur,oeuvre):
        """ Crée un DG 
        pre: isinstance(nomAuteur,str) and isinstance(oeuvre,Oeuvre) """
        nomDG = self.getDG(nomAuteur,oeuvre.getTitre())
        if not os.path.isdir(nomDG):
            os.mkdir(nomDG)
        auteur = self.getAuteur(nomAuteur)
        # print("12. Module: Controleurs.DGManager, Class: DGManager, Methode: addOeuvre, Ligne 100 ---> nomAuteur=[", nomAuteur, "], oeuvre=[", oeuvre, "], auteur=[", auteur, "] ")
        fim = Controleurs.FInfoManager.FInfoManager(auteur,oeuvre) # Modifiee 07.02.2019, ajoute: "Controleurs."
        fim.creerFichierInformations()
        
    def delOeuvre(self,nomAuteur,nomOeuvre):
        """ Supprime un DG 
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) """
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        # print("13. Module: Controleurs.DGManager, Class: DGManager, Methode: delOeuvre, Ligne 108 ---> nomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomDG=[", nomDG, "] ")
        if os.path.isdir(nomDG):
            self.recDelFolder(nomDG)
    def getListeOeuvresAuteur(self,nomAuteur):
        """ Renvoie une liste des oeuvres d'un auteur
        pre: isinstance(nomAuteur,str)
        post: isinstance(__return__,list) """
        dirAuteur = self.getDirAuteur(nomAuteur)
        ld = os.listdir(dirAuteur)  # Modifiee 07.02.2019,  ancienne ligne:  ld = dircache.listdir(dirAuteur)
        # print("14. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeOeuvresAuteur, Ligne 117 ---> nomAuteur=[", nomAuteur, "], dirAuteur=[", dirAuteur, "], ld=[", ld, "] ")
        ld = list(filter(lambda x: x != "CVS" and x!='.cvsignore' and x!= Utile.constantesDonnees.DG_AUTEUR_FILE, ld))      # Modifiee 18.06.2019, ancienne ligne: ld = filter(lambda x: x != "CVS" and x!='.cvsignore' and x!= Utile.constantesDonnees.DG_AUTEUR_FILE, ld)
        # print("15. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeOeuvresAuteur, Ligne 119 ---> nomAuteur=[", nomAuteur, "], dirAuteur=[", dirAuteur, "], ld=[", ld, "] ")
        return ld  
        
        
    def loadDG(self,nomAuteur,nomOeuvre):
        # print("16. Module: Controleurs.DGManager, Class: DGManager, Methode: loadDG, Ligne 124 ---> nomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "] ")
        pass        
        
    def addVersion(self,nomAuteur,nomOeuvre,nomVersion):
        """ Crée une version
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and isinstance(nomVersion,str)"""
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        dirVersion = os.path.join(nomDG,nomVersion)
        if not os.path.isdir(dirVersion):
            os.mkdir(dirVersion)
        # print("17. Module: Controleurs.DGManager, Class: DGManager, Methode: addVersion, Ligne 134 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], nomDG=[", nomDG, "], dirVersion=[", dirVersion, "] ")
        fim = Controleurs.FInfoManager.FInfoManager(Auteur(nomAuteur),Oeuvre(nomOeuvre))        # Modifiee 07.02.2019, ajoute: "Controleurs."
        fim.creerVersion(nomVersion)
    def delVersion(self,nomAuteur,nomOeuvre,nomVersion):
        """ Supprime une version
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and isinstance(nomVersion,str)"""
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        dirVersion = os.path.join(nomDG,nomVersion)
        # print("18. Module: Controleurs.DGManager, Class: DGManager, Methode: delVersion, Ligne 142 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], nomDG=[", nomDG, "], dirVersion=[", dirVersion, "] ")
        if os.path.isdir(dirVersion):
            self.recDelFolder(dirVersion)
        fim = Controleurs.FInfoManager.FInfoManager(Auteur(nomAuteur),Oeuvre(nomOeuvre)) # Modifiee 07.02.2019, ajoute: "Controleurs."
        fim.supprimerVersion(nomVersion)    
        
    def addEtat(self,nomAuteur,nomOeuvre,nomVersion,fileEtat,etatPere=""):
        """ Ajoute un état à partir du fichier fileEtat dans la version nomVersion
        et ayant éventellement un etatPere
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and \
             isinstance(nomVersion,str) and isinstance(fileEtat,str) and isinstance(etatPere,str)"""
        (filepath, nomEtat) = os.path.split(fileEtat) 
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        dirVersion = os.path.join(nomDG,nomVersion)
        cheminEtat=os.path.join(dirVersion,nomEtat)
        # print("19. Module: Controleurs.DGManager, Class: DGManager, Methode: addEtat, Ligne 157 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], fileEtat=", fileEtat, ", nomDG=[", nomDG, "], dirVersion=[", dirVersion, "], cheminEtat=[", cheminEtat, "] ")
        self.__copieFichier(fileEtat,cheminEtat)
        fim = Controleurs.FInfoManager.FInfoManager(Auteur(nomAuteur),Oeuvre(nomOeuvre)) # Modifiee 07.02.2019, ajoute: "Controleurs."
        fim.creerEtat(nomVersion, nomEtat, etatPere)
    
    def delEtat(self,nomAuteur,nomOeuvre,nomVersion,nomEtat):
        """ Supprime un état 
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and \
             isinstance(nomVersion,str) and isinstance(nomEtat,str) """
        fim = Controleurs.FInfoManager.FInfoManager(Auteur(nomAuteur),Oeuvre(nomOeuvre)) # Modifiee 07.02.2019, ajoute: "Controleurs."
        lEtatsFils = fim.getListeEtatsFils(nomVersion,nomEtat)
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        dirVersion = os.path.join(nomDG,nomVersion)
        cheminEtat=os.path.join(dirVersion,nomEtat)
        # print("20. Module: Controleurs.DGManager, Class: DGManager, Methode: delEtat, Ligne 171 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], nomEtat=[", nomEtat, "], nomDG=[", nomDG, "], dirVersion=[", dirVersion, "], cheminEtat=[", cheminEtat, "] ")
        self.recDelFolder(cheminEtat)
        if lEtatsFils != None and lEtatsFils != []:
            for etatFils in lEtatsFils: 
                cheminEtat=os.path.join(dirVersion,etatFils)
                self.recDelFolder(cheminEtat)  
        fim.supprimerEtat(nomVersion, nomEtat) 
        
    def getListeVersionsDG(self,nomAuteur,nomOeuvre):
        """ Renvoie une liste des versions d'un DG
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) 
        post: isinstance(__return__,list) """
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        print("20-2. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeVersionsDG, Ligne 186 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomDG=[", nomDG, "] ")
        ld = os.listdir(nomDG)                                                     # Modifiee 07.02.2019,  ancienne ligne:  ld = dircache.listdir(nomDG) 
        ld = list(filter(lambda x: x != "CVS" and x != '.cvsignore' and x!= Utile.constantesDonnees.DG_FILE_INFO, ld))        # Modifiee 18.06.2019, ancienne ligne: ld = filter(lambda x: x != "CVS" and x != '.cvsignore' and x!= Utile.constantesDonnees.DG_FILE_INFO, ld)
        # print("21. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeVersionsDG, Ligne 189 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomDG=[", nomDG, "], ld=[", ld, "] ")
        return ld  
    def getListeEtatsVersion(self,nomAuteur,nomOeuvre,nomVersion):
        """ Renvoie une liste des etats d'une  version
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and isinstance(nomVersion,str)
        post: isinstance(__return__,list) """
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        print("Controleurs.DGManager.py - 111. nomDG=", str(nomDG))
        path = os.path.join(nomDG,nomVersion)
        print("Controleurs.DGManager.py - 222. path=", str(path))
        #if not os.path.isdir(path): return [] 
        ld = os.listdir(path)  # Modifiee 07.02.2019,  ancienne ligne:  ld = dircache.listdir(path) 
        print("Controleurs.DGManager.py - 333. os.listdir(path)=", str(ld))
        ld = list(filter(lambda x: x != "CVS" and x != '.cvsignore', ld))     # Modifiee 18.06.2019, ancienne ligne: ld = filter(lambda x: x != "CVS" and x != '.cvsignore', ld)
        # print("22. Module: Controleurs.DGManager, Class: DGManager, Methode: getListeEtatsVersion, Ligne 200 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], nomDG=[", nomDG, "], path=[", path, "], ld=[", ld, "] ")
        return ld 
        
    def __copieFichier(self,pSource,pDestinataire):
        """ Methode privee qui sert a la copie d'un fichier """
        # print("23. Module: Controleurs.DGManager, Class: DGManager, Methode: __copieFichier, Ligne 205 ---> pSource=", pSource, ", pDestinataire=", pDestinataire, " ")
        if os.path.isfile(pDestinataire):
            raise Utile.exceptionsDonnees.FichierExistant(pSource,pDestinataire)
        else: shutil.copyfile(pSource,pDestinataire)

    def importTextFolder(self,nomAuteur,nomOeuvre,textFolder):  
        """ Importe un répertoire d'états 
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and isinstance(textFolder,str)  """
        # print("23. Module: Controleurs.DGManager, Class: DGManager, Methode: importTextFolder, Ligne 213 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], textFolder=[", textFolder, "] ")
        (filepath, nomVersion) = os.path.split(textFolder)
        # création de la version du nom du répertoire
        self.addVersion(nomAuteur,nomOeuvre,nomVersion)
        for fichier in os.listdir(textFolder):
            if os.path.isfile(os.path.join(textFolder,fichier)):
                self.addEtat(nomAuteur,nomOeuvre,nomVersion,os.path.join(textFolder,fichier))
              
    def getEtatPath(self,nomAuteur,nomOeuvre,nomVersion,nomEtat):
        """ Renvoie le fichier associé à un état 
        pre: isinstance(nomAuteur,str) and isinstance(nomOeuvre,str) and \
             isinstance(nomVersion,str) and isinstance(nomEtat,str) 
        post: isinstance(__return__,str) """
        nomDG = self.getDG(nomAuteur,nomOeuvre)
        dirVersion = os.path.join(nomDG,nomVersion)
        cheminEtat = os.path.join(dirVersion,nomEtat)
        # print("24. Module: Controleurs.DGManager, Class: DGManager, Methode: getEtatPath, Ligne 229 ---> \nnomAuteur=[", nomAuteur, "], nomOeuvre=[", nomOeuvre, "], nomVersion=[", nomVersion, "], nomEtat=[", nomEtat, "], nomDG=[", nomDG, "], dirVersion=[", dirVersion, "], cheminEtat=[", cheminEtat, "] ")
        return cheminEtat
        
    def recDelFolder(self,folder):
        """Efface récursivement un dossier et tout ce qu'il contient
        pre: isinstance(folder,str) or isinstance(folder,unicode) """
        # print("25. Module: Controleurs.DGManager, Class: DGManager, Methode: recDelFolder, Ligne 235 ---> folder=", folder, " ") 
        if os.path.isdir(folder):
            liste = os.listdir(folder)
            for i in liste :
                self.recDelFolder(os.path.join(folder,i))
            os.rmdir(folder)
        else:
            os.remove(folder)                

class Auteur(object):
    def __init__(self, pNom, pPrenom="", pNais="", pDeces=""):
        self.__nom = pNom
        self.__prenom = pPrenom
        self.__naissance = pNais
        self.__deces = pDeces
        # print("26. Module: Controleurs.DGManager, Class: Auteur, Methode: __init__, Ligne 250 ---> self.__nom=", self.__nom, ", self.__prenom=", self.__prenom, ", self.__naissance=", self.__naissance, ", self.__deces=", self.__deces, " ")
    def getNom(self):
        # print("27. Module: Controleurs.DGManager, Class: Auteur, Methode: getNom, Ligne 252 ---> self.__nom=", self.__nom, " ")
        return self.__nom
    def getPrenom(self):
        # print("28. Module: Controleurs.DGManager, Class: Auteur, Methode: getPrenom, Ligne 255 ---> self.__prenom=", self.__prenom, " ")
        return self.__prenom
    def getNaissance(self):
        print("29. Module: Controleurs.DGManager, Class: Auteur, Methode: getNaissance, Ligne 258 ---> self.__naissance=", self.__naissance, " ")
        return self.__naissance
    def getDeces(self):
        print("30. Module: Controleurs.DGManager, Class: Auteur, Methode: getDeces, Ligne 261 ---> self.__deces=", self.__deces, " ")
        return self.__deces
    def equals(self,auteur):
        """pre: isinstance(auteur,Auteur) """
        # print("31. Module: Controleurs.DGManager, Class: Auteur, Methode: equals, Ligne 265 ---> auteur.getNom()=", auteur.getNom(), ", auteur.getPrenom()=", auteur.getPrenom(), ", auteur.getNaissance()=", auteur.getNaissance(), ", auteur.getDeces()=", auteur.getDeces(), " ")
        return (self.getNom()==auteur.getNom() and self.getPrenom()==auteur.getPrenom() and
        self.getNaissance()==auteur.getNaissance() and self.getDeces()==auteur.getDeces())

class Oeuvre(object):
    def __init__(self, pTitre, pEdition="", pPublication=""):
        self.__titre = pTitre
        self.__edition = pEdition
        self.__publication = pPublication
        # print("32. Module: Controleurs.DGManager, Class: Oeuvre, Methode: __init__, Ligne 274 ---> self.__titre=", self.__titre, ", self.__edition=", self.__edition, ", self.__publication=", self.__publication, " ")
    def getTitre(self):
        # print("33. Module: Controleurs.DGManager, Class: Oeuvre, Methode: getTitre, Ligne 276 ---> self.__titre=", self.__titre, " ")
        return self.__titre
    def getEdition(self):
        # print("34. Module: Controleurs.DGManager, Class: Oeuvre, Methode: getEdition, Ligne 279 ---> self.__edition=", self.__edition, " ") 
        return self.__edition
    def getPublication(self):
        # print("35. Module: Controleurs.DGManager, Class: Oeuvre, Methode: getPublication, Ligne 282 ---> self.__publication=", self.__publication, " ")
        return self.__publication        
        
        
