# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#----------------------------------------------------------------------------
# Nom:          cDossierGenetique.py
# Objet:        Classe qui interface entre les dossiers et l'interface graphique
#----------------------------------------------------------------------------

import os                      # Modifiee 28.03.2019, ancienne ligne:  import os,dircache
import posixpath
import shutil

from Utile.constantesDonnees import *
from Donnees.donnees import *
from Donnees.donneesInformations import *

# C'est la classe de type 'Controleur' qui fait le lien entre les dossiers et l'interface
class CDossierGenetique:
    # Constructeur
    def __init__(self):
        self.__don = Donnees()
        # print("1. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: __init__, Ligne 36 ")

    # Methode  qui cree le dossier version dans le dossier genetique de
    # l'oeuvre passee en parametre
    def creerDossierVersion(self,pNom,pTitre,pVersion):
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        #print nomDG
        chem=os.path.join(LEURREP,pNom,nomDG,pVersion)
        #print chem
        os.mkdir(chem)
        # print("2. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: creerDossierVersion, Ligne 46, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", nomDG=", nomDG, ", chem=", chem)

    # Methode qui renvoie une liste des dossiers Version contenus
    # dans le dossier genetique de l'oeuvre passee en parametre
    def getListeDossiersVersions(self,pNom,pTitre):
        listeDossiersVersions=[]
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        cheminDG=os.path.join(LEURREP,pNom,nomDG)
        for X in os.listdir(cheminDG):
            chemin=os.path.join(cheminDG,X)
            if posixpath.isdir(chemin):
                listeDossiersVersions.append(X)
        # print("3. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: getListeDossiersVersions, Ligne 58, \npNom=", pNom, ", pTitre=", pTitre, ", nomDG=", nomDG, ", cheminDG=", cheminDG, ", \nlisteDossiersVersions=", listeDossiersVersions)
        return listeDossiersVersions

    # Methode qui renvoie une liste des dossiers Version contenus
    # dans le dossier genetique de l'oeuvre passee en parametre
    def getListeVersions(self,pNom,pTitre,pVersion):
        listeVersions=[]
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        cheminDG=os.path.join(LEURREP,pNom,nomDG)
        cheminVersion = os.path.join(LEURREP,cheminDG,pVersion)
        for X in os.listdir(cheminVersion):
            chemin=os.path.join(cheminVersion,X)
            if posixpath.isfile(chemin):
                listeVersions.append(X)
        # print("4. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: getListeVersions, Ligne 72, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", nomDG=", nomDG, ", cheminDG=", cheminDG, ", \ncheminVersion=", cheminVersion, ", listeVersions=", listeVersions)
        return listeVersions

    # Methode qui supprime un dossier Version et tout ce qu'il contient
    def supprimerDossierVersion(self,pNom,pTitre,pVersion):
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        chemin=os.path.join(LEURREP,pNom,nomDG,pVersion)
        self.__suppDos(chemin)
        # print("5. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: supprimerDossierVersion, Ligne 80, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", nomDG=", nomDG, ", chemin=", chemin)

    # Methode qui renvoie une liste des �tats contenus dans le dossier
    # version passe en parametre
    def getListeEtats(self,pNom,pTitre,pVersion):
        listeEtats=[]
        nomDG = self.__don.getNomDG(pNom, pTitre)
        cheminDG=os.path.join(LEURREP,pNom,nomDG,pVersion)
        # print("6. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: getListeEtats, Ligne 87, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", nomDG=", nomDG, ", cheminDG=", cheminDG, ", os.listdir(cheminDG)=", os.listdir(cheminDG))
        return os.listdir(cheminDG)

    # Methode qui cree un état a partir d'un fichier texte
    def creerEtat(self,pNom,pTitre,pVersion,pFichier):
        nomDG = self.__don.getNomDG(pNom, pTitre)
        (filepath, nomEtat) = os.path.split(pFichier) 
        cheminEtat=os.path.join(LEURREP,pNom,nomDG,pVersion,nomEtat)
        self.__copieFichier(pFichier,cheminEtat)
        # print("7. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: creerEtat, Ligne 97, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", \npFichier=", pFichier, ", filepath=", filepath, ", nomEtat=", nomEtat, ", cheminEtat=", cheminEtat)

    # Methode privee qui sert a la copie d'un fichier
    def __copieFichier(self,pSource,pDestinataire):
        if posixpath.isfile(pDestinataire):
##            print 'ce fic existe'
            raise FichierExistant(pSource,pDestinataire)
        else :
            shutil.copyfile(pSource,pDestinataire)
##            fichier=open(pSource,'r')
##            etat=open(pDestinataire,'w')
##            contenuFichier=fichier.readlines()
##            fichier.close()
##            etat.writelines(contenuFichier)
##            etat.close()
        # print("8. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: __copieFichier, Ligne 112, \npSource=", pSource, ", pDestinataire=", pDestinataire)

    # Methode privee qui sert à la copie d'un fichier
    def ecraserFichier(self,pSource,pDestinataire):
        shutil.copyfile(pSource,pDestinataire)
##        fichier=open(pSource,'r')
##        etat=open(pDestinataire,'w')
##        contenuFichier=fichier.readlines()
##        fichier.close()
##        etat.writelines(contenuFichier)
##        etat.close()
        # print("9. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: ecraserFichier, Ligne 123, \npSource=", pSource, ", pDestinataire=", pDestinataire)

    # Methode qui renvoie le chemin d'un état
    def cheminEtat(self,pNom,pTitre,pVersion,pEtat):
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        # print("10. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: cheminEtat, Ligne 128,  \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", pEtat=", pEtat, ", \nnomDG=", nomDG, ", os.path.join(LEURREP,pNom,nomDG,pVersion,pEtat)=", os.path.join(LEURREP,pNom,nomDG,pVersion,pEtat))
        return os.path.join(LEURREP,pNom,nomDG,pVersion,pEtat)

    # Methode qui supprime l'état
    def supprimerEtat(self,pNom,pTitre,pVersion,pEtat):
        donInf = DonneesInformations()
        nomDG = self.__don.getNomDG(pNom, pTitre)
        path = os.path.join(LEURREP, pNom, nomDG, FICHIER_INFO)
        lEtatsFils = donInf.getListeEtatsFils(path,pEtat,pVersion)
        chemin=self.cheminEtat(pNom,pTitre,pVersion,pEtat)
        self.__suppDos(chemin)
        if lEtatsFils != None and lEtatsFils != []:
            for etatFils in lEtatsFils:	
                chemin=self.cheminEtat(pNom,pTitre,pVersion,etatFils)
                self.__suppDos(chemin)
        # print("11. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: supprimerEtat, Ligne 143, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", pEtat=", pEtat, ", \nnomDG=", nomDG, ", path=", path, ", chemin=", chemin)

    # Methode qui supprime un dossier Auteur et tout ce qu'il contient
    def supprimerDossierAuteur(self, pNom):
        self.__suppDos(os.path.join(LEURREP,pNom))
        # print("12. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: supprimerDossierAuteur, Ligne 148, pNom=", pNom)

    # Methode qui supprime un dossier Oeuvre et tout ce qu'il contient
    def supprimerDossierOeuvre(self, pNom, pNomDG):
        rep = os.path.join(LEURREP, pNom, pNomDG)
        self.__suppDos(rep)
        # print("13. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: supprimerDossierOeuvre, Ligne 154, pNom=", pNom, ", pNomDG=", pNomDG, ", rep=", rep)

    # Methode privee qui supprime recursivement un dossier et tout ce qu'il contient
    def __suppDos(self, pPath):
        try:
            if posixpath.isdir(pPath):
                liste = os.listdir(pPath)
                for i in liste :
                    self.__suppDos(os.path.join(pPath,i))
                os.rmdir(pPath)
            else :
                os.remove(pPath)                
        except OSError:
            fake=1
        # print("14. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: __suppDos, Ligne 168, pPath=", pPath)

    # Methode qui copie un repertoire
    def importerRepertoire(self, pNom, pTitre, pPath, fenetreMedite):
        ## print 'ds CDG :  importerRepertoire'
        #nomDG =  self.__don.getNomDG(pNom, pTitre)
        (filepath, nomRep) = os.path.split(pPath)
        # cr�ation de la version du nom du répertoire
        self.creerDossierVersion(pNom,pTitre,nomRep)
        fenetreMedite.cinfos.enregistrerVersion(pNom,pTitre,nomRep)
        # chem = self.cheminVersion(pNom,pTitre,nomRep)
        for fichier in os.listdir(pPath):                             # Modifiee 28.03.2019, ancienne ligne:  for fichier in dircache.listdir(pPath):
            if os.path.isfile(os.path.join(pPath,fichier)):
                self.creerEtat(pNom,pTitre,nomRep,os.path.join(pPath,fichier))
            fenetreMedite.cinfos.enregistrerEtat(pNom,pTitre,fichier,nomRep,"")
#		(fpath, fname) = os.path.split(file)
#		shutil.copyfile(file,os.join(chem,fname))
#		self.creerEtat
        #shutil.copytree(pPath,chem)
        # print("15. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: importerRepertoire, Ligne 187, \npNom=", pNom, ", pTitre=", pTitre, ", \npPath=", pPath, ", filepath=", filepath, ", nomRep=", nomRep) 

    # Methode qui renvoie le chemin d'une version
    def cheminVersion(self,pNom,pTitre,pVersion):
        nomDG =  self.__don.getNomDG(pNom, pTitre)
        # print("16. Module: Controleurs.cDossierGenetique, Class: CDossierGenetique, Methode: cheminVersion, Ligne 192, \npNom=", pNom, ", pTitre=", pTitre, ", pVersion=", pVersion, ", nomDG=", nomDG)
        return os.path.join(LEURREP,pNom,nomDG,pVersion)
    
