# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#----------------------------------------------------------------------------
# Nom:          arbre.py
# Objet:        Classe qui regroupe les donnees d'un arbre
#
# Auteur:       Julien Bourdaillet
#
# Cree:         janvier 2005
#        
#----------------------------------------------------------------------------

# Arbre des versions
class Arbre:
    # Constucteur de la classe
    def __init__(self, pListe=[]):
        self.__listeVersions = pListe
        # print("1. Module: Donnees.arbre, Class: Arbre, Methode: __init__, Ligne 33, self.__listeVersions=", self.__listeVersions, " ")
        #self.__nomDG = nomDG

    # Methodes d'acces aux attributs prives
    def getListeVersions(self):
        # print("2. Module: Donnees.arbre, Class: Arbre, Methode: getListeVersions, Ligne 38, self.__listeVersions=", self.__listeVersions, " ")
        return self.__listeVersions

    # renvoie une liste plate (ce n'est plus un arbre) de tous les Etats de l'arbre
    # Chaque état est représenté par un couple (Etat,Version) où Version est la version
    # mére de l'Etat
    def getListePlateEtats(self):
        lEtats = []
        for version in self.getListeVersions():
            lEtats.extend(self.__getNode(version,version))
        # print("3. Module: Donnees.arbre, Class: Arbre, Methode: getListePlateEtats, Ligne 48, ---> lEtats=", lEtats, " ")
        return lEtats
        
    def __getNode(self,node, versionMere):
        lEtats = []
        if isinstance(node,Etat):
            lEtats.append((node,versionMere))
            for fils in node.getListeFils():
                lEtats.extend(self.__getNode(fils,versionMere))
        else: # Version
            for fils in node.getListeFils():
                lEtats.extend(self.__getNode(fils,node))
        # print("4. Module: Donnees.arbre, Class: Arbre, Methode: __getNode, Ligne 60, ---> lEtats=", lEtats, " ")
        return lEtats
              
    def addVersion(self, version):
        self.__listeVersions.append(version)
        # print("5. Module: Donnees.arbre, Class: Arbre, Methode: addVersion, Ligne 65, ---> self.__listeVersions=", self.__listeVersions, " ")
        
    def toString(self):
        s = "Arbre ["
        for v in self.__listeVersions:
            s = s + v.toString()
        s += "]"
        # print("6. Module: Donnees.arbre, Class: Arbre, Methode: toString, Ligne 72, ---> s=", s, " ")
        return s

# Version
class Version:
    def __init__(self, pNom, pListe=[]):
        self.__nom = pNom
        self.__listeVersionOREtats = pListe
        # print("7. Module: Donnees.arbre, Class: Version, Methode: __init__, Ligne 80, ---> self.__nom=", self.__nom, ", self.__listeVersionOREtats=", self.__listeVersionOREtats, " ")
        
    def getListeFils(self):
        # print("8. Module: Donnees.arbre, Class: Version, Methode: getListeFils, Ligne 83, ---> self.__listeVersionOREtats=", self.__listeVersionOREtats, " ")
        return self.__listeVersionOREtats
        
    def getNom(self):
        # print("9. Module: Donnees.arbre, Class: Version, Methode: getNom, Ligne 87, ---> self.__nom=", self.__nom, " ")
        return self.__nom
        
    def addFils(self, versionORetat):
        self.__listeVersionOREtats.append(versionORetat)
        # print("10. Module: Donnees.arbre, Class: Version, Methode: addFils, Ligne 92, ---> self.__listeVersionOREtats=", self.__listeVersionOREtats, " ")
        
    def toString(self):
        s = "V "+self.getNom()+ " ["
        for x in self.__listeVersionOREtats:
            s = s + x.toString() + " / "
        s += "]"
        # print("11. Module: Donnees.arbre, Class: Version, Methode: toString, Ligne 99, ---> s=", s, " ")
        return s
        
# Etat d'un version
class Etat:
    def __init__(self, pNom, pListe=[]):
        self.__nom = pNom
        self.__listeEtatsFils = pListe
        # print("12. Module: Donnees.arbre, Class: Etat, Methode: __init__, Ligne 107, ---> self.__nom=", self.__nom, ", self.__listeEtatsFils=", self.__listeEtatsFils, " ")
        
    def getListeFils(self):
        # print("13. Module: Donnees.arbre, Class: Etat, Methode: getListgetListeFilseFils, Ligne 110, ---> self.__listeEtatsFils=", self.__listeEtatsFils, " ")
        return self.__listeEtatsFils
        
    def getNom(self):
        # print("14. Module: Donnees.arbre, Class: Etat, Methode: getNom, Ligne 114, ---> self.__nom=", self.__nom, " ")
        return self.__nom       
        
    def addFils(self,etatFils):
        self.__listeEtatsFils.append(etatFils)
        # print("15. Module: Donnees.arbre, Class: Etat, Methode: addFils, Ligne 119, ---> self.__listeEtatsFils=", self.__listeEtatsFils, " ")
        
    def toString(self):
        s = "E "+self.getNom()+ " ["
        for x in self.__listeEtatsFils:
            s = s + x.toString() + " / "
        s += "]"
        # print("16. Module: Donnees.arbre, Class: Etat, Methode: toString, Ligne 126, ---> s=", s, " ")
        return s
        
        