# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#----------------------------------------------------------------------------
# Nom:          donnees.py
# Objet:        Classe qui accede au donnees concernant les auteurs
#----------------------------------------------------------------------------

from Utile.constantesDonnees import *
from Utile.exceptionsDonnees import *
from Donnees.auteur import *                                # Modifiee 28.02.2019, ancienne ligne:  from auteur import * 
from Donnees.oeuvre import *                                # Modifiee 28.02.2019, ancienne ligne:  from oeuvre import *  
from Donnees.arbre import *                                 # Modifiee 28.02.2019, ancienne ligne:  from arbre import *  
import shelve 

class Donnees:
    # Methode qui renvoie vrai si l'auteur existe dans notre fichier de donnees
    def __existeAuteurByNom(self, pNom):
        listeAuteurs = shelve.open(NOSDONNEES)  # Donne la liste des auteurs
        existe = (str(pNom) in listeAuteurs)            # Modifiee 16.04.2019, ancienne ligne:  existe = listeAuteurs.has_key(str(pNom))  
        listeAuteurs.close()
        # print("1. Module: Donnees.donnees, Class: Donnees, Methode: __existeAuteurByNom, Ligne 36, pNom=", pNom, ", str(pNom)=", str(pNom), ", listeAuteurs=", listeAuteurs, ", existe=", existe, " ")
        return existe

 
    # Methode qui retourne l'auteur dont le nom est passe en parametre
    # Une exception est lancee si l'auteur n'existe pas
    def getAuteurByNom(self,pNom):
##        print 'dans Donnees.getAuteurByNom'
        if self.__existeAuteurByNom(pNom):
            listeAuteurs = shelve.open(NOSDONNEES)
            auteur = listeAuteurs[str(pNom)]
            listeAuteurs.close()
            # print("2. Module: Donnees.donnees, Class: Donnees, Methode: getAuteurByNom, Ligne 48, pNom=", pNom, ", listeAuteurs=", listeAuteurs, ", auteur=", auteur, " ") 
            return auteur
        else :
            raise AuteurNonTrouve('l\'auteur "'+pNom+'" n\'a pas ete trouve')

    # Methode pour ajouter un auteur dans notre fichier de donnees
    # Une exception est lancee si l'auteur est deja enregistre
    def addAuteur(self, pAuteur):
##        print 'dans Donnees.addAuteur'
        try :
            auteur = self.getAuteurByNom(str(pAuteur.getNom()))
            # print("3. Module: Donnees.donnees, Class: Donnees, Methode: addAuteur, Ligne 59, pAuteur.getNom()=", pAuteur.getNom(), ", auteur=", auteur, " ") 
            raise AuteurDejaExistant('l\'auteur "'+pAuteur.getNom()+'" est deja existant',auteur)
        except AuteurNonTrouve :
            listeAuteurs = shelve.open(NOSDONNEES)
            listeAuteurs[str(pAuteur.getNom())]= pAuteur
            listeAuteurs.close()
            # print("4. Module: Donnees.donnees, Class: Donnees, Methode: addAuteur, Ligne 65, listeAuteurs[str(pAuteur.getNom())]=", listeAuteurs[str(pAuteur.getNom())], " ") 

    # Methode pour supprimer un auteur de notre fichier de donnees
    # Une exception est lancee, si l'auteur n'existe pas
    def deleteAuteur(self, pNom):
#        print 'dans Donnees.deleteAuteur'
        try:
            listeAuteurs = shelve.open(NOSDONNEES)
            # print("5. Module: Donnees.donnees, Class: Donnees, Methode: deleteAuteur, Ligne 73, listeAuteurs[str(pNom)]=", listeAuteurs[str(pNom)], " ") 
            del listeAuteurs[str(pNom)]
            listeAuteurs.close()
        except KeyError:
            raise AuteurNonTrouve('l\'auteur "'+pNom+'" n\'a pas ete trouve')

    # Methode de mise a jour d'un auteur
    def updateAuteur(self, pAuteur):
##        print 'dans Donnees.updateAuteur'
        auteur = self.getAuteurByNom(pAuteur.getNom())
        auteur.modifyAuteur(pAuteur.getPrenom(), pAuteur.getNaissance(), pAuteur.getDeces())
        self.deleteAuteur(pAuteur.getNom())
        self.addAuteur(auteur)
        # print("5. Module: Donnees.donnees, Class: Donnees, Methode: updateAuteur, Ligne 86, pAuteur.getNom()=", pAuteur.getNom(), " ")

    # Methode qui renvoie la liste des noms des auteurs
    def getListeNomsAuteurs(self):
##        print 'dans Donnees.getListeNomsAuteurs'
        listeAuteurs = shelve.open(NOSDONNEES)
        l=list(listeAuteurs.keys())                           # Modifiee 21.06.2019, ancienne ligne:  l=listeAuteurs.keys()
        listeAuteurs.close()
        # print("6. Module: Donnees.donnees, Class: Donnees, Methode: getListeNomsAuteurs, Ligne 94, (liste des noms des auteurs)=", l, " ")
        return l
    
    # Methode qui renvoie la liste des titres des oeuvres d'un auteur
    def getListeTitresByAuteur(self, pNom):
##        print 'dans Donnees.getListeTitresByAuteur'
        auteur = self.getAuteurByNom(pNom)
        # print("7. Module: Donnees.donnees, Class: Donnees, Methode: getListeTitresByAuteur, Ligne 101, auteur.getListeTitresOeuvres()=", auteur.getListeTitresOeuvres(), " ")
        return auteur.getListeTitresOeuvres()

    # Methode qui renvoie la liste des noms des dossiers genetiques des oeuvres d'un auteur
    def getListeDGByAuteur(self, pNom):
##        print 'dans Donnees.getListeDGByAuteur'
        auteur = self.getAuteurByNom(pNom)
        # print("8. Module: Donnees.donnees, Class: Donnees, Methode: getListeDGByAuteur, Ligne 108, (liste des noms des dossiers genetiques des oeuvres d\'un auteur)=", auteur.getListeTitresOeuvres(), " ")
        return auteur.getListeDGOeuvres()
       
    # Methode qui ajoute une oeuvre a un auteur
    # Une exception est lancee si l'oeuvre est deja existante
    def addOeuvre(self, pNom, pOeuvre):
##        print 'dans Donnees.addOeuvre'
        try :
            auteur = self.getAuteurByNom(pNom)
            oeuvre = auteur.getOeuvreByTitre(pOeuvre.getTitre())
            # print("9. Module: Donnees.donnees, Class: Donnees, Methode: addOeuvre, Ligne 118, pNom=", pNom, ", pOeuvre.getTitre()=", pOeuvre.getTitre(), ", auteur=", auteur, " oeuvre=", oeuvre, " ")
            # Si l'oeuvre existe, on lance une exception
            raise OeuvreDejaExistante('l\'oeuvre "'+pOeuvre.getTitre()+'"de "'+pNom+'" est deja enregistree',oeuvre)
        except OeuvreNonTrouvee:
            auteur.addOeuvre(pOeuvre)
            self.deleteAuteur(pNom)
            self.addAuteur(auteur)
    
    # Methode pour supprimer une oeuvre d'un auteur
    def deleteOeuvre(self, pNom, pTitre):
##        print 'dans Donnees.deleteOeuvre'
        auteur = self.getAuteurByNom(pNom)
        # print("10. Module: Donnees.donnees, Class: Donnees, Methode: deleteOeuvre, Ligne 130, pNom=", pNom, ", pTitre=", pTitre, " ")
        if auteur.existeOeuvreByTitre(pTitre):
            auteur.deleteOeuvreByTitre(pTitre)
            self.deleteAuteur(pNom)
            self.addAuteur(auteur)

    # Methode qui met a jour l'oeuvre d'un auteur
    def updateOeuvre(self, pNom, pOeuvre):
##        print 'dans Donnees.updateOeuvre'
        auteur = self.getAuteurByNom(pNom)
        # oeuvre = auteur.getOeuvreByTitre(pOeuvre.getTitre())
        auteur.deleteOeuvreByTitre(pOeuvre.getTitre())
        # oeuvre.modifyOeuvre(pOeuvre.getEdition(), pOeuvre.getPublication())
        auteur.addOeuvre(pOeuvre)
        self.deleteAuteur(pNom)
        self.addAuteur(auteur)    
        # print("11. Module: Donnees.donnees, Class: Donnees, Methode: updateOeuvre, Ligne 146, pNom=", pNom, ", pOeuvre.getTitre()=", pOeuvre.getTitre(), " ")

    # Methode qui retourne le nom du dossier genetique de l'oeuvre passee en parametre
    def getNomDG(self, pNom, pTitre):
##        print 'dans Donnees.getNomDG'
        auteur = self.getAuteurByNom(pNom)
        oeuvre = auteur.getOeuvreByTitre(pTitre)
        # print("12. Module: Donnees.donnees, Class: Donnees, Methode: getNomDG, Ligne 153, pNom=", pNom, ", pTitre=", pTitre, " ") 
        return oeuvre.getNomDG()

    