# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#----------------------------------------------------------------------------
# Nom:          resultatAppli.py
# Objet:        Classe qui regroupe les donnees d'une comparaison d'un couple
#               de etats  ( utilisee par l'interface )              
#----------------------------------------------------------------------------
import sys, logging, Utile.decodage                  # Modifiee 28.02.2019, ancienne ligne:  import sys,logging,contract,Utile.decodage 
import numpy
from . import planTravail                           # Modifiee 28.02.2019, ancienne ligne:  import planTravail   
import MediteApplication.utile
# contract.checkmod(__name__)                         # Desactivee 15.04.2019

class Resultat(object):
    """ Classe qui retourne le resulat obtenu apres comparaison de deux etats """    
    def __init__(self, pLI, pLS, pLD, pLR, pLgSource, pTextes, pBlocsCommuns, pairesBlocsDepl):
        self._li = pLI # liste des insertions
        self._ls = pLS # liste des suppression
        self._ld = pLD # liste des deplacements
        self._lr = pLR # liste des remplacements
        self._lgTexteS = pLgSource # longueur du texte source
        self._textes = pTextes # les deux etats concatenes
        self._blocsCom = pBlocsCommuns   # blocs communs    
        self._pairesBlocsDepl = pairesBlocsDepl #paires de blocs déplacés
        self._nonDef = []
        # print("1. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __init__, Ligne: 41 ---> \nself._li=", self._li, ", \nself._ls=", self._ls, ", \nself._ld=", self._ld, ", \nself._lr=", self._lr, ", \nself._lgTexteS=", self._lgTexteS, ", \nself._textes=", self._textes, ", \nself._blocsCom=", self._blocsCom, ", \nself._pairesBlocsDepl=", self._pairesBlocsDepl, ", \nself._nonDef=", self._nonDef, " ")

######################################################################
    def initialiser(self):
        self._li = []
        self._ls = []
        self._ld = []
        self._lr = []
        self._lgTexteS = 0
        self._textes =""
        self._blocsCom = []    
        self._pairesBlocsDepl = []
        self._nonDef = []
        # print("2. Module: Donnees.resultatAppli, Classe: Resultat, Methode: initialiser, Ligne: 54 ---> \nself._li=\n", self._li, ", \nself._ls=\n", self._ls, ", \nself._ld=\n", self._ld, ", \nself._lr=\n", self._lr, ", \nself._lgTexteS=\n", self._lgTexteS, ", \nself._textes=\n", self._textes, ", \nself._blocsCom=\n", self._blocsCom, ", \nself._pairesBlocsDepl=\n", self._pairesBlocsDepl, ", \nself._nonDef=\n", self._nonDef, " ")

#############################################################""


    def getListeInsertions(self):
        # print("3. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeInsertions, Ligne: 60 ---> \nself._li=\n", self._li, " ") 
        return self._li
    
    def getListeSuppressions(self):
        # print("4. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeSuppressions, Ligne: 64 ---> \nself._ls=\n", self._ls, " ")
        return self._ls

    def getListeDeplacements(self):
        # print("5. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeDeplacements, Ligne: 68 ---> \nself._ld=\n", self._ld, " ")
        return self._ld

    def _filtre(self,liste,deb,fin):
        """ Renvoie la sous-liste des items de liste compris entre deb et fin 
        
        pre: 0<=deb<fin #<=len(self._textes)
        post: isinstance(__return__,list)
            forall([x in __return__],deb<=x[0]<=x[1]<=fin)
        """
        res=[]
        for x in liste:
            if deb <= x[0] < x[1] <= fin:
                res.append(x)
        # print("6. Module: Donnees.resultatAppli, Classe: Resultat, Methode: _filtre, Ligne: 82 ---> res=", res, " ")
        return res
        ###test
    def shiba(self):
        # print("7. Module: Donnees.resultatAppli, Classe: Resultat, Methode: shiba, Ligne: 86 ---> test=", "test ")
        return 'test'
        
    def getListeDeplacementsT1(self):
        # print("8. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeDeplacementsT1, Ligne: 90 ---> self._filtre(self._ld,0,self._lgTexteS)=", self._filtre(self._ld,0,self._lgTexteS), " ")
        return self._filtre(self._ld,0,self._lgTexteS)
    
    def getListeDeplacementsT2(self):
        # print("9. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeDeplacementsT2, Ligne: 94 ---> self._filtre(self._ld,self._lgTexteS,len(self._textes))=", self._filtre(self._ld,self._lgTexteS,len(self._textes)), " ")
        return self._filtre(self._ld,self._lgTexteS,len(self._textes))
        
    def getListeRemplacements(self):
        # print("10. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeRemplacements, Ligne: 98 ---> self._lr=", self._lr, " ")
        return self._lr

    def getListeRemplacementsT1(self):
        # print("11. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeRemplacementsT1, Ligne: 102 ---> self._filtre(self._lr,0,self._lgTexteS)=", self._filtre(self._lr,0,self._lgTexteS), " ")
        return self._filtre(self._lr,0,self._lgTexteS)
    
    def getListeRemplacementsT2(self):
        # print("12. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeRemplacementsT1, Ligne: 106 ---> self._filtre(self._lr,self._lgTexteS,len(self._textes))=", self._filtre(self._lr,self._lgTexteS,len(self._textes)), " ")
        return self._filtre(self._lr,self._lgTexteS,len(self._textes))
        
    def getTextesConcatenes(self):
        # print("13. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getTextesConcatenes, Ligne: 110 ---> self._textes=\n[", self._textes, "] ")
        return self._textes

    def getLgSource(self):
        # print("14. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getLgSource, Ligne: 114 ---> self._lgTexteS=", self._lgTexteS, " ")
        return self._lgTexteS

    def getBlocsCommuns(self):
        # print("15. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getBlocsCommuns, Ligne: 118 ---> self._blocsCom=", self._blocsCom, " ")
        return self._blocsCom
        
    def getBlocsCommunsT1(self):
        # print("16. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getBlocsCommunsT1, Ligne: 122 ---> self._filtre(self._blocsCom,0,self._lgTexteS)=", self._filtre(self._blocsCom,0,self._lgTexteS), " ")
        return self._filtre(self._blocsCom,0,self._lgTexteS)
        
    def getBlocsCommunsT2(self):
        # print("17. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getBlocsCommunsT2, Ligne: 126 ---> self._filtre(self._blocsCom,self._lgTexteS,len(self._textes))=", self._filtre(self._blocsCom,self._lgTexteS,len(self._textes)), " ")
        return self._filtre(self._blocsCom,self._lgTexteS,len(self._textes))        

    def getPairesBlocsDeplaces(self):
        # print("18. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getPairesBlocsDeplaces, Ligne: 130 ---> self._pairesBlocsDepl=", self._pairesBlocsDepl, " ")
        return self._pairesBlocsDepl
        
    def setPairesBlocsDeplaces(self,liste):
        self._pairesBlocsDepl = liste
        # print("19. Module: Donnees.resultatAppli, Classe: Resultat, Methode: setPairesBlocsDeplaces, Ligne: 135 ---> self._pairesBlocsDepl=", self._pairesBlocsDepl, " ")

    def getNonDef(self):
        # print("20. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getNonDef, Ligne: 138 ---> self._nonDef=", self._nonDef, " ")
        return self._nonDef

    def setNonDef(self, NonDef):
        self._NonDef = NonDef
        # print("21. Module: Donnees.resultatAppli, Classe: Resultat, Methode: setNonDef, Ligne: 143 ---> self._nonDef=", self._nonDef, " ")

    def getListeTransformations(self):
    ##concatener les diff transfos en ajoutant etiquette
        lT=[]
        lI=self.getListeInsertions()
        lS=self.getListeSuppressions()
        lR=self.getListeRemplacements()
        lD=self.getListeDeplacements()
        lBC=self.getBlocsCommuns()
        lND=self.getNonDef()
        for x in lI:
            lT.append((x,'i'))
        for x in lS:
            lT.append((x,'s'))
        for x in lR:
            lT.append((x,'r'))
        for x in lD:
            lT.append((x,'d'))
        for x in lBC:
            lT.append((x,'bc'))
        for x in lND:
            lT.append((x,'nd'))
        lT.sort()
        # print("22. Module: Donnees.resultatAppli, Classe: Resultat, Methode: getListeTransformations, Ligne: 167 ---> lT=", str(lT), " ")
        return lT
    
    def toString_(self):
        s = u"Type RESULTAT \nInsertions : "
        for i in self._li:
            s += str(i).encode("iso-8859-2", "replace")+u" / "                          # Modifiee 28.02.2019, ancienne ligne:  s += unicode(i).encode("iso-8859-2", "replace")+u" / "  
        s += u"\nSuppressions : "
        for i in self._ls:
            s += str(i).encode("iso-8859-2", "replace")+u" / "                          # Modifiee 28.02.2019, ancienne ligne:   s += unicode(i).encode("iso-8859-2", "replace")+u" / "  
        s += u"\nDéplacements : "                                                       # Modifiee 28.02.2019, ancienne ligne:   s += u"\nD�placements : " 
        for i in self._ld:
            s += str(i).encode("iso-8859-2", "replace")+u" / "                          # Modifiee 28.02.2019, ancienne ligne:   s += unicode(i).encode("iso-8859-2", "replace")+u" / "  
        s += "\nRemplacements  : "
        if self._lr != []:
            for i in self._lr:
                s += str(i).encode("iso-8859-2", "replace")+" / "                           # Modifiee 28.02.2019, ancienne ligne:   s += unicode(i).encode("iso-8859-2", "replace")+" / "  
         
#       s2,x,y = Utile.decodage.decode_string_heuristically(self._texteCible.toxml())
#       s += s2
#       s += unicode(self._texteCible).encode("iso-8859-2", "replace")
        s += "\nBlocs communs : "
        if self._blocsCom != []:
            for i in self._blocsCom:
                s += "["
                for ii in i:
                    #iii,x,y=Utile.decodage.decode_string_heuristically(ii)
                    s += str(ii)+"/"    
                s +="]"
######################################################################

        s += "\nBlocs non definis : "
        if self._nonDef != []:
            for i in self._nonDef:
                s += str(i).encode("iso-8859-2", "replace")+" / "                                # Modifiee 28.02.2019, ancienne ligne:   s += unicode(i).encode("iso-8859-2", "replace")+" / " 
            
###################################################################
        s += "\n lgTexteSource  : "+str(self._lgTexteS)
        # print("23. Module: Donnees.resultatAppli, Classe: Resultat, Methode: toString_, Ligne: 205 ---> s(resultats)=\n[", s, "] ")
        sys.stderr.write(str(s).encode("iso-8859-2", "replace"))                             # Modifiee 28.02.2019, ancienne ligne:   sys.stderr.write(unicode(s).encode("iso-8859-2", "replace"))  
        sys.stderr.flush()
        
    def toString(self):
        """Retourne le texte correspondant aux listes de blocs"""
        f = lambda res,l: [res.append(self.getTextesConcatenes()[x[0]:x[1]]) for x in l]
        def f(res,l):
            """Fonction anonyme permettant de traiter les paires de blocs déplacés"""
            for x in l:
                if isinstance(x,tuple):#les paires de Bloc deplac� sont des tuples
                    res.extend(['(', self.getTextesConcatenes()[x[0][0]:x[0][1]],
                                '/',self.getTextesConcatenes()[x[1][0]:x[1][1]],')'])    
                else:# bloc normal
                    res.extend(['(',self.getTextesConcatenes()[x[0]:x[1]],')'])
        # print("24. Module: Donnees.resultatAppli, Classe: Resultat, Methode: toString/f, Ligne: 220 ---> self.__str(f)(resultats)=\n[", self.__str(f), "] ")
        return self.__str(f)
        
    def __str__(self):
        """Retourne uniquement les listes de blocs"""
        f = lambda res,l: [res.append(str(x)) for x in l]
        # print("25. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __str__, Ligne: 226 ---> longueur T1=\n[", str(self._lgTexteS), "\n", self.__str(f), "] ")
        return "longueur T1="+str(self._lgTexteS)+"\n"+self.__str(f)
        
    def __str(self,f):    
        """Fonction privée construisant la chaine à retourner
        
        f est une fonction concaténant de la manière souhaitée le bloc courant
        
        pre: callable(f)"""
        res = ['BC :']
        #f = lambda l: [res.append(str(x)) for x in l]
        f(res,self.getBlocsCommuns())
        res.append('\nDEP: ')
        f(res,self.getListeDeplacements())
        res.append('\nPDEP: ')
        f(res,self.getPairesBlocsDeplaces())
        res.append('\nSUP: ')
        f(res,self.getListeSuppressions())
        res.append('\nINS: ')
        f(res,self.getListeInsertions())
        res.append('\nREMP: ')
        f(res,self.getListeRemplacements())
##########################################################
        res.append('\nNONDEF: ')
        f(res,self.getNonDef())
        # print("26. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __str, Ligne: 251 ---> res=\n[", "".join(res), "] ")
        return ''.join(res)
########################################################
        
    def __xor__(self,res):
        """Différence symétrique
        
        Extrait les intervalles de chaque liste présents dans l'un ou l'autre des 2 objets
        mais pas dans les 2, cad une différence symétrique de chaque liste.
        
        pre: isinstance(res,Resultat) """
        assert self.getTextesConcatenes() == res.getTextesConcatenes() # on traite bien le meme texte
        assert self.getLgSource() == res.getLgSource()
        bc = self.__diff_sym(self.getBlocsCommuns(), res.getBlocsCommuns())
        dep = self.__diff_sym(self.getListeDeplacements(), res.getListeDeplacements())
        pdep = self.__diff_sym(self.getPairesBlocsDeplaces(), res.getPairesBlocsDeplaces())
        ins = self.__diff_sym(self.getListeInsertions(), res.getListeInsertions())
        sup = self.__diff_sym(self.getListeSuppressions(), res.getListeSuppressions())
        remp = self.__diff_sym(self.getListeRemplacements(), res.getListeRemplacements())
        
        # print("27. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __xor__, Ligne: 271 ---> \nbc=", bc, ", \ndep=", dep, ", \npdep=", pdep, ", \nins=", ins, ", \nsup=", sup, ", \nremp=", remp, " ")
        return Resultat(ins, sup, dep, remp, self.getLgSource(), 
                        self.getTextesConcatenes(),bc, pdep)
        
    def __diff_sym(self,L1,L2):
        """Différence symétrique effective de 2 listes: (L1 - L2) U (L2 - L1) """
        tool = MediteApplication.utile.Utile()
        res = []
        for x in L1:
            if x not in L2: res = tool.addition_intervalle(res,x) # addition sans fusion
        for x in L2:
            if x not in L1: res = tool.addition_intervalle(res,x)
        # print("28. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __diff_sym, Ligne: 283 ---> \nres=[", res, "] ")
        return res            
    
    def __sub__(self,res):
        """Différence: objet courant - res"""
        assert self.getTextesConcatenes() == res.getTextesConcatenes() # on traite bien le meme texte
        assert self.getLgSource() == res.getLgSource()
        bc = self.__diff(self.getBlocsCommuns(), res.getBlocsCommuns())
        dep = self.__diff(self.getListeDeplacements(), res.getListeDeplacements())
        pdep = self.__diff(self.getPairesBlocsDeplaces(), res.getPairesBlocsDeplaces())
        ins = self.__diff(self.getListeInsertions(), res.getListeInsertions())
        sup = self.__diff(self.getListeSuppressions(), res.getListeSuppressions())
        remp = self.__diff(self.getListeRemplacements(), res.getListeRemplacements())
        # print("29. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __sub__, Ligne: 296 ---> \nbc=", bc, ", \ndep=", dep, ", \npdep=", pdep, ", \nins=", ins, ", \nsup=", sup, ", \nremp=", remp, " ")
        return Resultat(ins, sup, dep, remp, self.getLgSource(), 
                        self.getTextesConcatenes(),bc, pdep)
    def __diff(self,L1,L2):
        """Différence effective de 2 listes: (L1 - L2)"""
        tool = MediteApplication.utile.Utile()
        res = []
        for x in L1:
            if x not in L2: res = tool.addition_intervalle(res,x) # addition sans fusion
        # print("30. Module: Donnees.resultatAppli, Classe: Resultat, Methode: __diff, Ligne: 305 ---> \nres=[", res, "] ")
        return res
    ###########################################
    ## POUR CHANGER LE RESULTAT MANUELLEMENT ##
    ###########################################

    def isInsertion (self,element):
        # print("31. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isInsertion, Ligne: 312 ---> element=", element, " ")
        deb,fin=element
        for x in self._li :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin):
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)
##        if element in self._li:
##            return True
##        return False
    def isSuppression(self,element):
        # print("32. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isSuppression, Ligne: 323 ---> element=", element, " ")
        deb,fin=element
        for x in self._ls :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin):
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)
##        if element in self._ls:
##            return True
##        return False
    def isRemplacement(self,element):
        # print("33. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isRemplacement, Ligne: 334 ---> element=", element, " ")
        deb,fin=element
        for x in self._lr :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin) :
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)
##        if element in self._lr:
##            return True
##        return False
    def isDeplacement(self, element):
        # print("34. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isDeplacement, Ligne: 345 ---> element=", element, " ")
        deb,fin=element
        for x in self._ld :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin):
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)
##        if element in self._ld:
##            return True
##        return False
    def isBlocCommun(self,element):
        # print("35. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isBlocCommun, Ligne: 356 ---> element=", element, " ")
        deb,fin=element
        for x in self._blocsCom :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin):
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)
##        if element in self._blocsCom:
##            return True
##        return False
    
    def isTransformation(self,element):
        # print("36. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isTransformation, Ligne: 368 ---> element=", element, " ")
        if self.isInsertion(element)[0] or self.isSuppression(element)[0] or self.isRemplacement(element)[0] or self.isDeplacement(element)[0] or self.isBlocCommun(element)[0]:
            return True
        return False

    def isNonDef(self,element):
        # print("37. Module: Donnees.resultatAppli, Classe: Resultat, Methode: isNonDef, Ligne: 374 ---> element=", element, " ")
        deb,fin=element
        for x in self._nonDef :
##            if (x[0]<=deb<=x[1] or x[0]<=fin<=x[1] or deb<=x[0]<=x[1]<=fin):
            if (x[0]<=deb<=fin<=x[1]):
                return (True,x)
        return (False,None)

###Methode pour identiifer les blocs selectionnes    
    def identifierBlocs(self,element):
        # print("38. Module: Donnees.resultatAppli, Classe: Resultat, Methode: identifierBlocs, Ligne: 384 ---> element=", element, " ")
        deb,fin=element
        lT= self.getListeTransformations()
        i=1
        while deb >= lT[i][0][0]:
            i+=1
        j=i-1
        
        while fin >= lT[j][0][1]:
            j+=1
        blocsID=[]
        for k in range(i-1,j+1):
            blocsID.append(lT[k])
            
        return blocsID


    def enleverTransformations(self,elts,txt):
        # print("39. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverTransformations, Ligne: 402 ---> elts=\n[", str(elts), "], \ntxt=\n[", txt, "] ")
        self.listeChangements = []
        insertions=self._li
        suppressions=self._ls
        deplacements=self._ld
        remplacements=self._lr
        blocsCommuns=self._blocsCom    
        blocsDeplaces=self._pairesBlocsDepl
        for element in elts :
            deb,fin = element
##            if txt ==2 :
##                element = (deb+self.getLgSource(),fin+self.getLgSource())
##                deb,fin=element

            nDef = self.isNonDef(element)
            if not nDef[0]:            
                ins = self.isInsertion(element)
                sup = self.isSuppression(element)
                remp = self.isRemplacement(element)
                depl = self.isDeplacement(element)
                blocCom = self.isBlocCommun(element)
                

                if (txt==2) & ins[0]:
                    self.listeChangements += self.enleverInsertion(element, ins[1])
                    

                elif (txt==1) & sup[0]:
                    self.listeChangements += self.enleverSuppression(element, sup[1])
                    

                elif remp[0]:
                    self.listeChangements += self.enleverRemplacement(element, remp[1], txt)
                    

                elif depl[0]:
                    self.listeChangements += self.enleverDeplacement(element, depl[1], txt)
                    

                elif blocCom[0]:
                    self.listeChangements += self.enleverBlocCommun(element, blocCom[1], txt)
                   

                self._nonDef.append(element)
                self._nonDef.sort()
                self.listeChangements.append(element)
                self.listeChangements = list(set(self.listeChangements))
                self.listeChangements.sort()
                for x in self._nonDef :
                    if self.isTransformation(x): self._nonDef.remove(x)
                self.checkNonDefinis()

            
                
    def enleverInsertion(self,element,ins):
        # print("40. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverInsertion, Ligne: 457 ---> element=\n[", element, "], \nins=\n[", ins, "] ")
        lC=[]
        insertions=self._li
        deb,fin=element
        if ins[0] == deb :
            if fin == ins[1]:
                insertions.remove(element)
                lC.append(element)
            else :
                insertions.append((fin,ins[1]))
                insertions.remove(ins)
                lC.append((fin,ins[1]))
                lC.append(ins)
                insertions.sort()
                        
        else :
            if fin == ins[1]:
                insertions.append((ins[0],deb))
                insertions.remove(ins)
                lC.append((ins[0],deb))
                lC.append(ins)
                insertions.sort()
            else:
                insertions.remove(ins)
                insertions.append((ins[0],deb))
                insertions.append((fin,ins[1]))
                lC.append(ins)
                lC.append((ins[0],deb))
                lC.append((fin,ins[1]))
                insertions.sort()
        # print("41. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverInsertion, Ligne: 487 ---> lC=\n[", lC, "] ")
        return lC
        
    def enleverSuppression(self,element, sup):
        # print("42. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverSuppression, Ligne: 491 ---> element=\n[", element, "], \nsup=\n[", sup, "] ")
        lC = []
        suppressions=self._ls
        deb,fin=element
        if sup[0] == deb :
            if fin == sup[1]:
                suppressions.remove(element)
                self.listeChangements.append(element)
            else :
                suppressions.append((fin,sup[1]))
                suppressions.remove(sup)
                suppressions.sort()
                lC.append((fin,sup[1]))
                lC.append(sup)
                        
        else :
            if fin == sup[1]:
                suppressions.append((sup[0],deb))
                suppressions.remove(sup)
                suppressions.sort()
                lC.append((sup[0],deb))
                lC.append(sup)
            else:
                suppressions.remove(sup)
                suppressions.append((sup[0],deb))
                suppressions.append((fin,sup[1]))
                suppressions.sort()
                lC.append(sup)
                lC.append((sup[0],deb))
                lC.append((fin,sup[1]))
        # print("43. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverSuppression, Ligne: 521 ---> lC=\n[", lC, "] ")
        return lC
                        
    def enleverRemplacement(self,element,remp,txt):
        # print("44. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverRemplacement, Ligne: 525 ---> element=\n[", element, "], \nremp=\n[", remp, "], \ntxt=\n", txt, " ")
        lC=[]
        remplacements=self._lr
        ln=len(remplacements)//2                                 # Modifiee 24.06.2019, ancienne ligne:  ln=len(remplacements)/2 
        item=remplacements.index(remp)
        deb,fin=element
        if txt==1 :
                if remp[0] == deb :
                    if fin == remp[1]:
                        remplacements.remove(element)
                        self._nonDef.append(remplacements[item+ln-1])
                        lC.append(element)
                        lC.append(remplacements[item+ln-1])
                        lC.append(remplacements[item+ln-1])
                        del remplacements[item+ln-1]
                        
                        
                    else :
                        remplacements.append((fin,remp[1]))
                        remplacements.remove(remp)
                        remplacements.sort()
                        lC.append((fin,remp[1]))
                        lC.append(remp)
                        
                else :
                    if fin == remp[1]:
                        remplacements.append((remp[0],deb))
                        remplacements.remove(remp)
                        remplacements.sort()
                        lC.append((remp[0],deb))
                        lC.append(remp)
                    
                    else:
                        pass
        elif txt==2 :
                if remp[0] == deb :
                    if fin == remp[1]:
                        remplacements.remove(element)
                        self._nonDef.append(remplacements[item-ln])
                        lC.append(element)
                        lC.append(remplacements[item-ln])
                        lC.append(remplacements[item-ln])
                        del remplacements[item-ln]
                    else :
                        remplacements.append((fin,remp[1]))
                        remplacements.remove(remp)
                        remplacements.sort()
                        lC.append((fin,remp[1]))
                        lC.append(remp)
                        
                else :
                    if fin == remp[1]:
                        remplacements.append((remp[0],deb))
                        remplacements.remove(remp)
                        remplacements.sort()
                        lC.append((remp[0],deb))
                        lC.append(remp)
                    else:
                        pass
        # print("45. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverRemplacement, Ligne: 584 ---> lC=\n[", lC, "] ")
        return lC
        
    def enleverDeplacement(self,element,depl,txt):
        # print("46. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverDeplacement, Ligne: 588 ---> element=\n[", element, "], \ndepl=\n[", depl, "], \ntxt=\n", txt, " ")
        lC=[]
        deplacements=self._ld
        blocsDeplaces=self._pairesBlocsDepl
        deb,fin=element
        if txt==1:
                paires = [x for x in blocsDeplaces if x[0]==depl]
                correspondants = [ x[1] for x in  paires]
                paires = [x for x in blocsDeplaces if x[1] in correspondants]
                elements = [x[0] for x in paires ]
                elements = list(set(elements))
                if depl[0]==deb:
                    if fin ==depl[1]:
                        deplacements.remove(element)
                        lC.append(element)
                        for x in paires :
                            if x[0] == element :
                                blocsDeplaces.remove(x)
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                self._nonDef.append(x)
                                lC.append(x)

                    else:
                        deplacements.remove(depl)
                        deplacements.append((fin,depl[1]))
                        lC.append(depl)
                        lC.append((fin,depl[1]))
                        for x in paires :
                            if x[0]==depl :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((fin,depl[1]),(x[1][0]+fin-depl[0],x[1][1])))
                                
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[0],x[1]))
                                self._nonDef.append((x[0],x[0]+fin-depl[0]))
                                lC.append(x)
                                lC.append((x[0]+fin-depl[0],x[1]))
                                lC.append((x[0],x[0]+fin-depl[0]))
                                
                        

                else:
                    if fin == depl[1]:
                        deplacements.remove(depl)
                        deplacements.append((depl[0],deb))
                        lC.append(depl)
                        lC.append((depl[0],deb))
                        for x in paires :
                            if x[0]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((depl[0],deb),(x[1][0],x[1][0]+deb-depl[0])))
                                
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0],x[0]+deb-depl[0]))
                                self._nonDef.append((x[0]+deb-depl[0],x[1]))
                                lC.append(x)
                                lC.append((x[0],x[0]+deb-depl[0]))
                                lC.append((x[0]+deb-depl[0],x[1]))
                                
                        

                    else:
                        deplacements.remove(depl)
                        deplacements.append((fin,depl[1]))
                        deplacements.append((depl[0],deb))
                        lC.append(depl)
                        lC.append((fin,depl[1]))
                        lC.append((depl[0],deb))
                        for x in paires :
                            if x[0]==depl :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((fin,depl[1]),(x[1][0]+fin-depl[0],x[1][1])))
                                blocsDeplaces.append(((depl[0],deb),(x[1][0],x[1][0]+deb-depl[0])))

                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[0],x[1]))
                                deplacements.append((x[0],x[0]+deb-depl[0]))
                                self._nonDef.append((x[0]+deb-depl[1][0],x[0]+fin-depl[0]))
                                lC.append(x)
                                lC.append((x[0]+fin-depl[0],x[1]))
                                lC.append((x[0],x[0]+deb-depl[0]))
                                lC.append((x[0]+deb-depl[0],x[0]+fin-depl[0]))
                                


        elif txt==2 :
            paires = [x for x in blocsDeplaces if x[1]==depl]
            antecedants = [ x[0] for x in  paires]
            paires = [x for x in blocsDeplaces if x[0] in antecedants]
            elements = [x[1] for x in paires]
            elements = list(set(elements))
            if depl[0]==deb:
                if fin ==depl[1]:
                    deplacements.remove(element)
                    lC.append(element)
                    for x in paires :
                        if x[1] == element :
                            blocsDeplaces.remove(x)
                    if len(elements) == 1:
                        for x in antecedants :
                            deplacements.remove(x)
                            self._nonDef.append(x)
                            lC.append(x)

                else:
                    deplacements.remove(depl)
                    deplacements.append((fin,depl[1]))
                    lC.append(depl)
                    lC.append((fin,depl[1]))
                    for x in paires :
                        if x[1]==depl :
                            blocsDeplaces.remove(x)
                            blocsDeplaces.append(((x[0][0]+fin-depl[0],x[0][1]),(fin,depl[1])))
                            
                            
                    if len(elements) == 1:
                        for x in antecedants :
                            deplacements.remove(x)
                            deplacements.append((x[0]+fin-depl[0],x[1]))
                            self._nonDef.append((x[0],x[0]+fin-depl[0]))
                            lC.append(x)
                            lC.append((x[0]+fin-depl[0],x[1]))
                            lC.append((x[0],x[0]+fin-depl[0]))
                            
                    

            else:
                if fin == depl[1]:
                    deplacements.remove(depl)
                    deplacements.append((depl[0],deb))
                    lC.append(depl)
                    lC.append((depl[0],deb))
                    for x in paires :
                        if x[1]==depl :
                            blocsDeplaces.remove(x)
                            blocsDeplaces.append(((x[0][0],x[0][0]+deb-depl[0]),(depl[0],deb)))
                            
                    if len(elements) == 1:
                        for x in antecedants :
                            deplacements.remove(x)
                            deplacements.append((x[0],x[0]+deb-depl[0]))
                            self._nonDef.append((x[0]+deb-depl[0],x[1]))
                            lC.append(x)
                            lC.append((x[0],x[0]+deb-depl[0]))
                            lC.append((x[0]+deb-depl[0],x[1]))
                            
                    

                else:
                    deplacements.remove(depl)
                    deplacements.append((fin,depl[1]))
                    deplacements.append((depl[0],deb))
                    lC.append(depl)
                    lC.append((fin,depl[1]))
                    lC.append((depl[0],deb))
                    for x in paires :
                        if x[1]==depl :
                            blocsDeplaces.remove(x)
                            blocsDeplaces.append(((x[0][0]+fin-depl[0],x[0][1]),(fin,depl[1])))
                            blocsDeplaces.append(((x[0][0],x[0][0]+deb-depl[0]),(depl[0],deb)))

                    if len(elements) == 1:
                        for x in antecedants :
                            deplacements.remove(x)
                            deplacements.append((x[0]+fin-depl[0],x[1]))
                            deplacements.append((x[0],x[0]+deb-depl[0]))
                            self._nonDef.append((x[0]+deb-depl[0],x[0]+fin-depl[0]))
                            lC.append(x)
                            lC.append((x[0]+fin-depl[0],x[1]))
                            lC.append((x[0],x[0]+deb-depl[0]))
                            lC.append((x[0]+deb-depl[0],x[0]+fin-depl[0]))
                            

            
        deplacements.sort()
        blocsDeplaces.sort()
        # print("47. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverDeplacement, Ligne: 772 ---> lC=\n[", lC, "] ")
        return lC
    def enleverBlocCommun(self,element,blocCom,txt):
        # print("48. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverBlocCommun, Ligne: 775 ---> element=\n[", element, "], \nblocCom=\n[", blocCom, "], \ntxt=\n", txt, " ")
        lC=[]
        blocsCommuns=self._blocsCom
        deb,fin=element
        if txt==1 :
                ln=len(blocsCommuns)//2                                      # Modifiee 24.06.2019, ancienne ligne:  ln=len(blocsCommuns)/2 
                item=blocsCommuns.index(blocCom)
                corres = blocsCommuns[item+ln]
                if blocCom[0] == deb :
                    if fin == blocCom[1]:
                        self._nonDef.append(corres)
                        blocsCommuns.remove(corres)
                        blocsCommuns.remove(element)
                        lC.append(corres)
                        lC.append(element)
                        
                        
                        
                    else :
                        blocsCommuns.append((fin,blocCom[1]))
                        blocsCommuns.append((corres[0]+fin-blocCom[0],corres[1]))
                        self._nonDef.append((corres[0],corres[0]+fin-blocCom[0]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        lC.append((fin,blocCom[1]))
                        lC.append((corres[0]+fin-blocCom[0],corres[1]))
                        lC.append((corres[0],corres[0]+fin-blocCom[0]))
                        lC.append(blocCom)
                        lC.append(corres)
                        
                        
                        blocsCommuns.sort()
                        
                else :
                    if fin == blocCom[1]:
                        blocsCommuns.append((blocCom[0],deb))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[0]))
                        self._nonDef.append((corres[0]+deb-blocCom[0],corres[1]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        lC.append((blocCom[0],deb))
                        lC.append((corres[0],corres[0]+deb-blocCom[0]))
                        lC.append((corres[0]+deb-blocCom[0],corres[1]))
                        lC.append(blocCom)
                        lC.append(corres)
                    else:
                        blocsCommuns.append((blocCom[0],deb))
                        blocsCommuns.append((fin,blocCom[1]))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[0]))
                        blocsCommuns.append((corres[0]+fin-blocCom[0],corres[1]))
                        self._nonDef.append((corres[0]+deb-blocCom[0],corres[0]+fin-blocCom[0]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        lC.append((blocCom[0],deb))
                        lC.append((fin,blocCom[1]))
                        lC.append((corres[0],corres[0]+deb-blocCom[0]))
                        lC.append((corres[0]+fin-blocCom[0],corres[1]))
                        lC.append((corres[0]+deb-blocCom[0],corres[0]+fin-blocCom[0]))
                        lC.append(blocCom)
                        lC.append(corres)
        elif txt==2:
                ln=len(blocsCommuns)//2                                          # Modifiee 24.06.2019, ancienne ligne:  ln=len(blocsCommuns)/2   
                item=blocsCommuns.index(blocCom)
                corres = blocsCommuns[item-ln]
                if blocCom[0] == deb :
                    if fin == blocCom[1]:
                        self._nonDef.append(corres)
                        blocsCommuns.remove(corres)
                        blocsCommuns.remove(element)
                        lC.append(corres)
                        lC.append(element)
                        
                        
                    else :
                        blocsCommuns.append((fin,blocCom[1]))
                        blocsCommuns.append((corres[0]+fin-blocCom[0],corres[1]))
                        self._nonDef.append((corres[0],corres[0]+fin-blocCom[0]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        lC.append((fin,blocCom[1]))
                        lC.append((corres[0]+fin-blocCom[0],corres[1]))
                        lC.append((corres[0],corres[0]+fin-blocCom[0]))
                        lC.append(blocCom)
                        lC.append(corres)
                        
                        
                        blocsCommuns.sort()
                        
                else :
                    if fin == blocCom[1]:
                        blocsCommuns.append((blocCom[0],deb))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[0]))
                        self._nonDef.append((corres[0]+deb-blocCom[0],corres[1]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        lC.append((blocCom[0],deb))
                        lC.append((corres[0],corres[0]+deb-blocCom[0]))
                        lC.append((corres[0]+deb-blocCom[0],corres[1]))
                        lC.append(blocCom)
                        lC.append(corres)
                    else:
                        blocsCommuns.append((blocCom[0],deb))
                        blocsCommuns.append((fin,blocCom[1]))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[0]))
                        blocsCommuns.append((corres[0]+fin-blocCom[0],corres[1]))
                        self._nonDef.append((corres[0]+deb-blocCom[0],corres[0]+fin-blocCom[0]))
                        blocsCommuns.remove(blocCom)
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        lC.append((blocCom[0],deb))
                        lC.append((fin,blocCom[1]))
                        lC.append((corres[0],corres[0]+deb-blocCom[0]))
                        lC.append((corres[0]+fin-blocCom[0],corres[1]))
                        lC.append((corres[0]+deb-blocCom[0],corres[0]+fin-blocCom[0]))
                        lC.append(blocCom)
                        lC.append(corres)
        # print("49. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverBlocCommun, Ligne: 894 ---> lC=\n[", lC, "] ")
        return lC
    
    def enleverChangement(self,element,txt):
        # print("50. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverChangement, Ligne: 898 ---> element=\n[", element, "], \ntxt=\n", txt, " ")
        self.listeChangements = []
        deb,fin= element
        insertions=self._li
        suppressions=self._ls
        deplacements=self._ld
        remplacements=self._lr
        blocsCommuns=self._blocsCom    
        blocsDeplaces=self._pairesBlocsDepl
        if txt == 1:
            ###SUPPR
            sup = self.isSuppression(element)
            remp = self.isRemplacement(element)
            depl = self.isDeplacement(element)
            blocCom= self.isBlocCommun(element)
            if sup[0]:
                if sup[1][0] == deb :
                    if fin == sup[1][1]:
                        suppressions.remove(element)
                        self.listeChangements.append(element)
                    else :
                        suppressions.append((fin,sup[1][1]))
                        suppressions.remove(sup[1])
                        suppressions.sort()
                        self.listeChangements.append((fin,sup[1][1]))
                        self.listeChangements.append(sup[1])
                        
                else :
                    if fin == sup[1][1]:
                        suppressions.append((sup[1][0],deb))
                        suppressions.remove(sup[1])
                        suppressions.sort()
                        self.listeChangements.append((sup[1][0],deb))
                        self.listeChangements.append(sup[1])
                    else:
                        suppressions.remove(sup[1])
                        suppressions.append((sup[1][0],deb))
                        suppressions.append((fin,sup[1][1]))
                        suppressions.sort()
                        self.listeChangements.append(sup[1])
                        self.listeChangements.append((sup[1][0],deb))
                        self.listeChangements.append((fin,sup[1][1]))

                            

            ###REMPL
            elif remp[0] :
                ln=len(remplacements)//2                                             # Modifiee 24.06.2019, ancienne ligne:  ln=len(remplacements)/2 
                item=remplacements.index(remp[1])
                if remp[1][0] == deb :
                    if fin == remp[1][1]:
                        remplacements.remove(element)
                        self._nonDef.append(remplacements[item+ln-1])
                        self.listeChangements.append(element)
                        self.listeChangements.append(remplacements[item+ln-1])
                        self.listeChangements.append(remplacements[item+ln-1])
                        del remplacements[item+ln-1]
                        
                        
                    else :
                        remplacements.append((fin,remp[1][1]))
                        remplacements.remove(remp[1])
                        remplacements.sort()
                        self.listeChangements.append((fin,remp[1][1]))
                        self.listeChangements.append(remp[1])
                        
                else :
                    if fin == remp[1][1]:
                        remplacements.append((remp[1][0],deb))
                        remplacements.remove(remp[1])
                        remplacements.sort()
                        self.listeChangements.append((remp[1][0],deb))
                        self.listeChangements.append(remp[1])
                    
                    else:
                        return None
                        
                

                
            

            ###BLOCCOM
            elif blocCom[0] :
                ln=len(blocsCommuns)//2                                          # Modifiee 24.06.2019, ancienne ligne:  ln=len(blocsCommuns)/2 
                item=blocsCommuns.index(blocCom[1])
                corres = blocsCommuns[item+ln]
                if blocCom[1][0] == deb :
                    if fin == blocCom[1][1]:
                        self._nonDef.append(corres)
                        blocsCommuns.remove(corres)
                        blocsCommuns.remove(element)
                        self.listeChangements.append(corres)
                        self.listeChangements.append(element)
                        
                        
                        
                    else :
                        blocsCommuns.append((fin,blocCom[1][1]))
                        blocsCommuns.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self._nonDef.append((corres[0],corres[0]+fin-blocCom[1][0]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        self.listeChangements.append((fin,blocCom[1][1]))
                        self.listeChangements.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self.listeChangements.append((corres[0],corres[0]+fin-blocCom[1][0]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
                        
                        
                        blocsCommuns.sort()
                        
                else :
                    if fin == blocCom[1][1]:
                        blocsCommuns.append((blocCom[1][0],deb))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self._nonDef.append((corres[0]+deb-blocCom[1][0],corres[1]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        self.listeChangements.append((blocCom[1][0],deb))
                        self.listeChangements.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self.listeChangements.append((corres[0]+deb-blocCom[1][0],corres[1]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
                    else:
                        blocsCommuns.append((blocCom[1][0],deb))
                        blocsCommuns.append((fin,blocCom[1][1]))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        blocsCommuns.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self._nonDef.append((corres[0]+deb-blocCom[1][0],corres[0]+fin-blocCom[1][0]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        self.listeChangements.append((blocCom[1][0],deb))
                        self.listeChangements.append((fin,blocCom[1][1]))
                        self.listeChangements.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self.listeChangements.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self.listeChangements.append((corres[0]+deb-blocCom[1][0],corres[0]+fin-blocCom[1][0]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
            ###DEPL
            elif depl[0]:
                paires = [x for x in blocsDeplaces if x[0]==depl[1]]
                correspondants = [ x[1] for x in  paires]
                paires = [x for x in blocsDeplaces if x[1] in correspondants]
                elements = [x[0] for x in paires ]
                elements = list(set(elements))
                if depl[1][0]==deb:
                    if fin ==depl[1][1]:
                        deplacements.remove(element)
                        self.listeChangements.append(element)
                        for x in paires :
                            if x[0] == element :
                                blocsDeplaces.remove(x)
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                self._nonDef.append(x)
                                self.listeChangements.append(x)

                    else:
                        deplacements.remove(depl[1])
                        deplacements.append((fin,depl[1][1]))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((fin,depl[1][1]))
                        for x in paires :
                            if x[0]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((fin,depl[1][1]),(x[1][0]+fin-depl[1][0],x[1][1])))
                                
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[1][0],x[1]))
                                self._nonDef.append((x[0],x[0]+fin-depl[1][0]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0]+fin-depl[1][0],x[1]))
                                self.listeChangements.append((x[0],x[0]+fin-depl[1][0]))
                                
                        

                else:
                    if fin == depl[1][1]:
                        deplacements.remove(depl[1])
                        deplacements.append((depl[1][0],deb))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((depl[1][0],deb))
                        for x in paires :
                            if x[0]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((depl[1][0],deb),(x[1][0],x[1][0]+deb-depl[1][0])))
                                
                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0],x[0]+deb-depl[1][0]))
                                self._nonDef.append((x[0]+deb-depl[1][0],x[1]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0],x[0]+deb-depl[1][0]))
                                self.listeChangements.append((x[0]+deb-depl[1][0],x[1]))
                                
                        

                    else:
                        deplacements.remove(depl[1])
                        deplacements.append((fin,depl[1][1]))
                        deplacements.append((depl[1][0],deb))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((fin,depl[1][1]))
                        self.listeChangements.append((depl[1][0],deb))
                        for x in paires :
                            if x[0]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((fin,depl[1][1]),(x[1][0]+fin-depl[1][0],x[1][1])))
                                blocsDeplaces.append(((depl[1][0],deb),(x[1][0],x[1][0]+deb-depl[1][0])))

                        if len(elements) == 1:
                            for x in correspondants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[1][0],x[1]))
                                deplacements.append((x[0],x[0]+deb-depl[1][0]))
                                self._nonDef.append((x[0]+deb-depl[1][0],x[0]+fin-depl[1][0]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0]+fin-depl[1][0],x[1]))
                                self.listeChangements.append((x[0],x[0]+deb-depl[1][0]))
                                self.listeChangements.append((x[0]+deb-depl[1][0],x[0]+fin-depl[1][0]))
                                

                
                deplacements.sort()
                blocsDeplaces.sort()

        elif txt == 2:
            deb,fin=element
            element = (deb+self.getLgSource(),fin+self.getLgSource())
            deb,fin=element
            ins = self.isInsertion(element)
            remp = self.isRemplacement(element)
            depl = self.isDeplacement(element)
            blocCom= self.isBlocCommun(element)
            ###INSERT
            if ins[0]:
                if ins[1][0] == deb :
                    if fin == ins[1][1]:
                        insertions.remove(element)
                        self.listeChangements.append(element)
                    else :
                        insertions.append((fin,ins[1][1]))
                        insertions.remove(ins[1])
                        self.listeChangements.append((fin,ins[1][1]))
                        self.listeChangements.append(ins[1])
                        insertions.sort()
                        
                else :
                    if fin == ins[1][1]:
                        insertions.append((ins[1][0],deb))
                        insertions.remove(ins[1])
                        self.listeChangements.append((ins[1][0],deb))
                        self.listeChangements.append(ins[1])
                        insertions.sort()
                    else:
                        insertions.remove(ins[1])
                        insertions.append((ins[1][0],deb))
                        insertions.append((fin,ins[1][1]))
                        self.listeChangements.append(ins[1])
                        self.listeChangements.append((ins[1][0],deb))
                        self.listeChangements.append((fin,ins[1][1]))
                        insertions.sort()

                
            ###REMPL
            elif remp[0] :
                ln=len(remplacements)//2                                     # Modifiee 24.06.2019, ancienne ligne:  ln=len(remplacements)/2 
                item=remplacements.index(remp[1])
                if remp[1][0] == deb :
                    if fin == remp[1][1]:
                        remplacements.remove(element)
                        self._nonDef.append(remplacements[item-ln])
                        self.listeChangements.append(element)
                        self.listeChangements.append(remplacements[item-ln])
                        self.listeChangements.append(remplacements[item-ln])
                        del remplacements[item-ln]
                    else :
                        remplacements.append((fin,remp[1][1]))
                        remplacements.remove(remp[1])
                        remplacements.sort()
                        self.listeChangements.append((fin,remp[1][1]))
                        self.listeChangements.append(remp[1])
                        
                else :
                    if fin == remp[1][1]:
                        remplacements.append((remp[1][0],deb))
                        remplacements.remove(remp[1])
                        remplacements.sort()
                        self.listeChangements.append((remp[1][0],deb))
                        self.listeChangements.append(remp[1])
                    else:
                        return None

                
            
            ###BLOCCOM
            elif blocCom[0]:
                ln=len(blocsCommuns)//2                                                   # Modifiee 24.06.2019, ancienne ligne:  ln=len(blocsCommuns)/2  
                item=blocsCommuns.index(blocCom[1])
                corres = blocsCommuns[item-ln]
                if blocCom[1][0] == deb :
                    if fin == blocCom[1][1]:
                        self._nonDef.append(corres)
                        blocsCommuns.remove(corres)
                        blocsCommuns.remove(element)
                        self.listeChangements.append(corres)
                        self.listeChangements.append(element)
                        
                        
                    else :
                        blocsCommuns.append((fin,blocCom[1][1]))
                        blocsCommuns.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self._nonDef.append((corres[0],corres[0]+fin-blocCom[1][0]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        self.listeChangements.append((fin,blocCom[1][1]))
                        self.listeChangements.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self.listeChangements.append((corres[0],corres[0]+fin-blocCom[1][0]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
                        
                        
                        blocsCommuns.sort()
                        
                else :
                    if fin == blocCom[1][1]:
                        blocsCommuns.append((blocCom[1][0],deb))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self._nonDef.append((corres[0]+deb-blocCom[1][0],corres[1]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        self.listeChangements.append((blocCom[1][0],deb))
                        self.listeChangements.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self.listeChangements.append((corres[0]+deb-blocCom[1][0],corres[1]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
                    else:
                        blocsCommuns.append((blocCom[1][0],deb))
                        blocsCommuns.append((fin,blocCom[1][1]))
                        blocsCommuns.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        blocsCommuns.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self._nonDef.append((corres[0]+deb-blocCom[1][0],corres[0]+fin-blocCom[1][0]))
                        blocsCommuns.remove(blocCom[1])
                        blocsCommuns.remove(corres)
                        blocsCommuns.sort()
                        self.listeChangements.append((blocCom[1][0],deb))
                        self.listeChangements.append((fin,blocCom[1][1]))
                        self.listeChangements.append((corres[0],corres[0]+deb-blocCom[1][0]))
                        self.listeChangements.append((corres[0]+fin-blocCom[1][0],corres[1]))
                        self.listeChangements.append((corres[0]+deb-blocCom[1][0],corres[0]+fin-blocCom[1][0]))
                        self.listeChangements.append(blocCom[1])
                        self.listeChangements.append(corres)
            ###DEPL
            elif depl[0]:
                paires = [x for x in blocsDeplaces if x[1]==depl[1]]
                antecedants = [ x[0] for x in  paires]
                paires = [x for x in blocsDeplaces if x[0] in antecedants]
                elements = [x[1] for x in paires]
                elements = list(set(elements))
                if depl[1][0]==deb:
                    if fin ==depl[1][1]:
                        deplacements.remove(element)
                        self.listeChangements.append(element)
                        for x in paires :
                            if x[1] == element :
                                blocsDeplaces.remove(x)
                        if len(elements) == 1:
                            for x in antecedants :
                                deplacements.remove(x)
                                self._nonDef.append(x)
                                self.listeChangements.append(x)

                    else:
                        deplacements.remove(depl[1])
                        deplacements.append((fin,depl[1][1]))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((fin,depl[1][1]))
                        for x in paires :
                            if x[1]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((x[0][0]+fin-depl[1][0],x[0][1]),(fin,depl[1][1])))
                                
                                
                        if len(elements) == 1:
                            for x in antecedants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[1][0],x[1]))
                                self._nonDef.append((x[0],x[0]+fin-depl[1][0]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0]+fin-depl[1][0],x[1]))
                                self.listeChangements.append((x[0],x[0]+fin-depl[1][0]))
                                
                        

                else:
                    if fin == depl[1][1]:
                        deplacements.remove(depl[1])
                        deplacements.append((depl[1][0],deb))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((depl[1][0],deb))
                        for x in paires :
                            if x[1]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((x[0][0],x[0][0]+deb-depl[1][0]),(depl[1][0],deb)))
                                
                        if len(elements) == 1:
                            for x in antecedants :
                                deplacements.remove(x)
                                deplacements.append((x[0],x[0]+deb-depl[1][0]))
                                self._nonDef.append((x[0]+deb-depl[1][0],x[1]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0],x[0]+deb-depl[1][0]))
                                self.listeChangements.append((x[0]+deb-depl[1][0],x[1]))
                                
                        

                    else:
                        deplacements.remove(depl[1])
                        deplacements.append((fin,depl[1][1]))
                        deplacements.append((depl[1][0],deb))
                        self.listeChangements.append(depl[1])
                        self.listeChangements.append((fin,depl[1][1]))
                        self.listeChangements.append((depl[1][0],deb))
                        for x in paires :
                            if x[1]==depl[1] :
                                blocsDeplaces.remove(x)
                                blocsDeplaces.append(((x[0][0]+fin-depl[1][0],x[0][1]),(fin,depl[1][1])))
                                blocsDeplaces.append(((x[0][0],x[0][0]+deb-depl[1][0]),(depl[1][0],deb)))

                        if len(elements) == 1:
                            for x in antecedants :
                                deplacements.remove(x)
                                deplacements.append((x[0]+fin-depl[1][0],x[1]))
                                deplacements.append((x[0],x[0]+deb-depl[1][0]))
                                self._nonDef.append((x[0]+deb-depl[1][0],x[0]+fin-depl[1][0]))
                                self.listeChangements.append(x)
                                self.listeChangements.append((x[0]+fin-depl[1][0],x[1]))
                                self.listeChangements.append((x[0],x[0]+deb-depl[1][0]))
                                self.listeChangements.append((x[0]+deb-depl[1][0],x[0]+fin-depl[1][0]))
                                

                
                deplacements.sort()
                blocsDeplaces.sort()

        
        self._nonDef.append(element)
        self._nonDef.sort()
        self.listeChangements.append(element)
        self.listeChangements = list(set(self.listeChangements))
        self.listeChangements.sort()
        self.checkNonDefinis()
        # print("51. Module: Donnees.resultatAppli, Classe: Resultat, Methode: enleverChangement, Ligne: 1358 ---> self.listeChangements=\n[", self.listeChangements, "] ")
##        self.updateBlocsCommuns(element)
##
##            
##    def updateBlocsCommuns(self,element):
##        deb,fin=element
##        self._blocsCom.append(element)
##        self._blocsCom.append((deb+self.getLgSource(),fin+self.getLgSource()))
##        self._blocsCom.sort()
##        
##        ln=len(self._blocsCom)//2                                    # Modifiee 24.06.2019, ancienne ligne:  ln=len(self._blocsCom)/2 
##        for i in range( ln - 1):
##            if self._blocsCom[i][1]==self._blocsCom[i+1][0] - 1 and self._blocsCom[i+ln][1]==self._blocsCom[i+ln+1][0] - 1:
##                print self._blocsCom[i][1]
##                
                
    def checkInsertions(self):
        # print("52. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkInsertions, Ligne: 1376 ---> self._li=\n[", self._li, "] ")
        ln = len(self._li)
        i = 0
        while i < ln - 1:
            if self._li[i][1]>=self._li[i+1][0]:
                self._li.append((self._li[i][0],self._li[i+1][1]))
                self._li.remove(self._li[i])
                self._li.remove(self._li[i])
                self._li.sort()
                i -= 1
                ln -= 1
            
            i += 1
        # print("53. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkInsertions, Ligne: 1388 ---> self._li=\n[", self._li, "] ")
        
    def checkSuppressions(self):
        # print("54. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkSuppressions, Ligne: 1391 ---> self._ls=\n[", self._ls, "] ")
        ln = len(self._ls)
        i = 0
        while i < ln - 1:
            if self._ls[i][1]>=self._ls[i+1][0]:
                self._ls.append((self._ls[i][0],self._ls[i+1][1]))
                self._ls.remove(self._ls[i])
                self._ls.remove(self._ls[i])
                self._ls.sort()
                i -= 1
                ln -= 1
                
            i += 1
        # print("55. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkSuppressions, Ligne: 1404 ---> self._ls=\n[", self._ls, "] ")
            
    def checkRemplacements(self):
        # print("56. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkRemplacements, Ligne: 1407 ---> self._lr=\n[", self._lr, "] ")
        ln=len(self._lr)//2                                                  # Modifiee 24.06.2019, ancienne ligne:  ln=len(self._lr)/2 
        i = 0
        while i < ln - 1:
            a = self._lr[i]
            b = self._lr[i+1]
            c= self._lr[i+ln]
            d = self._lr[i+ln+1]
            if a[1]>=b[0] and c[1]>=d[0] :
                self._lr.remove(a)
                self._lr.remove(b)
                self._lr.remove(c)
                self._lr.remove(d)
                self._lr.append((a[0],b[1]))
                self._lr.append((c[0],d[1]))
                self._lr.sort()
                i -= 1
                ln -= 1
                
            i += 1
        # print("57. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkRemplacements, Ligne: 1427 ---> self._lr=\n[", self._lr, "] ")

    def checkBlocsCommuns(self):
        # print("58. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkBlocsCommuns, Ligne: 1430 ---> self._blocsCom=\n[", self._blocsCom, "] ")
        ln=len(self._blocsCom)//2                                                # Modifiee 24.06.2019, ancienne ligne:  ln=len(self._blocsCom)/2  
        i = 0
        while i < ln - 1:
            a = self._blocsCom[i]
            b = self._blocsCom[i+1]
            c = self._blocsCom[i+ln]
            d = self._blocsCom[i+ln+1]
            if a[1]>=b[0] and c[1]>=d[0] :
                self._blocsCom.remove(a)
                self._blocsCom.remove(b)
                self._blocsCom.remove(c)
                self._blocsCom.remove(d)
                self._blocsCom.append((a[0],b[1]))
                self._blocsCom.append((c[0],d[1]))
                self._blocsCom.sort()
                i -= 1
                ln -= 1
                
            i += 1
        # print("59. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkBlocsCommuns, Ligne: 1450 ---> self._blocsCom=\n[", self._blocsCom, "] ")

    def checkDeplacements(self):
        # print("60. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkDeplacements, Ligne: 1453 ---> self._ld=\n[", self._ld, "] ")
        dep = self._ld
        pbDepl = self._pairesBlocsDepl
        ln = len(self.getListeDeplacementsT1())
        i = 0
        while i < ln - 1:
            a = dep[i]
            b = dep[i+1]
            
            if self.chevauche(a,b):
                p1 = [x for x in pbDepl if x[0] == a]
                p2 = [x for x in pbDepl if x[0] == b]
                for x in p1 :
                    for y in p2 :
                        alpha = x[1]
                        beta = y[1]
                        
                        if self.chevauche(alpha,beta):
                            
                            pbDepl.append(((a[0],b[1]),(alpha[0],beta[1])))
                            pbDepl.remove(x)
                            pbDepl.remove(y)
                            if len(p1) == 1:
                                dep.remove(a)
                                #print'a'
                            if len(p2) == 1:
                                dep.remove(b)
                                #print 'b'
                            dep.remove(alpha)
                            dep.remove(beta)
                            dep.append((a[0],b[1]))
                            dep.append((alpha[0],beta[1]))
                            dep.sort()
                            pbDepl.sort()
                            i -= 1
                            ln -= 1
                            
            
            i+=1
        # print("61. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkDeplacements, Ligne: 1492 ---> dep=\n[", dep, "] ")
        
    def chevauche(self , a , b) :
        # print("62. Module: Donnees.resultatAppli, Classe: Resultat, Methode: chevauche, Ligne: 1495 ---> a=[", a, "], \nb=[", b, "] ")
        if a[1]>=b[0] :
            return True
        return False
                
    def checkNonDefinis(self):
        # print("63. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkNonDefinis, Ligne: 1501 ---> self._nonDef=\n[", self._nonDef, "] ")
        ln = len(self._nonDef)
        i = 0
        while i < ln - 1:
            if self._nonDef[i][1]>=self._nonDef[i+1][0] and self._nonDef[i][1] != self.getLgSource():
                self._nonDef.append((self._nonDef[i][0],self._nonDef[i+1][1]))
                self._nonDef.remove(self._nonDef[i])
                self._nonDef.remove(self._nonDef[i])
                self._nonDef.sort()
                i -= 1
                ln -= 1
                
            i += 1
        # print("64. Module: Donnees.resultatAppli, Classe: Resultat, Methode: checkNonDefinis, Ligne: 1514 ---> self._nonDef=\n[", self._nonDef, "] ")
                
    def ajouterInsertion(self,select):
        # print("65. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterInsertion, Ligne: 1517 ---> select=\n[", select, "], \nself._li=\n[", self._li, "] ")
        deb,fin=select
        select=(deb+self.getLgSource(),fin+self.getLgSource())
        deb,fin=select
        insertions=self._li
        nonDefinis = self._nonDef
        self.listeChangements = []
        liste=self.identifierBlocs(select)
        liste[0]=((deb,liste[0][0][1]),liste[0][1])
        liste[-1]=((liste[-1][0][0],fin),liste[-1][1])
        listeBlocs = []
        for x in liste:
            listeBlocs.append(x[0])

        #print listeBlocs    
        self.enleverTransformations(listeBlocs,2)

        #ajout de l'insertion
        nDef = self.isNonDef(select)[1]
        if nDef[0] == deb :
                    if fin == nDef[1]:
                        nonDefinis.remove(select)
                        self.listeChangements.append(select)
                    else :
                        nonDefinis.append((fin,nDef[1]))
                        nonDefinis.remove(nDef)
                        nonDefinis.sort()
                        self.listeChangements.append((fin,nDef[1]))
                        self.listeChangements.append(nDef)
                        
        else :
                    if fin == nDef[1]:
                        nonDefinis.append((nDef[0],deb))
                        nonDefinis.remove(nDef)
                        nonDefinis.sort()
                        self.listeChangements.append((nDef[0],deb))
                        self.listeChangements.append(nDef)
                    else:
                        nonDefinis.remove(nDef)
                        nonDefinis.append((nDef[0],deb))
                        nonDefinis.append((fin,nDef[1]))
                        nonDefinis.sort()
                        self.listeChangements.append(nDef)
                        self.listeChangements.append((nDef[0],deb))
                        self.listeChangements.append((fin,nDef[1]))

        insertions.append(select)
        self.listeChangements.append(select)
        self.listeChangements = list(set(self.listeChangements))
        self.listeChangements.sort()
        insertions.sort()
        self.checkInsertions()
        # print("66. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterInsertion, Ligne: 1569 ---> insertions=\n[", insertions, "] ")

    def ajouterSuppression(self,select):
        # print("67. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterSuppression, Ligne: 1572 ---> select=\n[", select, "], \nself._ls=\n[", self._ls, "] ")
        deb,fin=select
        suppressions=self._ls
        nonDefinis = self._nonDef
        self.listeChangements = []
        liste=self.identifierBlocs(select)
        liste[0]=((deb,liste[0][0][1]),liste[0][1])
        liste[-1]=((liste[-1][0][0],fin),liste[-1][1])
        listeBlocs = []
        for x in liste:
            listeBlocs.append(x[0])
            
        
        
        #print listeBlocs
        self.enleverTransformations(listeBlocs,1)

        #ajout de la suppression
        nDef = self.isNonDef(select)[1]
        if nDef[0] == deb :
                    if fin == nDef[1]:
                        nonDefinis.remove(select)
                        self.listeChangements.append(select)
                    else :
                        nonDefinis.append((fin,nDef[1]))
                        nonDefinis.remove(nDef)
                        nonDefinis.sort()
                        self.listeChangements.append((fin,nDef[1]))
                        self.listeChangements.append(nDef)
                        
        else :
                    if fin == nDef[1]:
                        nonDefinis.append((nDef[0],deb))
                        nonDefinis.remove(nDef)
                        nonDefinis.sort()
                        self.listeChangements.append((nDef[0],deb))
                        self.listeChangements.append(nDef)
                    else:
                        nonDefinis.remove(nDef)
                        nonDefinis.append((nDef[0],deb))
                        nonDefinis.append((fin,nDef[1]))
                        nonDefinis.sort()
                        self.listeChangements.append(nDef)
                        self.listeChangements.append((nDef[0],deb))
                        self.listeChangements.append((fin,nDef[1]))

        suppressions.append(select)
        self.listeChangements.append(select)
        self.listeChangements = list(set(self.listeChangements))
        self.listeChangements.sort()
        suppressions.sort()
        self.checkSuppressions()
        # print("68. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterSuppression, Ligne: 1624 ---> suppressions=\n[", suppressions, "] ")

    def ajouterDeplacement(self,select,txt):
        # print("69. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterDeplacement, Ligne: 1627 ---> select=\n[", select, "], \ntxt=\n[", txt, "] ")    
        if txt == 1 :
            self.listeChangements = []
            self.deplacementSource = select
            debSource,finSource = select 
        elif txt == 2 :
            debSource, finSource = self.deplacementSource
            debCible,finCible=select
            self.listeChangements = []
            deplCible = (debCible+self.getLgSource(),finCible+self.getLgSource())
            debCible,finCible=deplCible
            
            deplacements=self._ld
            nonDefinis = self._nonDef
            
            listeSource=self.identifierBlocs(self.deplacementSource)
            listeSource[0]=((debSource,listeSource[0][0][1]),listeSource[0][1])
            listeSource[-1]=((listeSource[-1][0][0],finSource),listeSource[-1][1])
            
            listeCible=self.identifierBlocs(deplCible)
            listeCible[0]=((debCible,listeCible[0][0][1]),listeCible[0][1])
            listeCible[-1]=((listeCible[-1][0][0],finCible),listeCible[-1][1])

            listeBlocsSource = []
            listeBlocsCible = []
            for x in listeSource:
                listeBlocsSource.append(x[0])

            for x in listeCible:
                listeBlocsCible.append(x[0])

            #print listeBlocsSource
            #print listeBlocsCible

            self.enleverTransformations(listeBlocsSource,1)
            lc = self.listeChangements
            #print lc
            self.enleverTransformations(listeBlocsCible,2)
            lc += self.listeChangements
            self.listeChangements = lc
            #print lc

            nDefSource = self.isNonDef(self.deplacementSource)[1]
            nDefCible = self.isNonDef(deplCible)[1]  
            
            if nDefSource[0] == debSource :
                if finSource == nDefSource[1]:
                    nonDefinis.remove(self.deplacementSource)
                    self.listeChangements.append(self.deplacementSource)
                else :
                    nonDefinis.append((finSource,nDefSource[1]))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((finSource,nDefSource[1]))
                    self.listeChangements.append(nDefSource)
                
            else :
                if finSource == nDefSource[1]:
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append(nDefSource)
                else :
                    nonDefinis.remove(nDefSource)
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.append((finSource,nDefSource[1]))
                    nonDefinis.sort()
                    self.listeChangements.append(nDefSource)
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append((finSource,nDefSource[1]))

                
            if nDefCible[0] == debCible :
                if finCible == nDefCible[1]:
                    nonDefinis.remove(deplCible)
                    self.listeChangements.append(deplCible)
                else :
                    nonDefinis.append((finCible,nDefCible[1]))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((finCible,nDefCible[1]))
                    self.listeChangements.append(nDefCible)
                        
            else :
                if finCible == nDefCible[1]:
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append(nDefCible)
                else :
                    nonDefinis.remove(nDefCible)
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.append((finCible,nDefCible[1]))                   # Modifiee 20.03.2019, ancienne ligne:  nonDefinis.append((fin,nDefCible[1])) 
                    nonDefinis.sort()
                    self.listeChangements.append(nDefCible)
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append((finCible,nDefCible[1]))
                
            deplacements.append(self.deplacementSource)
            deplacements.append(deplCible)
            self._pairesBlocsDepl.append((self.deplacementSource,deplCible))
            self.listeChangements.append(self.deplacementSource)
            self.listeChangements.append(deplCible)
            self.listeChangements = list(set(self.listeChangements))
            self.listeChangements.sort()
            deplacements.sort()
            self._pairesBlocsDepl.sort()
            #self.checkDeplacements()
            # print("70. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterDeplacement, Ligne: 1737 ---> self.listeChangements=\n[", self.listeChangements, "], \ndeplacements=\n[", deplacements, "] ")  

    def ajouterRemplacement(self,select,txt):

        # print("71. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterRemplacement, Ligne: 1741 ---> select=\n[", select, "], \ntxt=\n[", txt, "] ") 
        if txt == 1 :
            self.listeChangements = []
            self.remplacementSource = select
            debSource,finSource = select
        elif txt == 2 :
            debSource, finSource = self.remplacementSource
            debCible,finCible=select
            self.listeChangements =[]
            rempCible = (debCible+self.getLgSource(),finCible+self.getLgSource())
            debCible,finCible=rempCible
                
            remplacements=self._lr               
            nonDefinis = self._nonDef

            listeSource=self.identifierBlocs(self.remplacementSource)
            listeSource[0]=((debSource,listeSource[0][0][1]),listeSource[0][1])
            listeSource[-1]=((listeSource[-1][0][0],finSource),listeSource[-1][1])
                
            listeCible=self.identifierBlocs(rempCible)
            listeCible[0]=((debCible,listeCible[0][0][1]),listeCible[0][1])
            listeCible[-1]=((listeCible[-1][0][0],finCible),listeCible[-1][1])

            listeBlocsSource = []
            listeBlocsCible = []
            for x in listeSource:
                listeBlocsSource.append(x[0])

            for x in listeCible:
                listeBlocsCible.append(x[0])

            #print listeBlocsSource
            #print listeBlocsCible

            self.enleverTransformations(listeBlocsSource,1)
            lc = self.listeChangements
            #print lc
            self.enleverTransformations(listeBlocsCible,2)
            lc += self.listeChangements
            self.listeChangements = lc
            #print lc
            nDefSource = self.isNonDef(self.remplacementSource)[1]
            nDefCible = self.isNonDef(rempCible)[1]
                           
                
            if nDefSource[0] == debSource :
                if finSource == nDefSource[1]:
                    nonDefinis.remove(self.remplacementSource)
                    self.listeChangements.append(self.remplacementSource)
                else :
                    nonDefinis.append((finSource,nDefSource[1]))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((finSource,nDefSource[1]))
                    self.listeChangements.append(nDefSource)
                    
            else :
                if finSource == nDefSource[1]:
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append(nDefSource)
                else:
                    nonDefinis.remove(nDefSource)
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.append((finSource,nDefSource[1]))
                    nonDefinis.sort()
                    self.listeChangements.append(nDefSource)
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append((finSource,nDefSource[1]))

                
                
            if nDefCible[0] == debCible :
                if finCible == nDefCible[1]:
                    nonDefinis.remove(rempCible)
                    self.listeChangements.append(rempCible)
                else :
                    nonDefinis.append((finCible,nDefCible[1]))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((finCible,nDefCible[1]))
                    self.listeChangements.append(nDefCible)
                    
            else :
                if finCible == nDefCible[1]:
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append(nDefCible)
                else:
                    nonDefinis.remove(nDefCible)
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.append((finCible,nDefCible[1]))
                    nonDefinis.sort()
                    self.listeChangements.append(nDefCible)
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append((finCible,nDefCible[1]))
            
            remplacements.append(self.remplacementSource)
            remplacements.append(rempCible)
            self.listeChangements.append(self.remplacementSource)
            self.listeChangements.append(rempCible)
            self.listeChangements = list(set(self.listeChangements))
            self.listeChangements.sort()
            remplacements.sort()
            self.checkRemplacements()
            # print("72. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterRemplacement, Ligne: 1850 ---> self.listeChangements=\n[", self.listeChangements, "], \nremplacements=\n[", remplacements, "] ")  
        
    def ajouterBlocCommun(self,select,txt):
        # print("73. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterBlocCommun, Ligne: 1853 ---> select=\n[", select, "], \ntxt=\n[", txt, "] ") 
        if txt == 1 :
            self.listeChangements = []
            self.blocCommunSource = select
            debSource,finSource = select
        elif txt == 2 :
            debSource, finSource = self.blocCommunSource
            debCible,finCible=select
            self.listeChangements =[]
            blocComCible = (debCible+self.getLgSource(),finCible+self.getLgSource())
            debCible,finCible=blocComCible
                
            blocsCommuns=self._blocsCom              
            nonDefinis = self._nonDef

            listeSource=self.identifierBlocs(self.blocCommunSource)
            listeSource[0]=((debSource,listeSource[0][0][1]),listeSource[0][1])
            listeSource[-1]=((listeSource[-1][0][0],finSource),listeSource[-1][1])
                
            listeCible=self.identifierBlocs(blocComCible)
            listeCible[0]=((debCible,listeCible[0][0][1]),listeCible[0][1])
            listeCible[-1]=((listeCible[-1][0][0],finCible),listeCible[-1][1])

            listeBlocsSource = []
            listeBlocsCible = []
            for x in listeSource:
                listeBlocsSource.append(x[0])

            for x in listeCible:
                listeBlocsCible.append(x[0])

            #print listeBlocsSource
            #print listeBlocsCible

            self.enleverTransformations(listeBlocsSource,1)
            lc = self.listeChangements
            #print lc
            self.enleverTransformations(listeBlocsCible,2)
            lc += self.listeChangements
            self.listeChangements = lc
            #print lc
            nDefSource = self.isNonDef(self.blocCommunSource)[1]
            nDefCible = self.isNonDef(blocComCible)[1]
                
            if nDefSource[0] == debSource :
                if finSource == nDefSource[1]:
                    nonDefinis.remove(self.blocCommunSource)
                    self.listeChangements.append(self.blocCommunSource)
                else :
                    nonDefinis.append((finSource,nDefSource[1]))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((finSource,nDefSource[1]))
                    self.listeChangements.append(nDefSource)
                            
            else :
                if finSource == nDefSource[1]:
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.remove(nDefSource)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append(nDefSource)
                else:
                    nonDefinis.remove(nDefSource)
                    nonDefinis.append((nDefSource[0],debSource))
                    nonDefinis.append((finSource,nDefSource[1]))                                  # Modifiee 20.03.2019, ancienne ligne:  nonDefinis.append((fin,nDefSource[1]))
                    nonDefinis.sort()
                    self.listeChangements.append(nDefSource)
                    self.listeChangements.append((nDefSource[0],debSource))
                    self.listeChangements.append((finSource,nDefSource[1]))
                
            if nDefCible[0] == debCible :
                if finCible == nDefCible[1]:
                    nonDefinis.remove(blocComCible)
                    self.listeChangements.append(blocComCible)
                else :
                    nonDefinis.append((finCible,nDefCible[1]))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((finCible,nDefCible[1]))
                    self.listeChangements.append(nDefCible)
                    
            else :
                if finCible == nDefCible[1]:
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.remove(nDefCible)
                    nonDefinis.sort()
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append(nDefCible)
                else:
                    nonDefinis.remove(nDefCible)
                    nonDefinis.append((nDefCible[0],debCible))
                    nonDefinis.append((finCible,nDefCible[1]))
                    nonDefinis.sort()
                    self.listeChangements.append(nDefCible)
                    self.listeChangements.append((nDefCible[0],debCible))
                    self.listeChangements.append((finCible,nDefCible[1]))
            
            blocsCommuns.append(self.blocCommunSource)
            blocsCommuns.append(blocComCible)
            self.listeChangements.append(self.blocCommunSource)
            self.listeChangements.append(blocComCible)
            self.listeChangements = list(set(self.listeChangements))
            self.listeChangements.sort()
            blocsCommuns.sort()
            self.checkBlocsCommuns()
            # print("74. Module: Donnees.resultatAppli, Classe: Resultat, Methode: ajouterBlocCommun, Ligne: 1959 ---> self.listeChangements=\n[", self.listeChangements, "], \nblocsCommuns=\n[", blocsCommuns, "] ")  

    def SupprimerInsertions(self):
        # print("75. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerInsertions, Ligne: 1962 ---> self._li=\n[", self._li, "], ") 
        for i in range(len(self._li)) :
            x=(self._li[0][0]-self.getLgSource(),self._li[0][1]-self.getLgSource())
            self.enleverChangement(x,2)
        # print("76. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerInsertions, Ligne: 1966 ---> self._li=\n[", self._li, "], ") 

    def SupprimerSuppressions(self):
        # print("77. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerSuppressions, Ligne: 1969 ---> self._ls=\n[", self._ls, "], ") 
        for i in range(len(self._ls)) :
            self.enleverChangement(self._ls[0],1)
        # print("78. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerSuppressions, Ligne: 1972 ---> self._ls=\n[", self._ls, "], ") 
    def SupprimerRemplacements(self):
        # print("79. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerRemplacements, Ligne: 1974 ---> self._lr=\n[", self._lr, "], ") 
        for i in range(len(self._lr)//2):                                   # Modifiee 24.06.2019, ancienne ligne:   for i in range(len(self._lr)/2) :                                                          
            self.enleverChangement(self._lr[0],1)
        # print("80. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerRemplacements, Ligne: 1977 ---> self._lr=\n[", self._lr, "], ") 
    def SupprimerDeplacements(self):
        # print("81. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerDeplacements, Ligne: 1979 ---> self._ld=\n[", self._ld, "], ") 
        while len(self._ld) != 0 :
            self.enleverChangement(self._ld[0],1)
        # print("82. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerDeplacements, Ligne: 1982 ---> self._ld=\n[", self._ld, "], ") 
    def SupprimerBlocsCommuns(self):
        # print("83. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerBlocsCommuns, Ligne: 1984 ---> self._blocsCom=\n[", self._blocsCom, "], ")
        for i in range(len(self._blocsCom)//2):                             # Modifiee 24.06.2019, ancienne ligne:  for i in range(len(self._blocsCom)/2) :
            self.enleverChangement(self._blocsCom[0],1)
        # print("84. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerBlocsCommuns, Ligne: 1987 ---> self._blocsCom=\n[", self._blocsCom, "], ")

    def SupprimerToutesTransformations(self):
        # print("85. Module: Donnees.resultatAppli, Classe: Resultat, Methode: SupprimerToutesTransformations, Ligne: 1990 ---> ")
        self.SupprimerDeplacements()
        self.SupprimerInsertions()
        self.SupprimerSuppressions()
        self.SupprimerRemplacements()
        self.SupprimerBlocsCommuns()
        
class Resultat__(Resultat): #buggé 
    def __init__(self, pLI, pLS, pLD, pLR, pLgSource, pTextes, pBlocsCommuns, pairesBlocsDepl):
        Resultat.__init__(self,[], [], [], [], pLgSource, pTextes, [], [])        # Modifiee 28.02.2019, ancienne ligne:  ResultatListe.__init__(self,[], [], [], [], pLgSource, pTextes, [], []) 
        self._li = self.__listeToTab(pLI) 
        self._ls = self.__listeToTab(pLS) 
        self._ld = self.__listeToTab(pLD) 
        self._lr = self.__listeToTab(pLR) 
        self._blocsCom = self.__listeToTab(pBlocsCommuns)   
        self._pairesBlocsDepl = self.__listeToTab(pairesBlocsDepl) 
        
    def __listeToTab(self, liste):
        if isinstance(liste,list):
            tab = numpy.array(liste)#, Numeric.Int16)
        else: tab = liste # c deja un array
        return tab
        
    def getListeInsertions(self):
        return self._li.tolist()
    def getListeSuppressions(self):
        return self._ls.tolist()
    def getListeDeplacements(self):
        return self._ld.tolist()
    def getListeDeplacementsT1(self):
        return self._filtre(self._ld,0,self._lgTexteS)
    def getListeDeplacementsT2(self):
        return self._filtre(self._ld,self._lgTexteS,len(self._textes))
    def getListeRemplacements(self):
        return self._lr.tolist()
    def getListeRemplacementsT1(self):
        return self._filtre(self._lr,0,self._lgTexteS)
    def getListeRemplacementsT2(self):
        return self._filtre(self._lr,self._lgTexteS,len(self._textes))
    def getBlocsCommuns(self):
        return self._blocsCom.tolist()
    def getBlocsCommunsT1(self):
        return self._filtre(self._blocsCom,0,self._lgTexteS)
    def getBlocsCommunsT2(self):
        return self._filtre(self._blocsCom,self._lgTexteS,len(self._textes))        
    def getPairesBlocsDeplaces(self):
        return self._pairesBlocsDepl.tolist()
    def setPairesBlocsDeplaces(self,liste):
        self._pairesBlocsDepl = self.__listeToTab(liste)
       
class Informations(object):
    def __init__(self, pVersionSource, pEtatSource, pVersionCible, pEtatCible,
                 pParametres, pTransformations, pCommentaire = None):
        """ pre: isinstance(pVersionSource,str) and isinstance(pEtatSource,str) 
                 isinstance(pVersionCible,str) and isinstance(pEtatCible,str) 
                 isinstance(pParametres,planTravail.Parametres)  
                 isinstance(pTransformations,Transformations) 
                 (pCommentaire==None or isinstance(pCommentaire,Commentaire)) """
        self.__versionSource = pVersionSource
        self.__etatSource = pEtatSource 
        self.__versionCible = pVersionCible
        self.__etatCible = pEtatCible
        self.__parametres = pParametres
        self.__transformations = pTransformations
        self.__commentaire = pCommentaire
        # print("90. Module: Donnees.resultatAppli, Classe: Informations, Methode: __init__, Ligne: 2055 ---> \nself.__versionSource=", self.__versionSource, ", self.__etatSource=", self.__etatSource, ", self.__versionCible=", self.__versionCible, ", self.__etatCible=", self.__etatCible, ", \nself.__parametres=", str(self.__parametres), ", \nself.__transformations=", self.__transformations, ", \nbself.__commentaire=", self.__commentaire, " ") 


    # Methodes d'acces aux attributs
    def getVersionSource(self):
        # print("91. Module: Donnees.resultatAppli, Classe: Informations, Methode: getVersionSource, Ligne: 2060 ---> self.__versionSource=", self.__versionSource, " ")
        return self.__versionSource
    def getEtatSource(self):
        # print("92. Module: Donnees.resultatAppli, Classe: Informations, Methode: getEtatSource, Ligne: 2063 ---> self.__etatSource=", self.__etatSource, " ")
        return self.__etatSource
    def getVersionCible(self):
        # print("93. Module: Donnees.resultatAppli, Classe: Informations, Methode: getVersionCible, Ligne: 2066 ---> self.__versionCible=", self.__versionCible, " ")
        return self.__versionCible
    def getEtatCible(self):
        # print("94. Module: Donnees.resultatAppli, Classe: Informations, Methode: getEtatCible, Ligne: 2069 ---> self.__etatCible=", self.__etatCible, " ")
        return self.__etatCible
    def getParametres(self):
        # print("95. Module: Donnees.resultatAppli, Classe: Informations, Methode: getParametres, Ligne: 2072 ---> self.__parametres=", str(self.__parametres), " ")
        return self.__parametres
    def getTransformations(self):
        # print("96. Module: Donnees.resultatAppli, Classe: Informations, Methode: getTransformations, Ligne: 2075 ---> self.__transformations=", self.__transformations, " ")
        return self.__transformations
    def getCommentaire(self):
        # print("97. Module: Donnees.resultatAppli, Classe: Informations, Methode: getCommentaire, Ligne: 2078 ---> self.__commentaire=", self.__commentaire, " ")
        return self.__commentaire
    
    def existeCommentaire(self):
        """ Verifie si le commentaire existe """
        # print("98. Module: Donnees.resultatAppli, Classe: Informations, Methode: existeCommentaire, Ligne: 2083 ---> self.__commentaire=", self.__commentaire, " ")
        if self.__commentaire == None:
            return 1    ## FAUX
        else:
            return 0    ## VRAI
    def existeTransformations(self):
        """ Verifie si les transformations existent """
        # print("99. Module: Donnees.resultatAppli, Classe: Informations, Methode: existeTransformations, Ligne: 2090 ---> self.__transformations=", self.__transformations, " ")
        if self.__transformations == None:
            return 1    ## FAUX
        else:
            return 0    ## VRAI        
    # Methode de modification d'attribut
    # Seul le commentaire est modifiable
    def setCommentaire(self, pCommentaire):
        self.__commentaire = pCommentaire
        # print("100. Module: Donnees.resultatAppli, Classe: Informations, Methode: setCommentaire, Ligne: 2099 ---> self.__commentaire=", self.__commentaire, " ")
        
        
class Transformations(object):
    def __init__(self, pInsertions, pSuppressions, pDeplacements, pRemplacements, pTexteSource, pTexteCible):
        ## les attributs ci-dessous sont des listes de sequence de caracteres
        self.__insertions =pInsertions
        self.__suppressions =pSuppressions
        self.__deplacements = pDeplacements
        self.__remplacements = pRemplacements
        ## les attributs ci-dessous sont des noeuds xml
        self.__texteSource = pTexteSource
        self.__texteCible = pTexteCible
        # print("101. Module: Donnees.resultatAppli, Classe: Transformations, Methode: __init__, Ligne: 2112 ---> \nself.__insertions=", self.__insertions, ", \nself.__suppressions=", self.__suppressions, ", \nself.__deplacements=", self.__deplacements, ", \nself.__remplacements=", self.__remplacements, ", \nself.__texteSource=", str(self.__texteSource), ", \nself.__texteCible=", str(self.__texteCible), " ") 

    def getInsertions(self):
        # print("102. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getInsertions, Ligne: 2115 ---> \nself.__insertions=", self.__insertions, " ")
        return self.__insertions

    def getSuppressions(self):
        # print("103. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getSuppressions, Ligne: 2119 ---> \nself.__suppressions=", self.__suppressions, " ")
        return self.__suppressions

    def getDeplacements(self):
        # print("104. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getDeplacements, Ligne: 2123 ---> \nself.__deplacements=", self.__deplacements, " ")
        return self.__deplacements

    def getRemplacements(self):
        # print("105. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getRemplacements, Ligne: 2127 ---> \nself.__remplacements=", self.__remplacements, " ")
        return self.__remplacements

    def getSource(self):
        # print("106. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getSource, Ligne: 2131 ---> \nself.__texteSource=", str(self.__texteSource), " ")
        return self.__texteSource

    def getCible(self):
        # print("107. Module: Donnees.resultatAppli, Classe: Transformations, Methode: getCible, Ligne: 2135 ---> \nself.__texteCible=", str(self.__texteCible), " ")
        return self.__texteCible
        
    def toString(self):
        s = u"Type TRANSFORMATION\nInsertions : "
        for i in self.__insertions:
            s += i+u" / "
        s += u"\nSuppressions : "
        for i in self.__suppressions:
            s += i+u" / "
        s += u"\nDéplacements : "                                       # Modifiee 28.02.2019, ancienne ligne: s += u"\nD�placements : " 
        for i in self.__deplacements:
            s += i+u" / "
        s += "\nRemplacements  : "
        if self.__remplacements:
            for i in self.__remplacements:
                s += str(i).encode("iso-8859-2", "replace")+" / "                        # Modifiee 28.02.2019, ancienne ligne:   s += unicode(i).encode("iso-8859-2", "replace")+" / " 
        s += "\nTexteSource : "
        #s += unicode(self.__texteSource.toxml()).encode("iso-8859-2", "replace")
        s2,x,y = Utile.decodage.decode_string_heuristically(self.__texteSource.toxml())
        s += s2
#       if self.__texteSource != []:
#         for i in self.__texteSource:
#           s += unicode(i).encode("iso-8859-2", "replace")+" / "    
        s += "\nTexteCible : "
        s2,x,y = Utile.decodage.decode_string_heuristically(self.__texteCible.toxml())
        s += s2
#       s += unicode(self.__texteCible).encode("iso-8859-2", "replace")
#       if self.__texteCible != []:
#         for i in self.__texteCible:
#           s += unicode(i).encode("iso-8859-2", "replace")+" / "    
        sys.stderr.write(str(s).encode("iso-8859-2", "replace"))                         # Modifiee 28.02.2019, ancienne ligne:  sys.stderr.write(unicode(s).encode("iso-8859-2", "replace")) 
        sys.stderr.flush()
        # print("108. Module: Donnees.resultatAppli, Classe: Transformations, Methode: toString, Ligne: 2168 ---> s=\n[", s, "] ")
        
class Commentaire(object):
    def __init__(self, pCommentaire):
        self.__commentaire = pCommentaire
        # print("109. Module: Donnees.resultatAppli, Classe: Commentaire, Methode: __init__, Ligne: 2173 ---> self.__commentaire=\n[", self.__commentaire, "] ")
    # Methode d'acces a l'attribut
    def getCommentaire(self):
        # print("110. Module: Donnees.resultatAppli, Classe: Commentaire, Methode: getCommentaire, Ligne: 2176 ---> self.__commentaire=\n[", self.__commentaire, "] ")
        return self.__commentaire
    # Methodes de modification de l'attribut
    # Methode qui remplace le commentaire
    def setCommentaire(self, pCommentaire):
        self.__commentaire = pCommentaire
        # print("111. Module: Donnees.resultatAppli, Classe: Commentaire, Methode: setCommentaire, Ligne: 2182 ---> self.__commentaire=\n[", self.__commentaire, "] ")
    # Methode qui ajoute du texte au commentaire
    def addCommentaire(self, pCommentaire):
        # print("112. Module: Donnees.resultatAppli, Classe: Commentaire, Methode: addCommentaire, Ligne: 2185 ---> self.__commentaire=\n[", self.__commentaire, "] ")
        self.__commentaire += ' ' + pCommentaire
        # print("113. Module: Donnees.resultatAppli, Classe: Commentaire, Methode: addCommentaire, Ligne: 2185 ---> self.__commentaire=\n[", self.__commentaire, "] ")

        
