# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#import la lib hmm ici
import sys,logging
import hmmlearn                          # Ajoutee 25.02.2019. 
from hmmlearn import hmm

# from hmmlearn import hmm                 # Ajoutee 26.02.2019. 
from decimal import *
#from numpy import exp,log,seterr
#from math import log as mlog
from math import exp, log                  # Modifiee 01.03.2019, ancienne ligne:  from math import exp
#from numpy import Infinity
import numpy as np
from numpy import *                    
from datetime import datetime
import platform
if platform.system().lower() == 'linux':
    from hmmlearn import *                             # Modifiee 04.03.2019, ancienne ligne:  from ghmm import * 
elif platform.system().lower() == 'windows':       # Ajoutee 01.03.2019. 
    from hmmlearn import *                         # Ajoutee 01.03.2019. 

def mlog(x): return x
def log(x):
    try:
        return mlog(x)
    except OverflowError:
        #si x = 0
        return -1 * Infinity
#   if x !=0 :
#       return mlog(x)
#   else :
#       return -Infinity
        

class algoHMM:
    """classe effectuant l'algorithme HMM viterbi pour l'algorithme de Feng & Manmatha
    """ 
    def __init__(self,termes1,termes2,debugLevel=1):
        """Constructeur
        @param termes1,termes2: la liste des blocs que l'on va comparer
        @type termes1,termes2: [bloc(val,index),...] (au moins les champs val et index doivent etre renseignés) 
        @param debugLevel : voir feng.py, optionnel
        """
        #Definition des constantes:
        self.__k1 = 0.001
        self.__logk1 = mlog(self.__k1)
        self.__k2 = 0.8
        self.__logk2 = mlog(self.__k2)
        self.__lambda = 1.04617083
        #self.__lambda = 1.04617082
        self.__mu1 = 0.99
        self.__logmu1 = mlog(self.__mu1)
        self.__mu2 = 0.0001
        self.__logmu2 = mlog(self.__mu2)
        self.__logZero = 0#-Infinity
        self.__logUn = mlog(1)
        logging.basicConfig(level=debugLevel,datefmt='%a, %d %b %Y %H:%M:%S',format='%(asctime)s %(levelname)s %(message)s')#,filename='hmm.log',filemode='w')
        #TODO : DEFINIR LE FORMAT D ENTREE ([(mot/hash,pos),...])
        # G liste des positions dans le texte1 de taille m
        self.listeTexte1 = [(termes1.index(x),x.index,x.val) for x in termes1]
        #O liste des positions dans le texte2 de taille n
        self.listeTexte2 = [(termes2.index(x),x.index,x.val) for x in termes2]

#       if len(self.listeTexte2)>len(self.listeTexte1) :
#           tmp = self.listeTexte1[:]
#           self.listeTexte1 = self.listeTexte2[:]
#           self.listeTexte2 = tmp[:]

        
        #par defaut numpy affiche un warning pour log(0), mais renvoie quand meme -infini
        #on ne souhaite pas afficher ce warning
#       e = seterr(divide='ignore')
#       e=seterr(**e)
#       seterr(**e) #bizarre, il faut le faire 2 fois ...

        
    def calculeAlignementPy(self):
        """
        Lance l'alignement HMM, ancienne version en python pur, lente car python
        @return la sequence de termes sur laquelle on aligne 
        @rtype [[posListe1,posListe2]...]
        """
        #print ("Debut viterbi :",len(self.listeTexte1),len(self.listeTexte2))    # Modifiee 26.02.2019, ancienne ligne:  #print "Debut viterbi :",len(self.listeTexte1),len(self.listeTexte2) 
        #print (self.listeTexte1,"\n____\n",self.listeTexte2)                     # Modifiee 26.02.2019, ancienne ligne:  #print self.listeTexte1,"\n____\n",self.listeTexte2
        logging.log(2,"Appel de l'alignement HMM")
        viterbi = self.__viterbi()[1]
        res=[]
        if viterbi == None : res = []
        else:
            for index in range(len(viterbi)-1):                # Modifiee 19.02.2019, ancienne ligne:  for index in xrange(len(viterbi)-1):
                res.append((self.listeTexte1[viterbi[index]][1],self.listeTexte2[index][1]))

        logging.log(1,"liste de termes 1 : %s",self.listeTexte1)
        logging.log(1,"liste de termes 2 : %s",self.listeTexte2)
        logging.log(1,"Termes alignés par le HMM : %s",res)
        logging.log(2,"Fin de l'alignement HMM")
        return res
        
    def calculeAlignement(self):
        """ 
        Calcul l'alignement de Viterbi avec la librairie GHMM
        @return liste de couples apparies entre texte 1 et texte 2 
        @rtype liste de couples (index texte 1, index texte 2) 
        """
        # print("1_1 Module--->HMM, ligne 122, liste de termes 1 : %s",self.listeTexte1)
        # print("1_2 Module--->HMM, ligne 123, liste de termes 2 : %s",self.listeTexte2)
        # print("1_3 Module--->HMM, ligne 124, Debut de l'alignement HMM")
        #matrice A des probas de transition
        if self.listeTexte1 == [] or self.listeTexte2 == [] : return []
        # sigma = liste/alphabet des etats possibles 
        sigma =  np.arange(0,max(len(self.listeTexte1),len(self.listeTexte2)))     # !!!-Important-!!! Modifiee 25.02.2019, ancienne ligne:  sigma =  IntegerRange(0,max(len(self.listeTexte1),len(self.listeTexte2))) 
        # A: matrice de transition
        A=[] 
        sumtemp = 0.0
        # print("Sommes d'elements de la ligne pour les lignes de la matrice des probabilites de transition A[] ---> \n")
        for i in range(len(self.listeTexte1)):               # Modifiee 25.02.2019, ancienne ligne:  for i in xrange(len(self.listeTexte1)):
            ATmp=[]
            for j in range(len(self.listeTexte1)):          # Modifiee 25.02.2019, ancienne ligne:  for j in xrange(len(self.listeTexte1)): 
                ATmp.append(self.__probaTransition(j, i))
                #print("Probabilite de transition: ", "i=", "%4.0f" %i, "  j=", "%4.0f" %j, "  proba_transition=", "%2.10f" %self.__probaTransition(j, i))
            list(ATmp)                                                                                                                                          # Ajoutee 09.05.2019
            sumtemp = 0.0 
            sumtemptotal = 0.0                                                                                                                                       # Ajoutee 09.05.2019
            if i < len(self.listeTexte1) - 1:
                for j in range(len(self.listeTexte1)):                                                                                                              # Ajoutee 09.05.2019
                    if j != i + 1: 
                        sumtemp += ATmp[j] 
                        #print("i=" + str(i), " j=" + str(j),  "ATmp[j]=", "%2.10f" %ATmp[j],  "sumtemp=", "%2.10f" %sumtemp)
                    sumtemptotal += ATmp[j]
                    #print("i=" + str(i), " j=" + str(j),  "ATmp[j]=", "%2.10f" %ATmp[j],  "sumtemptotal=", "%2.10f" %sumtemptotal)
                ATmp[i + 1] = 1.0 - sumtemp
            elif i == len(self.listeTexte1) - 1:
                ATmp[len(self.listeTexte1) - 1] = 1.0                                                                           # Ajoutee 09.05.2019
            # print("Ancienne somme d'elements de la ligne:  i=" + str(i), "  sumtemp(sauf element (" + str(i) + ", " + str(i+1) + "))=" + "%6.10f" %sumtemp, "somme complet=" + "%6.10f" %sumtemptotal)                                                                                              # Ajoutee 09.05.2019
            #for j in range(len(self.listeTexte1)):                                                                                                              # Ajoutee 09.05.2019
                #if len(self.listeTexte1) > 1 and i < (len(self.listeTexte1) - 1) and j == i + 1: 
                    #ATmp[j] = 1.0 - sumtemp     
                    #print("i=" + str(i), " j=" + str(j),  " ATmp[j]=" + "%6.10f" %ATmp[j])                                    # Ajoutee 09.05.2019
            #if len(self.listeTexte1) >= 1 and i == len(self.listeTexte1) - 1: 
                #ATmp[len(self.listeTexte1) - 1] = 1.0  
                #print("i=" + str(i), " j=" + str(j),  " ATmp[j]=" + "%2.10f" %ATmp[j])                                             # Ajoutee 09.05.2019
            #if len(self.listeTexte1) > 2 and i == len(self.listeTexte1) - 1: ATmp[len(self.listeTexte1) - 1] = 1.                                               # Ajoutee 09.05.2019
            sumtempnew = 0.0                                                                                                                                     # Ajoutee 09.05.2019
            for j in range(len(self.listeTexte1)):                                                                                                              # Ajoutee 09.05.2019
                sumtempnew += ATmp[j]                                                                                                                           # Ajoutee 09.05.2019
            # print("Nouvelle somme d'elements de la ligne:  i=" + str(i), "  sumtempnew=" + "%2.10f" %sumtempnew)                                                                                        # Ajoutee 09.05.2019
            tuple(ATmp)                                                                                                                                         # Ajoutee 09.05.2019
            A.append(ATmp)
            #sumtemp = 0.
            #for j in range(len(self.listeTexte1)):
            #    if j != i + 2: sumtemp += A[i, j]
            #A[i, i + 2] = 1. - sumtemp
        # print("****************************************************************************************************\n")
        #matrice B des probas d'emissions
        B=[]
        for i in range(len(self.listeTexte1)):             # Modifiee 25.02.2019, ancienne ligne:  for i in xrange(len(self.listeTexte1)): 
            BTmp=[]
            for j in range(len(self.listeTexte2)):        # Modifiee 25.02.2019, ancienne ligne:  for j in xrange(len(self.listeTexte2)):
                BTmp.append(self.__probaEmmision(self.listeTexte2[j][2], i))
            #la proba d'emission est 0 pour les termes1 non présents dans le texte2
            # cf le code de ghmm
            for k in range(len(self.listeTexte1)-len(self.listeTexte2)):        # Modifiee 25.02.2019, ancienne ligne:  for k in xrange(len(self.listeTexte1)-len(self.listeTexte2)):
                BTmp.append(0)
            B.append(BTmp)
        
        #matrice pi des proba initiales
        PI = [0]*len(self.listeTexte1)
        if len(PI) > 0 : PI[0]=1 # éta 0, proba intiale de 1, les autres de 0
# ----------------------------------------------------------------------------------------------------------------------------------- 
        # print("\n****************************************************************************************************")    
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 21 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")  
        # print("Longueur des sequences des blocs d'entree et de sortie avant Viterbi--->\n****************************************************************************************************\n")      
        # print("len(self.listeTexte1) = " + str(len(self.listeTexte1))) 
        # print("len(self.listeTexte2) = " + str(len(self.listeTexte2)))         
        # print("****************************************************************************************************")
        # print("sigma --->\n" + str(sigma))                                                       # Ajoutee 18.04.2019
        # print("****************************************************************************************************")
        AA = np.array(A)
        # print ("Matrix A ---> A["+("%d" %AA.shape[0])+"]["+("%d" %AA.shape[1])+"]")
        rowsA = AA.shape[0]
        colsA = AA.shape[1]
        # print("rowsA = " + str(rowsA) + ",  colsA = " + str(colsA) + ".  ")
        # print("\n****************************************************************************************************")  
        # print("AA ---> \n" + str(AA))
        # print("****************************************************************************************************")  
        # print("****************************************************************************************************")
        SumLigneA = []
        for ia in range(0,rowsA): 
            SumLigneValA = 0.
            for ja in range(0,colsA):
                SumLigneValA += AA[ia,ja]
                #print("%2.10f" %AA[ia,ja],  "%2.10f" %SumLigneValA)
            SumLigneA.append(SumLigneValA)
            #print("****************************************************************************************************")
        #print("****************************************************************************************************")  
        SumColonneA = []
        for ia in range(0,colsA): 
            SumColonneValA = 0.
            for ja in range(0,rowsA):
                SumColonneValA += AA[ja,ia]
                #print("%2.10f" %AA[ja,ia],  "%6.10f" %SumColonneValA)
            SumColonneA.append(SumColonneValA)
            #print("****************************************************************************************************")
        #print("****************************************************************************************************") 
            #print("%6.20f" %SumLigneA[ia])
        # print("\n****************************************************************************************************")    
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 22 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")       
        # print("Sommes des probabilites cumulees dans chaque ligne A --->\n****************************************************************************************************\n") 
        # for iia in range(len(SumLigneA)): 
            # print("%2.10f" %SumLigneA[iia])
        # print("****************************************************************************************************")   
        # print("Sommes des probabilites cumulees dans chaque colonne A --->\n****************************************************************************************************\n") 
        # for iia in range(len(SumColonneA)): 
            # print("%2.10f" %SumColonneA[iia])
        # print("\n****************************************************************************************************")   
        #print("Matrice des probabilites de transition A[] avant Viterbi --->\n****************************************************************************************************\n" + str(np.matrix(A)))        
        # print("Matrice des probabilites de transition A[] avant Viterbi --->\n****************************************************************************************************\n")
        # for iiia in range(0, rowsA):
            # print(["{:2.10f}".format(AA[iiia, jjja]) for jjja in range(0, colsA)])
        # print("****************************************************************************************************")
        # print("****************************************************************************************************")
        BB = np.array(B)
        # print ("Matrix B ---> B["+("%d" %BB.shape[0])+"]["+("%d" %BB.shape[1])+"]")
        rowsB = BB.shape[0]
        colsB = BB.shape[1]
        # print("rowsB = " + str(rowsB) + ",  colsB = " + str(colsB) + ".  ")
        # print("****************************************************************************************************")  
        # print("BB ---> \n" + str(BB))
        # print("****************************************************************************************************")  
        SumLigneB = []
        for ib in range(0,rowsB): 
            SumLigneValB = 0.
            for jb in range(0,colsB):
                SumLigneValB += BB[ib,jb]
            SumLigneB.append(SumLigneValB)
        # print("\n****************************************************************************************************")
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 23 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")
        # print("Sommes des probabilites cumulees dans chaque ligne B --->\n****************************************************************************************************\n")        # Ajoutee 20.03.2019.  ne marche pas:  str(np.matrix(np.array_str(A, 36, precision=3)))
        # for iib in range(len(SumLigneB)): 
            # print("%2.10f" %SumLigneB[iib])
        # print("\n****************************************************************************************************")
        #print("Matrice des probabilites d'emissions B[] avant Viterbi --->\n****************************************************************************************************\n" + str(np.matrix(B)))  
        # print("Matrice des probabilites d'emissions B[] avant Viterbi --->\n****************************************************************************************************\n")          # Ajoutee 20.03.2019.
        # for iiib in range(0, rowsB):
            # print(["{:2.10f}".format(BB[iiib, jjjb]) for jjjb in range(0, colsB)])
        # print("****************************************************************************************************")
        # print("****************************************************************************************************")
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 24 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")
        # print("Matrice des probabilites initiales pi avant Viterbi (1)--->\n****************************************************************************************************\n" + str(np.matrix(PI)))            # Ajoutee 20.03.2019.
        # print("****************************************************************************************************")
        # print("Matrice des probabilites initiales pi avant Viterbi (2)--->\n****************************************************************************************************")
        #print("****************************************************************************************************")
        PIPI = np.array(PI)
        #print ("Matrix PI ---> PI["+("%d" %PIPI.shape[0])+"]["+("%d" %PIPI.shape[1])+"]")
        #rowsPI = PIPI.shape[0]
        #colsPI = PIPI.shape[1]
        #print("rowsPI = " + str(rowsPI) + ",  colsPI = " + str(colsPI) + ".  ")
        # print("****************************************************************************************************")  
        # print("PIPI ---> \n" + str(PIPI) + " \n  et sous une forme de colonne: ")
        # for iiipipi in range(0, len(PIPI)): 
            # print("{:2.10f}".format(PIPI[iiipipi])) 
        # print("****************************************************************************************************")  
        #SumLignePI = []
        #for ipi in range(0,rowsPI): 
        #    SumLigneValPI = 0.
        #    for jpi in range(0,colsPI):
        #        SumLigneValPI =+ PIPI[jpi,ipi]
        #    SumLignePI.append(SumLigneValPI)
        # print("\n****************************************************************************************************")
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 25 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")
        # print("\nSommes des probabilites cumulees dans la ligne PI (1)--->\n****************************************************************************************************")        # Ajoutee 20.03.2019.  ne marche pas:  str(np.matrix(np.array_str(A, 36, precision=3)))
        #for iipi in range(len(SumLignePI)): 
        #    print("%2.10f" %SumLignePI[iipi])
        sumPI=0
        for iipi in range(0, len(PI)):
            sumPI += PI[iipi]
        #print("Sommes des probabilites cumulees dans la ligne PI (2) --->\n****************************************************************************************************\n") 
        # print("\nsumPI="+str(sumPI))
        #print("\n****************************************************************************************************")
        #print("Matrice des probabilites initiales pi[] avant Viterbi --->\n****************************************************************************************************\n" + str(np.matrix(B)))  
        #print("Matrice des probabilites initiales pi[] avant Viterbi --->\n****************************************************************************************************\n")          # Ajoutee 20.03.2019.
        #for iiipi in range(0, rowsPI):
        #    print(["{:2.10f}".format(PIPI[iiipi, jjjpi]) for jjjpi in range(0, colsPI)])
        # print("****************************************************************************************************")        
        # print("Sequences des blocs d'entree et de sortie avant Viterbi--->\n****************************************************************************************************")        
        # print ("\nself.listeTexte1 avant Viterbi ---> \n" + str(self.listeTexte1))                                # Ajoutee 20.03.2019 
        # print ("\nself.listeTexte2 avant Viterbi ---> \n" + str(self.listeTexte2))                                # Ajoutee 20.03.2019 
        #print("\nSequence d'entree  self.listeTexte2[0] avant Viterbi--->")                                     # Ajoutee 09.05.2019      
        #print ("\nself.listeTexte2 avant Viterbi ---> \n" + str(self.listeTexte2[0]))                             # Ajoutee 09.05.2019         
        # print("****************************************************************************************************")
# -----------------------------------------------------------------------------------------------------------------------------------        
#         m = HMMFromMatrices(sigma, DiscreteDistribution(sigma), A, B, pi)
#       print (m)            # Modifiee 26.02.2019, ancienne ligne:  print m
#         test = EmissionSequence(sigma,[x[0] for x in self.listeTexte2])
#         v = m.viterbi(test)
# -----------------------------------------------------------------------------------------------------------------------------------
# Tests avec librairie hmmlearn. 
        model = hmm.MultinomialHMM(n_components=len(self.listeTexte1))
        model.startprob_ = PI
        model.transmat_ = A
        model.emissionprob_ = B
        
        logprob, vitseq = model.decode(np.array([[x[0] for x in self.listeTexte2]]).T, algorithm = "viterbi")
        # print("Sequence de sortie self.listeTexte1[0] apres Viterbi--->\n")                                     # Ajoutee 09.05.2019
        # print ("self.listeTexte1 apres Viterbi ---> \n" + str(vitseq))                             # Ajoutee 09.05.2019                
        # print("****************************************************************************************************")    
        # print("!!! "+datetime.now().strftime("%d/%m/%Y - %H:%M:%S")+" !!!"+"   Affichage N° 26 HMM.algoHMM.calculeAlignement pour debogage")
        # print("****************************************************************************************************")  
        # print("\nSequences des blocs d'entree et de sortie apres Viterbi--->\n****************************************************************************************************\n")   
        # print ("self.listeTexte1 apres Viterbi ---> \n{" + str(self.listeTexte1) + "} ")            # Ajoutee 20.03.2019. 
        # print ("\nself.listeTexte2 apres Viterbi ---> \n{" + str(self.listeTexte2) + "} ")            # Ajoutee 20.03.2019. 
        #print ("\nprob apres viterbi(format str(np.exp(logprob))) ---> {" + str(np.exp(logprob)) + "} ")                           # Ajoutee 20.03.2019. 
        # print ("\nprob apres viterbi(format str(logprob) sans exp) ---> {" + str(logprob) + "} \n")
        # print("\nprob apres viterbi(format {:2.50f}.format(logprob) sans exp) ---> {", "{:2.50f}".format(logprob), "} \n")
        print (vitseq)                                                                              # Modifiee 26.02.2019, ancienne ligne:   # print v
        viterbiseq = vitseq                                                                         # Modifiee, ancienne ligne:  viterbi = v[0]
        res=[]
        if viterbiseq.any() == None : res = []                                                      # Modifiee 07.05.2019, ancienne ligne:  if viterbi == None : res = []
        else:
            for index in range(len(viterbiseq)):                                                      # Modifiee 20.03.2019, ancienne ligne:  for index in xrange(len(viterbi)): 
                res.append((self.listeTexte1[viterbiseq[index]][1],self.listeTexte2[index][1]))       # Modifiee 20.03.2019, ancienne ligne:  res.append((self.listeTexte1[viterbi[index]][1],self.listeTexte2[index][1]))
        # print ("\nprob (resultats) (format str sans exp)---> {" + str(logprob) + "} ") 
        # print ("\nprob (resultats) (format repr sans exp) ---> {", repr(logprob), "} \n")
        # print("****************************************************************************************************")
        # print("\nprob (resultats) (format {:2.50f}.format(np.exp(logprob) ---> {", "{:2.50f}".format(np.exp(logprob)), "} \n")
        # print("****************************************************************************************************")
        # print ("\nself.listeTexte1 (resultats) ---> \n{" + str(self.listeTexte1) + "} ")          # Modifiee 26.02.2019, ancienne ligne:  print self.listeTexte1
        # print ("\nself.listeTexte2 (resultats) ---> \n{" + str(self.listeTexte2) + "} ")          # Modifiee 26.02.2019, ancienne ligne:  print self.listeTexte2
        # print("****************************************************************************************************")
        # print ("\nres (resultats) ---> {" + str(res) + "} ")                       # Modifiee 26.02.2019, ancienne ligne:  print res
        # print("\n****************************************************************************************************")
        # print("****************************************************************************************************\n") 
        #sys.exit(0)
        # res: liste de couples appariés entre texte 1 et texte 2
        print("1-10 Module--->HMM, ligne 354, liste de termes 1 : %s",self.listeTexte1)
        print("1-11 Module--->HMM, ligne 355, liste de termes 2 : %s",self.listeTexte2)
        print("1-12 Module--->HMM, ligne 356, Termes alignés par le HMM : %s",res)
        print("1-13 Module--->HMM, ligne 357, Fin de l'alignement HMM")
        return res  
    ##########
    #Methodes privées
    ###########
    
    
    def __probaTransition(self, etatSuivant, etat):
        """P(Si|Si-1), et etat1 apres etat2
        Partie 2.1 => (3)
        """
        if etatSuivant < etat : res = self.__logZero
        elif etatSuivant == etat : res = 0.001
        elif etatSuivant - etat == 1 : res = 0.8    
        #elif etatSuivant == etat : res = self.__logk1
        #elif etatSuivant - etat == 1 : res = self.__logk2
        elif etatSuivant - etat > 1 : res = log(self.__lambda * exp (-1 * self.__lambda * (etatSuivant-etat))) 
        #print("-1 * self.__lambda * (etatSuivant-etat): ", "etat=", "%3.0f" %etat, "  etatSuivant=", "%3.0f" %etatSuivant, "  proba_transition=", "%6.30f" %(-1 * self.__lambda * (etatSuivant-etat)))
        #print("exp (-1 * self.__lambda * (etatSuivant-etat)): ", "etat=", "%3.0f" %etat, "  etatSuivant=", "%3.0f" %etatSuivant, "  proba_transition=", "%6.30f" %exp (-1 * self.__lambda * (etatSuivant-etat)))
        # print("9. Module HMM, Methode __probaTransition, ligne 379, Probabilite de transition: ", "etat=", "%3.0f" %etat, "  etatSuivant=", "%3.0f" %etatSuivant, "  proba_transition=", "%6.30f" %res) 
        return res
    
    def __probaEmmision(self, observation, etat):
        """P(Oi|Si)
        Partie 2.1 => (4)
        """
        if observation == self.listeTexte1[etat][2] : res = self.__logmu1
        else : res = self.__logmu2
        return res
    
    def __probaInitiale(self,etat):
        """p(Si)
        """
        if etat == 0 : res = self.__logUn
        else : res = self.__logZero
        return res
#       return len(self.listeTexte1)**(-1)




    def __viterbi(self):
        """Algorithme viterbi en Python, lent
        @return: la proba de l'etat, la séquence d'etats, la probabilité d'apparition de la sequence
        @rtype: decimal,[[posListe1,posListe2]...],decimal
        """     
        T = {}
        for etat in self.listeTexte1:
            #           prob.      V. path  V. prob.
            #T[etat[1]] = (self.__probaInitiale(etat[0]), [etat[0]], self.__probaInitiale(etat[0]))
            T[etat[1]] = ([], [etat[0]], self.__probaInitiale(etat[0]))
        logging.log(1,"Debut viterbi")
        logging.log(1,"T : %s",T)
        if __debug__ :
            cpt = 0
            len2 = len(self.listeTexte2)+1
            argMax=[]       
        for observation in self.listeTexte2:
            U = {}
            for etatSuivant in self.listeTexte1:
                #total = log(0)
                #argMax = None
                valMax = self.__logZero
                for etat in self.listeTexte1:
                    (prob, viterbiSequence, viterbiProb) = T[etat[1]]
                    p = self.__probaEmmision(observation[2], etat[0]) + self.__probaTransition(etatSuivant[1], etat[1])
                    viterbiProb += p
                    #total = log( exp(total)+  exp(p))
                    if viterbiProb > valMax:
                        argMax = viterbiSequence+[etatSuivant[0]]
                        valMax = viterbiProb
#               U[etatSuivant[1]] = (total, argMax, valMax)
                U[etatSuivant[1]] = ([], argMax, valMax)
            if __debug__ :
                cpt=len(argMax)
                logging.log(3, "Viterbi : %0.0f %% Done",float(cpt*len2**-1)*100)
            T = U
        ## apply sum/max to the final states:
        #total = log(0)
        argMax = None
        valMax = self.__logZero
        for etat in self.listeTexte1:
            (prob, viterbiSequence, viterbiProb) = T[etat[1]]
            #total += prob
            if viterbiProb > valMax:
                argMax = viterbiSequence
                valMax = viterbiProb
        logging.log(1,"T final : %s",T)
        # print("10. Module HMM, Methode __viterbi, Ligne 448, T final : \n%s",T)
        return ([], argMax, valMax)
        

    