# -*- coding: iso-8859-1 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import unittest, logging, os, random, time, sys          # Modifiee 15.04.2019, ancienne ligne:  import unittest, contract, logging, os, random, time, cPickle, sys  
import pickle                       # Ajoutee 15.04.2019
#import numpy.numarray as numarray
import numpy
import Controleurs.cInformations, Controleurs.DGManager
import Donnees.resultatAppli
from . import test_data
import math
import MediteApplication.MediteAppli, MediteApplication.synthetic

import Donnees.planTravail

import platform
if platform.system().lower() == 'linux':
    from MediteAppli.mediteFeng import *
    
# vrification des contrats
# contract.checkmod(MediteAppli.MediteAppli)                # Desactivee 15.04.2019
# contract.checkmod(Donnees.resultatAppli)                    # Desactivee 15.04.2019


class SyntheticData(object):
    """ Gnre un texte synthtique  partir d'un texte donn.
    On connait galement l'alignement entre les 2 textes"""
    def __init__(self,texte='ABC', tx=None, seed=None, moves=True, moveOnly=False):
        """texte=texte en entre
        tx=taux de modification
        seed= seed pour initialiser le random
        moves=si true des dplacement sont gnrs par le modle"""
        self.t1 = texte
        self.moves = moves
        self.moveOnly = moveOnly
        self.len_t1 = len(self.t1)
        if tx is not None:
            self.TxIns = self.TxSup = self.TxRemp = self.TxMov = tx
        else:
            self.TxIns = self.TxSup = self.TxRemp = self.TxMov = 0.05
        if not self.moves:  self.TxMov = 0
        if self.moveOnly:
            self.TxIns = self.TxSup = self.TxRemp = 0.0
        self.TxInv = 1 - (self.TxIns + self.TxSup + self.TxRemp + self.TxMov)
        
        if seed is not None:
            random.seed(seed)
        else:            
            random.seed(222)
    
        self.taille_bloc = 100
        self.sep = """ .!\r,\n:\t;-?"'`()"""
        
        self.distrib_taille = 'GAUSS' # gaussienne pour la taille des squences modifiantes
        self.mu = 15 
        self.variance = 10
        
    def getModifiedText(self):
        listePos, len_lPos, lTexte = self._getModifiedTextLN()
        self._getModifiedTextLN2(listePos, len_lPos, lTexte)
        
    def _getModifiedText1(self):
        """Cre le texte modifi et l'alignement"""
        # nobmre de modifications pour chaque opration
        NbIns = int(self.TxIns * self.len_t1) ; NbSup = int(self.TxSup * self.len_t1) 
        NbRemp = int(self.TxRemp * self.len_t1) ; NbMov = int(self.TxMov * self.len_t1)
        # liste stockant les caractres du nouveau texte + les suppresions
        lTexte = list(self.t1)
        # liste stockant les oprations  chaque position dabns les 2 textes
        lPos = [0 for i in range(self.len_t1)]                             # Modifiee 15.04.2019, ancienne ligne:  lPos = [0 for i in xrange(self.len_t1)] 
        len_lPos = self.len_t1
        while NbIns > 0 or NbSup > 0 or NbRemp > 0 or NbMov > 0:
            assert len(lPos) == len(lTexte) == len_lPos
            if NbIns > 0: #INS = 1
                # recherche d'une position vierge
                pos = random.randint(0,len_lPos-1)
                while lPos[pos] != 0:
                    pos = random.randint(0,len_lPos-1)
                # taille du bloc insr
                nb = min(random.randint(1,self.taille_bloc),NbIns)
                # tq le bloc ne chevauche pas un autre bloc
                while nb > 0 and pos < len_lPos and lPos[pos] == 0:                        
                    lPos.insert(pos,1)
                    #char = 'X'# chr(ord(lTexte[pos]) + 1)
                    # choix du caractre insr
                    ch = random.randint(65,122)
                    while ch == ord(lTexte[pos]): ch = random.randint(65,122)
                    char = chr(ch)
                    lTexte.insert(pos, char)
                    len_lPos += 1 ;  NbIns -= 1
                    nb -= 1 ; pos += 1
           
            if NbSup > 0: # SUP = 2
                pos = random.randint(0,len_lPos-1)
                while lPos[pos] != 0:
                    pos = random.randint(0,len_lPos-1)
                nb = min(random.randint(1,self.taille_bloc),NbSup)
                while nb > 0 and pos < len_lPos and lPos[pos] == 0: 
                    lPos[pos] = 2
                    lTexte[pos] = '*'
                    #len_lPos += 1 ;  
                    NbSup -= 1
                    nb -= 1 ; pos += 1
           
            if NbRemp > 0: # REMP = 3
                pos = random.randint(0,len_lPos-1)
                while lPos[pos] != 0:
                    pos = random.randint(0,len_lPos-1)
                nb = min(random.randint(1,self.taille_bloc),NbRemp)
                while nb > 0 and pos < len_lPos and lPos[pos] == 0: 
                    lPos[pos] = 3
                    #char = 'W' #chr(ord(lTexte[pos]) + 1)
                    ch = random.randint(65,122)
                    while ch == ord(lTexte[pos]): ch = random.randint(65,122)
                    char = chr(ch)
                    lTexte[pos] =  char
                    #len_lPos += 1 ;  
                    NbRemp -= 1
                    nb -= 1 ; pos += 1
                    
            if NbMov > 0: # MOV = 4 / 5
                nb = min(random.randint(1,self.taille_bloc),NbMov)
                pos1 = random.randint(0,len_lPos-1)
                # recherche d'une pos1 OK , sans chevauchements
                pos1OK = False
                while not pos1OK:
                    #i = pos1
                    #while i < pos1+nb and i < len_lPos and lPos[i] == 0: i+= 1
                    bu = pos1
                    #assert type(pos1) == type(2), type(pos1)
                    #assert type(pos1+nb) == type(2), str(type(pos1+nb))+str(nb)+'/'+str(pos1+nb)
                    t = numpy.array(lPos[pos1:pos1+nb])
                    if pos1+nb < len_lPos and numpy.sum(t) == 0: pos1OK = True
                    else: 
                        pos1 = random.randint(0,len_lPos-1)
                        while bu == pos1: pos1 = random.randint(10,len_lPos-10)
                    #if i == pos1+nb-1: pos1OK = True
                    #else: pos1 = random.randint(0,len_lPos-1)
                    assert pos1OK or bu != pos1
                    
                pos2 = random.randint(0,len_lPos-1)
                pos2OK = False
                while not pos2OK:
                    #i = pos2
                    #while i < pos2+nb and i < len_lPos and lPos[i] == 0: i+= 1
                    bu = pos2
                    t = numpy.array(lPos[pos2:pos2+nb])
                    if pos2+nb < len_lPos and numpy.sum(t) == 0: pos2OK = True
                    else: 
                        pos2 = random.randint(0,len_lPos-1)
                        while bu == pos2: pos2 = random.randint(10,len_lPos-10)
                    #if i == pos2+nb-1: pos2OK = True
                    #else: pos2 = random.randint(0,len_lPos-1)
                    assert pos2OK or bu != pos2
                #while lPos[pos2] != 0:
                #    pos2 = random.randint(0,len_lPos-1)
                
                #while nb > 0: # and pos1 < len_lPos and pos2 < len_lPos and lPos[pos1] == 0 and lPos[pos2] == 0:
                #    lPos[pos1] = 4 ; lPos.insert(pos2,5)
                #    c = lTexte[pos1]
                #    lTexte[pos1] = '#'
                #    lTexte.insert(pos2, c)
                #    len_lPos += 1   
                #    NbMov -= 1 
                #    nb -= 1 ; pos1 += 1 ; pos2 += 1 
                #insertion des 2 blocs
                lPos[pos1:pos1+nb] = [4] * nb
                lPos[pos2:pos2] = [5] * nb
                t1 = lTexte[pos1:pos1+nb]
                lTexte[pos1:pos1+nb] = ['#'] * nb
                lTexte[pos2:pos2] = t1
                len_lPos += nb ; NbMov -= nb
        return lPos, len_lPos, lTexte

    def _getModifiedText2(self, lPos, len_lPos, lTexte):            
        t2 = []  ; compt_t1 = compt_t2 = 0
        #oprations produite sur le texte1 pour donner le texte2
        self.t1ToT2 = numpy.zeros(self.len_t1,numpy.int8) 
        #oprations produite sur le texte 2 pour donner le 1
        self.t2ToT1 = numpy.zeros(self.len_t1,numpy.int8) 
        #print lPos, lTexte
        # cration du nouveau texte et de l'alignement
        for pos in range(len_lPos):                                # Modifiee 15.04.2019, ancienne ligne:  for pos in xrange(len_lPos): 
            if compt_t2 >= self.len_t1: break
            assert compt_t1 < self.len_t1
            assert compt_t2 < self.len_t1, (compt_t2, '/', self.len_t1, '/', pos, '/', len_lPos)
            operation = lPos[pos] ; char = lTexte[pos]
            if operation == 0: # INV
                t2.append(char)
                compt_t1 += 1 ; compt_t2 += 1
            elif operation == 1: # INS = 1
                t2.append(char)
                self.t2ToT1[compt_t2] = 1
                #compt_t1 += 1 ; 
                compt_t2 += 1
            elif operation == 2: # SUP = 2
                self.t1ToT2[compt_t1] = 2
                compt_t1 += 1 #; compt_t2 += 1
            elif operation == 3: # REMP = 3
                t2.append(char)
                self.t1ToT2[compt_t1] = 3
                self.t2ToT1[compt_t2] = 3
                compt_t1 += 1 ; compt_t2 += 1
            elif operation == 4: # DEP T1 = 4
                self.t1ToT2[compt_t1] = 4
                compt_t1 += 1 #; compt_t2 += 1
            elif operation == 5: # DEP T2 = 5
                t2.append(char)
                self.t2ToT1[compt_t2] = 5
                #compt_t1 += 1 ; 
                compt_t2 += 1
        self.t2 = ''.join(t2)    
        #print t2
        #print self.t2
        
    def pick_mot(self, dico_mot, liste_taille_mot, taille):
        longueur_liste_taille_mot = len(liste_taille_mot)
        chaine = ''
        taille_courante = 0
        while taille_courante < taille:
            
            longueur_mot_courant = 100000
            #logging.debug(('bf',taille,longueur_mot_courant,taille_courante))
            while taille_courante + longueur_mot_courant > taille:
                pos_liste_taille_mot = random.randint(0,longueur_liste_taille_mot-1)
                longueur_mot_courant = liste_taille_mot[pos_liste_taille_mot]
                #logging.debug(('in',taille,longueur_mot_courant,taille_courante))
            #logging.debug(('ok',taille,longueur_mot_courant,taille_courante))
            longueur_liste = len(dico_mot[longueur_mot_courant])
            pos_mot = random.randint(0,longueur_liste-1)
            mot = list(dico_mot[longueur_mot_courant].keys())[pos_mot]                    # Modifiee 21.06.2019, ancienne ligne:  mot = dico_mot[longueur_mot_courant].keys()[pos_mot] 
            chaine += mot 
            taille_courante += len(mot)
            if taille_courante < taille:
                chaine += ' '
                taille_courante += 1
        assert len(chaine) == taille, (len(chaine), taille)
        return chaine
        
    def _getTailleSeq(self):
        taille = 0
        if self.distrib_taille == 'GAUSS':
            taille = random.gauss(self.mu, self.variance)
        taille = int(taille)
        if taille <= 0: taille  = 1
        return taille
            
    def _getModifiedTextLN(self):
        """Cre le texte modifi et l'alignement"""
        path=os.path.join(os.getcwd(),"MediteAppli","test",'liste_mots.txt')
        f = open(path,'r')
        lignes = f.readlines()
        f.close()
        dico_mot = {}
        for i in range(len(lignes)):                               # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(len(lignes)):  
            mot = lignes[i][:-1]
            longueur = len(mot)
            if not (longueur in dico_mot): dico_mot[longueur] = {}                  # Modifiee 16.04.2019, ancienne ligne:  if not dico_mot.has_key(longueur): dico_mot[longueur] = {}  
            if not (mot in dico_mot[longueur]): dico_mot[longueur][mot] = 0         # Modifiee 16.04.2019, ancienne ligne:  if not dico_mot[longueur].has_key(mot): dico_mot[longueur][mot] = 0 
        if 1 in dico_mot: del dico_mot[1]                                     # Modifiee 16.04.2019, ancienne ligne:  if dico_mot.has_key(1): del dico_mot[1]  
        if not (1 in dico_mot): dico_mot[1] = {'':0}                           # Modifiee 16.04.2019, ancienne ligne:  if not dico_mot.has_key(1): dico_mot[1] = {'':0}  
        if 2 in dico_mot: del dico_mot[2]                                     # Modifiee 16.04.2019, ancienne ligne:  if dico_mot.has_key(2): del dico_mot[2] 
        if not (2 in dico_mot): dico_mot[2] = {'ah':0, 'as':0, 'au':0,  'bi':0, 'de':0,  'do':0, 'du':0, 'eh':0, 'en':0, 'et':0, 'ex':0, 'fa':0, 'fi':0, 'go':0, 'if':0, 'in':0, 'je':0, 'l':0, 'la':0, 'le':0, 'ma':0,  'mi':0, 'ni':0, 'nu':0,  'il':0,  'uf':0, 'oh':0, 'on':0, 'r':0}     # Modifiee 16.04.2019, ancienne ligne:  if not dico_mot.has_key(2): dico_mot[2] = {'ah':0, 'as':0, 'au':0,  'bi':0, 'de':0,  'do':0, 'du':0, 'eh':0, 'en':0, 'et':0, 'ex':0, 'fa':0, 'fi':0, 'go':0, 'if':0, 'in':0, 'je':0, 'l':0, 'la':0, 'le':0, 'ma':0,  'mi':0, 'ni':0, 'nu':0,  'il':0,  'uf':0, 'oh':0, 'on':0, 'r':0} 
        liste_taille_mot = []
        liste_taille_mot = list(dico_mot.keys())                              # Modifiee 21.06.2019, ancienne ligne:  liste_taille_mot = dico_mot.keys() 
        logging.debug('len(dico_mot)='+str(len(dico_mot)))
        logging.debug('liste_taille_mot='+str(liste_taille_mot))
        if False:
            for cle,val in iter(dico_mot.items()):                        # Modifiee 16.04.2019, ancienne ligne:  for cle,val in dico_mot.iteritems(): 
                f = open(os.path.join(os.getcwd(),"MediteAppli","test",'dico_mots'+str(cle)+'.txt'),'w')
                f.write(str(val))
                f.close()
        
        # nobmre de modifications pour chaque opration
        NbIns = int(self.TxIns * self.len_t1) ; NbSup = int(self.TxSup * self.len_t1) 
        NbRemp = int(self.TxRemp * self.len_t1)  
        NbMov = int(self.TxMov * self.len_t1)
        # liste stockant les caractres du nouveau texte + les suppresions
        lTexte = list(self.t1)
        # liste stockant les oprations  chaque position dans les 2 textes
        lPos = [(0,lTexte[i]) for i in range(self.len_t1)]                     # Modifiee 15.04.2019, ancienne ligne:  lPos = [(0,lTexte[i]) for i in xrange(self.len_t1)]
        len_lPos = self.len_t1
        while NbIns > 0 or NbSup > 0 or NbRemp > 0 or NbMov > 0:
            assert len(lPos) == len(lTexte) == len_lPos
            if NbIns > 0: #INS = 1
                # recherche d'une position vierge
                pos = random.randint(1,len_lPos-1)
                while lPos[pos][0] != 0 and lPos[pos-1][1] not in self.sep:
                    pos = random.randint(1,len_lPos-1)
                # taille du bloc insr
                nb = min(self._getTailleSeq(),NbIns)
                chaine = self.pick_mot(dico_mot, liste_taille_mot, nb)
                logging.debug((nb, len(chaine),chaine))
                assert nb == len(chaine)
                nb = min(len(chaine),NbIns)
                pos_chaine = 0
                # tq le bloc ne chevauche pas un autre bloc
                while nb > 0 and pos < len_lPos and lPos[pos][0] == 0:                        
                    
                    #char = 'X'# chr(ord(lTexte[pos]) + 1)
                    # choix du caractre insr
                    #ch = random.randint(65,122)
                    #while ch == ord(lTexte[pos]): ch = random.randint(65,122)
                    #char = chr(ch)
                    char = chaine[pos_chaine]
                    lTexte.insert(pos, char)
                    lPos.insert(pos,(1,char))
                    len_lPos += 1 ;  NbIns -= 1
                    nb -= 1 ; pos += 1 ; pos_chaine += 1
           
            if NbSup > 0: # SUP = 2
                pos = random.randint(1,len_lPos-1)
                while lPos[pos][0] != 0 and lPos[pos-1][1] not in self.sep:
                    pos = random.randint(1,len_lPos-1)
                nb = min(self._getTailleSeq(),NbSup)
                while nb > 0 and pos < len_lPos and lPos[pos][0] == 0: 
                    lPos[pos] = (2, '*')
                    lTexte[pos] = '*'
                    #len_lPos += 1 ;  
                    NbSup -= 1
                    nb -= 1 ; pos += 1
           
            if NbRemp > 0: # REMP = 3
                pos = random.randint(1,len_lPos-1)
                while lPos[pos][0] != 0 and lPos[pos-1][1] not in self.sep:
                    pos = random.randint(1,len_lPos-1)
                nb = min(self._getTailleSeq(),NbRemp)
                chaine = self.pick_mot(dico_mot, liste_taille_mot, nb)
                logging.debug((nb, len(chaine),chaine))
                assert nb == len(chaine)
                nb = min(len(chaine),NbRemp)
                pos_chaine = 0
                while nb > 0 and pos < len_lPos and lPos[pos][0] == 0: 
                    #char = 'W' #chr(ord(lTexte[pos]) + 1)
                    #ch = random.randint(65,122)
                    #while ch == ord(lTexte[pos]): ch = random.randint(65,122)
                    #char = chr(ch)
                    char = chaine[pos_chaine]
                    lPos[pos] = (3,char)
                    lTexte[pos] =  char
                    #len_lPos += 1 ;  
                    NbRemp -= 1
                    nb -= 1 ; pos += 1 ; pos_chaine += 1
                    
            if NbMov > 0: # MOV = 4 / 5
                nb = min(self._getTailleSeq(),NbMov)
                pos1 = random.randint(1,len_lPos-nb)
                # recherche d'une pos1 OK , sans chevauchements
                pos1OK = False
                while not pos1OK:
                    #i = pos1
                    #while i < pos1+nb and i < len_lPos and lPos[i] == 0: i+= 1
                    bu = pos1
                    #assert type(pos1) == type(2), type(pos1)
                    #assert type(pos1+nb) == type(2), str(type(pos1+nb))+str(nb)+'/'+str(pos1+nb)
                    #t = numpy.array(lPos[pos1:pos1+nb])
                    t = numpy.array([lPos[i][0] for i in range(pos1,pos1+nb)])                     # Modifiee 15.04.2019, ancienne ligne:  t = numpy.array([lPos[i][0] for i in xrange(pos1,pos1+nb)])
                    if (pos1+nb < len_lPos and numpy.sum(t) == 0 and 
                        lPos[pos1-1][1] in self.sep): pos1OK = True
                    else: 
                        pos1 = random.randint(0,len_lPos-1)
                        while (bu == pos1 or pos1+nb >= len_lPos or lPos[pos1-1][1] not in self.sep): 
                            pos1 = random.randint(1,len_lPos-nb)
                    #if i == pos1+nb-1: pos1OK = True
                    #else: pos1 = random.randint(0,len_lPos-1)
                    assert pos1OK or bu != pos1
                    
                pos2 = random.randint(1,len_lPos-nb)
                pos2OK = False
                while not pos2OK:
                    #i = pos2
                    #while i < pos2+nb and i < len_lPos and lPos[i] == 0: i+= 1
                    bu = pos2
                    #t = numpy.array(lPos[pos2:pos2+nb])
                    t = numpy.array([lPos[i][0] for i in range(pos2,pos2+nb)])                     # Modifiee 15.04.2019, ancienne ligne:  t = numpy.array([lPos[i][0] for i in xrange(pos2,pos2+nb)])
                    if (pos2+nb < len_lPos and numpy.sum(t) == 0 and
                        lPos[pos2-1][1] in self.sep): pos2OK = True
                    else: 
                        pos2 = random.randint(0,len_lPos-1)
                        while (bu == pos2 or pos2+nb >= len_lPos or lPos[pos2-1][1] not in self.sep):
                            pos2 = random.randint(1,len_lPos-nb)
                    #if i == pos2+nb-1: pos2OK = True
                    #else: pos2 = random.randint(0,len_lPos-1)
                    assert pos2OK or bu != pos2
                #while lPos[pos2] != 0:
                #    pos2 = random.randint(0,len_lPos-1)
                
                #while nb > 0: # and pos1 < len_lPos and pos2 < len_lPos and lPos[pos1] == 0 and lPos[pos2] == 0:
                #    lPos[pos1] = 4 ; lPos.insert(pos2,5)
                #    c = lTexte[pos1]
                #    lTexte[pos1] = '#'
                #    lTexte.insert(pos2, c)
                #    len_lPos += 1   
                #    NbMov -= 1 
                #    nb -= 1 ; pos1 += 1 ; pos2 += 1 
                #insertion des 2 blocs
                t1 = lTexte[pos1:pos1+nb]
                lPos[pos1:pos1+nb] = [(4,'#') for i in range(nb)] #[4] * nb                    # Modifiee 15.04.2019, ancienne ligne:  lPos[pos1:pos1+nb] = [(4,'#') for i in xrange(nb)] #[4] * nb   
                lPos[pos2:pos2] = [(5,t1[i]) for i in range(nb)]  # ][5] * nb                    # Modifiee 15.04.2019, ancienne ligne:  lPos[pos2:pos2] = [(5,t1[i]) for i in xrange(nb)]  # ][5] * nb 
                
                lTexte[pos1:pos1+nb] = ['#'] * nb
                lTexte[pos2:pos2] = t1
                len_lPos += nb ; NbMov -= nb
        assert len(lPos) == len(lTexte) == len_lPos
        return lPos, len_lPos, lTexte
    
    def _getModifiedTextLN2(self, lPos, len_lPos, lTexte):            
        t2 = []  ; compt_t1 = compt_t2 = 0
        #oprations produite sur le texte1 pour donner le texte2
        self.t1ToT2 = numpy.zeros(self.len_t1,numpy.int8) 
        #oprations produite sur le texte 2 pour donner le 1
        self.t2ToT1 = numpy.zeros(self.len_t1,numpy.int8) 
        #print lPos, lTexte
        # cration du nouveau texte et de l'alignement
        for pos in range(len_lPos):                                                    # Modifiee 15.04.2019, ancienne ligne:  for pos in xrange(len_lPos): 
            if compt_t2 >= self.len_t1: break
            assert compt_t1 < self.len_t1
            assert compt_t2 < self.len_t1, (compt_t2, '/', self.len_t1, '/', pos, '/', len_lPos)
            operation = lPos[pos][0] ; char = lTexte[pos]
            if operation == 0: # INV
                t2.append(char)
                compt_t1 += 1 ; compt_t2 += 1
            elif operation == 1: # INS = 1
                t2.append(char)
                self.t2ToT1[compt_t2] = 1
                #compt_t1 += 1 ; 
                compt_t2 += 1
            elif operation == 2: # SUP = 2
                self.t1ToT2[compt_t1] = 2
                compt_t1 += 1 #; compt_t2 += 1
            elif operation == 3: # REMP = 3
                t2.append(char)
                self.t1ToT2[compt_t1] = 3
                self.t2ToT1[compt_t2] = 3
                compt_t1 += 1 ; compt_t2 += 1
            elif operation == 4: # DEP T1 = 4
                self.t1ToT2[compt_t1] = 4
                compt_t1 += 1 #; compt_t2 += 1
            elif operation == 5: # DEP T2 = 5
                t2.append(char)
                self.t2ToT1[compt_t2] = 5
                #compt_t1 += 1 ; 
                compt_t2 += 1
        self.t2 = ''.join(t2)    
        #print t2
        #print self.t2
        
   

class Test_AlignSynthetic(unittest.TestCase):
    def _printFile(self,name,string):
        """ Lit un fichier dans le dossier courant et renvoie une chaine """
        path=os.path.join(os.getcwd(),"MediteAppli","test",name)
        f = open(path,'w')
        f.write(string)
        f.close()
        
    def ecartType(self, population):
        n = 0
        sum1 = 0.0
        for x in population:
            n = n + 1
            sum1 = sum1 + x
        mean = sum1/n
        
        sum2 = 0.0
        for x in population:
            sum2 = sum2 + pow(x - mean,2)
        variance2 = sum2/(n)
        # methode + robuste aux pb vigule flotante
        sum2 = 0.0
        sumc = 0.0
        for x in population:
            sum2 = sum2 + pow(x - mean,2)
            sumc = sumc + (x - mean)
        variance22 = (sum2 - pow(sumc,2)/n)/(n)
        ecartType = math.sqrt(variance22)
        return ecartType
            
    def test_alignSynthetic(self):
        """Mta-Fonction lancant les alignements 1 par 1 et sauvegardant les rsultats 
        """
        tdf = test_data.TestDataFactory()
        
        #self.td = f.getTestData(test_data.ClaudeBernard,"2")
        #self.td = f.getTestData(test_data.Traduction,"1")
        #self.td = f.getTestData(test_data.Chedid,"1")
        #self.td.texte1 = 'ABCDA' ; self.td.texte2 = 'ZADW'
        #self.td = f.getTestData(test_data.Bovary,"1")
        #td = tdf.getTestData(test_data.Leroux,"1")
        td = tdf.getTestData(test_data.Althusser, "1")
        #self.td = f.getTestData(test_data.Verne,"1")
        #self.td = f.getTestData(test_data.Brousse,"1")
           
    
        planTravail = Donnees.planTravail.PlanTravail('Traduction', 'Traduction', 
            'pVS', td.file1, 'pVC', td.file2, Donnees.planTravail.Parametres(1, #self.td.getp1(),
            td.getp2()*25, td.getp3()*1000, td.getpcarOuMot(), td.getpcaseSensitive(), 
            td.getpseparatorSensivitive(), td.getpdiacriticSensitive()))
        logRes = logging.FileHandler(filename=os.path.join(os.getcwd(), 'log',' logRes_'+algo+'.txt'), mode='a')
        logRes1 = logging.FileHandler(filename=os.path.join(os.getcwd(), 'log', 'logRes1_'+algo+'.txt'), mode='a')
        try :
            nom_csv1 = os.path.join(os.getcwd(), 'res_'+algo+'.csv') 
            open(nom_csv1)                                              # Modifiee 15.04.2019, ancienne ligne:  file(nom_csv1)  
            #csv = open(os.path.join(os.getcwd(), 'res_'+algo+'.csv'), 'a') #mode append 
        except :
            #le fichier csv n'existe pas, on crit sur la premiere ligne le nom des params 
            csv = open(nom_csv1, 'a') #mode append 
            csv.write("nom_fich;tx_modif;nb_test;dep;x;y;z;x\';z\';y1;y2;y3;y4;y5;z0;z1;z2;l_inv;nb_inv1;nb_inv2;l_sup;nb_sup;l_ins;nb_ins;l_remp;nb_remp_1;nb_remp_2;l_dep;nb_dep1;nd_dep2;l_texte1;l_texte2;tps;precision;precision pond\n")
            csv.close()
        try :
            nom_csv2 = os.path.join(os.getcwd(), 'res2_'+algo+'.csv')
            open(nom_csv2)                                              # Modifiee 15.04.2019, ancienne ligne:  file(nom_csv2)
            #csv2 = open(os.path.join(os.getcwd(), 'res2_'+algo+'.csv'), 'a') #mode append 
        except :
            #le fichier csv n'existe pas, on crit sur la premiere ligne le nom des params 
            csv2 = open(nom_csv2, 'a') #mode append 
            csv2.write("nom_fich;tx_modif;nb_test;dep;"+
                       "Pri;etPri;Moy;etMoy;Sep;etSep;Sim;etSim;Tps;etTps;"+
                       "FMPond;etFMPond;RappPond;etRappPond;PrecPond;etPrecPond;"+
                       "Pri1;etPri1;Pri2;etPri2;Pri3;etPri3;"+
                       "y1;y2;y3;y4;y5;ety1;ety2;ety3;ety4;ety5;"+
                       "l_inv;nb_inv1;nb_inv2;l_sup;nb_sup;l_ins;nb_ins;l_remp;nb_remp_1;nb_remp_2;l_dep;nb_dep1;nd_dep2;l_texte1;l_texte2;front_bloc;front_bloc_sep" + 
                       "FM;etFM;Prec;etPrec;Rapp;etRapp\n")
            csv2.close()
        logging.getLogger('').addHandler(logRes)     
        logRes.setLevel(logging.WARNING)
        logRes.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
        logRes1.setLevel(logging.WARNING)
        logRes1.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))

       
        # les 3 params en dur  modifier  la main, non paramtralbes par le batch
        lFile=['ClaudeBernard']#,'Leroux']#'Chedid','Althusser','ClaudeBernard','Traduction','ClaudeBernard']#],'Althusser',#Leroux']#'Leroux']#,'Bovary']#'Ramuz',
        #lTaux = [0.01, 0.025, 0.05, 0.075, 0.1, 0.125, 0.15, 0.175, 0.2]#[0.10]#,0.1]#0.05, 0.0375,0.075 0.05,
        lTaux = [0.01 ]#, 0.025, 0.05, 0.075, 0.1, 0.125, 0.15, 0.175, 0.2]
        #lTaux.extend([ 0.225, 0.25, 0.275, 0.3, 0.325, 0.35, 0.375, 0.4])
        #lTaux.extend([ 0.425, 0.45, 0.475, 0.5, 0.525, 0.55, 0.575, 0.6])
        #lTaux.extend([ 0.625, 0.65, 0.675, 0.7, 0.725, 0.75, 0.775, 0.8])
        #lSeed = [9, 33, 222, 2228, 4750, 5666, 7564, 8654, 9123, 11235]
        lSeed = [1 ]#, 2, 3, 4, 11, 6, 7, 8, 9, 10, 33, 222, 2228, 4750, 5666, 7564, 8654, 9123, 11235, 352]#, 1222, 568, 6574, 6487, 11114]
        #lSeed2 = [x * 23 for x in lSeed]
        #lSeed.extend(lSeed2)
        #lSeed2 = [x * 13 for x in lSeed]
        #lSeed.extend(lSeed2)
        var = vars(self) #; print var 
        #self.cpt=1
        if 'f' in var: # on reprend un prcdent run qui a plant            # Modifiee 16.04.2019, ancienne ligne:  if var.has_key('f'): # on reprend un prcdent run qui a plant 
            reprise = True ; f_backup = self.f ; tx_backup = self.tx ; seed_backup = self.seed 
        else: reprise = False
        next_iteration = False
        
        for self.f in lFile:
            if next_iteration  or not reprise: pass
            elif reprise and self.f != f_backup: continue
            for self.tx in lTaux:
                if next_iteration  or not reprise: pass
                elif reprise and self.tx != tx_backup: continue
                if next_iteration or not reprise: 
                    self.cumMoy = self.cumMoyPond = self.cumTemps = self.nb = 0.0
                    self.cumRappMoy = self.cumRappMoyPond = self.cumFMMoy = self.cumFMMoyPond = 0.0
                    self.cumX = self.cumY = self.cumZ = self.cumSim = 0.0 
                    self.cumPri1 = self.cumPri2 = self.cumPri3 = self.cumSep = 0.0
                    self.listeMoy = [] ; self.listeMoyPond = [] 
                    self.listeRappMoy = [] ; self.listeRappMoyPond = [] 
                    self.listeFMMoy = [] ; self.listeFMMoyPond = [] 
                    self.listeSim = [] ; self.listeTemps = [] 
                    self.listeObjPri = [] ; self.listeObjMoy = [] ; self.listeObjSep = []
                    self.listeObjPri1 = [] ; self.listeObjPri2 = [] ; self.listeObjPri3 = []
                    self.listeObjY1 = [] ; self.listeObjY2 = [] ; self.listeObjY3 = [] ; self.listeObjY4 = [] ; self.listeObjY5 = [] 
                    self.cumConfusionMatrice = numpy.zeros((5, 5), float)   
                    self.cumXprime=0.0
                    self.cumLY = [0.0]*5
                    self.cumLZ = [0.0]*3
                    self.cumInv=[0.0]*3
                    self.cumSup = [0.0]*2
                    self.cumIns = [0.0]*2
                    self.cumRemp = [0.0]*3
                    self.cumDep = [0.0]*3
                    self.cumLTexte1 = 0.0
                    self.cumLTexte2 = 0.0  
                    self.cumTps= 0.0
                    self.cumFront_bloc = 0.0
                    self.cumFront_bloc_sep = 0.0
                for self.seed in lSeed:
                    if next_iteration  and reprise :
                        reprise = False
                        next_iteration = False # on est sur le bon
                    elif reprise and self.seed != seed_backup: continue
                    elif reprise and not next_iteration: next_iteration = True ; continue # on est sur la dernire effectue, on passe  la suivante
                    #elif reprise and next_iteration: 
                    #    reprise = False ; next_iteration = False # on est sur le bon
                    logging.warning("========================")
                    logging.warning("Iteration "+str(lSeed.index(self.seed)+1)+ " / "+str(len(lFile)*len(lTaux)*len(lSeed)))
                    logging.warning("=====================")
                    td = tdf.getTestData(eval('test_data.'+self.f), "1")
                    synData =  SyntheticData(td.gettexte1(), self.tx, self.seed, moveOnly=False)#+self.td.gettexte2())
                    log = logging.FileHandler(filename=os.path.join(os.getcwd(), 'log', 'log_'+self.f+'_'+str(self.tx)+'_mv'+str(synData.moves)+'_'+algo+'_'+str(self.seed)+'.txt'), mode='w')
                    log.setLevel(5)
                    log.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
                    logging.getLogger('').addHandler(log)
                    logging.debug('===================================================')
                    logging.debug('log fichier='+self.f+' / Taux ins/del/remp='+str(self.tx)+' / seed='+str(self.seed)+' / moves='+str(synData.moves))
                    logging.debug('cararctres gnrs au hasard par blocs de 1  25')
                    moy, moyPond, temps, confusionMatrice, x, y, z, sim, xPrime, listeY, listeZ, dico,tps, (pri1,pri2,pri3), sep, Rappmoy, RappmoyPond, fmes, fmesPond = self._mainTest(td, planTravail, synData)
                    logging.debug('===================================================\n\n')
                    logging.getLogger('').removeHandler(log)
                    log.close()
                    
                    self.cumTps+=tps
                    self.cumMoy += moy ; self.cumMoyPond += moyPond 
                    self.cumRappMoy += Rappmoy ; self.cumRappMoyPond += RappmoyPond 
                    self.cumFMMoy += fmes ; self.cumFMMoyPond += fmesPond
                    self.cumTemps += temps ; self.nb += 1
                    self.cumX += x ; self.cumY += y ; self.cumZ += z ; self.cumSim += sim
                    self.cumPri1 += pri1; self.cumPri2 += pri2 ; self.cumPri3 += pri3
                    self.cumSep += sep
                    self.cumXprime += xPrime
                    
                    # somme l'itration courante avec les prcdentes
                    self.cumLY=[x[0]+x[1] for x in zip(self.cumLY, listeY)]
                    self.cumLZ=[x[0]+x[1] for x in zip(self.cumLZ, listeZ)]
                    self.cumInv=[x[0]+x[1] for x in zip(self.cumInv, dico['inv'])]
                    self.cumSup = [x[0]+x[1] for x in zip(self.cumSup, dico['sup'])]
                    self.cumIns = [x[0]+x[1] for x in zip(self.cumIns, dico['ins'])]
                    self.cumRemp = [x[0]+x[1] for x in zip(self.cumRemp, dico['remp'])]
                    self.cumDep = [x[0]+x[1] for x in zip(self.cumDep, dico['dep'])]
                    self.cumLTexte1 += dico['lTexte1']
                    self.cumLTexte2 += dico['lTexte2']    
                    self.cumFront_bloc += dico['front'][0]
                    self.cumFront_bloc_sep += dico['front'][1]
                    numpy.add(self.cumConfusionMatrice, confusionMatrice, self.cumConfusionMatrice)
                    arrondi = 2
                    cumPri =(1.0/3)*self.cumPri1 + (1.0/3)*self.cumPri2 + (1.0/3)*self.cumPri3
                    moyPri = round(100 *  cumPri / self.nb,arrondi)
                    moyY = round(100 * self.cumY / self.nb,arrondi)
                    moySep = round(100 * self.cumSep / self.nb,arrondi)
                    moySim = round((1.0/3)*moyPri + (1.0/3)*moyY + (1.0/3)*moySep, arrondi)
                    
                    Pri = (1.0/3)*pri1 + (1.0/3)*pri2 + (1.0/3)*pri3
                    self.listeMoy.append(moy) ; self.listeMoyPond.append(moyPond) 
                    self.listeRappMoy.append(Rappmoy) ; self.listeRappMoyPond.append(RappmoyPond)
                    self.listeFMMoy.append(fmes) ; self.listeFMMoyPond.append(fmesPond) 
                    
                    self.listeSim.append(sim) ; self.listeTemps.append(temps)
                    self.listeObjPri.append(Pri) ; self.listeObjMoy.append(y) ; self.listeObjSep.append(sep)
                    self.listeObjPri1.append(pri1) ; self.listeObjPri2.append(pri2) ; self.listeObjPri3.append(pri3)
                    self.listeObjY1.append(listeY[0]) ; self.listeObjY2.append(listeY[1]) ; self.listeObjY3.append(listeY[2])
                    self.listeObjY4.append(listeY[3]) ; self.listeObjY5.append(listeY[4])
                    
                    output = open(os.path.join(os.getcwd(), 'test_MediteAppli_'+algo+'.pkl'), 'wb')
                    pickle.dump(self, output)                                                      # Modifiee 15.04.2019, ancienne ligne:  cPickle.dump(self, output) 
                    output.close()
                    #self.cpt+=1
                    
                if not reprise:    # log des cumuls 
                    
                    logging.warning('log fichier='+self.f+' / Taux ins/del/remp='+str(self.tx)+' / moves='+str(synData.moves))
                    logging.warning('moyenne des moyennes = %f', 100*self.cumMoy/self.nb)
                    logging.warning('moyenne des moyennes pondres = %f', 100*self.cumMoyPond/self.nb)
                    m = int((self.cumTemps/self.nb)/60) ; s = (self.cumTemps/self.nb)-(m*60)
                    logging.warning('moyenne des temps = '+str(self.cumTemps/self.nb)+' = '+str(m)+' mn '+str(s)+' s')
                    logging.warning('moyenne des matrice de confusion = '+str(self.cumConfusionMatrice/self.nb))
                    logging.warning('moyenne des x = '+str(100*self.cumX/self.nb)+', des y = '+str(100*self.cumY/self.nb)+', des z = '+str(100*self.cumZ/self.nb))
                    logging.warning('moyenne des sim 0.5x+0.35y+0.15z = '+str(100*self.cumSim/self.nb))
                    logging.warning("moyenne des X\'= "+str(100*self.cumXprime/self.nb))
                    logging.warning("moyenne des PRI = "+str(moyPri))
                    logging.warning("moyenne des SEP = "+str(moySep))
                    logging.warning("moyenne des SIM = "+str(moySim))
                    logging.warning('===================================================\n\n')
                    logRes.flush()
                    logging.getLogger('').removeHandler(logRes)
                    logging.getLogger('').addHandler(logRes1)
                    logging.warning('log fichier='+self.f+' / Taux ins/del/remp='+str(self.tx)+' / moves='+str(synData.moves))
                    logging.warning('moyenne des moyennes = %f', 100*self.cumMoy/self.nb)
                    logging.warning('moyenne des moyennes pondres = %f', 100*self.cumMoyPond/self.nb)
                    logging.warning('moyenne des temps = '+str(self.cumTemps/self.nb)+' = '+str(m)+' mn '+str(s)+' s')
                    logging.warning('moyenne des matrice de confusion = '+str(self.cumConfusionMatrice/self.nb))
                    logging.warning('moyenne des x = '+str(100*self.cumX/self.nb)+', des y = '+str(100*self.cumY/self.nb)+', des z = '+str(100*self.cumZ/self.nb))
                    logging.warning('moyenne des sim 0.5x+0.35y+0.15z = '+str(100*self.cumSim/self.nb))
                    logging.warning("moyenne des X\'= "+str(100*self.cumXprime/self.nb))
                    logging.warning("moyenne des PRI = "+str(moyPri))
                    logging.warning("moyenne des SEP = "+str(moySep))
                    logging.warning("moyenne des SIM = "+str(moySim))
                    
                    afficheY= [round(100*x/self.nb,arrondi) for x in self.cumLY ]
                    logging.warning('moyenne des y1 = '+str(afficheY[0])+', moyenne des y2 = '+str(afficheY[1])+', moyenne des y3 = '+str(afficheY[2])+', moyenne des y4 = '+str(afficheY[3])+', moyenne des y5 = '+str(afficheY[4]))
                    afficheZ = [round(100*x/self.nb,arrondi) for x in self.cumLZ]
                    logging.warning('moyenne des z0 = '+str(afficheZ[0])+', moyenne des z1 = '+str(afficheZ[1])+', moyenne des z2 = '+str(afficheZ[2]))
                    afficheInv = [round(x/self.nb,arrondi) for x in self.cumInv]
                    logging.warning('Invariants : longueur moyenne = '+str(afficheInv[0])+', nombre moyen de blocs pour le texte 1 = '+str(afficheInv[1])+', nombre moyen de blocs pour le texte2 = '+str(afficheInv[2]))
                    afficheSup = [round(x/self.nb,arrondi) for x in self.cumSup]
                    logging.warning('Supprims : longueur moyenne = '+str(afficheSup[0])+', nombre moyen de blocs = '+str(afficheSup[1]))
                    afficheIns = [round(x/self.nb,arrondi) for x in self.cumIns]
                    logging.warning('Insers : longueur moyenne = '+str(afficheIns[0])+', nombre moyen de blocs = '+str(afficheIns[1]))
                    afficheRemp = [round(x/self.nb,arrondi) for x in self.cumRemp]
                    logging.warning('Remplacs : longueur moyenne = '+str(afficheRemp[0])+', nombre moyen de blocs pour le texte1 = '+str(afficheRemp[1])+', nombre moyen de blocs pour le texte2 = '+str(afficheRemp[2]))
                    afficheDep = [round(x/self.nb,arrondi) for x in self.cumDep]
                    logging.warning('Dplacs : longueur moyenne = '+str(afficheDep[0])+', nombre moyen de blocs pour le texte1 = '+str(afficheDep[1])+', nombre moyen de blocs pour le texte2 = '+str(afficheDep[2]))
                    logging.warning('longueur texte1 '+str(self.cumLTexte1/self.nb))
                    logging.warning('longueur texte2 '+str(self.cumLTexte2/self.nb))
                    logging.warning("moyenne des z\' = "+str(1.0/3*(afficheZ[0]+afficheZ[1]+afficheZ[2])))
                
                    logging.warning('===================================================\n\n')                   
                    logging.getLogger('').removeHandler(logRes1)
                    logRes1.flush()
                    logging.getLogger('').addHandler(logRes)
                    #str(afficheY)[1:-1].replace(',', ';')
                    # le [1:-1] enlve les crochets de dbut ef fin de afficheY qui est la liste des
                    # % des Y pondrs de tous les rsultats
                    # et le replace pour avoir les sparateurs de csv
                    #csv.write('type fichier;x;y;z;x\';z\';y1;y2;y3;y4;y5;z0;z1;z2;l_inv;nb_inv1;nb_inv2;l_sup;nb_sup;l_ins;nb_ins;l_remp;nb_remp_1;nb_remp_2;l_dep;nb_dep1;nd_dep2;l_texte1;l_texte2;\n')
                    strCsv=self.f+';'+str(self.tx)+';'+str(len(lSeed))+';'+ \
                        str(synData.moves)+';'+str(100*self.cumX/self.nb)+';'+\
                        str(100*self.cumY/self.nb)+';'+str(100*self.cumZ/self.nb)+';'+\
                        str(100*self.cumXprime/self.nb)+';'+\
                        str(1.0/3*(afficheZ[0]+afficheZ[1]+afficheZ[2]))+';'+\
                        str(afficheY)[1:-1].replace(',', ';')+';'+\
                        str(afficheZ)[1:-1].replace(',', ';')+';'+\
                        str(afficheInv)[1:-1].replace(',', ';')+';'+\
                        str(afficheSup)[1:-1].replace(',', ';')+';'+\
                        str(afficheIns)[1:-1].replace(',', ';')+';'+\
                        str(afficheRemp)[1:-1].replace(',', ';')+';'+\
                        str(afficheDep)[1:-1].replace(',', ';')+';'+\
                        str(self.cumLTexte1/self.nb)+';'+str(self.cumLTexte2/self.nb)+';'+\
                        str(self.cumTps/self.nb)+';'+str(100*self.cumMoy/self.nb)+';'+\
                        str(100*self.cumMoyPond/self.nb)
                    csv = open(nom_csv1, 'a') #mode append 
                    csv.write(strCsv+';\n')
                    csv.close()
                    # ===================================================
                    strCsv2=self.f+';'+str(self.tx)+';'+str(len(lSeed))+';'+ \
                        str(synData.moves)+';'+ \
                        str(moyPri)+';'+\
                        str(round(100 *self.ecartType(self.listeObjPri),arrondi))+';'+\
                        str(moyY)+';'+ \
                        str(round(100 *self.ecartType(self.listeObjMoy),arrondi))+';'+\
                        str(moySep)+';'+\
                        str(round(100 *self.ecartType(self.listeObjSep),arrondi))+';'+\
                        str(moySim)+';'+\
                        str(round(100 *self.ecartType(self.listeSim),arrondi))+';'+\
                        str(round(self.cumTps/self.nb,arrondi))+';'+\
                        str(round(self.ecartType(self.listeTemps),arrondi))+';'+\
                        str(round(100*self.cumFMMoyPond/self.nb,arrondi))+ ';' +\
                        str(round(100 *self.ecartType(self.listeFMMoyPond),arrondi))+';'+\
                        str(round(100*self.cumRappMoyPond/self.nb,arrondi))+ ';' +\
                        str(round(100 *self.ecartType(self.listeRappMoyPond),arrondi))+';'+\
                        str(round(100*self.cumMoyPond/self.nb,arrondi))+ ';' +\
                        str(round(100 *self.ecartType(self.listeMoyPond),arrondi))+';'+\
                        str(round(100*self.cumPri1/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjPri1),arrondi))+';'+\
                        str(round(100*self.cumPri2/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjPri2),arrondi))+';'+\
                        str(round(100*self.cumPri3/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjPri3),arrondi))+';'+\
                        str(afficheY)[1:-1].replace(',', ';')+';'+\
                        str(round(100 *self.ecartType(self.listeObjY1),arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjY2),arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjY3),arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjY4),arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeObjY5),arrondi))+';'+\
                        str(afficheInv)[1:-1].replace(',', ';')+';'+\
                        str(afficheSup)[1:-1].replace(',', ';')+';'+\
                        str(afficheIns)[1:-1].replace(',', ';')+';'+\
                        str(afficheRemp)[1:-1].replace(',', ';')+';'+\
                        str(afficheDep)[1:-1].replace(',', ';')+';'+\
                        str(round(self.cumLTexte1/self.nb,arrondi))+';'+\
                        str(round(self.cumLTexte2/self.nb,arrondi))+';'+\
                        str(round(self.cumFront_bloc/self.nb,arrondi))+';'+\
                        str(round(self.cumFront_bloc_sep/self.nb,arrondi))+';'+\
                        str(round(100*self.cumFMMoy/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeFMMoy),arrondi))+';'+\
                        str(round(100*self.cumRappMoy/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeRappMoy),arrondi))+';'+\
                        str(round(100*self.cumMoy/self.nb,arrondi))+';'+\
                        str(round(100 *self.ecartType(self.listeMoy),arrondi))
                    csv2 = open(nom_csv2, 'a') #mode append 
                    csv2.write(strCsv2+';\n')
                    csv2.close()
        
    def test_alignSyntheticLeraningPoids(self):
        """Mta-Fonction lancant les alignements 1 par 1
        et sauvegardant les rsultats """
        tdf = test_data.TestDataFactory()
        td = tdf.getTestData(test_data.Leroux,"1")
           
        planTravail = Donnees.planTravail.PlanTravail('Traduction', 'Traduction', 
            'pVS', td.file1, 'pVC', td.file2, Donnees.planTravail.Parametres(1,#self.td.getp1(),
            td.getp2()*25,td.getp3()*1000,td.getpcarOuMot(),td.getpcaseSensitive(),
            td.getpseparatorSensivitive(),td.getpdiacriticSensitive()))
        logRes = logging.FileHandler(filename=os.path.join(os.getcwd(),'logRes.txt'),mode='a')
        logRes.setLevel(logging.WARNING)
        logRes.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
        logging.getLogger('').addHandler(logRes)
        outPoids = open(os.path.join(os.getcwd(),'logPoids.txt'),'a')
        lFile=['Althusser'] #'Leroux']#'Chedid','Althusser','ClaudeBernard','Traduction','ClaudeBernard']#],'Althusser',#Leroux']#'Leroux']#,'Bovary']#'Ramuz',
        lTaux = [0.1]#,0.1]#,0.1]#0.05, 0.0375,0.075 0.05,
        lSeed = [9,33,222,2228, 4750] #9,33,
        lCoeff = self.generateTupleCoeff()
        #print lCoeff
        self.pos_lCoeff = 0
        var = vars(self) #; print var 
        if 'f' in var: # on reprend un prcdent run qui a plant            # Modifiee 16.04.2019, ancienne ligne:  if var.has_key('f'): # on reprend un prcdent run qui a plant  
            reprise = True ; f_backup = self.f ; tx_backup = self.tx 
            seed_backup = self.seed ; coeff_backup = self.coeff 
        else: reprise = False
        next_iteration = False
        for self.f in lFile:
            if next_iteration  or not reprise: pass
            elif reprise and self.f != f_backup: continue
            for self.tx in lTaux:
                if next_iteration  or not reprise: pass
                elif reprise and self.tx != tx_backup: continue
                for self.coeff in lCoeff:
                    if next_iteration  or not reprise: pass
                    elif reprise and self.coeff != coeff_backup: continue
                    
                    if next_iteration or not reprise: 
                        self.cumMoy = self.cumMoyPond = self.cumTemps = self.nb = 0.0
                        self.cumX = self.cumY = self.cumZ = self.cumSim = 0.0 
                        self.cumConfusionMatrice = numpy.zeros((5,5),float)   
                    for self.seed in lSeed:
                        if next_iteration  and reprise:
                            reprise = False ; next_iteration = False # on est sur le bon
                        elif reprise and self.seed != seed_backup: continue
                        elif reprise and not next_iteration: 
                            next_iteration = True ; continue # on est sur la dernire effectue, on passe  la suivante
                        #elif reprise and next_iteration: 
                        #    reprise = False ; next_iteration = False # on est sur le bon
                            
                        td = tdf.getTestData(eval('test_data.'+self.f),"1")
                        synData =  SyntheticData(td.gettexte1(), self.tx, self.seed)#+self.td.gettexte2())
                        log = logging.FileHandler(filename=os.path.join(os.getcwd(),'log_'+self.f+'_'+str(self.tx)+'_'+str(self.seed)+'_mv'+str(synData.moves)+'.txt'),mode='w')
                        log.setLevel(5)
                        log.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
                        logging.getLogger('').addHandler(log)
                        logging.debug('===================================================')
                        logging.debug('log fichier='+self.f+' / Taux ins/del/remp='+str(self.tx)+
                            ' / seed='+str(self.seed)+' / moves='+str(synData.moves)+' / coeff='+str(self.coeff))  
                        logging.debug('chararctres gnrs au hasard par blocs de 1  25')
                        moy,moyPond,temps,confusionMatrice,x,y,z,sim = self._mainTest(td,planTravail,synData, self.coeff)
                        logging.debug('===================================================\n\n')
                        logging.getLogger('').removeHandler(log)
                        log.close()
                        self.cumMoy += moy ; self.cumMoyPond += moyPond ; self.cumTemps += temps ; self.nb += 1
                        self.cumX += x ; self.cumY += y ; self.cumZ += z ; self.cumSim += sim
                        numpy.add(self.cumConfusionMatrice,confusionMatrice, self.cumConfusionMatrice)
                        output = open(os.path.join(os.getcwd(),'test_MediteAppli.pkl'), 'wb')
                        pickle.dump(self, output)                                          # Modifiee 15.04.2019, ancienne ligne:  cPickle.dump(self, output) 
                        output.close()
                    
                    if not reprise:    # log des cumuls 
                        logging.warning('log fichier='+self.f+' / Taux ins/del/remp='+str(self.tx)+' / moves='+str(synData.moves)+' / coeff='+str(self.coeff))                      
                        logging.warning('moyenne des moyennes = %f',100*self.cumMoy/self.nb)
                        logging.warning('moyenne des moyennes pondres = %f',100*self.cumMoyPond/self.nb)
                        m = int((self.cumTemps/self.nb)/60) ; s = (self.cumTemps/self.nb)-(m*60)
                        logging.warning('moyenne des temps = '+str(self.cumTemps/self.nb)+' = '+str(m)+' mn '+str(s)+' s')
                        logging.warning('moyenne des matrice de confusion = '+str(self.cumConfusionMatrice/self.nb))
                        logging.warning('moyenne des x = '+str(100*self.cumX/self.nb)+', des y = '+str(100*self.cumY/self.nb)+', des z = '+str(100*self.cumZ/self.nb))
                        logging.warning('moyenne des sim 0.5x+0.35y+0.15z = '+str(100*self.cumSim/self.nb))
                        logging.warning('===================================================\n\n')
                        logRes.flush()
                        moyMoy = (100*self.cumMoy/self.nb + 100*self.cumMoyPond/self.nb) / 2
                        poids = str(self.coeff[0])+';'+str(self.coeff[1])+';'+str(self.coeff[2])+';'+str(100*self.cumMoy/self.nb)+';'+str(100*self.cumMoyPond/self.nb)+';'+str(moyMoy)+';'+str(self.cumTemps/self.nb)+'\n'
                        outPoids.write(poids)
                        outPoids.flush()
        outPoids.close()
        
    def generateTupleCoeff(self, granularite=0.1, nbCoeff=3):
        tupleListe = []
        for i in range(20,-1,-1):#[1, 0.5, 0] #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:        # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(20,-1,-1):#[1, 0.5, 0] #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]: 
            for j in range(20,-1,-1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:                # Modifiee 15.04.2019, ancienne ligne:  for j in xrange(20,-1,-1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]: 
                for k in range(20,-1,-1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:            # Modifiee 15.04.2019, ancienne ligne:  for k in xrange(20,-1,-1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]: 
                    if i + j + k != 20: continue
                    tupleListe.append((1.0*i/20,1.0*j/20,1.0*k/20))
        # print tupleListe
        print (len(tupleListe))                                               # Modifiee 15.04.2019, ancienne ligne:  print len(tupleListe) 
        #for i,j,k in tupleListe:
        #    assert int(i+j+k) == 1, int(i+j+k)
        return tupleListe
        
    def _mainTest(self, td, planTravail, synData, coeff=None):
        """Fonction lanant l'alignement"""
        
        
        #trace("res = et.run(textesApparies=True, dossierRapport=dgm.getDG('Traduction', 'Traduction'))",locals())
        #self._printFile('ChedidSynthe.txt', synData.t2)
        #self.texte = self.synData.t1 + self.synData.t2
        #print len(self.texte),len(self.td.gettexte1()+self.td.gettexte2())
        #print self.synData.t2[:500]    
        synData.getModifiedText()
        f1 = open(os.path.join(os.getcwd(),'t1.txt'),'w')
        f2 = open(os.path.join(os.getcwd(),'t2.txt'),'w')
        f1.write(synData.t1) ; f2.write(synData.t2)
        f1.close() ; f2.close()
        
        a = time.time()
        if algo.lower() == 'HMMDep'.lower():
            et = MediteApplication.mediteFeng.fengAlgoMedite(synData.t1, synData.t2, td.getp2(), td.getpcaseSensitive(), td.getpseparatorSensivitive(), td.getpdiacriticSensitive(), planTravail,calculDep=True)
            dgm = Controleurs.DGManager.DGManager()
            et.run()
        elif algo.lower() == 'HMMNoDep'.lower():
            et = MediteApplication.mediteFeng.fengAlgoMedite(synData.t1, synData.t2, td.getp2(), td.getpcaseSensitive(), td.getpseparatorSensivitive(), td.getpdiacriticSensitive(), planTravail,calculDep=False)
            dgm = Controleurs.DGManager.DGManager()
            et.run()
            
        else :
            et = MediteApplication.MediteAppli.ecartTextesRecur2(td.gettexte1(), td.gettexte2(), #synData.t1, synData.t2, 
                td.getp1(), 
                td.getp2(), td.getp3(), 
                td.getpcarOuMot(), td.getpcaseSensitive(), 
                td.getpseparatorSensivitive(), td.getpdiacriticSensitive(), 
                planTravail, 
                algo)
            dgm = Controleurs.DGManager.DGManager()
            et.run(textesApparies=False, dossierRapport=dgm.getDG('Traduction', 'Traduction'), coeff=coeff)
        b =  time.time()
        m = (int(b) - int(a))/60 ; s = b-a-m*60
        logging.debug('dure = '+str(b-a)+' / '+str(m)+' mn '+str(s)+' s')
        bbl = et.bbl
        #print bbl.liste
        moy, moyPond, confusionMatrice, Rappmoy, RappmoyPond, fmes, fmesPond = self.compare2(bbl, synData, len(synData.t1))
        x, y, z, sim, dicoSommes, listeY, listeZ, (pri1,pri2,pri3), sep = bbl.evaluation()
        xPrime = float(dicoSommes['inv'][0]+dicoSommes['dep'][0])/float(dicoSommes['lTexte1']+dicoSommes['lTexte2'])
        logging.debug('FIN')
        return moy, moyPond, b-a, confusionMatrice, x, y, z, sim, xPrime, listeY, listeZ, dicoSommes, b-a, (pri1,pri2,pri3), sep, Rappmoy, RappmoyPond, fmes, fmesPond 
    
    
    def compare2(self,bbl,synData, len_t1):
        """Comparaison avec l'alignement de rfrence"""
        t1ToT2 = synData.t1ToT2
        t2ToT1 = synData.t2ToT1
        res1 = numpy.zeros(len_t1,numpy.int8)
        res2 = numpy.zeros(len_t1,numpy.int8)
        # parcours de l'alignement et cration de 2 tableaux correspondant aux 
        # oprations effectues sur chaque texte par l'aligneur
        for (B1,B2) in bbl.liste:
            if B1 is not None:
                B1_type = B1[0] ; B1_deb = B1[1] ; B1_fin = B1[2]
                if B1_type == 'S': # SUP = 2
                    for i in range(B1_deb,B1_fin): res1[i] = 2                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): res1[i] = 2 
                elif B1_type == 'R':  # REMP = 3
                    for i in range(B1_deb,B1_fin): res1[i] = 3                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): res1[i] = 3 
                elif B1_type == 'BC':  # INV = 0
                    for i in range(B1_deb,B1_fin): res1[i] = 0                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): res1[i] = 0  
                elif B1_type == 'D': 
                    if synData.moves: 
                        for i in range(B1_deb,B1_fin): res1[i] = 4                     # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): res1[i] = 4 
                    else: 
                        for i in range(B1_deb,B1_fin): res1[i] = 2                     # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): res1[i] = 2 
            if B2 is not None:
                B2_type = B2[0] ; B2_deb = B2[1] - len_t1 ; B2_fin = B2[2] - len_t1               
                if B2_type == 'I':  # INS = 1
                    for i in range(B2_deb,B2_fin): res2[i] = 1                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): res2[i] = 1  
                elif B2_type == 'R':  # REMP = 3
                    for i in range(B2_deb,B2_fin): res2[i] = 3                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): res2[i] = 3 
                elif B2_type == 'BC':  # INV = 0
                    for i in range(B2_deb,B2_fin): res2[i] = 0                         # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): res2[i] = 0 
                elif B2_type == 'D': 
                    if synData.moves: 
                        for i in range(B2_deb,B2_fin): res2[i] = 5                     # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): res2[i] = 5
                    else: 
                        for i in range(B2_deb,B2_fin): res2[i] = 1                     # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): res2[i] = 1 
             
        # comparaison entre les rsultats de l'aligneur et de l'alignement de rfrence
        ins = insOK = dele = deleOK = remp = rempOK = mov = movOK = inv = invOK = 0.0   
        dAppli = {}
        dAppli[0] = dAppli[1] = dAppli[2] = dAppli[3] = dAppli[4] = dAppli[5] = 0.0
        dinv={}; dsup={}; dins={}; dmov={}; dremp= {}
        for i in range(len_t1):                                                        # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(len_t1):
            if t1ToT2[i] == 0:            
                if res1[i] == 0: invOK += 1
                try: dinv[res1[i]] += 1
                except KeyError: dinv[res1[i]] = 1.0
                inv += 1
                dAppli[res1[i]] += 1
            if t1ToT2[i] == 2: # SUP
                if res1[i] == 2: deleOK += 1 ; dele += 1
                #elif res1[i] == 3: rempOK += 1 ; remp += 1
                else: dele += 1
                try: dsup[res1[i]] += 1
                except KeyError: dsup[res1[i]] = 1.0
                dAppli[res1[i]] += 1
            if t1ToT2[i] == 3: # REMP
                #if res1[i] == 2: deleOK += 1 ; dele += 1
                if res1[i] == 3: rempOK += 1 ; remp += 1
                else: remp += 1
                try: dremp[res1[i]] += 1
                except KeyError: dremp[res1[i]] = 1.0
                dAppli[res1[i]] += 1
            if t1ToT2[i] == 4: # MOV T1
                assert synData.moves
                if res1[i] == 4: movOK += 1 ; mov += 1
                else: mov += 1
                try: dmov[res1[i]] += 1
                except KeyError: dmov[res1[i]] = 1.0
                dAppli[res1[i]] += 1
                    
            if t2ToT1[i] == 0:            
                if res2[i] == 0: invOK += 1
                inv += 1
                try: dinv[res2[i]] += 1
                except KeyError: dinv[res2[i]] = 1.0
                dAppli[res2[i]] += 1
            if t2ToT1[i] == 1: # INS
                if res2[i] == 1: insOK += 1 ; ins += 1
                #elif res2[i] == 3: rempOK += 1 ; remp += 1
                else: ins += 1
                try: dins[res2[i]] += 1
                except KeyError: dins[res2[i]] = 1.0
                dAppli[res2[i]] += 1
            if t2ToT1[i] == 3: # REMP
                #if res2[i] == 1: insOK += 1 ; ins += 1
                if res2[i] == 3: rempOK += 1 ; remp += 1
                else: remp += 1
                try: dremp[res2[i]] += 1
                except KeyError: dremp[res2[i]] = 1.0
                dAppli[res2[i]] += 1
            if t2ToT1[i] == 5: # MOV T2
                assert synData.moves
                if res2[i] == 5: movOK += 1 ; mov += 1
                else: mov += 1
                try: dmov[res2[i]] += 1
                except KeyError: dmov[res2[i]] = 1.0
                dAppli[res2[i]] += 1
                
        # calcul des stats
        totOK = 0.0 + insOK + deleOK + rempOK + invOK
        totREF = 0.0 + ins + dele + remp + inv
        totSYS = 0.0 + dAppli[0] + dAppli[1] + dAppli[2] + dAppli[3]  
        invSYS = dAppli[0] ; insSYS = dAppli[1] ; supSYS = dAppli[2]
        rempSYS = dAppli[3] ; movSYS = dAppli[4] + dAppli[5]   
        if synData.moves: 
            totOK += movOK ; totREF += mov ; totSYS += dAppli[4] + dAppli[5] 
        if dele > 0: 
            logging.debug('deleOK/dele = %f', deleOK/dele)
            #print 'deleOK/dele = ', 100.0*deleOK/dele
        if ins > 0: 
            logging.debug('insOK/ins = %f', insOK/ins)
            #print 'insOK/ins = ', 100.0*insOK/ins
        if remp > 0: 
            logging.debug('rempOK/remp = %f', rempOK/remp)
            #print 'rempOK/remp = ', 100.0*rempOK/remp
        if mov > 0: 
            logging.debug('movOK/mov = %f', movOK/mov)
            #print 'movOK/mov = ', 100.0*movOK/mov
        if inv > 0: 
            logging.debug('invOK/inv = %f', invOK/inv)
            #print 'invOK/inv = ', 100.0*invOK/inv
        if totREF > 0: 
            logging.debug('totOK/tot = %f', totOK/totREF)
            #print 'totOK/tot = ', 100.0*totOK/tot
        
        if synData.moveOnly:
            moy = movOK/mov    
            moyPond = (synData.TxInv*invOK/inv + synData.TxMov*movOK/mov)
        elif synData.moves:
            Precmoy = (invOK/invSYS + deleOK/supSYS + insOK/insSYS +rempOK/rempSYS + movOK/movSYS)/5
            PrecmoyPond = (synData.TxInv*invOK/invSYS + synData.TxSup*deleOK/supSYS + synData.TxIns*insOK/insSYS +synData.TxRemp*rempOK/rempSYS + synData.TxMov*movOK/movSYS)
            logging.debug('Precmoy(ins,remp,sup,mov,inv) = %f', 100.0*Precmoy)                        
            logging.debug('PrecmoyPond(ins,remp,sup,mov,inv) = %f', 100.0*PrecmoyPond)
            
            Rappmoy = (invOK/inv + deleOK/dele + insOK/ins +rempOK/remp + movOK/mov)/5
            RappmoyPond = (synData.TxInv*invOK/inv + synData.TxSup*deleOK/dele + synData.TxIns*insOK/ins +synData.TxRemp*rempOK/remp + synData.TxMov*movOK/mov)
            logging.debug('Rappmoy(ins,remp,sup,mov,inv) = %f', 100.0*Rappmoy)                        
            logging.debug('RappmoyPond(ins,remp,sup,mov,inv) = %f', 100.0*RappmoyPond)
        else:
            Precmoy = (invOK/inv + deleOK/dele + insOK/ins +rempOK/remp)/4
            PrecmoyPond = (synData.TxInv*invOK/inv + synData.TxSup*deleOK/dele + synData.TxIns*insOK/ins +synData.TxRemp*rempOK/remp)
            #print 'moy(ins,remp,sup,inv) = ', 100.0*moy
            logging.debug('Precmoy(ins,remp,sup,inv) = %f', 100.0*Precmoy)                        
            #print 'moyPond(ins,remp,sup,inv) = ', 100.0*moyPond
            logging.debug('PrecmoyPond(ins,remp,sup,inv) = %f', 100.0*PrecmoyPond)
        # calcul des matrices de confusion    
        import decimal
        decimal.getcontext()
        decimal.getcontext().prec = 3
        m = numpy.zeros((5,5),float)            
        for ligne,d,nom in [(0,dinv,'inv'),(1,dins,'ins'),(2,dsup,'sup'),(3,dremp,'remp'),(4,dmov,'mov')]:
            s = 0 ; ch = ''
            for cle,nb in iter(d.items()): s+=nb                              # Modifiee 16.04.2019, ancienne ligne:  for cle,nb in d.iteritems(): s+=nb 
            for cle,nb in iter(d.items()):                                        # Modifiee 16.04.2019, ancienne ligne:  for cle,nb in d.iteritems():  
                if cle == 0: str2 = 'inv'
                elif cle == 1: str2 = 'ins'
                elif cle == 2: str2 = 'sup'
                elif cle == 3: str2 = 'remp'
                else: cle = 4 ; str2 = 'mov' # MOV4 et 5
                ch += str2+'/'+nom+'='+str(nb/s)+' // '
                m[ligne,cle] = nb/s
            m[ligne,4] = 0
            if 4 in d: m[ligne,4] += d[4]/s                   # Modifiee 16.04.2019, ancienne ligne:  if d.has_key(4): m[ligne,4] += d[4]/s  
            if 5 in d: m[ligne,4] += d[5]/s # les moves        # Modifiee 16.04.2019, ancienne ligne:  if d.has_key(5): m[ligne,4] += d[5]/s # les moves
            #print ch
            logging.debug(ch)
        #numpy.around(m,3,m)
        #print m
        logging.debug(m)
        fmes = (2.0*Precmoy*Rappmoy)/(0.0+Precmoy+Rappmoy)
        fmesPond = (2.0*PrecmoyPond*RappmoyPond)/(0.0+PrecmoyPond+RappmoyPond)
        return Precmoy,PrecmoyPond,m, Rappmoy, RappmoyPond, fmes, fmesPond
        
    def compare(self, bbl, synData, len_t1):
        """bugg"""
        t1ToT2 = synData.t1ToT2
        t2ToT1 = synData.t2ToT1
        #print t1ToT2
        #print t2ToT1
        ins = insOK = dele = deleOK = remp = rempOK = inv = invOK = 0.0
        for (B1,B2) in bbl.liste:
            if B1 is not None:
                B1_type = B1[0] ; B1_deb = B1[1] ; B1_fin = B1[2]
                if B1_type == 'S': # SUP = 2
                    for i in range(B1_deb,B1_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin):   
                        if t1ToT2[i] == 2: deleOK += 1
                        dele += 1
                        #print 'dele1'
                elif B1_type == 'R':  # REMP = 3
                    for i in range(B1_deb,B1_fin):                                 # Modifiee 15.04.2019, ancienne ligne:   for i in xrange(B1_deb,B1_fin): 
                        if t1ToT2[i] == 3 : rempOK += 1 ; remp += 1 #; print 'remp1'
                        elif t1ToT2[i] == 2: deleOK += 1; dele += 1 #; print 'dele1'
                        elif t1ToT2[i] == 0 and t2ToT1[i] == 1: continue#insOK += 1; ins += 1
                        else: inv += 1
                elif B1_type == 'BC':  # INV = 0
                    for i in range(B1_deb,B1_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin): 
                        if t1ToT2[i] == 0: invOK += 1
                        inv += 1
                        #print 'INV1'
                elif B1_type == 'D': 
                    for i in range(B1_deb,B1_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B1_deb,B1_fin):  
                        if t1ToT2[i] == 2: deleOK += 1
                        dele += 1
            if B2 is not None:
                B2_type = B2[0] ; B2_deb = B2[1] - len_t1 ; B2_fin = B2[2] - len_t1               
                if B2_type == 'I':  # INS = 1
                    for i in range(B2_deb,B2_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin):  
                        if t2ToT1[i] == 1: insOK += 1
                        ins += 1
                        #print 'ins2'
                elif B2_type == 'R':  # REMP = 3
                    for i in range(B2_deb,B2_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): 
                        if t2ToT1[i] == 3 : rempOK += 1 ; remp += 1 #; print 'remp22'
                        elif t2ToT1[i] == 1: insOK += 1; ins += 1 #;print 'ins22'
                        elif t2ToT1[i] == 0 and t1ToT2[i] == 2: continue
                        else: inv += 1
                elif B2_type == 'BC':  # INV = 0
                    for i in range(B2_deb,B2_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin): 
                        if t2ToT1[i] == 0: invOK += 1
                        inv += 1
                        #print 'INV2'
                elif B2_type == 'D': 
                    for i in range(B2_deb,B2_fin):                                 # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(B2_deb,B2_fin):
                        if t2ToT1[i] == 1: insOK += 1
                        ins += 1
            #print insOK , deleOK , rempOK , invOK
            #print ins , dele , remp , inv
        totOK = 0.0 + insOK + deleOK + rempOK + invOK
        tot = 0.0 + ins + dele + remp + inv
        if dele > 0: 
            logging.debug('deleOK/dele = %f', deleOK/dele)
            print ('deleOK/dele = ', 100.0*deleOK/dele)                               # Modifiee 15.04.2019, ancienne ligne:  print 'deleOK/dele = ', 100.0*deleOK/dele
        if ins > 0: 
            logging.debug('insOK/ins = %f', insOK/ins)
            print ('insOK/ins = ', 100.0*insOK/ins)                                   # Modifiee 15.04.2019, ancienne ligne:  print 'insOK/ins = ', 100.0*insOK/ins
        if remp > 0: 
            logging.debug('rempOK/remp = %f', rempOK/remp)
            print ('rempOK/remp = ', 100.0*rempOK/remp)                               # Modifiee 15.04.2019, ancienne ligne:  print 'rempOK/remp = ', 100.0*rempOK/remp
        if inv > 0: 
            logging.debug('invOK/inv = %f', invOK/inv)
            print ('invOK/inv = ', 100.0*invOK/inv)                                   # Modifiee 15.04.2019, ancienne ligne:  print 'invOK/inv = ', 100.0*invOK/inv  
        if tot > 0: 
            logging.debug('totOK/tot = %f', totOK/tot)
            print ('totOK/tot = ', 100.0*totOK/tot)                                   # Modifiee 15.04.2019, ancienne ligne:  print 'totOK/tot = ', 100.0*totOK/tot
        
        print ('moy(ins,remp,sup,inv) = ', 100.0*(invOK/inv + deleOK/dele + insOK/ins +rempOK/remp)/4)            # Modifiee 15.04.2019, ancienne ligne:  print 'moy(ins,remp,sup,inv) = ', 100.0*(invOK/inv + deleOK/dele + insOK/ins +rempOK/remp)/4
        logging.debug('moy(ins,remp,sup,inv) = %f', 100.0*(invOK/inv + deleOK/dele + insOK/ins +rempOK/remp)/4)
        
        print ('moyPond(ins,remp,sup,inv) = ', 100.0*(synData.TxInv*invOK/inv + synData.TxSup*deleOK/dele + synData.TxIns*insOK/ins +synData.TxRemp*rempOK/remp))         # Modifiee 15.04.2019, ancienne ligne:  print 'moyPond(ins,remp,sup,inv) = ', 100.0*(synData.TxInv*invOK/inv + synData.TxSup*deleOK/dele + synData.TxIns*insOK/ins +synData.TxRemp*rempOK/remp)
        logging.debug('moyPond(ins,remp,sup,inv) = %f', 100.0*(synData.TxInv*invOK/inv + synData.TxSup*deleOK/dele + synData.TxIns*insOK/ins +synData.TxRemp*rempOK/remp))
        
def suite():
    global algo
    suite = unittest.TestSuite()
    fichier = os.path.join(os.getcwd(), 'test_MediteAppli_'+algo+'.pkl')
    if os.path.isfile(fichier):
        pkl_file = open(fichier, 'rb')
        tas = pickle.load(pkl_file)                                # Modifiee 15.04.2019, ancienne ligne:  tas = cPickle.load(pkl_file) 
        pkl_file.close()
        print ("Attention !! fichier backup existant")                # Modifiee 15.04.2019, ancienne ligne:  print "Attention !! fichier backup existant"
        print ("itration termine: file = "+tas.f+" / taux = "+str(tas.tx)+" / seed = "+str(tas.seed))           # Modifiee 15.04.2019, ancienne ligne:  print "itration termine: file = "+tas.f+" / taux = "+str(tas.tx)+" / seed = "+str(tas.seed)
        print ('Continuer ce test ? 0/1 \n')                          # Modifiee 15.04.2019, ancienne ligne:  print 'Continuer ce test ? 0/1 \n' 
        sys.stdout.flush()
        #reponse = str()
        if input('\n'):
            print ("OK: reprise  l'interruption")                    # Modifiee 15.04.2019, ancienne ligne: print "OK: reprise  l'interruption" 
            suite.addTest(tas)
            return suite
    
    #suite.addTest(Test_AlignSynthetic("test_alignSynthetic"))
    suite.addTest(Test_AlignSynthetic("test_alignSynthetic"))
    return suite
   
def generateTupleCoeff22(granularite=0.1, nbCoeff=3):
        tupleListe = []
        for i in range(20, -1, -1):#[1, 0.5, 0] #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:                    # Modifiee 15.04.2019, ancienne ligne:  for i in xrange(20, -1, -1):#[1, 0.5, 0] #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]: 
            for j in range(20, -1, -1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:                            # Modifiee 15.04.2019, ancienne ligne:  for k in xrange(20, -1, -1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]: 
                for k in range(20, -1, -1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:                        # Modifiee 15.04.2019, ancienne ligne:  for k in xrange(20, -1, -1): #[1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0]:
                    if i + j + k != 20: continue
                    tupleListe.append((1.0*i/20, 1.0*j/20, 1.0*k/20))
        print (tupleListe)                                                 # Modifiee 15.04.2019, ancienne ligne: print tupleListe 
        print (len(tupleListe))                                            # Modifiee 15.04.2019, ancienne ligne: print len(tupleListe)
        for i, j, k in tupleListe:
            assert i+j+k == 1
        return tupleListe
        
if __name__ == '__main__':

    #on fait un traitement des parametres tres sommaire
    # en param, len nom des algos  lancer parmi:
    # LIS  ASTAR  glouton  dyn2 HMMDep HMMNoDep dyn
    global algo
    algo=sys.argv[1]

    print ("Algo : ", algo)                                                # Modifiee 15.04.2019, ancienne ligne: print "Algo : ", algo 
    logging.basicConfig(level=5, #logging.INFO,#DEBUG,#INFO,
                            format='%(asctime)s %(levelname)s %(message)s', 
                            #datefmt='%H:%M:%S',
                            filename=os.path.join(os.getcwd(), 'log.txt'), 
                            filemode='w')
    console = logging.StreamHandler()
    console.setLevel(logging.WARNING)  
    logging.getLogger('').addHandler(console)
    
    unittest.TextTestRunner(verbosity=2).run(suite())
    #generateTupleCoeff22()

    
