# -*- coding: iso-8859-1 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import unittest            # Modifiee 15.04.2019, ancienne ligne: import unittest, contract
#from test import test_support
import MediteApplication.synthetic   # Modifiee 08.02.2019, ancienne ligne: import MediteAppli.synthetic, test_data
import MediteApplication.test
from MediteApplication.test import test_data    # Ajoutee 08.02.2019
import Donnees.resultatAppli 
import numpy as np          # Ajoutee 04.04.2019 
# import numpy.oldnumeric as Numeric    # Desactivee 04.04.2019 
#import scipy
# vrification des contrats
# contract.checkmod(MediteAppli.synthetic)            # Desactivee 15.04.2019 
# contract.checkmod(Donnees.resultatAppli)            # Desactivee 15.04.2019


class Test_Synthetic(unittest.TestCase):
        def setUp(self):
            ds = test_data.Data_Synthetic()
            self.taggerExtension = MediteApplication.synthetic.TaggerExtension(ds.jeu_essai_Ramuz())
            self.tm = MediteApplication.synthetic.TagManager()
        
        def test_etiquetage(self):
            self.lTokenTag = self.taggerExtension.etiquetage()
            self.assertNotEqual(len(self.lTokenTag),0)
            for (token,tag) in self.lTokenTag:
                self.assert_(isinstance(token,str))
                self.assert_(isinstance(tag,str))
                self.assertNotEqual(len(token),0)
                self.assertNotEqual(len(tag),0)
                self.assert_(tag in self.tm.getListe(), "tag "+tag+" unknown pour le token "+token) # le tag doit faire partie de la liste des tags
                
        def test_extractLTokenPos(self):
            lTokenPos = self.taggerExtension.extractLTokenPos_()
            self.assertNotEqual(len(lTokenPos),0)
            lgSource=self.taggerExtension.resultatMedite.getLgSource()
            texte=self.taggerExtension.resultatMedite.getTextesConcatenes()
            lgTotal=len(texte)
            for (token,pos) in lTokenPos:
                self.assert_(isinstance(token,str))
                self.assert_(isinstance(pos,int))
                self.assert_(0<=pos<lgTotal)
                fin = pos+len(token)
                self.assertEqual(token,texte[pos:fin])
                self.assert_((0<=pos<lgSource and 0<=fin<=lgSource) or 
                    (lgSource<=pos<lgTotal and lgSource<=fin<=lgTotal),
                    "debut="+str(pos)+" fin="+str(fin)+" / "+texte[pos:fin]+" / lgSource="+str(lgSource)+" lgTotal="+str(lgTotal))
   
        def test_alignement(self):
            self.lTokenDebTokenFinTag = self.taggerExtension.alignement()
            self.assertNotEqual(len(self.lTokenDebTokenFinTag),0)
            lgSource=self.taggerExtension.resultatMedite.getLgSource()
            texte=self.taggerExtension.resultatMedite.getTextesConcatenes()
            lgTotal=len(texte)
            for (debut,fin,tag) in self.lTokenDebTokenFinTag:
                self.assert_(isinstance(debut,int))
                self.assert_(isinstance(fin,int))
                self.assert_(isinstance(tag,str))
                self.assert_((0<=debut<lgSource and 0<=fin<=lgSource) or 
                    (lgSource<=debut<lgTotal and lgSource<=fin<=lgTotal),
                    "debut="+str(debut)+" fin="+str(fin)+" / "+texte[debut:fin]+" / lgSource="+str(lgSource)+" lgTotal="+str(lgTotal))
    
        def test_post_traitement_alignement(self):
            self.dicoPosTag = self.taggerExtension.post_traitement_alignement(self.taggerExtension.alignement())
            self.assert_(len(self.dicoPosTag)>0) 
            lgTotal=len(self.taggerExtension.resultatMedite.getTextesConcatenes())
            for debut in self.dicoPosTag:
                (fin,tag) = self.dicoPosTag[debut]
                self.assert_(0<=debut<=fin<=lgTotal)
                self.assert_(tag in self.tm.getListe())
    
        def test_calcul_frequences(self):
            la = self.taggerExtension.alignement()
            da = self.taggerExtension.post_traitement_alignement(la)
            self.fm = self.taggerExtension.calcul_frequences(la,da)
            self.assert_(isinstance(self.fm, MediteApplication.synthetic.TagFrequencyMatrix))
            # le nb de BC doit tre gal pout T1 et T
            self.assert_(sum(self.fm.freqGeneticModifs[6]) == sum(self.fm.freqGeneticModifs[7])) # 
            
        def text_visualisation_statistiques(self):
            pass

class Test_SyntheticRamuz(Test_Synthetic):
        def setUp(self):
            ds = test_data.Data_Synthetic()
            self.taggerExtension = MediteApplication.synthetic.TaggerExtension(ds.jeu_essai_Ramuz())
            self.tm = MediteApplication.synthetic.TagManager()
class Test_SyntheticAlthusser(Test_Synthetic):
        def setUp(self):
            ds = test_data.Data_Synthetic()
            self.taggerExtension = MediteApplication.synthetic.TaggerExtension(ds.jeu_essai_Althusser())
            self.tm = MediteApplication.synthetic.TagManager()            
class Test_SyntheticCharroi(Test_Synthetic):
        def setUp(self):
            ds = test_data.Data_Synthetic()
            self.taggerExtension = MediteApplication.synthetic.TaggerExtension(ds.jeu_essai_Charroi())
            self.tm = MediteApplication.synthetic.TagManager()  

class Test_SyntheticAragon(Test_Synthetic):
        def setUp(self):
            ds = test_data.Data_Synthetic()
            self.taggerExtension = MediteApplication.synthetic.TaggerExtension(ds.jeu_essai_Aragon())
            self.tm = MediteApplication.synthetic.TagManager()  
            
class Test_FrequencyMatrix(unittest.TestCase):
    def test_fm(self): 
        fm = MediteApplication.synthetic.FrequencyMatrix(5,2)
        self.assertEqual(fm.getNbLine(),5)
        self.assertEqual(fm.getNbColumn(),2)
        
        
class Test_TagFrequencyMatrix(unittest.TestCase):
    def setUp(self):
        self.tm = MediteApplication.synthetic.TagManager()
        self.potm = MediteApplication.synthetic.PartOfTextManager()
    def get_fm(self):
        l = self.tm.getNbItems()
        freqGeneticModifs = np.array(
            [[0 for i in range(l)], 
            [1 for i in range(l)], 
            [2 for i in range(l)], 
            [3 for i in range(l)], 
            [4 for i in range(l)], 
            [5 for i in range(l)]])      # Modifiee 08.02.2019, ancienne ligne:  [[0 for i in xrange(l)],[1 for i in xrange(l)],[2 for i in xrange(l)],[3 for i in xrange(l)],[4 for i in xrange(l)],[5 for i in xrange(l)]]), remplace Numeric par np
        freqTexts = np.array(
            [[6 for i in range(l)], 
            [7 for i in range(l)]])      # Modifiee 08.02.2019, ancienne ligne:  [[6 for i in xrange(l)],[7 for i in xrange(l)]]), remplace Numeric par np
        fm = MediteApplication.synthetic.TagFrequencyMatrix(freqGeneticModifs,freqTexts)
        fm.setRelation([0,2,4],0)
        fm.setRelation([1,3,5],1)
        return fm
    def test_fm(self):    
        fm = self.get_fm()
        self.assertEqual(fm.getRelation(0),0)
        self.assertEqual(fm.getRelation(2),0)
        self.assertEqual(fm.getRelation(4),0)
        self.assertEqual(fm.getRelation(1),1)
        self.assertEqual(fm.getRelation(3),1)
        self.assertEqual(fm.getRelation(5),1)
        
        for x in fm.getFreqLine(0): self.assertEqual(x,0)
        for x in fm.getFreqLine(self.potm.posToItem(0)): self.assertEqual(x,0)
        for x in fm.getFreqLine(5): self.assertEqual(x,5)
        for x in fm.getFreqLine(self.potm.posToItem(5)): self.assertEqual(x,5)
        
        self.assertEqual(fm.getFreqColumn(0),[0,1,2,3,4,5])
        self.assertEqual(fm.getFreqColumn(5),[0,1,2,3,4,5])
        
class Test_TagFrequencyMatrix2(Test_TagFrequencyMatrix):
    def test_fm2(self):
        fm = self.get_fm()
        fm2 = MediteApplication.synthetic.TagFrequencyMatrix2(fm)
        for i in range(fm.getNbColumn()):        # Modifiee 08.02.2019, ancienne ligne:  for i in xrange(fm.getNbColumn()):
            self.assert_(fm2.getFreqLine(0)[i]==fm.getFreqLine(0)[i]+fm.getFreqLine(2)[i])
            self.assert_(fm2.getFreqLine(1)[i]==fm.getFreqLine(1)[i]+fm.getFreqLine(3)[i])
            self.assert_(fm2.getFreqLine(2)[i]==fm.getFreqLine(4)[i])
            self.assert_(fm2.getFreqLine(3)[i]==fm.getFreqLine(5)[i])
            self.assert_(fm2.getFreqLine(4)[i]==fm.getFreqLine(6)[i])
            self.assert_(fm2.getFreqLine(5)[i]==fm.getFreqLine(7)[i])
 
class Test_BiBlocList(unittest.TestCase):
    def test_constructor(self):
        ds = test_data.Data_Synthetic()
        p = Donnees.planTravail.PlanTravail('pAuteur', 'pOeuvre', 
            'pVS', 'pFS', 'pVC', 'pFC', Donnees.planTravail.Parametres())
        biBlocList = MediteApplication.synthetic.BiBlocList(ds.jeu_essai_Althusser(),p)
        ###biBlocList = MediteAppli.synthetic.BiBlocList(ds.jeu_essai_Althusser2(),p)
        biBlocList = MediteApplication.synthetic.BiBlocList(ds.jeu_essai_Althusser3(),p)
        biBlocList = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Ramuz(),p)
        biBlocList = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Charroi(),p)
        ###biBlocList = MediteAppli.synthetic.BiBlocList(ds.jeu_essai_CB(),p)
        biBlocList = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Python(),p)
        biBlocList = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Condorcet(),p)
        res = biBlocList.toResultat()
        biBlocList.print_html()
        biBlocList.evaluation()
        
    def test_decoreDep(self):
        ds = test_data.Data_Synthetic()
        p = Donnees.planTravail.PlanTravail('pAuteur', 'pOeuvre', 
            'pVS', 'pFS', 'pVC', 'pFC', Donnees.planTravail.Parametres())
        biBlocList1 = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Python(),p,1)
        res1 = biBlocList1.toResultat()
        #biBlocList.print_html()
        #biBlocList.evaluation()      
        biBlocList2 = MediteApplication.synthetic.BiBlocListWD(ds.jeu_essai_Python(),p,2)
        res2 = biBlocList2.toResultat()
        print (res1 ^ res2)   # Modifiee 08.02.2019, ancienne ligne:  print res1 ^ res2
        
def suite():
    suite1 = unittest.TestSuite()
    suite1.addTest(Test_SyntheticRamuz("test_etiquetage"))
    suite1.addTest(Test_SyntheticRamuz("test_extractLTokenPos"))
    suite1.addTest(Test_SyntheticRamuz("test_alignement"))
    suite1.addTest(Test_SyntheticRamuz("test_post_traitement_alignement"))
    suite1.addTest(Test_SyntheticRamuz("test_calcul_frequences"))
    suite2 = unittest.TestSuite()
    suite2.addTest(Test_SyntheticAlthusser("test_etiquetage"))
    suite2.addTest(Test_SyntheticAlthusser("test_extractLTokenPos"))
    suite2.addTest(Test_SyntheticAlthusser("test_alignement"))
    suite2.addTest(Test_SyntheticAlthusser("test_post_traitement_alignement"))
    suite2.addTest(Test_SyntheticAlthusser("test_calcul_frequences"))
    suite3 = unittest.TestSuite()
    suite3.addTest(Test_SyntheticCharroi("test_etiquetage"))
    suite3.addTest(Test_SyntheticCharroi("test_extractLTokenPos"))
    suite3.addTest(Test_SyntheticCharroi("test_alignement"))
    suite3.addTest(Test_SyntheticCharroi("test_post_traitement_alignement"))
    suite3.addTest(Test_SyntheticCharroi("test_calcul_frequences"))
    suite4 = unittest.TestSuite()
    #suite4.addTest(Test_SyntheticAragon("test_etiquetage"))
    #suite4.addTest(Test_SyntheticAragon("test_extractLTokenPos"))
    #suite4.addTest(Test_SyntheticAragon("test_alignement"))
    #suite4.addTest(Test_SyntheticAragon("test_post_traitement_alignement"))
    suite4.addTest(Test_SyntheticAragon("test_calcul_frequences"))
    suite5 = unittest.TestSuite()
    suite5.addTest(Test_BiBlocList("test_constructor"))
    suite5.addTest(Test_BiBlocList("test_decoreDep"))
    #suite = unittest.TestSuite((suite1, suite2, suite3, suite4,suite5))
    suite = unittest.TestSuite((suite4))
    return suite

def suiteFM():
    suite = unittest.TestSuite()
    suite.addTest(Test_FrequencyMatrix("test_fm"))
    #suite.addTest(Test_TagFrequencyMatrix("test_fm"))
    suite.addTest(Test_TagFrequencyMatrix2("test_fm2"))
    return suite
    
def trace(commande,dic_locals):
    import profile,pstats,sys
    #profile.runctx(commande,globals(), locals(),'Z:\workspace\medite\statFile')
    profile.runctx(commande,globals(), dic_locals,'c:\workspace\medite\statFile')
    s = pstats.Stats('c:\workspace\medite\statFile')
    s.sort_stats('time')
    s.print_stats()
    sys.stderr.flush()
    sys.stdout.flush()
    
if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())
    #unittest.TextTestRunner(verbosity=2).run(suiteFM())
    