# -*- coding: UTF-8 -*-
# Copyright 20003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys,os,string
import Donnees.resultatAppli
from apply import apply

class Factory(object):      
    def register(self, methodName, constructor, *args, **kargs):
        """register a constructor"""
        _args = [constructor]
        _args.extend(args)
        setattr(self, methodName,apply(Functor,_args, kargs))
        
    def unregister(self, methodName):
        """unregister a constructor"""
        delattr(self, methodName)

class Functor(object):
    def __init__(self, function, *args, **kargs):
        assert callable(function), "function should be a callable obj"
        self._function = function
        self._args = args
        self._kargs = kargs
        
    def __call__(self, *args, **kargs):
        """call function"""
        _args = list(self._args)
        _args.extend(args)
        _kargs = self._kargs.copy()
        _kargs.update(kargs)
        return apply(self._function,_args,_kargs)
        
class TestDataFactory(object):
    def getTestData(self,nom,cas):
        return apply(nom,cas)

class Test_Data(object):
    def _readFile(self,name):
        """ Lit un fichier dans le dossier courant et renvoie une chaine """
        path=os.path.join(os.getcwd(), "textes",name)                      # Modifiee 23.07.2019, ancienne ligne:  path=os.path.join(os.getcwd(),"MediteAppli","test", "textes",name) 
        # print("5. Module MediteAppli.test.test_data, Class: Test_Data, Methode: _readFile, ligne 57, \npath=", path)
        f = open(path)
        res = f.read()
        f.close()
        return res
        
class Data_Alignement(Test_Data):
    """ Jeu d'essais pour test_alignement """
    def getp1(self): return self.p1
    def getp2(self): return self.p2
    def getp3(self): return self.p3
    def getpcarOuMot(self): return self.pcarOuMot
    def getpcaseSensitive(self): return self.pcaseSensitive
    def getpseparatorSensivitive(self): return self.pseparatorSensivitive
    def getpdiacriticSensitive(self): return self.pdiacriticSensitive
    def getL1(self): return self.L1
    def getL2(self): return self.L2
    def gettexte1(self): return self.texte1
    def gettexte2(self): return self.texte2
    
class ClaudeBernard(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        elif cas=='2': self.cas2()
        elif cas=='3': self.cas3()
        elif cas=='4': self.cas4()
        elif cas=='5': self.cas5()
    def cas5(self):
        self.cas2()
        self.L1 = [[0, 3], [3, 6], [6, 12], [12, 19], [19, 26], [26, 36], [51, 55], [55, 59], [69, 72], [72, 80]]
        self.L2 = [[115, 118], [136, 143], [158, 168], [186, 190], [231, 239], [291, 295], [363, 370], [376, 379], [384, 387], [408, 414]]
        self.texte1 = """.de.la.patte.droite.on.lui.fait.une.incision.sur.le.dos.et.on.y.place.du.curare.a.2h.5.a.2h.20.la.grenouille.est"""
        self.texte2 = """.et.de.l.artere.crurales.droites.apres.quoi.on.fait.une.plaie.a.la.peau.du.dos.par.laquelle.on.introduit.un.fragment.de.curare.cette.operation.fut.faite.a.deux.heures.cinq.minutes.et.a.deux.heures.vingt.minutes.l.animal.est.completement.immobile.quand.on.lui.pince.la.peau.du.corps.celle.des.deux.pattes"""        
    def cas4(self):
        self.cas2()
        self.L1 = [[0, 6], [48, 72], [72, 79], [116, 126], [138, 145]]
        self.L2 = [[145, 151], [153, 163], [184, 194], [241, 248], [248, 272], [284, 291]]
        self.texte1 = """celle.qui.a.ete.operee.laquelle.manifeste.encore.des.mouvements.reflexes.il.y.a.encore.du.mouvement.quand.on.pince.a.la.patte.liee.elle.se.retire"""
        self.texte2 = """celle.de.la.patte.droite.posterieure.et.la.patte.posterieure.gauche.dont.les.vaisseaux.sont.lies.il.y.a.des.mouvements.reflexes.et.l.animal.retire"""        
    def cas3(self):
        self.cas1()
        self.L1 = [[0, 6], [49, 73], [74, 81], [118, 128], [141, 148]]
        self.L2 = [[148, 154], [156, 166], [188, 198], [247, 254], [254, 278], [291, 298]]
        self.texte1 = """celle qui a été opérée, laquelle manifeste encore des mouvements réflexes; il y a encore du mouvement quand on pince à la patte liée, elle se retire"""
        self.texte2 = """celle de la patte droite postérieure, et la patte postérieure gauche, dont les vaisseaux sont liés, il y a des mouvements réflexes, et l'animal retire"""
    def cas2(self):
        self.p1 = 5
        self.p2 = 2.0 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible � la casse
        self.pseparatorSensivitive = 0  #insensible aux s�parateurs
        self.pdiacriticSensitive = 0  #insensible aux diacritiques
        # listes de blocs r�p�t�s � aligner
        self.L1 = []#[[0, 18], [18, 25], [30, 37], [37, 50], [56, 68], [68, 71], [71, 74], [88, 91], [105, 110], [114, 120], [120, 141], [151, 157], [157, 164], [164, 167], [167, 176], [176, 196], [196, 203], [207, 220], [234, 239], [239, 245], [245, 257], [270, 288], [288, 290], [290, 299], [299, 320], [325, 328], [333, 339], [339, 356], [356, 365], [365, 376], [382, 406], [446, 450], [466, 469], [469, 472], [472, 479], [494, 497], [503, 519], [534, 541], [541, 544], [548, 564], [564, 570], [570, 575], [575, 586], [586, 592], [597, 603], [604, 614], [641, 649], [658, 663], [666, 670], [670, 678], [699, 702], [702, 705], [705, 714], [720, 752], [752, 764], [764, 838], [838, 845], [862, 865], [865, 889], [889, 902], [908, 918], [926, 937], [937, 969], [978, 990], [1014, 1020], [1020, 1025], [1030, 1039], [1070, 1087], [1087, 1096], [1096, 1140], [1148, 1172], [1172, 1178], [1203, 1217], [1243, 1254], [1266, 1277], [1291, 1294], [1325, 1333], [1336, 1340], [1340, 1345], [1372, 1382], [1382, 1442], [1466, 1470], [1470, 1507], [1507, 1527], [1527, 1548], [1548, 1556], [1556, 1571], [1571, 1593], [1593, 1615], [1615, 1618], [1648, 1653], [1656, 1662], [1681, 1686], [1690, 1723], [1740, 1773], [1773, 1791], [1798, 1802], [1826, 1830], [1830, 1832], [1844, 1849], [1849, 1860], [1868, 1871], [1874, 1880], [1881, 1891], [1891, 1894], [1894, 1903], [1903, 1920], [1926, 1931], [1935, 1940], [1940, 1948], [1948, 1953], [1962, 1967], [1974, 1990], [1991, 1996], [1996, 2004], [2004, 2010], [2015, 2021], [2030, 2034], [2036, 2042], [2050, 2055], [2060, 2064], [2070, 2087], [2087, 2097], [2097, 2104], [2104, 2119], [2119, 2125], [2125, 2134], [2134, 2156], [2156, 2164], [2165, 2172], [2172, 2177], [2177, 2178], [2178, 2183], [2186, 2191], [2191, 2219], [2226, 2250], [2265, 2270], [2270, 2279], [2316, 2320], [2327, 2332], [2332, 2340], [2340, 2346], [2346, 2353], [2369, 2373], [2373, 2392], [2392, 2395], [2395, 2399], [2408, 2413], [2414, 2433], [2433, 2435], [2438, 2459], [2459, 2463], [2469, 2479], [2495, 2499], [2504, 2509], [2509, 2513], [2531, 2553], [2553, 2564], [2564, 2580], [2583, 2635], [2635, 2638], [2638, 2646], [2649, 2652], [2652, 2657], [2659, 2687], [2687, 2696], [2696, 2728], [2740, 2742], [2742, 2764], [2764, 2772], [2776, 2783], [2783, 2786], [2786, 2795], [2795, 2800], [2810, 2813], [2813, 2817], [2817, 2821], [2821, 2845], [2845, 2851], [2851, 2860], [2881, 2884], [2887, 2891], [2898, 2905], [2919, 2926], [2934, 2939], [2955, 2961], [2961, 2974], [2974, 2979], [2984, 2995], [3005, 3009], [3009, 3013], [3017, 3021], [3037, 3041], [3041, 3056], [3056, 3063], [3063, 3079], [3079, 3087], [3094, 3099], [3099, 3108], [3108, 3119], [3119, 3127], [3127, 3131], [3137, 3142], [3152, 3155], [3155, 3175], [3180, 3185], [3193, 3194], [3198, 3200], [3211, 3221], [3231, 3233], [3233, 3240], [3240, 3246], [3249, 3256], [3256, 3261], [3261, 3274], [3274, 3281], [3281, 3284], [3284, 3310], [3313, 3318], [3318, 3327], [3333, 3349], [3354, 3369], [3369, 3391], [3391, 3412], [3420, 3425], [3425, 3429], [3429, 3443], [3443, 3461], [3461, 3465], [3465, 3472], [3472, 3481], [3486, 3498], [3498, 3504], [3513, 3521], [3528, 3531], [3531, 3541], [3550, 3554], [3565, 3569], [3569, 3574], [3574, 3585], [3597, 3599], [3599, 3610], [3620, 3666], [3673, 3711], [3724, 3747], [3771, 3796], [3796, 3799], [3799, 3801], [3808, 3820], [3820, 3822], [3822, 3832], [3832, 3839], [3846, 3858], [3872, 3874], [3878, 3884], [3927, 3928], [3936, 3941], [3952, 3958], [3972, 3975], [3991, 3999], [3999, 4016], [4037, 4058], [4074, 4099], [4099, 4103], [4103, 4115], [4127, 4142], [4142, 4153], [4163, 4169], [4169, 4186], [4211, 4217], [4217, 4225], [4225, 4259], [4259, 4265], [4269, 4285], [4285, 4293], [4306, 4317], [4352, 4386], [4386, 4388], [4388, 4395], [4395, 4400], [4400, 4441], [4441, 4446], [4459, 4470], [4479, 4491], [4491, 4517], [4517, 4527], [4527, 4531], [4533, 4535], [4535, 4547], [4547, 4569], [4569, 4580], [4580, 4593], [4598, 4601], [4602, 4608], [4621, 4627], [4627, 4644], [4644, 4652], [4663, 4668], [4668, 4685], [4685, 4718], [4731, 4735], [4740, 4742], [4742, 4747], [4747, 4757], [4757, 4764], [4773, 4779], [4807, 4809], [4809, 4835], [4838, 4841], [4841, 4850], [4850, 4867], [4873, 4882], [4882, 4899], [4899, 4914], [4914, 4920], [4920, 4929], [4938, 4945], [4948, 4956]]
        self.L2 = []#[[4973, 4976], [4984, 5005], [5005, 5023], [5023, 5031], [5031, 5035], [5043, 5046], [5046, 5055], [5063, 5070], [5079, 5086], [5086, 5087], [5087, 5093], [5098, 5106], [5126, 5133], [5137, 5146], [5146, 5149], [5149, 5151], [5171, 5174], [5174, 5181], [5181, 5184], [5192, 5197], [5306, 5313], [5313, 5322], [5336, 5342], [5342, 5346], [5346, 5358], [5358, 5360], [5370, 5376], [5376, 5396], [5399, 5410], [5443, 5467], [5467, 5489], [5501, 5510], [5510, 5517], [5517, 5520], [5535, 5541], [5541, 5559], [5559, 5562], [5579, 5583], [5597, 5621], [5621, 5624], [5631, 5638], [5725, 5738], [5758, 5763], [5763, 5769], [5769, 5774], [5774, 5776], [5776, 5782], [5782, 5784], [5784, 5792], [5842, 5845], [5845, 5848], [5850, 5888], [5888, 5896], [5896, 5907], [5915, 5927], [5927, 5930], [5943, 5945], [5951, 5983], [5983, 5990], [5990, 5993], [5998, 6004], [6019, 6024], [6024, 6033], [6033, 6107], [6123, 6131], [6160, 6177], [6177, 6182], [6182, 6185], [6185, 6194], [6203, 6209], [6228, 6238], [6238, 6240], [6256, 6258], [6268, 6278], [6290, 6307], [6307, 6310], [6310, 6354], [6367, 6391], [6391, 6415], [6420, 6425], [6425, 6431], [6431, 6454], [6454, 6467], [6474, 6480], [6480, 6485], [6485, 6489], [6500, 6504], [6514, 6519], [6535, 6539], [6573, 6577], [6627, 6635], [6668, 6674], [6674, 6680], [6697, 6703], [6739, 6744], [6744, 6747], [6747, 6759], [6759, 6765], [6765, 6768], [6768, 6771], [6771, 6772], [6772, 6775], [6785, 6799], [6803, 6819], [6853, 6864], [6878, 6902], [6902, 6907], [6915, 6939], [7003, 7008], [7008, 7017], [7032, 7092], [7092, 7096], [7096, 7107], [7114, 7119], [7119, 7156], [7156, 7175], [7175, 7196], [7250, 7271], [7283, 7288], [7337, 7342], [7342, 7352], [7370, 7373], [7421, 7425], [7433, 7437], [7437, 7438], [7473, 7495], [7500, 7509], [7528, 7531], [7531, 7532], [7532, 7535], [7557, 7560], [7614, 7619], [7625, 7632], [7643, 7667], [7704, 7716], [7728, 7735], [7768, 7772], [7795, 7802], [7812, 7821], [7831, 7836], [7837, 7841], [7841, 7848], [7865, 7872], [7872, 7881], [7883, 7886], [7886, 7889], [7898, 7899], [7899, 7914], [7924, 7928], [7933, 7949], [8017, 8032], [8051, 8055], [8070, 8078], [8114, 8124], [8141, 8158], [8158, 8163], [8163, 8178], [8191, 8195], [8218, 8224], [8257, 8263], [8281, 8287], [8287, 8295], [8318, 8325], [8330, 8338], [8367, 8372], [8430, 8434], [8439, 8443], [8443, 8449], [8465, 8470], [8470, 8474], [8485, 8513], [8513, 8516], [8516, 8524], [8544, 8547], [8552, 8558], [8565, 8617], [8667, 8671], [8763, 8767], [8793, 8799], [8815, 8839], [8844, 8866], [8902, 8906], [8906, 8911], [8914, 8921], [8930, 8949], [8949, 8953], [8953, 8964], [8964, 8967], [8967, 8973], [9004, 9009], [9012, 9020], [9033, 9037], [9037, 9041], [9041, 9052], [9063, 9071], [9088, 9092], [9122, 9128], [9160, 9168], [9172, 9178], [9197, 9203], [9241, 9257], [9274, 9285], [9285, 9301], [9301, 9306], [9306, 9338], [9338, 9344], [9344, 9377], [9377, 9382], [9382, 9388], [9408, 9418], [9424, 9435], [9447, 9452], [9468, 9471], [9495, 9501], [9501, 9503], [9503, 9507], [9568, 9575], [9575, 9579], [9579, 9582], [9590, 9595], [9614, 9638], [9710, 9727], [9737, 9759], [9759, 9764], [9780, 9787], [9787, 9795], [9855, 9860], [9886, 9893], [9893, 9897], [9910, 9922], [9932, 9937], [9944, 9946], [9964, 9969], [9975, 9984], [9996, 10014], [10030, 10041], [10041, 10044], [10044, 10046], [10049, 10059], [10066, 10071], [10077, 10088], [10110, 10116], [10134, 10154], [10164, 10171], [10171, 10177], [10236, 10242], [10242, 10244], [10264, 10267], [10267, 10274], [10278, 10291], [10307, 10314], [10338, 10341], [10341, 10347], [10373, 10380], [10380, 10406], [10408, 10420], [10431, 10433], [10439, 10455], [10481, 10485], [10485, 10507], [10517, 10538], [10551, 10556], [10559, 10573], [10581, 10586], [10596, 10603], [10607, 10611], [10621, 10626], [10674, 10677], [10697, 10700], [10706, 10723], [10754, 10800], [10802, 10814], [10817, 10820], [10820, 10831], [10850, 10873], [10946, 10951], [10951, 10960], [10965, 10970], [10970, 10976], [10976, 11008], [11020, 11031], [11057, 11065], [11077, 11087], [11087, 11090], [11090, 11097], [11100, 11109], [11109, 11127], [11133, 11138], [11145, 11179], [11200, 11221], [11228, 11232], [11232, 11237], [11262, 11287], [11305, 11317], [11317, 11321], [11321, 11328], [11330, 11345], [11360, 11367], [11391, 11408], [11458, 11468], [11504, 11509], [11529, 11547], [11559, 11565], [11580, 11596], [11596, 11624], [11624, 11649], [11658, 11669], [11669, 11703], [11703, 11711], [11711, 11731], [11740, 11781], [11781, 11787], [11796, 11812], [11826, 11831], [11831, 11840], [11868, 11871], [11871, 11881], [11899, 11903], [11903, 11907], [11910, 11912], [11912, 11914], [11927, 11935], [11935, 11948], [11948, 11951], [11951, 11963], [11963, 11970], [11970, 11973], [11973, 11974], [11974, 11977], [11984, 12010], [12010, 12014], [12022, 12027], [12027, 12049], [12057, 12065], [12065, 12067], [12067, 12082], [12085, 12098], [12098, 12106], [12118, 12123], [12123, 12156], [12156, 12166], [12166, 12176], [12195, 12199], [12208, 12211], [12222, 12227], [12244, 12270], [12270, 12277], [12290, 12293], [12293, 12298], [12321, 12338], [12338, 12349], [12371, 12388], [12388, 12393], [12412, 12423], [12435, 12441], [12441, 12443], [12443, 12447], [12467, 12469], [12503, 12510], [12510, 12514], [12568, 12575], [12609, 12612], [12617, 12622], [12683, 12689], [12689, 12696], [12708, 12713], [12723, 12727], [12741, 12745], [12771, 12779], [12779, 12782], [12821, 12826], [12868, 12875], [12890, 12894], [12898, 12905], [12982, 12986], [12986, 12989], [12989, 12994], [13001, 13003], [13016, 13025], [13058, 13063], [13068, 13073], [13116, 13120], [13146, 13154], [13166, 13170], [13170, 13181], [13196, 13202], [13250, 13253], [13255, 13262], [13262, 13266]]
        self.file1 = "CB_experience.txt"
        self.file2 = "CB_Texte_de_1857.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
    def cas1(self):
        # sensible � tout
        # liste des r�p�titions de claude bernard: experience.txt.Texte_de_1857.txt
        self.p1 = 5
        self.p2 = 2.0 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 1 # sensible à la casse
        self.pseparatorSensivitive = 1  #sensible aux séparateurs
        self.pdiacriticSensitive = 1  #sensible aux diacritiques
        self.L1 = []#[[0, 18], [21, 26], [31, 51], [57, 69], [69, 74], [79, 84], [107, 112], [112, 117], [117, 124], [124, 143], [149, 153], [161, 167], [167, 170], [170, 179], [179, 198], [202, 206], [211, 224], [224, 228], [238, 243], [246, 249], [252, 261], [278, 296], [298, 307], [307, 328], [348, 365], [365, 366], [366, 375], [375, 385], [392, 416], [417, 424], [457, 466], [476, 481], [481, 484], [484, 491], [491, 493], [493, 498], [498, 510], [510, 533], [548, 555], [555, 559], [565, 581], [581, 587], [587, 592], [592, 603], [609, 613], [613, 621], [621, 629], [629, 631], [632, 637], [648, 652], [652, 653], [653, 663], [664, 669], [678, 683], [686, 690], [690, 699], [702, 707], [736, 775], [775, 779], [783, 788], [788, 812], [817, 863], [863, 869], [887, 890], [890, 909], [909, 914], [914, 928], [942, 944], [947, 953], [953, 964], [964, 998], [1006, 1018], [1018, 1038], [1043, 1049], [1049, 1054], [1059, 1068], [1068, 1074], [1074, 1075], [1075, 1076], [1079, 1084], [1097, 1101], [1101, 1112], [1113, 1118], [1118, 1128], [1128, 1173], [1173, 1181], [1181, 1188], [1188, 1191], [1191, 1206], [1206, 1211], [1233, 1253], [1282, 1293], [1302, 1313], [1313, 1326], [1329, 1331], [1351, 1360], [1360, 1365], [1365, 1373], [1379, 1386], [1402, 1405], [1412, 1424], [1424, 1470], [1471, 1485], [1508, 1514], [1514, 1524], [1524, 1552], [1552, 1572], [1572, 1594], [1594, 1602], [1617, 1639], [1639, 1661], [1689, 1693], [1693, 1699], [1699, 1703], [1734, 1738], [1738, 1771], [1788, 1823], [1837, 1841], [1849, 1854], [1863, 1869], [1876, 1880], [1895, 1900], [1900, 1912], [1925, 1931], [1932, 1939], [1939, 1946], [1946, 1948], [1956, 1973], [1977, 1980], [1980, 1986], [1988, 1994], [1994, 2002], [2002, 2007], [2013, 2017], [2029, 2045], [2045, 2046], [2046, 2051], [2054, 2063], [2070, 2076], [2085, 2089], [2091, 2097], [2105, 2110], [2118, 2127], [2127, 2133], [2133, 2144], [2144, 2154], [2154, 2161], [2161, 2176], [2176, 2183], [2193, 2207], [2207, 2210], [2210, 2215], [2215, 2221], [2229, 2237], [2238, 2243], [2243, 2247], [2247, 2252], [2252, 2282], [2282, 2286], [2286, 2288], [2288, 2312], [2312, 2326], [2326, 2327], [2327, 2332], [2374, 2377], [2377, 2382], [2398, 2405], [2411, 2418], [2422, 2438], [2438, 2457], [2457, 2460], [2460, 2464], [2469, 2473], [2473, 2478], [2479, 2498], [2503, 2525], [2525, 2528], [2529, 2532], [2535, 2544], [2560, 2565], [2568, 2571], [2571, 2576], [2576, 2580], [2580, 2581], [2581, 2600], [2600, 2621], [2621, 2632], [2655, 2705], [2717, 2722], [2722, 2724], [2739, 2758], [2758, 2767], [2771, 2800], [2813, 2815], [2815, 2837], [2837, 2845], [2849, 2850], [2856, 2862], [2862, 2868], [2868, 2873], [2883, 2886], [2886, 2890], [2894, 2918], [2918, 2925], [2956, 2959], [2961, 2967], [2967, 2972], [2994, 3002], [3032, 3038], [3051, 3056], [3061, 3071], [3082, 3086], [3086, 3089], [3119, 3134], [3141, 3158], [3158, 3166], [3172, 3178], [3178, 3199], [3203, 3213], [3218, 3223], [3223, 3226], [3227, 3236], [3236, 3255], [3274, 3279], [3279, 3281], [3296, 3301], [3317, 3324], [3327, 3330], [3330, 3340], [3340, 3345], [3345, 3358], [3358, 3365], [3365, 3369], [3369, 3395], [3395, 3402], [3402, 3405], [3405, 3411], [3411, 3414], [3414, 3437], [3452, 3457], [3457, 3459], [3459, 3480], [3480, 3501], [3502, 3509], [3509, 3515], [3515, 3519], [3519, 3536], [3536, 3551], [3551, 3555], [3555, 3563], [3563, 3564], [3564, 3573], [3580, 3584], [3592, 3596], [3623, 3634], [3634, 3639], [3643, 3647], [3654, 3660], [3668, 3679], [3695, 3705], [3715, 3716], [3730, 3751], [3759, 3763], [3772, 3782], [3783, 3809], [3835, 3847], [3871, 3890], [3890, 3892], [3892, 3897], [3897, 3921], [3923, 3933], [3933, 3939], [3939, 3940], [3948, 3960], [4035, 4040], [4052, 4057], [4057, 4063], [4083, 4086], [4086, 4089], [4089, 4104], [4104, 4115], [4116, 4121], [4144, 4165], [4165, 4170], [4170, 4181], [4184, 4207], [4211, 4227], [4227, 4232], [4232, 4233], [4233, 4235], [4235, 4252], [4275, 4279], [4279, 4297], [4322, 4329], [4329, 4337], [4337, 4371], [4371, 4378], [4382, 4398], [4399, 4407], [4440, 4451], [4466, 4501], [4510, 4515], [4515, 4558], [4558, 4563], [4572, 4576], [4576, 4587], [4587, 4596], [4596, 4608], [4624, 4636], [4636, 4646], [4646, 4650], [4654, 4666], [4666, 4679], [4684, 4699], [4699, 4712], [4714, 4717], [4721, 4726], [4729, 4734], [4743, 4747], [4747, 4765], [4772, 4773], [4773, 4780], [4783, 4788], [4805, 4840], [4854, 4859], [4870, 4880], [4894, 4897], [4897, 4903], [4915, 4920], [4934, 4960], [4960, 4967], [4967, 4969], [4977, 4994], [4994, 5000], [5011, 5017], [5017, 5028], [5028, 5043], [5043, 5050], [5059, 5069], [5069, 5076], [5079, 5087]]
        self.L2 = []#[[5090, 5109], [5115, 5120], [5166, 5169], [5169, 5178], [5178, 5181], [5181, 5190], [5199, 5205], [5226, 5230], [5239, 5249], [5264, 5271], [5276, 5285], [5332, 5337], [5355, 5360], [5371, 5375], [5387, 5392], [5404, 5408], [5411, 5417], [5445, 5450], [5450, 5457], [5457, 5463], [5481, 5487], [5487, 5491], [5491, 5503], [5503, 5514], [5516, 5522], [5522, 5541], [5554, 5567], [5615, 5616], [5616, 5638], [5638, 5647], [5654, 5660], [5660, 5667], [5686, 5692], [5702, 5714], [5735, 5736], [5795, 5800], [5814, 5820], [5846, 5861], [5873, 5894], [5894, 5897], [5914, 5919], [5931, 5939], [5965, 5971], [5979, 5984], [6002, 6006], [6009, 6019], [6020, 6046], [6046, 6055], [6055, 6067], [6067, 6069], [6069, 6073], [6077, 6081], [6104, 6106], [6106, 6145], [6145, 6152], [6152, 6155], [6170, 6174], [6183, 6188], [6188, 6198], [6198, 6222], [6227, 6273], [6289, 6297], [6312, 6326], [6326, 6343], [6349, 6353], [6353, 6362], [6371, 6377], [6396, 6407], [6411, 6418], [6429, 6432], [6432, 6447], [6447, 6449], [6461, 6472], [6473, 6478], [6530, 6535], [6566, 6573], [6576, 6591], [6601, 6607], [6607, 6618], [6618, 6630], [6630, 6644], [6653, 6657], [6657, 6662], [6662, 6666], [6666, 6677], [6677, 6681], [6692, 6697], [6699, 6702], [6773, 6782], [6784, 6787], [6794, 6808], [6843, 6849], [6849, 6855], [6862, 6868], [6878, 6883], [6920, 6925], [6930, 6942], [6942, 6948], [6948, 6953], [6958, 6960], [6964, 6984], [6988, 7004], [7016, 7021], [7040, 7051], [7090, 7095], [7103, 7127], [7127, 7131], [7138, 7152], [7191, 7198], [7208, 7223], [7223, 7269], [7270, 7284], [7290, 7301], [7301, 7305], [7305, 7308], [7308, 7313], [7322, 7323], [7323, 7351], [7351, 7370], [7370, 7371], [7371, 7393], [7404, 7408], [7447, 7469], [7480, 7486], [7492, 7496], [7542, 7554], [7637, 7641], [7671, 7678], [7678, 7700], [7715, 7720], [7732, 7735], [7742, 7744], [7767, 7789], [7800, 7804], [7852, 7871], [7914, 7926], [7976, 7982], [8005, 8014], [8024, 8033], [8041, 8047], [8048, 8052], [8094, 8097], [8097, 8101], [8110, 8115], [8140, 8147], [8147, 8163], [8187, 8193], [8235, 8250], [8282, 8287], [8291, 8299], [8318, 8327], [8339, 8349], [8385, 8390], [8390, 8405], [8405, 8406], [8420, 8424], [8428, 8435], [8487, 8494], [8513, 8519], [8522, 8527], [8541, 8544], [8563, 8569], [8601, 8606], [8664, 8668], [8673, 8677], [8677, 8683], [8700, 8705], [8720, 8750], [8753, 8760], [8789, 8795], [8804, 8854], [8854, 8855], [8886, 8895], [8981, 8982], [8982, 8989], [9034, 9040], [9056, 9080], [9080, 9083], [9085, 9107], [9110, 9114], [9114, 9116], [9142, 9145], [9145, 9149], [9149, 9154], [9157, 9164], [9173, 9192], [9201, 9206], [9258, 9266], [9289, 9299], [9310, 9318], [9408, 9413], [9469, 9482], [9536, 9553], [9563, 9592], [9592, 9599], [9599, 9600], [9600, 9633], [9633, 9638], [9638, 9644], [9664, 9674], [9680, 9690], [9703, 9708], [9724, 9727], [9751, 9755], [9759, 9764], [9817, 9821], [9829, 9834], [9850, 9855], [9857, 9863], [9875, 9899], [9956, 9959], [9966, 9972], [9972, 9990], [10000, 10021], [10052, 10061], [10121, 10127], [10155, 10160], [10178, 10188], [10215, 10233], [10243, 10264], [10264, 10266], [10266, 10284], [10284, 10288], [10325, 10330], [10344, 10347], [10350, 10361], [10384, 10387], [10408, 10427], [10439, 10446], [10446, 10448], [10448, 10452], [10486, 10490], [10495, 10500], [10512, 10519], [10550, 10555], [10555, 10568], [10584, 10591], [10614, 10616], [10619, 10625], [10647, 10652], [10652, 10659], [10659, 10685], [10685, 10688], [10688, 10695], [10712, 10714], [10714, 10737], [10769, 10790], [10800, 10821], [10834, 10840], [10843, 10860], [10866, 10871], [10881, 10889], [10889, 10894], [10894, 10899], [10913, 10916], [10970, 10975], [10984, 10986], [10986, 10990], [10995, 11012], [11021, 11026], [11041, 11045], [11045, 11090], [11094, 11106], [11106, 11109], [11109, 11112], [11112, 11122], [11143, 11154], [11154, 11166], [11213, 11218], [11241, 11246], [11246, 11256], [11261, 11266], [11266, 11272], [11272, 11306], [11317, 11328], [11377, 11387], [11387, 11390], [11390, 11396], [11396, 11400], [11400, 11409], [11409, 11427], [11431, 11434], [11434, 11439], [11441, 11446], [11446, 11480], [11481, 11484], [11502, 11523], [11530, 11534], [11567, 11590], [11608, 11624], [11633, 11650], [11664, 11665], [11665, 11673], [11673, 11675], [11695, 11713], [11713, 11716], [11716, 11722], [11726, 11732], [11742, 11747], [11764, 11771], [11781, 11788], [11820, 11831], [11831, 11851], [11867, 11874], [11890, 11906], [11906, 11916], [11916, 11935], [11935, 11954], [11954, 11969], [11969, 11974], [11974, 11979], [11979, 11980], [11980, 12015], [12015, 12023], [12023, 12043], [12052, 12095], [12110, 12126], [12140, 12145], [12145, 12154], [12160, 12169], [12195, 12209], [12209, 12214], [12218, 12222], [12228, 12234], [12245, 12265], [12269, 12281], [12285, 12290], [12295, 12297], [12317, 12333], [12365, 12377], [12377, 12385], [12405, 12418], [12418, 12426], [12438, 12443], [12443, 12478], [12478, 12488], [12488, 12498], [12517, 12521], [12530, 12533], [12536, 12539], [12559, 12567], [12567, 12593], [12600, 12606], [12616, 12621], [12634, 12641], [12646, 12652], [12652, 12663], [12663, 12674], [12743, 12754], [12767, 12774], [12784, 12787], [12787, 12790], [12800, 12804], [12804, 12806], [12806, 12816], [12841, 12846], [12874, 12878], [12905, 12912], [12926, 12945], [12955, 12960], [12962, 12968], [13027, 13035], [13062, 13066], [13076, 13081], [13111, 13119], [13122, 13123], [13144, 13162], [13231, 13235], [13241, 13245], [13270, 13274], [13325, 13334], [13359, 13368], [13401, 13409], [13414, 13419], [13435, 13440], [13493, 13501], [13518, 13530], [13544, 13550], [13600, 13606], [13606, 13607], [13607, 13612], [13612, 13616]]
        self.file1 = "CB_experience.txt"
        self.file2 = "CB_Texte_de_1857.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
   
class Charroi(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 1 # sensible à la casse
        self.pseparatorSensivitive = 1  # sensible aux séparateurs
        self.pdiacriticSensitive = 1  # sensible aux diacritiques
        # listes de blocs répétés à aligner
        self.L1 = []
        self.L2 = []
        sepTable = str.maketrans("$",".")                                    # Modifiee 13.06.2019, ancienne ligne:   sepTable = string.maketrans("$",".")
        self.texte1 = self._readFile("Debut_A.txt").translate(sepTable)
        self.texte2 = self._readFile("Debut_E.txt").translate(sepTable)
           
class Bio(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 100
        # listes de blocs répétés à aligner
        self.L1 = []
        self.L2 = []
        sepTable = str.maketrans(""" !\r,\n:\t;-?"'`’()""","................")               # Modifiee 13.06.2019, ancienne ligne:   sepTable = string.maketrans(""" !\r,\n:\t;-?"'`�()""","................")  
        self.texte1 = self._readFile("Chr_21.fasta").lower().translate(sepTable)
        self.texte2 = self._readFile("Chr_22.fasta").lower().translate(sepTable)            
 
class Condorcet(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        elif cas=='2': self.cas2()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 1 # sensible à la casse
        self.pseparatorSensivitive = 1  # sensible aux séparateurs
        self.pdiacriticSensitive = 1  # sensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.texte1 = self._readFile("Condorcet_Esimpr1.txt")
        self.texte2 = self._readFile("Condorcet_Esimpr2.txt")
    def cas2(self):        
        self.cas1()
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques

class Althusser(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        elif cas=='2': self.cas2()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "Alth1v2.txt"
        self.file2 = "Alth1v2.txt"
        self.texte1 = self._readFile("Alth1v2.txt")
        self.texte2 = self._readFile("Alth1v2.txt")
    def cas2(self):        
        self.cas1()
        self.pcaseSensitive = 1 # sensible à la casse
        self.pseparatorSensivitive = 1  # sensible aux séparateurs
        self.pdiacriticSensitive = 1  # sensible aux diacritiques
        
        
class Traduction(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        elif cas=='2': self.cas2()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 1  # sensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        #self.file1 = "train.f.src"  
        #self.file2 = "train.f.trans" 
        self.file1 = "final.fr100.txt" 
        self.file2 = "final-en2fr100.txt"
        #self.file1 = "src100000.txt" 
        #self.file2 = "trans100000.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        #self.texte1 = """nous devons garantir que ces pays sont prêts à adhérer et , en particulier , nous devons garantir qu ' ils ont la capacité administrative de faire face à l ' acquis communautaire ."""
        #self.texte2 = """nous devons faire en sorte que les pays sont prêts à rejoindre et en particulier qu ' ils ont la capacité administrative de traiter l ' acquis communautaire ."""
        #self.texte1 =  "j ' espère vraiment , et j ' insiste très fort , que l ' on verra se présenter une occasion réelle d ' incorporer un jour ce document dans les traités "
        #self.texte2 = "j ' espère de tout cur , et je dois le dire très expressément que l ' opportunité qui se pose lorsque ce document puisse être incorporée dans les traités sur certains points à l ' avenir ." 
        
    def cas2(self):        
        self.cas1()
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques

class Bovary(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "bovary1.txt" 
        self.file2 = "bovary2.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)

class Ramuz(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "Village ds la montagne orig.txt" 
        self.file2 = "Village dans montagne OC.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class Leroux(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "chambre_jaune.txt" 
        self.file2 = "chambre_jaune.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)

class Miserables(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "miserablesT1.txt" 
        self.file2 = "miserablesT1.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class BovaryFR(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "BovaryFR.txt" 
        self.file2 = "BovaryFR.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class RougeNoir(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "RougeNoir.txt" 
        self.file2 = "RougeNoir.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class Verne(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "20000leagues1.txt" 
        self.file2 = "20000leagues2.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class Chedid(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "chedidtapuscrit.txt" 
        self.file2 = "chedidcor.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
      
class Brousse(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        
    def cas1(self):
        self.p1 = 5
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        self.file1 = "IP_823.txt" 
        self.file2 = "Macadam.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)        
            
class Resto(Data_Alignement):
    def __init__(self,cas):
        if cas=='1': self.cas1()
        
    def cas1(self):
        self.p1 = 20
        self.p2 = 2.0 #0.01 
        self.p3 = 0.05
        self.pcarOuMot = 1 
        self.pcaseSensitive = 0 # insensible à la casse
        self.pseparatorSensivitive = 0  # insensible aux séparateurs
        self.pdiacriticSensitive = 0  # insensible aux diacritiques
        self.L1 = [] # listes de blocs répétés à aligner
        self.L2 = []
        #self.file1 = "zagats.txt" 
        #self.file2 = "fodors.txt" 
        self.file1 = "zagats-nophone.txt" 
        self.file2 = "fodors-nophone.txt"
        self.texte1 = self._readFile(self.file1)
        self.texte2 = self._readFile(self.file2)
        
class Data_Synthetic(Test_Data):
    """ Jeu d'essais pour test_synthetic """
    def jeu_essai_Ramuz(self):
            """Assertion d'ordre sur occs_deplaces vérifiée"""
            insertions = [[9365, 9371], [9414, 9417], [10287, 10296], [10579, 10582], [10600, 10617], [10656, 10672], [10808, 10813], [11020, 11026], [11088, 11102], [11136, 11158], [11277, 11295], [11379, 11399], [11491, 11502], [11508, 11542], [11691, 11702], [12144, 12163], [12310, 12315], [12373, 12376], [12552, 12567], [12591, 12596], [12636, 12640], [12895, 12904], [12941, 12942], [13032, 13033], [13152, 13154], [13218, 13219], [13501, 13503], [13595, 13623], [13730, 13738], [13754, 13771], [14011, 14013], [14020, 14029], [14268, 14275], [14367, 14373], [15092, 15105], [15323, 15327], [15585, 15586], [16040, 16063], [16075, 16083], [16170, 16186], [16727, 16737]]
            suppressions = [[147, 184], [209, 256], [299, 329], [463, 471], [539, 543], [567, 578], [622, 631], [714, 799], [1005, 1013], [1197, 1201], [1453, 1456], [1693, 1707], [1851, 1898], [1974, 1995], [2022, 2046], [2055, 2059], [2068, 2073], [2115, 2200], [2252, 2254], [2319, 2472], [2555, 2598], [2710, 3045], [3052, 3055], [3061, 3065], [3452, 3502], [3563, 3575], [3659, 3719], [3742, 3756], [3765, 3773], [3795, 3854], [3859, 3932], [4047, 4089], [4116, 4295], [4352, 4499], [4703, 4787], [4815, 4830], [4937, 4942], [5025, 5033], [5503, 5507], [5770, 5776], [5799, 5866], [5870, 5908], [5918, 5933], [5962, 6013], [6061, 6080], [6193, 6202], [6341, 6372], [6386, 6387], [6509, 6512], [6719, 6736], [7019, 7020], [7025, 7027], [7147, 7149], [7177, 7189], [7308, 7314], [7352, 7353], [7371, 7382], [7471, 7477], [7533, 7534], [7638, 7639], [7984, 8000], [8086, 8092], [8125, 8139], [8170, 8190], [8382, 8403], [8471, 8477], [8489, 8517], [8572, 8585], [8625, 8639], [8823, 8860], [8987, 9003], [9034, 9048], [9135, 9138], [9178, 9189], [9200, 9205], [9213, 9240]]
            occs_deplaces = [[95, 101], [144, 147], [147, 150], [176, 180], [221, 227], [227, 232], [234, 241], [247, 250], [250, 255], [274, 275], [299, 305], [525, 526], [539, 540], [791, 792], [797, 799], [1055, 1057], [1130, 1131], [1174, 1175], [1431, 1434], [1492, 1501], [1501, 1503], [1613, 1614], [1693, 1698], [1848, 1851], [1851, 1854], [1867, 1873], [1884, 1886], [1958, 1961], [2003, 2004], [2098, 2103], [2103, 2109], [2109, 2115], [2115, 2125], [2147, 2152], [2319, 2325], [2334, 2372], [2395, 2401], [2433, 2441], [2441, 2450], [2456, 2472], [2555, 2560], [2565, 2567], [2664, 2670], [2762, 2769], [2833, 2838], [2859, 2862], [2871, 2875], [2889, 2895], [2941, 2945], [2955, 2959], [2959, 2963], [2991, 3002], [3011, 3014], [3068, 3069], [3452, 3457], [3465, 3468], [3497, 3502], [3521, 3523], [3523, 3541], [3791, 3795], [3795, 3800], [3805, 3811], [3812, 3819], [3845, 3850], [3850, 3854], [3885, 3889], [3923, 3928], [4003, 4004], [4120, 4124], [4151, 4156], [4207, 4213], [4249, 4253], [4273, 4287], [4291, 4295], [4355, 4358], [4358, 4361], [4361, 4363], [4440, 4449], [4453, 4458], [4488, 4492], [4520, 4521], [4595, 4600], [4622, 4623], [4696, 4697], [4697, 4703], [4703, 4708], [4721, 4725], [5081, 5082], [5109, 5110], [5129, 5130], [5220, 5221], [5400, 5401], [5686, 5687], [5799, 5803], [5821, 5832], [5847, 5851], [5859, 5863], [5874, 5881], [5881, 5883], [5883, 5902], [5902, 5903], [5962, 5968], [5985, 5991], [5997, 6002], [6048, 6056], [6056, 6061], [6061, 6067], [6151, 6157], [6159, 6171], [6340, 6341], [6341, 6348], [6577, 6578], [6621, 6627], [6722, 6728], [7008, 7014], [7058, 7059], [7175, 7177], [7177, 7185], [7312, 7314], [7376, 7381], [7381, 7382], [7697, 7701], [7955, 7956], [7990, 8000], [8042, 8043], [8164, 8165], [8615, 8616], [8851, 8857], [8961, 8962], [8987, 8993], [9178, 9179], [9200, 9201], [9442, 9452], [9494, 9502], [9636, 9643], [9697, 9698], [9711, 9715], [9783, 9786], [9786, 9791], [9874, 9878], [9878, 9883], [9994, 9997], [10054, 10057], [10057, 10062], [10140, 10141], [10214, 10216], [10279, 10282], [10397, 10398], [10479, 10480], [10494, 10495], [10608, 10617], [10656, 10662], [10727, 10728], [10728, 10729], [10818, 10824], [10895, 10896], [10907, 10912], [11047, 11049], [11067, 11073], [11073, 11079], [11088, 11093], [11097, 11102], [11141, 11145], [11148, 11152], [11152, 11157], [11285, 11290], [11314, 11315], [11393, 11399], [11434, 11472], [11472, 11480], [11491, 11497], [11514, 11520], [11520, 11526], [11526, 11542], [11568, 11569], [11690, 11691], [11691, 11697], [11953, 11965], [11968, 11973], [11974, 11976], [11976, 11978], [12091, 12102], [12102, 12105], [12132, 12135], [12140, 12144], [12144, 12150], [12234, 12235], [12278, 12283], [12331, 12332], [12397, 12399], [12473, 12478], [12549, 12552], [12552, 12554], [12591, 12592], [12624, 12633], [12633, 12636], [12636, 12638], [12747, 12753], [12818, 12819], [12885, 12886], [13068, 13069], [13189, 13197], [13351, 13352], [13566, 13567], [13587, 13588], [13588, 13595], [13595, 13614], [13646, 13652], [13749, 13754], [13754, 13758], [13765, 13771], [13867, 13877], [13900, 13907], [14022, 14029], [14099, 14100], [14201, 14202], [14438, 14439], [14646, 14649], [14654, 14657], [14657, 14662], [14698, 14699], [14787, 14788], [14814, 14823], [14864, 14865], [14943, 14946], [14984, 14985], [15092, 15097], [15231, 15232], [15372, 15373], [15391, 15392], [15453, 15454], [15466, 15467], [15732, 15743], [15844, 15845], [15965, 15970], [15970, 15971], [16022, 16023], [16060, 16063], [16075, 16079], [16138, 16139], [16152, 16153], [16170, 16174], [16380, 16386], [16408, 16409], [16488, 16490], [16546, 16560], [16560, 16561], [16648, 16650], [16685, 16686], [16691, 16697], [16697, 16699], [16710, 16716], [16716, 16719], [16727, 16729], [16767, 16768]]
            tous_remplacements = [[902, 910], [967, 973], [994, 997], [1014, 1023], [1148, 1155], [1597, 1600], [1712, 1721], [1796, 1806], [2273, 2279], [2630, 2706], [3098, 3105], [3513, 3557], [5373, 5379], [6125, 6128], [6148, 6171], [6225, 6235], [6281, 6288], [7539, 7543], [8247, 8251], [8693, 8701], [9986, 9997], [10057, 10066], [10087, 10091], [10100, 10108], [10233, 10238], [10709, 10714], [10818, 10831], [10907, 10918], [11229, 11237], [11431, 11480], [11575, 11583], [11953, 11978], [13188, 13197], [13805, 13808], [13828, 13845], [13900, 13909], [13955, 13959], [15166, 15169], [15830, 15833], [16249, 16258]]
            lg_texte1 = 9270
            texte_original = """Le Tout-Vieux

Les hommes fauchaient l'herbe au-dessus des rochers du Vanil, dans une espèce de creux qu'il y a entre deux parois toutes droites et où la terre est épaisse et noire ; vers midi ils s'arrêtèrent de faucher, parce que le soleil devenait chaud et ils s'assirent  à l'ombre pour manger ; puis, quand ils eurent mangé, ils retournèrent le foin étendu de la veille qui séchait ; vers le soir, il fut sec ; alors ils le nouèrent dans les grands filards ; et, les portant au bord du rocher, ils les précipitèrent l'un
après l'autre, on les voyait rouler vite, ressauter, puis bondir et s'élancer dans le grand trou, où ils tombaient d'une seule haleine - et ils arrivaient dans le pâturage où est le chalet, parmi de gros blocs, tout près de la montagne. L'ouvrage allait vite ainsi. Le soir n'était pas encore là que les hommes redescendirent.
Ils étaient trois, deux jeunes et un vieux, vêtus de même d'un pantalon de laine brune et d'une chemise de couleur, avec de gros souliers à longs clous au pied ; mais l
e vieux avait une drôle de figure ; ses cheveux blancs et bouclés sortaient de dessous son chapeau de feutre, il avait le dos voûté ; il était tout rasé, et il parlait seul en marchant.
Il n'y a pas de sentier pour descendre du Vanil, mais des traces seulement, de distance en distance dans la roche ; il faut connaître les passages, car il est facile de s'égarer et si on s'égare on est vite perdu à la vie ; et même le bon chemin ce n'est pas facile, à cause qu'on est suspendu à certains endroits aux rochers qui sont souvent lisses et des cheminées où il faut se laisser aller sur le dos, mais les gens de la montagne ont tellement l'habitude de ces choses qu'ils n'y prennent même plus garde ;
c'est leurs jambes et leurs bras qui vont pour eux ; et le vieux aux mauvais endroits continuait à fumer tranquillement sa pipe et hochait la tête comme il faisait tout le temps. Les deux jeunes étaient devant, on ne les voyait pas, tant la montagne est hérissée, creusée et ravinée, mais on entendait le grincement de leurs
 souliers sur la roche dure.
Quand ils furent en bas, ils attendirent un moment le vieux qui ne tarda pas et ils entrèrent ensemble au chalet et ils prirent chacun un des filards sur leurs épaules, ils les emportèrent au fenil. C'est un bon poids, heureusement que le fenil était près - puis ils remontèrent prendre les autres - et ainsi jusqu'à ce qu'ils eussent fini ; il y avait douze filards, puis ils entrèrent au chalet. Ils marchaient tout voûtés sous la grosse boule de foin où les mailles du filet se dessinait tellement elles
étaient serrées, et ils fléchissaient les jambes,  assurant leur pied chaque fois qu'ils le posaient devant eux sur la pente.
Le maître vacher était près de la grande chaudière suspendue à une poutre de bois qui tourne sur un pivot, avec le feu sombre dessous, près du tas de bois d'éclats de sapin et de la table bien récurée et raclée avec du verre, où sont les bols de bois, les cuillers  et le quartier de fromage et la gatte du petit lait ; et à ce moment le soleil se couchait. Le
 maître vacher leur dit:
- Eh bien ! où en êtes-vous ?
Ils répondirent :
- On a fait quatorze filards.
- Voilà, c'est une bonne journée.
- Oh ! quand on a le beau !
Comme ils avaient faim, ils bâillèrent l'un après l'autre et ils se tenaient assis sur le banc, penchés en avant et les coudes sur leurs genoux. Le jour n'entrait que par la porte et il faisait sombre. Puis les bergers qui venaient de compter le troupeau avant la nuit rentrèrent un à un ; et ils s'assirent tout près de la table pour manger. Il y avait d'abord de la soupe au lait et au pain que le bovairon avait fait cuire, dans une grande seille plate au chacun puisait à son tour, et ensuite du séré, du
fromage et le pain dur et noir. Mais ils avaient les mâchoires fortes et les dents dures ; et ils serraient les mâchoires pour manger, faisant un grand bruit et sans rien dire ; et ils puisaient de temps en temps une cuillère de petit-lait pour faire descendre le manger.
Une fois qu'ils furent rassasiés quand l'estomac se tient tranquille, i
ls s'assirent autour du foyer sur les escabeaux à un pied dont on se sert pour traire les vaches et ils s'appuyaient sur leurs genoux pour garder l'équilibre, car les nuits sont fraîches à la montagne. Et les branches de sapin brûlaient en jetant une grande flamme claire et des pétillements vifs comme le claquement d'un fouet et parfois des étincelles qui s'éteignaient lentement sur la terre battue. La flamme montait et retombait, parfois il faisait presque nuit et parfois clair comme en plein jour ; et ils
parlaient avec lenteur, comme s'ils avaient eu des pierres sur la langue et un poids sur la tête, parce que leurs pensées se soulevaient péniblement et ne sortaient qu'avec effort. Et ils parlaient d'un petit berger qui était mort l'année d'avant dans les rochers en cueillant des edelweiss ; il avait d'abord roulé sur la pente d'herbe, puis de ressaut en ressaut, jusqu'au grand trou au bas de la montagne comme les filards de foin.
- Ah ! dit un des faucheurs, j'ai vu l'endroit  ça n'est pas mauvais.
- Et
 moi, dit le maître vacher, j'ai vu quand on l'a retrouvé, que la justice et le médecin sont montés de Château d'Oex, qu'il est resté en attendant où il était toute la journée ; c'était pas du beau à voir.
Un troisième demanda :
C'est sur la tête qu'il est tombé ?
- Ah ! répondit le maître vacher, c'est peut-être sur la tête, c'est peut-être sur le dos. En tous cas, il avait la cervelle qui lui sortait et on ne savait pas où était le devant de sa tête, parce que les cheveux étaient loin et le nez et la bouche et tout.
- Seulement il est mort du coup.
- Est-ce qu'on sait.
Ils se turent. A ce moment, le vieux se leva sans rien dire et prenant la lanterne monta au fenil où on couche, les uns, sur le foin, les autres, dans des lits de planches. On le vit qui grimpait à l'échelle droite, puis la clarté de la lanterne entre les poutres du plafond et on entendit le foin crier sous ses pieds lourds.
- Qu'est-ce qu'il a ? dirent-ils.
- Voilà, dit le faucheur, c'est sa maladie.
Alors ils se turent de nouveau. 
Et comme ils montaient se coucher à leur tour, le vieux lisait dans un livre à côté de la lanterne pendant au mur. Ils ne lui dirent rien et se couchèrent. Et ils ronflèrent bientôt, mais le vieux lisait toujours. Le livre c'était la Bible. Il la tenait ouverte à deux mains devant lui, ayant mis ses grosses lunettes rondes et plissait la peau du front ; puis il secouait la tête et il parlait bas, disant :
- Va-t'en !
Ensuite il recommençait à lire, dans l'Apocalypse où les temps futurs sont arrivés, avec la venue du Christ sur la terre et toutes les calamités pour la punition des hommes pêcheurs. Et les paroles de la prophétie sont obscures pour qui est aveugle, mais pour qui sait voir, elles sont plus claires que la lumière du soleil. Et ceux qui savent voir sont ceux qui ont la foi. Ils savent l'heure où la Bête viendra et la ruine de Babylone et le nom des anges, car leurs mains pèsent déjà sur le monde et les temps
sont bientôt révolus. Mais le vieil Elie soupira et secoua de nouveau la tête, regardant 
autour de lui avec crainte ; ensuite, il se mit à prier et il disait :
- Seigneur délivre-moi, car c'est toujours la même chose, il me tient autour des épaules et il n'a pas le respect de toi, ni de ton livre. Je sais bien, mon Dieu, quelles sont mes fautes et que ce châtiment est juste, mais donne-moi ton pardon et fais qu'il s'éloigne, parce que sa présence est un sujet de grandes douleurs.
Il serrait ses mains jointes qu'il élevait devant son visage en fermant les yeux avec ferveurs et parlait à mi-voix ; et sa Bible était restée posée sur les genoux. Cependant sa prière ne servit à rien car il recommença de gémir et de s'agiter et il répétait :
- Va-t'en !
Et puis à un moment il tomba à la renverse sur le foin et demeura là sans faire un mouvement comme un homme mort, jusqu'à ce que minuit fut passé. Alors il se releva, ferma sa Bible, pria de nouveau et, soufflant sa lanterne, s'étendit pour dormir.

*
**

Le lendemain matin, il fut debout en même temps que tout le monde. Ils sortirent au petit 
matin avec leurs fourches et montèrent aux rochers. Ils fauchèrent comme la veille ; derrière eux il y avait deux montagnes, l'une grise, l'autre verte, qui se tenaient assises et qui se regardaient. Le soleil se posa au sommet des rochers comme un gros oiseau rose. Et les faux avançaient par larges ronds parmi les pierres où elles courent et se brisent.
Le vieil Elie n'était pas marié. Il gagnait sa vie à travailler durant l'été dans les chalets ; l'hiver il descendait, à la vallée, il tressait des paniers ou fendait les bardeaux dont on couvre les toits. Il vivait de peu et il était pieux. Et au commencement de l'hiver, un soir qu'il faisait froid, un mouchoir où il allait se coucher, il se mettait au lit, il avait senti que quelqu'un entrait, mais il n'avait rien vu, ni rien entendu, il avait seulement senti comme il disait. Car il y a beaucoup d'esprits
à la montagne qui habitent les grottes et les endroits où on ne peut pas aller et dans les forêts ; puis viennent des fois vers les hommes pour leur fair
e du mal ou leur du bien, selon leur nature. Et lui comprit qu'il était visité.

Le Tout-Vieux

Les hommes fauchaient l'herbe au-dessus des rochers du Vanil, dans une espèce de combe qu'il y a entre deux parois toutes droites. Vers midi ils s'arrêtèrent un moment et ils s'assirent à l'ombre pour manger ; ensuite ils retournèrent le foin étendu de la veille qui séchait ; vers le soir, il fut sec ; alors ils le nouèrent dans les grands filards ; et les portant au bord du rocher, ils les précipitèrent l'un après l'autre. On les voyait rouler vite, puis bondir et s'élancer dans le grand trou où ils
tombaient d'une seule haleine - et ils arrivaient dans le pâturage où est le chalet. La nuit n'était pas encore là que les hommes redescendirent.
Ils étaient trois, deux jeunes et un vieux, vêtus tous trois d'un pantalon de laine brune et d'une chemise de couleur et chaussés de gros souliers à larges clous ; mais le vieux avait une drôle de figure, ses cheveux blancs et bouclés sortaient de dessous son chapeau de feutre ; il avait le dos rond, il était tout rasé et il parlait seul en hochant la 
tête.
Il n'y a pas de sentier pour descendre du Vanil, mais des traces seulement, de distance en distance, dans la roche ; il faut connaître les passages, car il est facile de s'égarer et, si on s'égare, on est vite perdu à la vie ; même le bon chemin n'est pas facile, à cause qu'on est en certains endroits presque suspendu aux rochers qui sont souvent lisses et parce qu'il y a des cheminées où il faut se laisser glisser sur le dos ; mais les gens de la montagne ont tellement l'habitude de ces choses qu'ils n'y
font plus attention ; c'est leurs jambes et leurs bras qui vont pour eux ; et le vieux, aux mauvaises places, continuait à fumer tranquillement sa pipe. Les deux jeunes étaient devant, on ne les voyait pas, tant cette montagne est ravinée ; mais on entendait crier leurs souliers qui mordaient la roche.
Quand ils furent en bas tous les trois, prenant chacun un des filards sur leurs épaules, ils les portèrent au fenil. C'était un bon poids, heureusement que le fenil n'était pas loin. Ils marchaient tou
t voûtés sous la grosse boule de foin où les mailles du filet se marquaient en creux et ils fléchissaient les jambes. Puis ils remontèrent prendre les autres filards.
Le soleil était déjà couché lorsqu'ils arrivèrent au chalet. Le maître vacher leur dit :
- Eh ! bien, où en êtes-vous ?
Ils répondirent :
- On a fait quatorze filards.
- Voilà, c'est une bonne journée.
- Oh ! dirent-ils, quand on a le beau !
Comme ils avaient faim, ils bâillèrent l'un après l'autre et ils se tenaient assis sur le banc, penchés en avant et les coudes sur leurs genoux. Le jour n'entrait que par la porte et il faisait sombre. Puis les bergers rentrèrent à leur tour. On se mit à table. Il y avait d'abord de la soupe au lait et au pain que le bovairon avait fait cuire et ensuite du séré, du fromage et du pain dur. Mais ils avaient la mâchoire forte ; et ils buvaient de temps en temps une cuillère de petit-lait pour faire descendre
le manger.
Une fois qu'ils furent rassasiés, ils s'assirent autour du foyer. Les branches de sap
in brûlaient en jetant une grande flamme qui montait et retombait ; parfois il faisait presque nuit et parfois clair comme en plein jour ; et ils parlaient avec lenteur comme s'ils avaient eu des pierres sur la langue et dans la bouche et un poids sur la tête. Ils parlaient d'un petit berger qui s'était tué l'année d'avant dans les rochers en cueillant des edelweiss ; il avait d'abord roulé sur la pente d'herbe, puis, de ressaut en ressaut, jusqu'au grand trou au bas de la montagne, comme les filards.
- Ah ! dit un des faucheurs, j'ai vu l'endroit, ça n'est pourtant pas mauvais.
- Et moi, dit le maître-vacher, j'ai vu quand on l'a retrouvé, que la justice et le médecin sont montés de Château-d'Oex, qu'il est resté en attendant  où il était toute la journée ; c'était pas du beau à voir.
Un troisième demanda :
- C'est sur la tête qu'il est tombé !
- Voilà ! répondit le maître-vacher, c'est peut-être sur la tête, c'est peut-être sur le dos. En tous cas, il avait la cervelle qui sortait et on ne savait pas o
ù était le devant de sa tête, parce que les cheveux étaient loin et le nez et la bouche et tout.
- Seulement il est mort du coup.
- Est-ce qu'on sait ?
Ils se turent. A ce moment, le vieux se leva sans rien dire et, prenant la lanterne, grimpa à l'échelle droite qui mène au fenil où on couche, et on vit la clarté entre les poutres du plafond.
- Qu'est-ce qu'il a ? dirent-ils.
- C'est sa maladie, dit un des faucheurs.
Alors ils se turent de nouveau. Et, comme ils montaient dormir eux aussi, le vieux lisait dans sa Bible, à côté de la lanterne pendue au mur. Ils ne lui dirent rien et se couchèrent. Ils ronflèrent bientôt, mais le vieux lisait toujours. Il tenait le livre ouvert à deux mains devant lui, ayant mis ses grosses lunettes rondes, et plissait la peau du front ; puis il secouait la tête et parlait bas, disant :
- Va-t'en !
Ensuite, il recommençait à lire dans l'Apocalypse où les temps futurs sont annoncés, avec la venue du Christ sur la terre et toutes les calamités pour la punition des hommes.
 Les paroles de la prophétie sont obscures pour qui est aveugle, mais, pour qui sait voir, elles sont plus claires que la lumière du soleil. Et ceux qui savent voir sont ceux qui ont la foi. Ils savent l'heure où la Bête viendra et la ruine de Babylone et le nom des anges, car la main de Dieu pèse déjà sur le monde et les temps sont bientôt révolus. Mais le vieil Elie soupira et secoua de nouveau la tête, regardant autour de lui avec crainte, puis il se mit à prier et il disait :
- Seigneur, délivre-moi, car c'est toujours la même chose, il me tient autour des épaules et n'a pas le respect de toi, ni de ton Livre. Je sais bien quelles sont mes fautes et que ce châtiment est juste, mais donne-moi ton pardon et fais que celui-là s'éloigne, parce que sa présence est un sujet de grande douleur.
Il serrait ses mains jointes qu'il élevait devant son visage, en fermant les yeux avec ferveur et parlait à mi-voix ; et sa Bible était restée posée sur ses genoux. Cependant sa prière ne servit à rien, car il recommença
 de gémir et de s'agiter et il répétait :
- Va-t'en !
Et puis, à un moment, il tomba à la renverse sur le foin et demeura là sans faire un mouvement comme un homme mort, jusqu'à ce que minuit fût passé. Alors il se releva, pria de nouveau et, soufflant sa lanterne s'étendit pour dormir.

*
**

Le lendemain, il fut debout en même temps que les autres. Ils sortirent au petit matin et montèrent aux rochers. Ils fauchèrent comme la veille. Derrière eux, il y avait deux montagnes, l'une grise, l'autre verte, qui se tenaient assises et qui se regardaient. Le soleil se posa dessus comme un gros oiseau rose. Et les faux avançaient, par larges ronds, dans l'herbe haute où les pierres se cachent.
Le vieil Elie n'était pas marié. Il gagnait sa vie, durant l'été, dans les chalets de la montagne ; l'hiver, à la vallée, il tressait des paniers ou fendait les tavillons dont on couvre les toits. Il vivait de peu et il était pieux. Et au commencement de l'hiver, un soir qu'il faisait froid, comme il se mettait au lit,
 il avait senti que quelqu'un entrait, mais il n'avait rien vu, ni rien entendu ; il avait seulement senti. Car il y a beaucoup d'esprits à la montagne, qui habitent les grottes et les endroits où on ne peut pas aller et dans les forêts ; ils viennent des fois vers les hommes, pour le faire du mal ou le faire du bien ; et Elie comprit qu'il était visité.



        """
            blocsCommuns = [[0, 95], [101, 144], [184, 209], [256, 274], [275, 299], [329, 463], [471, 525], [526, 539], [543, 567], [578, 622], [631, 714], [799, 902], [910, 967], [973, 994], [997, 1005], [1013, 1014], [1023, 1055], [1057, 1130], [1131, 1148], [1155, 1174], [1175, 1197], [1201, 1206], [1206, 1307], [1307, 1388], [1388, 1402], [1402, 1431], [1434, 1453], [1456, 1492], [1503, 1521], [1521, 1560], [1560, 1597], [1600, 1613], [1614, 1693], [1707, 1712], [1721, 1785], [1785, 1796], [1806, 1848], [1898, 1958], [1961, 1974], [1995, 2003], [2004, 2022], [2046, 2055], [2059, 2068], [2073, 2098], [2200, 2252], [2254, 2273], [2279, 2319], [2472, 2491], [2491, 2555], [2598, 2630], [2706, 2710], [3045, 3052], [3055, 3061], [3065, 3068], [3069, 3092], [3092, 3098], [3105, 3212], [3212, 3452], [3502, 3513], [3557, 3563], [3575, 3659], [3719, 3742], [3756, 3765], [3773, 3791], [3854, 3859], [3932, 4003], [4004, 4047], [4089, 4116], [4295, 4311], [4311, 4352], [4499, 4520], [4521, 4595], [4600, 4622], [4623, 4672], [4672, 4696], [4787, 4815], [4830, 4937], [4942, 5007], [5007, 5025], [5033, 5081], [5082, 5091], [5091, 5109], [5110, 5129], [5130, 5220], [5221, 5256], [5256, 5339], [5339, 5373], [5379, 5400], [5401, 5503], [5507, 5537], [5537, 5686], [5687, 5750], [5750, 5770], [5776, 5799], [5866, 5870], [5908, 5918], [5933, 5962], [6013, 6016], [6016, 6048], [6080, 6091], [6091, 6125], [6128, 6148], [6171, 6193], [6202, 6225], [6235, 6281], [6288, 6340], [6372, 6379], [6379, 6386], [6387, 6450], [6450, 6509], [6512, 6554], [6554, 6577], [6578, 6621], [6627, 6719], [6736, 6801], [6801, 7008], [7014, 7019], [7020, 7025], [7027, 7058], [7059, 7147], [7149, 7175], [7189, 7230], [7230, 7308], [7314, 7352], [7353, 7371], [7382, 7471], [7477, 7533], [7534, 7539], [7543, 7605], [7605, 7638], [7639, 7697], [7701, 7746], [7746, 7764], [7764, 7825], [7825, 7837], [7837, 7955], [7956, 7984], [8000, 8042], [8043, 8086], [8092, 8125], [8139, 8140], [8140, 8164], [8165, 8170], [8190, 8247], [8251, 8262], [8262, 8382], [8403, 8454], [8454, 8471], [8477, 8489], [8517, 8572], [8585, 8598], [8598, 8615], [8616, 8625], [8639, 8693], [8701, 8823], [8860, 8882], [8882, 8961], [8962, 8987], [9003, 9034], [9048, 9135], [9138, 9173], [9173, 9178], [9189, 9200], [9205, 9213], [9240, 9270], [9270, 9365], [9371, 9414], [9417, 9442], [9452, 9470], [9470, 9494], [9502, 9636], [9643, 9697], [9698, 9711], [9715, 9739], [9739, 9783], [9791, 9874], [9883, 9986], [9997, 10054], [10066, 10087], [10091, 10099], [10099, 10100], [10108, 10140], [10141, 10214], [10216, 10233], [10238, 10257], [10257, 10279], [10282, 10287], [10296, 10397], [10398, 10479], [10480, 10494], [10495, 10524], [10524, 10543], [10543, 10579], [10582, 10600], [10617, 10656], [10672, 10709], [10714, 10727], [10729, 10808], [10813, 10818], [10831, 10895], [10896, 10907], [10918, 10960], [10960, 11020], [11026, 11039], [11039, 11047], [11049, 11067], [11079, 11088], [11102, 11111], [11111, 11136], [11158, 11210], [11210, 11229], [11237, 11277], [11295, 11314], [11315, 11379], [11399, 11431], [11480, 11484], [11484, 11491], [11502, 11508], [11542, 11545], [11545, 11568], [11569, 11575], [11583, 11690], [11702, 11942], [11942, 11953], [11978, 11984], [11984, 12068], [12068, 12091], [12105, 12114], [12114, 12132], [12135, 12140], [12163, 12234], [12235, 12278], [12283, 12310], [12315, 12331], [12332, 12373], [12376, 12397], [12399, 12473], [12478, 12500], [12500, 12549], [12567, 12591], [12596, 12624], [12640, 12747], [12753, 12818], [12819, 12837], [12837, 12885], [12886, 12895], [12904, 12922], [12922, 12941], [12942, 13032], [13033, 13068], [13069, 13152], [13154, 13188], [13197, 13218], [13219, 13321], [13321, 13351], [13352, 13501], [13503, 13566], [13567, 13587], [13623, 13646], [13652, 13656], [13656, 13666], [13666, 13695], [13695, 13698], [13698, 13730], [13738, 13749], [13771, 13805], [13808, 13828], [13845, 13867], [13877, 13900], [13909, 13955], [13959, 14011], [14013, 14020], [14029, 14036], [14036, 14099], [14100, 14159], [14159, 14201], [14202, 14225], [14225, 14268], [14275, 14367], [14373, 14438], [14439, 14646], [14649, 14654], [14662, 14667], [14667, 14698], [14699, 14787], [14788, 14814], [14823, 14864], [14865, 14943], [14946, 14984], [14985, 15003], [15003, 15092], [15105, 15161], [15161, 15166], [15169, 15231], [15232, 15265], [15265, 15323], [15327, 15372], [15373, 15391], [15392, 15453], [15454, 15466], [15467, 15585], [15586, 15614], [15614, 15656], [15656, 15699], [15699, 15732], [15743, 15744], [15744, 15768], [15768, 15773], [15773, 15830], [15833, 15844], [15845, 15965], [15971, 16022], [16023, 16040], [16063, 16075], [16083, 16138], [16139, 16152], [16153, 16170], [16186, 16195], [16195, 16249], [16258, 16380], [16386, 16408], [16409, 16488], [16490, 16515], [16515, 16546], [16561, 16648], [16650, 16685], [16686, 16691], [16699, 16710], [16719, 16727], [16737, 16767]]
            lDepl = [([95, 101], [11393, 11399]), ([144, 147], [10054, 10057]), ([144, 147], [12549, 12552]), ([144, 147], [14943, 14946]), ([147, 150], [9783, 9786]), ([147, 150], [16060, 16063]), ([176, 180], [12140, 12144]), ([221, 227], [10656, 10662]), ([227, 232], [15092, 15097]), ([247, 250], [12633, 12636]), ([250, 255], [10057, 10062]), ([274, 275], [9697, 9698]), ([274, 275], [10727, 10728]), ([274, 275], [11568, 11569]), ([274, 275], [11690, 11691]), ([274, 275], [13068, 13069]), ([274, 275], [14698, 14699]), ([274, 275], [14787, 14788]), ([299, 305], [12747, 12753]), ([525, 526], [11314, 11315]), ([525, 526], [12234, 12235]), ([525, 526], [12331, 12332]), ([525, 526], [13351, 13352]), ([525, 526], [15391, 15392]), ([525, 526], [16408, 16409]), ([525, 526], [16767, 16768]), ([539, 540], [10140, 10141]), ([539, 540], [10397, 10398]), ([539, 540], [10479, 10480]), ([539, 540], [10494, 10495]), ([539, 540], [10895, 10896]), ([539, 540], [12818, 12819]), ([539, 540], [12885, 12886]), ([539, 540], [13566, 13567]), ([539, 540], [13587, 13588]), ([539, 540], [14099, 14100]), ([539, 540], [14201, 14202]), ([539, 540], [14438, 14439]), ([539, 540], [14864, 14865]), ([539, 540], [15231, 15232]), ([539, 540], [15372, 15373]), ([539, 540], [15453, 15454]), ([539, 540], [15466, 15467]), ([539, 540], [15844, 15845]), ([539, 540], [16022, 16023]), ([539, 540], [16138, 16139]), ([539, 540], [16152, 16153]), ([539, 540], [16560, 16561]), ([539, 540], [16685, 16686]), ([791, 792], [9697, 9698]), ([791, 792], [10727, 10728]), ([791, 792], [11568, 11569]), ([791, 792], [11690, 11691]), ([791, 792], [13068, 13069]), ([791, 792], [14698, 14699]), ([791, 792], [14787, 14788]), ([797, 799], [16697, 16699]), ([1055, 1057], [10214, 10216]), ([1055, 1057], [11047, 11049]), ([1055, 1057], [12397, 12399]), ([1055, 1057], [16488, 16490]), ([1055, 1057], [16727, 16729]), ([1130, 1131], [10140, 10141]), ([1130, 1131], [10397, 10398]), ([1130, 1131], [10479, 10480]), ([1130, 1131], [10494, 10495]), ([1130, 1131], [10895, 10896]), ([1130, 1131], [12818, 12819]), ([1130, 1131], [12885, 12886]), ([1130, 1131], [13566, 13567]), ([1130, 1131], [13587, 13588]), ([1130, 1131], [14099, 14100]), ([1130, 1131], [14201, 14202]), ([1130, 1131], [14438, 14439]), ([1130, 1131], [14864, 14865]), ([1130, 1131], [15231, 15232]), ([1130, 1131], [15372, 15373]), ([1130, 1131], [15453, 15454]), ([1130, 1131], [15466, 15467]), ([1130, 1131], [15844, 15845]), ([1130, 1131], [16022, 16023]), ([1130, 1131], [16138, 16139]), ([1130, 1131], [16152, 16153]), ([1130, 1131], [16560, 16561]), ([1130, 1131], [16685, 16686]), ([1174, 1175], [10140, 10141]), ([1174, 1175], [10397, 10398]), ([1174, 1175], [10479, 10480]), ([1174, 1175], [10494, 10495]), ([1174, 1175], [10895, 10896]), ([1174, 1175], [12818, 12819]), ([1174, 1175], [12885, 12886]), ([1174, 1175], [13566, 13567]), ([1174, 1175], [13587, 13588]), ([1174, 1175], [14099, 14100]), ([1174, 1175], [14201, 14202]), ([1174, 1175], [14438, 14439]), ([1174, 1175], [14864, 14865]), ([1174, 1175], [15231, 15232]), ([1174, 1175], [15372, 15373]), ([1174, 1175], [15453, 15454]), ([1174, 1175], [15466, 15467]), ([1174, 1175], [15844, 15845]), ([1174, 1175], [16022, 16023]), ([1174, 1175], [16138, 16139]), ([1174, 1175], [16152, 16153]), ([1174, 1175], [16560, 16561]), ([1174, 1175], [16685, 16686]), ([1431, 1434], [10054, 10057]), ([1431, 1434], [12549, 12552]), ([1431, 1434], [14943, 14946]), ([1492, 1501], [10608, 10617]), ([1501, 1503], [11976, 11978]), ([1613, 1614], [10140, 10141]), ([1613, 1614], [10397, 10398]), ([1613, 1614], [10479, 10480]), ([1613, 1614], [10494, 10495]), ([1613, 1614], [10895, 10896]), ([1613, 1614], [12818, 12819]), ([1613, 1614], [12885, 12886]), ([1613, 1614], [13566, 13567]), ([1613, 1614], [13587, 13588]), ([1613, 1614], [14099, 14100]), ([1613, 1614], [14201, 14202]), ([1613, 1614], [14438, 14439]), ([1613, 1614], [14864, 14865]), ([1613, 1614], [15231, 15232]), ([1613, 1614], [15372, 15373]), ([1613, 1614], [15453, 15454]), ([1613, 1614], [15466, 15467]), ([1613, 1614], [15844, 15845]), ([1613, 1614], [16022, 16023]), ([1613, 1614], [16138, 16139]), ([1613, 1614], [16152, 16153]), ([1613, 1614], [16560, 16561]), ([1613, 1614], [16685, 16686]), ([1693, 1698], [11152, 11157]), ([1848, 1851], [10054, 10057]), ([1848, 1851], [12549, 12552]), ([1848, 1851], [14943, 14946]), ([1851, 1854], [10279, 10282]), ([1867, 1873], [16380, 16386]), ([1884, 1886], [12636, 12638]), ([1958, 1961], [12132, 12135]), ([1958, 1961], [14646, 14649]), ([2003, 2004], [10140, 10141]), ([2003, 2004], [10397, 10398]), ([2003, 2004], [10479, 10480]), ([2003, 2004], [10494, 10495]), ([2003, 2004], [10895, 10896]), ([2003, 2004], [12818, 12819]), ([2003, 2004], [12885, 12886]), ([2003, 2004], [13566, 13567]), ([2003, 2004], [13587, 13588]), ([2003, 2004], [14099, 14100]), ([2003, 2004], [14201, 14202]), ([2003, 2004], [14438, 14439]), ([2003, 2004], [14864, 14865]), ([2003, 2004], [15231, 15232]), ([2003, 2004], [15372, 15373]), ([2003, 2004], [15453, 15454]), ([2003, 2004], [15466, 15467]), ([2003, 2004], [15844, 15845]), ([2003, 2004], [16022, 16023]), ([2003, 2004], [16138, 16139]), ([2003, 2004], [16152, 16153]), ([2003, 2004], [16560, 16561]), ([2003, 2004], [16685, 16686]), ([2098, 2103], [12278, 12283]), ([2103, 2109], [10818, 10824]), ([2109, 2115], [11691, 11697]), ([2115, 2125], [9442, 9452]), ([2147, 2152], [11285, 11290]), ([2319, 2325], [11491, 11497]), ([2334, 2372], [11434, 11472]), ([2395, 2401], [11514, 11520]), ([2433, 2441], [11472, 11480]), ([2441, 2450], [14814, 14823]), ([2456, 2472], [11526, 11542]), ([2555, 2560], [15965, 15970]), ([2565, 2567], [12636, 12638]), ([2664, 2670], [11514, 11520]), ([2762, 2769], [13900, 13907]), ([2833, 2838], [15965, 15970]), ([2859, 2862], [9994, 9997]), ([2871, 2875], [16170, 16174]), ([2941, 2945], [11141, 11145]), ([2955, 2959], [11148, 11152]), ([2959, 2963], [11141, 11145]), ([2991, 3002], [12091, 12102]), ([3011, 3014], [12102, 12105]), ([3068, 3069], [11314, 11315]), ([3068, 3069], [12234, 12235]), ([3068, 3069], [12331, 12332]), ([3068, 3069], [13351, 13352]), ([3068, 3069], [15391, 15392]), ([3068, 3069], [16408, 16409]), ([3068, 3069], [16767, 16768]), ([3452, 3457], [11088, 11093]), ([3465, 3468], [14654, 14657]), ([3497, 3502], [9878, 9883]), ([3521, 3523], [10214, 10216]), ([3521, 3523], [11047, 11049]), ([3521, 3523], [12397, 12399]), ([3521, 3523], [16488, 16490]), ([3521, 3523], [16727, 16729]), ([3791, 3795], [11141, 11145]), ([3805, 3811], [12144, 12150]), ([3812, 3819], [9636, 9643]), ([3845, 3850], [11097, 11102]), ([3850, 3854], [11141, 11145]), ([3885, 3889], [13754, 13758]), ([3923, 3928], [12473, 12478]), ([4003, 4004], [9697, 9698]), ([4003, 4004], [10727, 10728]), ([4003, 4004], [11568, 11569]), ([4003, 4004], [11690, 11691]), ([4003, 4004], [13068, 13069]), ([4003, 4004], [14698, 14699]), ([4003, 4004], [14787, 14788]), ([4120, 4124], [11141, 11145]), ([4151, 4156], [11968, 11973]), ([4207, 4213], [11073, 11079]), ([4249, 4253], [11141, 11145]), ([4273, 4287], [16546, 16560]), ([4291, 4295], [11141, 11145]), ([4355, 4358], [16716, 16719]), ([4358, 4361], [10054, 10057]), ([4358, 4361], [12549, 12552]), ([4358, 4361], [14943, 14946]), ([4361, 4363], [12552, 12554]), ([4440, 4449], [12624, 12633]), ([4453, 4458], [11097, 11102]), ([4488, 4492], [9874, 9878]), ([4520, 4521], [10140, 10141]), ([4520, 4521], [10397, 10398]), ([4520, 4521], [10479, 10480]), ([4520, 4521], [10494, 10495]), ([4520, 4521], [10895, 10896]), ([4520, 4521], [12818, 12819]), ([4520, 4521], [12885, 12886]), ([4520, 4521], [13566, 13567]), ([4520, 4521], [13587, 13588]), ([4520, 4521], [14099, 14100]), ([4520, 4521], [14201, 14202]), ([4520, 4521], [14438, 14439]), ([4520, 4521], [14864, 14865]), ([4520, 4521], [15231, 15232]), ([4520, 4521], [15372, 15373]), ([4520, 4521], [15453, 15454]), ([4520, 4521], [15466, 15467]), ([4520, 4521], [15844, 15845]), ([4520, 4521], [16022, 16023]), ([4520, 4521], [16138, 16139]), ([4520, 4521], [16152, 16153]), ([4520, 4521], [16560, 16561]), ([4520, 4521], [16685, 16686]), ([4595, 4600], [9786, 9791]), ([4622, 4623], [10140, 10141]), ([4622, 4623], [10397, 10398]), ([4622, 4623], [10479, 10480]), ([4622, 4623], [10494, 10495]), ([4622, 4623], [10895, 10896]), ([4622, 4623], [12818, 12819]), ([4622, 4623], [12885, 12886]), ([4622, 4623], [13566, 13567]), ([4622, 4623], [13587, 13588]), ([4622, 4623], [14099, 14100]), ([4622, 4623], [14201, 14202]), ([4622, 4623], [14438, 14439]), ([4622, 4623], [14864, 14865]), ([4622, 4623], [15231, 15232]), ([4622, 4623], [15372, 15373]), ([4622, 4623], [15453, 15454]), ([4622, 4623], [15466, 15467]), ([4622, 4623], [15844, 15845]), ([4622, 4623], [16022, 16023]), ([4622, 4623], [16138, 16139]), ([4622, 4623], [16152, 16153]), ([4622, 4623], [16560, 16561]), ([4622, 4623], [16685, 16686]), ([4696, 4697], [10140, 10141]), ([4696, 4697], [10397, 10398]), ([4696, 4697], [10479, 10480]), ([4696, 4697], [10494, 10495]), ([4696, 4697], [10895, 10896]), ([4696, 4697], [12818, 12819]), ([4696, 4697], [12885, 12886]), ([4696, 4697], [13566, 13567]), ([4696, 4697], [13587, 13588]), ([4696, 4697], [14099, 14100]), ([4696, 4697], [14201, 14202]), ([4696, 4697], [14438, 14439]), ([4696, 4697], [14864, 14865]), ([4696, 4697], [15231, 15232]), ([4696, 4697], [15372, 15373]), ([4696, 4697], [15453, 15454]), ([4696, 4697], [15466, 15467]), ([4696, 4697], [15844, 15845]), ([4696, 4697], [16022, 16023]), ([4696, 4697], [16138, 16139]), ([4696, 4697], [16152, 16153]), ([4696, 4697], [16560, 16561]), ([4696, 4697], [16685, 16686]), ([4697, 4703], [10656, 10662]), ([4703, 4708], [15092, 15097]), ([4721, 4725], [16075, 16079]), ([5081, 5082], [9697, 9698]), ([5081, 5082], [10727, 10728]), ([5081, 5082], [11568, 11569]), ([5081, 5082], [11690, 11691]), ([5081, 5082], [13068, 13069]), ([5081, 5082], [14698, 14699]), ([5081, 5082], [14787, 14788]), ([5109, 5110], [11314, 11315]), ([5109, 5110], [12234, 12235]), ([5109, 5110], [12331, 12332]), ([5109, 5110], [13351, 13352]), ([5109, 5110], [15391, 15392]), ([5109, 5110], [16408, 16409]), ([5109, 5110], [16767, 16768]), ([5129, 5130], [9697, 9698]), ([5129, 5130], [10727, 10728]), ([5129, 5130], [11568, 11569]), ([5129, 5130], [11690, 11691]), ([5129, 5130], [13068, 13069]), ([5129, 5130], [14698, 14699]), ([5129, 5130], [14787, 14788]), ([5220, 5221], [9697, 9698]), ([5220, 5221], [10727, 10728]), ([5220, 5221], [11568, 11569]), ([5220, 5221], [11690, 11691]), ([5220, 5221], [13068, 13069]), ([5220, 5221], [14698, 14699]), ([5220, 5221], [14787, 14788]), ([5400, 5401], [9697, 9698]), ([5400, 5401], [10727, 10728]), ([5400, 5401], [11568, 11569]), ([5400, 5401], [11690, 11691]), ([5400, 5401], [13068, 13069]), ([5400, 5401], [14698, 14699]), ([5400, 5401], [14787, 14788]), ([5686, 5687], [12591, 12592]), ([5799, 5803], [11141, 11145]), ([5821, 5832], [15732, 15743]), ([5847, 5851], [16170, 16174]), ([5859, 5863], [9711, 9715]), ([5874, 5881], [13588, 13595]), ([5881, 5883], [11974, 11976]), ([5883, 5902], [13595, 13614]), ([5902, 5903], [10140, 10141]), ([5902, 5903], [10397, 10398]), ([5902, 5903], [10479, 10480]), ([5902, 5903], [10494, 10495]), ([5902, 5903], [10895, 10896]), ([5902, 5903], [12818, 12819]), ([5902, 5903], [12885, 12886]), ([5902, 5903], [13566, 13567]), ([5902, 5903], [13587, 13588]), ([5902, 5903], [14099, 14100]), ([5902, 5903], [14201, 14202]), ([5902, 5903], [14438, 14439]), ([5902, 5903], [14864, 14865]), ([5902, 5903], [15231, 15232]), ([5902, 5903], [15372, 15373]), ([5902, 5903], [15453, 15454]), ([5902, 5903], [15466, 15467]), ([5902, 5903], [15844, 15845]), ([5902, 5903], [16022, 16023]), ([5902, 5903], [16138, 16139]), ([5902, 5903], [16152, 16153]), ([5902, 5903], [16560, 16561]), ([5902, 5903], [16685, 16686]), ([5962, 5968], [13646, 13652]), ([5985, 5991], [11067, 11073]), ([5997, 6002], [10907, 10912]), ([6048, 6056], [13189, 13197]), ([6056, 6061], [13749, 13754]), ([6061, 6067], [16691, 16697]), ([6061, 6067], [16710, 16716]), ([6159, 6171], [11953, 11965]), ([6340, 6341], [14984, 14985]), ([6341, 6348], [14022, 14029]), ([6577, 6578], [10140, 10141]), ([6577, 6578], [10397, 10398]), ([6577, 6578], [10479, 10480]), ([6577, 6578], [10494, 10495]), ([6577, 6578], [10895, 10896]), ([6577, 6578], [12818, 12819]), ([6577, 6578], [12885, 12886]), ([6577, 6578], [13566, 13567]), ([6577, 6578], [13587, 13588]), ([6577, 6578], [14099, 14100]), ([6577, 6578], [14201, 14202]), ([6577, 6578], [14438, 14439]), ([6577, 6578], [14864, 14865]), ([6577, 6578], [15231, 15232]), ([6577, 6578], [15372, 15373]), ([6577, 6578], [15453, 15454]), ([6577, 6578], [15466, 15467]), ([6577, 6578], [15844, 15845]), ([6577, 6578], [16022, 16023]), ([6577, 6578], [16138, 16139]), ([6577, 6578], [16152, 16153]), ([6577, 6578], [16560, 16561]), ([6577, 6578], [16685, 16686]), ([6621, 6627], [11520, 11526]), ([6722, 6728], [13765, 13771]), ([7008, 7014], [11073, 11079]), ([7058, 7059], [11314, 11315]), ([7058, 7059], [12234, 12235]), ([7058, 7059], [12331, 12332]), ([7058, 7059], [13351, 13352]), ([7058, 7059], [15391, 15392]), ([7058, 7059], [16408, 16409]), ([7058, 7059], [16767, 16768]), ([7175, 7177], [10214, 10216]), ([7175, 7177], [11047, 11049]), ([7175, 7177], [12397, 12399]), ([7175, 7177], [16488, 16490]), ([7175, 7177], [16727, 16729]), ([7177, 7185], [9494, 9502]), ([7312, 7314], [16648, 16650]), ([7376, 7381], [14657, 14662]), ([7381, 7382], [10140, 10141]), ([7381, 7382], [10397, 10398]), ([7381, 7382], [10479, 10480]), ([7381, 7382], [10494, 10495]), ([7381, 7382], [10895, 10896]), ([7381, 7382], [12818, 12819]), ([7381, 7382], [12885, 12886]), ([7381, 7382], [13566, 13567]), ([7381, 7382], [13587, 13588]), ([7381, 7382], [14099, 14100]), ([7381, 7382], [14201, 14202]), ([7381, 7382], [14438, 14439]), ([7381, 7382], [14864, 14865]), ([7381, 7382], [15231, 15232]), ([7381, 7382], [15372, 15373]), ([7381, 7382], [15453, 15454]), ([7381, 7382], [15466, 15467]), ([7381, 7382], [15844, 15845]), ([7381, 7382], [16022, 16023]), ([7381, 7382], [16138, 16139]), ([7381, 7382], [16152, 16153]), ([7381, 7382], [16560, 16561]), ([7381, 7382], [16685, 16686]), ([7697, 7701], [11141, 11145]), ([7955, 7956], [15970, 15971]), ([7990, 8000], [13867, 13877]), ([8042, 8043], [10140, 10141]), ([8042, 8043], [10397, 10398]), ([8042, 8043], [10479, 10480]), ([8042, 8043], [10494, 10495]), ([8042, 8043], [10895, 10896]), ([8042, 8043], [12818, 12819]), ([8042, 8043], [12885, 12886]), ([8042, 8043], [13566, 13567]), ([8042, 8043], [13587, 13588]), ([8042, 8043], [14099, 14100]), ([8042, 8043], [14201, 14202]), ([8042, 8043], [14438, 14439]), ([8042, 8043], [14864, 14865]), ([8042, 8043], [15231, 15232]), ([8042, 8043], [15372, 15373]), ([8042, 8043], [15453, 15454]), ([8042, 8043], [15466, 15467]), ([8042, 8043], [15844, 15845]), ([8042, 8043], [16022, 16023]), ([8042, 8043], [16138, 16139]), ([8042, 8043], [16152, 16153]), ([8042, 8043], [16560, 16561]), ([8042, 8043], [16685, 16686]), ([8164, 8165], [11314, 11315]), ([8164, 8165], [12234, 12235]), ([8164, 8165], [12331, 12332]), ([8164, 8165], [13351, 13352]), ([8164, 8165], [15391, 15392]), ([8164, 8165], [16408, 16409]), ([8164, 8165], [16767, 16768]), ([8615, 8616], [9697, 9698]), ([8615, 8616], [10727, 10728]), ([8615, 8616], [11568, 11569]), ([8615, 8616], [11690, 11691]), ([8615, 8616], [13068, 13069]), ([8615, 8616], [14698, 14699]), ([8615, 8616], [14787, 14788]), ([8961, 8962], [10140, 10141]), ([8961, 8962], [10397, 10398]), ([8961, 8962], [10479, 10480]), ([8961, 8962], [10494, 10495]), ([8961, 8962], [10895, 10896]), ([8961, 8962], [12818, 12819]), ([8961, 8962], [12885, 12886]), ([8961, 8962], [13566, 13567]), ([8961, 8962], [13587, 13588]), ([8961, 8962], [14099, 14100]), ([8961, 8962], [14201, 14202]), ([8961, 8962], [14438, 14439]), ([8961, 8962], [14864, 14865]), ([8961, 8962], [15231, 15232]), ([8961, 8962], [15372, 15373]), ([8961, 8962], [15453, 15454]), ([8961, 8962], [15466, 15467]), ([8961, 8962], [15844, 15845]), ([8961, 8962], [16022, 16023]), ([8961, 8962], [16138, 16139]), ([8961, 8962], [16152, 16153]), ([8961, 8962], [16560, 16561]), ([8961, 8962], [16685, 16686]), ([8987, 8993], [16380, 16386]), ([9178, 9179], [9697, 9698]), ([9178, 9179], [10727, 10728]), ([9178, 9179], [11568, 11569]), ([9178, 9179], [11690, 11691]), ([9178, 9179], [13068, 13069]), ([9178, 9179], [14698, 14699]), ([9178, 9179], [14787, 14788]), ([9200, 9201], [9697, 9698]), ([9200, 9201], [10727, 10728]), ([9200, 9201], [11568, 11569]), ([9200, 9201], [11690, 11691]), ([9200, 9201], [13068, 13069]), ([9200, 9201], [14698, 14699]), ([9200, 9201], [14787, 14788])]
            resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
            return resultat
        
    def jeu_essai_Althusser(self):
        """freudEtLacan manuscrit1 V1.txt / M1 v2.txt 
        Sensible à la casse, aux séparateurs et aux signes diacritiques
        Pas d'assertion d'ordre sur occs_deplaces / inclusion de dep dans d'autres"""
        insertions = [[39713, 39714], [39968, 39977], [40166, 40168], [40383, 40384], [40503, 40504], [40674, 40675], [40722, 40723], [40915, 40916], [40956, 40957], [41037, 41038], [41289, 41290], [41984, 41985], [42055, 42056], [42529, 42530], [42551, 42570], [42618, 42620], [42634, 42635], [42654, 42655], [42663, 42664], [42863, 42868], [42884, 42886], [43051, 43059], [43066, 43081], [43244, 43245], [43253, 43256], [43559, 43560], [43930, 43931], [44098, 44099], [44158, 44159], [44219, 44220], [44668, 44674], [45379, 45381], [45460, 45470], [45672, 45689], [45817, 45845], [46521, 46522], [46781, 46785], [46890, 46891], [47310, 47311], [47601, 47602], [47851, 47855], [48082, 48083], [48703, 48709], [48922, 48937], [49061, 49087], [49489, 49490], [49745, 49747], [49806, 49807], [50145, 50146], [50498, 50500], [50796, 50808], [50839, 50840], [50983, 50984], [50999, 51000], [51112, 51138], [51161, 51163], [51282, 51283], [51334, 51335], [51344, 51346], [51435, 51436], [51444, 51445], [51464, 51465], [51560, 51561], [51583, 51584], [51615, 51645], [51962, 51963], [52053, 52054], [52306, 52309], [52333, 52342], [52621, 52623], [52653, 52660], [52668, 52679], [52810, 52824], [52842, 52845], [52869, 52870], [53030, 53034], [53287, 53288], [53446, 53457], [53736, 53746], [53803, 53804], [53833, 53848], [54110, 54111], [54127, 54128], [54700, 54701], [54968, 55003], [55124, 55133], [55142, 55146], [55635, 55637], [55913, 55919], [55954, 55955], [56515, 56517], [56566, 56567], [56939, 56940], [57193, 57213], [57242, 57243], [57254, 57307], [57428, 57429], [57496, 57497], [57640, 57647], [58050, 58051], [58160, 58168], [58497, 58498], [58553, 58558], [59132, 59133], [59161, 59162], [59221, 59222], [59283, 59284], [59374, 59391], [59544, 59563], [59685, 59686], [59748, 59749], [59982, 59996], [60010, 60015], [60068, 60093], [60116, 60125], [60152, 60153], [60286, 60293], [60584, 60592], [60724, 60729], [60755, 60774], [60930, 60933], [60980, 60982], [61011, 61012], [61026, 61028], [61052, 61069], [61121, 61198], [61206, 61288], [61312, 61331], [61468, 61480], [61578, 61589], [61600, 61618], [61805, 61812], [62185, 62186], [62221, 62222], [62352, 62357], [62424, 62438], [62619, 62644], [62714, 62729], [62887, 62904], [62933, 62937], [63168, 63189], [63451, 63452], [63620, 63621], [63762, 63769], [63809, 63816], [63914, 63915], [64172, 64197], [64306, 64307], [64548, 64572], [64579, 64643], [64648, 64651], [64814, 64817], [64838, 64849], [64894, 64907], [64913, 64925], [64938, 64939], [64955, 64957], [64963, 64964], [65014, 65018], [65224, 65232], [65243, 65244], [65437, 65438], [65447, 65456], [65495, 65503], [65692, 65695], [65747, 65783], [66076, 66077], [66195, 66230], [66509, 66511], [66738, 66742], [66753, 66764], [66864, 66919], [66924, 66926], [67313, 67314], [67491, 67492], [67674, 67677], [67700, 67701], [68059, 68060], [69070, 69071], [69312, 69313], [69490, 69491], [69508, 69540], [69806, 69807], [69838, 69840], [70078, 70084], [70266, 70267], [70375, 70376], [70701, 70720], [70739, 70744], [70752, 70753], [70787, 70794], [70864, 70870], [70924, 70925], [70991, 70999], [71457, 71458], [71489, 71499], [71595, 71597], [71637, 71638], [71722, 71723], [71780, 71781], [72044, 72045], [72169, 72170], [72439, 72467], [72503, 72514], [72720, 72729], [73079, 73080], [73328, 73334], [73726, 73727], [73912, 73913], [74110, 74114], [74291, 74295], [74372, 74373], [74480, 74481], [74565, 74573], [74783, 74794], [74908, 74919], [75877, 75878], [76023, 76037], [76323, 76362], [76449, 76464], [76656, 76670], [76681, 77027], [77364, 77386], [77427, 77429], [77436, 77674], [77801, 77802], [77847, 77852], [77945, 77946], [78135, 78136], [78181, 78182], [78192, 78193], [78340, 78341], [78360, 78364], [79019, 79020], [79867, 79868]]
        suppressions =  [[17, 88], [677, 691], [701, 706], [1069, 1070], [1105, 1106], [1198, 1199], [1399, 1404], [1638, 1639], [1698, 1699], [2773, 2785], [3259, 3260], [3281, 3290], [3338, 3339], [3794, 3799], [3839, 3851], [3953, 3954], [4257, 4258], [5281, 5282], [5832, 5833], [5909, 5911], [6067, 6072], [6147, 6148], [7151, 7152], [7412, 7425], [8475, 8479], [8534, 8537], [9564, 9568], [9683, 9685], [9744, 9746], [10355, 10362], [10416, 10418], [10423, 10426], [10563, 10565], [11021, 11022], [11562, 11567], [11918, 11932], [12030, 12042], [12405, 12407], [12415, 12416], [12743, 12745], [12908, 12910], [12980, 12989], [13252, 13253], [13273, 13276], [13359, 13362], [13367, 13387], [14010, 14022], [14371, 14378], [15150, 15152], [15497, 15505], [15597, 15599], [15764, 15765], [16063, 16064], [16414, 16417], [16838, 16842], [16872, 16889], [16903, 16904], [17313, 17314], [17685, 17689], [17757, 17760], [17853, 17864], [18039, 18057], [18093, 18095], [18109, 18111], [18244, 18245], [18457, 18458], [19004, 19005], [19824, 19831], [19971, 19972], [20403, 20417], [20837, 20838], [20899, 20900], [20946, 20950], [20960, 20963], [21068, 21100], [21127, 21140], [21321, 21322], [21369, 21524], [21532, 21533], [21544, 21545], [21555, 21556], [21629, 21630], [21646, 21659], [21667, 21681], [21931, 21933], [21944, 21950], [22322, 22323], [22535, 22571], [22717, 22731], [22798, 22801], [22954, 22956], [23192, 23198], [23227, 23242], [23369, 23370], [23392, 23397], [23484, 23489], [23533, 23562], [23946, 23957], [24098, 24118], [24254, 24265], [24761, 24771], [24879, 24886], [24891, 24907], [25107, 25127], [25162, 25187], [25359, 25361], [25373, 25374], [25491, 25503], [25696, 25700], [25996, 26007], [26048, 26049], [26465, 26473], [26718, 26723], [26732, 26737], [26939, 26948], [27059, 27061], [27066, 27072], [27101, 27104], [27850, 27867], [29273, 29274], [29588, 29590], [29651, 29660], [30203, 30204], [30837, 30838], [30950, 30973], [31027, 31028], [31669, 31672], [32546, 32547], [32565, 32567], [32623, 32658], [35029, 35067], [35095, 35102], [35359, 35361], [36373, 36382], [36393, 36403], [36414, 36418], [37347, 37349], [37720, 37723], [38599, 38600], [39043, 39045], [39354, 39356]]
        occs_deplaces = [[43, 49], [61, 66], [75, 83], [446, 449], [1078, 1084], [1650, 1651], [1651, 1658], [1904, 1905], [1909, 1913], [2008, 2009], [2773, 2779], [3161, 3164], [3287, 3290], [3579, 3583], [3599, 3600], [3728, 3732], [3736, 3741], [3795, 3798], [3840, 3845], [3847, 3850], [5565, 5571], [7389, 7390], [7414, 7420], [7421, 7423], [7530, 7531], [8488, 8493], [9346, 9351], [9564, 9565], [11130, 11132], [11865, 11867], [12557, 12559], [12879, 12884], [12980, 12986], [13275, 13276], [13763, 13768], [14016, 14022], [15512, 15516], [16872, 16877], [16879, 16883], [18042, 18047], [18102, 18103], [18128, 18131], [18134, 18144], [18483, 18484], [18486, 18491], [18611, 18620], [19017, 19023], [19389, 19393], [20878, 20881], [21135, 21140], [21375, 21384], [21386, 21395], [21418, 21422], [21432, 21440], [21440, 21444], [21453, 21456], [21472, 21476], [21487, 21492], [21500, 21507], [21718, 21725], [21730, 21757], [21944, 21949], [22582, 22587], [22852, 22858], [23270, 23276], [23472, 23479], [23537, 23539], [23544, 23554], [23554, 23558], [23718, 23722], [23954, 23957], [24258, 24260], [24487, 24488], [24496, 24497], [24519, 24520], [24879, 24881], [25112, 25115], [25176, 25181], [25185, 25187], [25216, 25217], [25500, 25503], [25937, 25938], [26006, 26007], [26466, 26473], [26734, 26737], [26750, 26752], [26752, 26755], [27089, 27094], [30950, 30956], [30964, 30967], [31641, 31645], [32516, 32519], [32531, 32533], [32642, 32645], [32800, 32809], [33407, 33411], [34840, 34846], [35053, 35058], [36373, 36381], [36376, 36378], [37852, 37857], [40476, 40477], [43141, 43150], [44584, 44586], [45146, 45151], [45456, 45460], [45460, 45466], [45687, 45689], [45823, 45828], [47839, 47842], [47896, 47898], [49075, 49080], [50951, 50954], [51619, 51622], [51622, 51627], [52675, 52679], [52702, 52707], [52810, 52814], [53213, 53220], [53451, 53457], [54622, 54625], [54989, 54994], [55014, 55017], [55799, 55800], [56352, 56355], [56399, 56404], [56404, 56409], [57202, 57208], [57274, 57280], [57284, 57285], [57646, 57647], [57699, 57709], [59374, 59375], [59401, 59407], [59559, 59563], [59982, 59988], [60010, 60014], [60072, 60076], [61104, 61105], [61140, 61146], [61212, 61219], [61225, 61252], [61355, 61361], [61475, 61480], [61805, 61807], [61808, 61812], [62464, 62465], [62565, 62566], [62566, 62569], [62580, 62587], [62892, 62896], [62897, 62900], [63109, 63115], [63168, 63173], [63179, 63189], [63389, 63392], [63392, 63395], [63809, 63812], [64189, 64191], [64434, 64441], [64554, 64559], [64609, 64615], [64841, 64846], [64894, 64897], [65104, 65106], [65226, 65229], [65447, 65450], [65497, 65500], [65749, 65752], [65780, 65783], [66523, 66525], [66573, 66574], [66741, 66742], [68755, 68758], [68778, 68786], [68786, 68790], [69508, 69512], [69525, 69530], [70790, 70794], [71195, 71201], [72442, 72443], [72448, 72457], [72460, 72462], [72462, 72467], [74914, 74919], [75016, 75024], [75281, 75285], [76032, 76037], [76344, 76352], [76373, 76374], [76451, 76453], [76626, 76635], [76659, 76664], [76691, 76693], [76705, 76706], [76821, 76827], [76848, 76851], [76942, 76945], [76979, 76986], [76996, 76997], [77019, 77021], [77072, 77076], [77446, 77448], [77503, 77509], [77665, 77673], [79957, 79958]]
        tous_remplacements = [[446, 451], [457, 458], [729, 730], [1060, 1063], [1077, 1084], [1218, 1220], [1333, 1335], [1579, 1580], [1618, 1619], [1646, 1659], [1780, 1781], [1863, 1864], [1904, 1914], [2223, 2224], [3149, 3164], [3315, 3316], [3690, 3691], [3728, 3742], [3966, 3967], [4170, 4171], [4677, 4680], [4733, 4742], [5565, 5575], [5820, 5821], [6051, 6060], [6199, 6201], [6231, 6232], [6355, 6357], [6398, 6401], [6509, 6510], [6776, 6777], [7640, 7643], [7657, 7658], [8596, 8598], [9034, 9042], [10135, 10141], [11130, 11150], [11394, 11395], [11401, 11402], [12393, 12398], [12451, 12452], [12551, 12559], [12851, 12855], [13112, 13122], [13763, 13774], [15239, 15249], [15339, 15341], [15428, 15436], [15511, 15519], [15660, 15669], [15800, 15805], [16446, 16448], [17041, 17042], [17869, 17871], [17976, 17990], [18016, 18020], [18125, 18144], [18483, 18491], [18509, 18510], [19370, 19373], [19386, 19393], [19839, 19856], [20877, 20894], [20909, 20920], [21303, 21307], [21717, 21760], [22852, 22863], [22874, 22875], [22907, 22943], [23061, 23067], [23073, 23086], [23270, 23278], [23308, 23310], [23325, 23326], [23718, 23731], [23768, 23769], [24195, 24197], [24859, 24865], [25948, 25961], [26749, 26755], [26767, 26775], [27612, 27613], [27635, 27640], [27723, 27724], [28209, 28217], [28917, 28921], [29302, 29304], [29317, 29320], [29328, 29329], [29338, 29339], [29554, 29555], [30254, 30255], [30368, 30372], [31290, 31298], [31615, 31616], [31638, 31645], [32045, 32047], [32531, 32539], [36206, 36207], [36304, 36306], [36720, 36743], [36796, 36804], [37056, 37057], [37852, 37860], [37995, 37998], [38532, 38535], [39732, 39737], [39743, 39744], [40005, 40006], [40338, 40342], [40355, 40362], [40496, 40498], [40612, 40615], [40856, 40857], [40895, 40896], [40923, 40941], [41063, 41064], [41146, 41147], [41187, 41195], [41504, 41505], [42420, 42434], [42595, 42596], [42976, 42977], [43014, 43026], [43268, 43269], [43472, 43473], [43980, 43982], [44035, 44042], [44875, 44888], [45133, 45135], [45367, 45372], [45521, 45523], [45553, 45555], [45695, 45699], [45740, 45743], [45879, 45880], [46146, 46147], [47000, 47005], [47019, 47020], [47957, 47958], [48395, 48399], [49527, 49531], [50511, 50526], [50770, 50771], [50777, 50778], [51821, 51826], [51876, 51877], [51977, 51987], [52278, 52282], [52535, 52552], [53213, 53225], [54713, 54718], [54808, 54809], [54896, 54907], [55009, 55017], [55169, 55178], [55308, 55313], [55960, 55962], [56548, 56549], [57434, 57435], [57541, 57553], [57579, 57585], [57674, 57684], [58031, 58038], [58057, 58059], [58917, 58920], [58933, 58939], [59399, 59429], [60516, 60528], [60542, 60548], [60911, 60916], [61343, 61395], [62490, 62498], [62509, 62510], [62542, 62608], [62764, 62767], [62773, 62781], [62965, 62969], [62999, 63001], [63016, 63017], [63392, 63404], [63441, 63442], [63853, 63857], [64529, 64541], [65705, 65712], [66546, 66549], [66561, 66574], [67459, 67460], [67482, 67486], [67570, 67571], [68043, 68050], [68751, 68758], [69151, 69153], [69166, 69169], [69177, 69178], [69187, 69188], [69404, 69405], [70134, 70135], [70248, 70253], [71195, 71203], [71531, 71532], [71554, 71564], [71966, 71968], [72479, 72486], [76145, 76146], [76243, 76245], [77060, 77076], [77129, 77134], [77408, 77409], [78454, 78463], [78598, 78603], [79138, 79140]]
        lg_texte1 = 39356
        texte_original = self._readFile("Alth1v1.txt") + self._readFile("Alth1v2.txt") 
        blocsCommuns = [[0, 17], [88, 428], [428, 446], [451, 457], [458, 677], [691, 696], [696, 701], [706, 729], [730, 890], [890, 1060], [1063, 1069], [1070, 1077], [1084, 1105], [1106, 1198], [1199, 1218], [1220, 1225], [1225, 1333], [1335, 1394], [1394, 1399], [1404, 1446], [1446, 1579], [1580, 1618], [1619, 1638], [1639, 1646], [1659, 1674], [1674, 1698], [1699, 1755], [1755, 1780], [1781, 1863], [1864, 1899], [1899, 1904], [1914, 2008], [2009, 2223], [2224, 2703], [2703, 2773], [2785, 3149], [3164, 3259], [3260, 3281], [3290, 3315], [3316, 3338], [3339, 3353], [3353, 3372], [3372, 3380], [3380, 3579], [3583, 3599], [3600, 3605], [3605, 3690], [3691, 3728], [3742, 3767], [3767, 3774], [3774, 3794], [3799, 3839], [3851, 3945], [3945, 3953], [3954, 3966], [3967, 4170], [4171, 4257], [4258, 4628], [4628, 4677], [4680, 4733], [4742, 4798], [4798, 4857], [4857, 4917], [4917, 5281], [5282, 5364], [5364, 5565], [5575, 5820], [5821, 5832], [5833, 5909], [5911, 6051], [6060, 6067], [6072, 6147], [6148, 6199], [6201, 6231], [6232, 6349], [6349, 6355], [6357, 6398], [6401, 6475], [6475, 6509], [6510, 6776], [6777, 7151], [7152, 7389], [7390, 7412], [7425, 7530], [7531, 7640], [7643, 7657], [7658, 7948], [7948, 8238], [8238, 8469], [8469, 8475], [8479, 8488], [8493, 8534], [8537, 8596], [8598, 8722], [8722, 9034], [9042, 9346], [9351, 9564], [9568, 9683], [9685, 9694], [9694, 9744], [9746, 10098], [10098, 10130], [10130, 10135], [10141, 10355], [10362, 10416], [10418, 10423], [10426, 10563], [10565, 10766], [10766, 11021], [11022, 11119], [11119, 11130], [11150, 11394], [11395, 11401], [11402, 11420], [11420, 11451], [11451, 11562], [11567, 11596], [11596, 11611], [11611, 11723], [11723, 11746], [11746, 11865], [11867, 11918], [11932, 11941], [11941, 12030], [12042, 12050], [12050, 12069], [12069, 12164], [12164, 12186], [12186, 12217], [12217, 12393], [12398, 12405], [12407, 12415], [12416, 12451], [12452, 12537], [12537, 12551], [12559, 12625], [12625, 12736], [12736, 12743], [12745, 12851], [12855, 12879], [12884, 12908], [12910, 12980], [12989, 13112], [13122, 13191], [13191, 13221], [13221, 13229], [13229, 13252], [13253, 13273], [13276, 13359], [13362, 13367], [13387, 13400], [13400, 13424], [13424, 13584], [13584, 13763], [13774, 13836], [13836, 13994], [13994, 14010], [14022, 14285], [14285, 14342], [14342, 14371], [14378, 14640], [14640, 14656], [14656, 15150], [15152, 15227], [15227, 15239], [15249, 15339], [15341, 15428], [15436, 15497], [15505, 15511], [15519, 15597], [15599, 15628], [15628, 15637], [15637, 15660], [15669, 15764], [15765, 15800], [15805, 16063], [16064, 16128], [16128, 16290], [16290, 16297], [16297, 16403], [16403, 16414], [16417, 16441], [16441, 16446], [16448, 16838], [16842, 16872], [16889, 16903], [16904, 17010], [17010, 17041], [17042, 17059], [17059, 17313], [17314, 17432], [17432, 17678], [17678, 17685], [17689, 17711], [17711, 17718], [17718, 17729], [17729, 17757], [17760, 17853], [17864, 17869], [17871, 17932], [17932, 17976], [17990, 18016], [18020, 18039], [18057, 18093], [18095, 18102], [18103, 18109], [18111, 18119], [18119, 18125], [18144, 18159], [18159, 18244], [18245, 18457], [18458, 18478], [18478, 18483], [18491, 18503], [18503, 18509], [18510, 18611], [18620, 18949], [18949, 19004], [19005, 19017], [19023, 19370], [19373, 19386], [19393, 19586], [19586, 19614], [19614, 19673], [19673, 19734], [19734, 19819], [19819, 19824], [19831, 19839], [19856, 19971], [19972, 20094], [20094, 20156], [20156, 20389], [20389, 20403], [20417, 20470], [20470, 20493], [20493, 20520], [20520, 20653], [20653, 20837], [20838, 20877], [20894, 20899], [20900, 20909], [20920, 20946], [20950, 20960], [20963, 21068], [21100, 21127], [21140, 21166], [21166, 21296], [21296, 21303], [21307, 21316], [21316, 21321], [21322, 21369], [21524, 21532], [21533, 21544], [21545, 21555], [21556, 21570], [21570, 21594], [21594, 21629], [21630, 21646], [21659, 21667], [21681, 21705], [21705, 21717], [21760, 21833], [21833, 21931], [21933, 21944], [21950, 22137], [22137, 22322], [22323, 22511], [22511, 22535], [22571, 22582], [22587, 22717], [22731, 22798], [22801, 22827], [22827, 22852], [22863, 22874], [22875, 22907], [22943, 22954], [22956, 23026], [23026, 23061], [23067, 23073], [23086, 23192], [23198, 23227], [23242, 23270], [23278, 23308], [23310, 23325], [23326, 23369], [23370, 23392], [23397, 23424], [23424, 23472], [23479, 23484], [23489, 23533], [23562, 23718], [23731, 23768], [23769, 23778], [23778, 23946], [23957, 24098], [24118, 24158], [24158, 24195], [24197, 24254], [24265, 24482], [24482, 24487], [24488, 24496], [24497, 24504], [24504, 24514], [24514, 24519], [24520, 24525], [24525, 24634], [24634, 24761], [24771, 24859], [24865, 24872], [24872, 24879], [24886, 24891], [24907, 25070], [25070, 25091], [25091, 25107], [25127, 25156], [25156, 25162], [25187, 25200], [25200, 25216], [25217, 25223], [25223, 25273], [25273, 25359], [25361, 25373], [25374, 25480], [25480, 25491], [25503, 25696], [25700, 25709], [25709, 25748], [25748, 25937], [25938, 25948], [25961, 25996], [26007, 26048], [26049, 26301], [26301, 26419], [26419, 26465], [26473, 26706], [26706, 26718], [26723, 26732], [26737, 26749], [26755, 26767], [26775, 26939], [26948, 26959], [26959, 27059], [27061, 27066], [27072, 27089], [27094, 27101], [27104, 27467], [27467, 27612], [27613, 27635], [27640, 27645], [27645, 27723], [27724, 27827], [27827, 27850], [27867, 28209], [28217, 28226], [28226, 28917], [28921, 28941], [28941, 29221], [29221, 29273], [29274, 29302], [29304, 29317], [29320, 29328], [29329, 29338], [29339, 29463], [29463, 29554], [29555, 29588], [29590, 29642], [29642, 29651], [29660, 29668], [29668, 29934], [29934, 29965], [29965, 30203], [30204, 30254], [30255, 30368], [30372, 30385], [30385, 30493], [30493, 30818], [30818, 30837], [30838, 30846], [30846, 30880], [30880, 30950], [30973, 31027], [31028, 31094], [31094, 31290], [31298, 31303], [31303, 31552], [31552, 31583], [31583, 31615], [31616, 31638], [31645, 31669], [31672, 31679], [31679, 31719], [31719, 31803], [31803, 31860], [31860, 32045], [32047, 32123], [32123, 32247], [32247, 32516], [32519, 32531], [32539, 32546], [32547, 32557], [32557, 32565], [32567, 32623], [32658, 32800], [32809, 33159], [33159, 33407], [33411, 33803], [33803, 33988], [33988, 34185], [34185, 34362], [34362, 34439], [34439, 34546], [34546, 34630], [34630, 34840], [34846, 34960], [34960, 35029], [35067, 35095], [35102, 35359], [35361, 35953], [35953, 36098], [36098, 36206], [36207, 36298], [36298, 36304], [36306, 36373], [36382, 36393], [36403, 36414], [36418, 36493], [36493, 36655], [36655, 36676], [36676, 36687], [36687, 36720], [36743, 36796], [36804, 37034], [37034, 37056], [37057, 37075], [37075, 37082], [37082, 37209], [37209, 37254], [37254, 37347], [37349, 37538], [37538, 37583], [37583, 37593], [37593, 37720], [37723, 37743], [37743, 37762], [37762, 37852], [37860, 37995], [37998, 38414], [38414, 38532], [38535, 38599], [38600, 39043], [39045, 39265], [39265, 39354], [39356, 39373], [39373, 39713], [39714, 39732], [39737, 39743], [39744, 39963], [39963, 39968], [39977, 39982], [39982, 40005], [40006, 40166], [40168, 40338], [40342, 40348], [40348, 40355], [40362, 40383], [40384, 40476], [40477, 40496], [40498, 40503], [40504, 40612], [40615, 40674], [40675, 40680], [40680, 40722], [40723, 40856], [40857, 40895], [40896, 40915], [40916, 40923], [40941, 40956], [40957, 40981], [40981, 41037], [41038, 41063], [41064, 41146], [41147, 41182], [41182, 41187], [41195, 41289], [41290, 41504], [41505, 41984], [41985, 42055], [42056, 42420], [42434, 42529], [42530, 42551], [42570, 42595], [42596, 42618], [42620, 42634], [42635, 42654], [42655, 42663], [42664, 42863], [42868, 42884], [42886, 42891], [42891, 42976], [42977, 43014], [43026, 43051], [43059, 43066], [43081, 43101], [43101, 43141], [43150, 43244], [43245, 43253], [43256, 43268], [43269, 43472], [43473, 43559], [43560, 43930], [43931, 43980], [43982, 44035], [44042, 44098], [44099, 44158], [44159, 44219], [44220, 44584], [44586, 44668], [44674, 44875], [44888, 45133], [45135, 45146], [45151, 45227], [45227, 45367], [45372, 45379], [45381, 45456], [45470, 45521], [45523, 45553], [45555, 45672], [45689, 45695], [45699, 45740], [45743, 45817], [45845, 45879], [45880, 46146], [46147, 46521], [46522, 46759], [46759, 46781], [46785, 46890], [46891, 47000], [47005, 47019], [47020, 47310], [47311, 47601], [47602, 47833], [47833, 47839], [47842, 47851], [47855, 47896], [47898, 47957], [47958, 48082], [48083, 48395], [48399, 48703], [48709, 48922], [48937, 49052], [49052, 49061], [49087, 49137], [49137, 49489], [49490, 49522], [49522, 49527], [49531, 49745], [49747, 49801], [49801, 49806], [49807, 49944], [49944, 50145], [50146, 50401], [50401, 50498], [50500, 50511], [50526, 50770], [50771, 50777], [50778, 50796], [50808, 50839], [50840, 50951], [50954, 50983], [50984, 50999], [51000, 51112], [51138, 51161], [51163, 51282], [51283, 51334], [51335, 51344], [51346, 51435], [51436, 51444], [51445, 51464], [51465, 51560], [51561, 51583], [51584, 51615], [51645, 51821], [51826, 51833], [51833, 51841], [51841, 51876], [51877, 51962], [51963, 51977], [51987, 52053], [52054, 52165], [52165, 52172], [52172, 52278], [52282, 52306], [52309, 52333], [52342, 52412], [52412, 52535], [52552, 52621], [52623, 52653], [52660, 52668], [52679, 52702], [52707, 52727], [52727, 52810], [52824, 52829], [52829, 52842], [52845, 52869], [52870, 53030], [53034, 53213], [53225, 53287], [53288, 53446], [53457, 53473], [53473, 53736], [53746, 53803], [53804, 53833], [53848, 54110], [54111, 54127], [54128, 54622], [54625, 54700], [54701, 54713], [54718, 54808], [54809, 54896], [54907, 54968], [55003, 55009], [55017, 55095], [55095, 55124], [55133, 55142], [55146, 55169], [55178, 55273], [55273, 55308], [55313, 55571], [55571, 55635], [55637, 55799], [55800, 55807], [55807, 55913], [55919, 55930], [55930, 55954], [55955, 55960], [55962, 56352], [56355, 56385], [56385, 56399], [56409, 56515], [56517, 56548], [56549, 56566], [56567, 56821], [56821, 56939], [56940, 57186], [57186, 57193], [57213, 57235], [57235, 57242], [57243, 57254], [57307, 57335], [57335, 57428], [57429, 57434], [57435, 57496], [57497, 57541], [57553, 57579], [57585, 57604], [57604, 57640], [57647, 57654], [57654, 57660], [57660, 57668], [57668, 57674], [57684, 57699], [57709, 57794], [57794, 58006], [58006, 58026], [58026, 58031], [58038, 58050], [58051, 58057], [58059, 58160], [58168, 58497], [58498, 58553], [58558, 58570], [58570, 58917], [58920, 58933], [58939, 59132], [59133, 59161], [59162, 59221], [59222, 59283], [59284, 59369], [59369, 59374], [59391, 59399], [59429, 59544], [59563, 59685], [59686, 59748], [59749, 59982], [59996, 60010], [60015, 60068], [60093, 60116], [60125, 60152], [60153, 60286], [60293, 60477], [60477, 60516], [60528, 60533], [60533, 60542], [60548, 60574], [60574, 60584], [60592, 60697], [60697, 60724], [60729, 60755], [60774, 60904], [60904, 60911], [60916, 60925], [60925, 60930], [60933, 60980], [60982, 60990], [60990, 61001], [61001, 61011], [61012, 61026], [61028, 61052], [61069, 61104], [61105, 61121], [61198, 61206], [61288, 61312], [61331, 61343], [61395, 61468], [61480, 61578], [61589, 61600], [61618, 61805], [61812, 61997], [61997, 62185], [62186, 62210], [62210, 62221], [62222, 62352], [62357, 62424], [62438, 62464], [62465, 62490], [62498, 62509], [62510, 62542], [62608, 62619], [62644, 62714], [62729, 62764], [62767, 62773], [62781, 62887], [62904, 62933], [62937, 62965], [62969, 62999], [63001, 63016], [63017, 63060], [63060, 63082], [63082, 63109], [63115, 63163], [63163, 63168], [63189, 63233], [63233, 63389], [63404, 63441], [63442, 63451], [63452, 63620], [63621, 63762], [63769, 63809], [63816, 63853], [63857, 63914], [63915, 64132], [64132, 64137], [64137, 64145], [64145, 64152], [64152, 64162], [64162, 64167], [64167, 64172], [64197, 64306], [64307, 64434], [64441, 64529], [64541, 64548], [64572, 64579], [64643, 64648], [64651, 64814], [64817, 64838], [64849, 64865], [64865, 64894], [64907, 64913], [64925, 64938], [64939, 64955], [64957, 64963], [64964, 65014], [65018, 65104], [65106, 65118], [65118, 65224], [65232, 65243], [65244, 65437], [65438, 65447], [65456, 65495], [65503, 65692], [65695, 65705], [65712, 65747], [65783, 65824], [65824, 66076], [66077, 66195], [66230, 66276], [66276, 66509], [66511, 66523], [66525, 66534], [66534, 66546], [66549, 66561], [66574, 66738], [66742, 66753], [66764, 66864], [66919, 66924], [66926, 66943], [66943, 66950], [66950, 67313], [67314, 67459], [67460, 67482], [67486, 67491], [67492, 67570], [67571, 67674], [67677, 67700], [67701, 68043], [68050, 68059], [68060, 68751], [68758, 68778], [68790, 69070], [69071, 69123], [69123, 69151], [69153, 69166], [69169, 69177], [69178, 69187], [69188, 69312], [69313, 69404], [69405, 69438], [69438, 69490], [69491, 69500], [69500, 69508], [69540, 69806], [69807, 69838], [69840, 70078], [70084, 70134], [70135, 70248], [70253, 70266], [70267, 70375], [70376, 70701], [70720, 70739], [70744, 70752], [70753, 70787], [70794, 70864], [70870, 70924], [70925, 70991], [70999, 71195], [71203, 71208], [71208, 71457], [71458, 71489], [71499, 71531], [71532, 71554], [71564, 71588], [71588, 71595], [71597, 71637], [71638, 71722], [71723, 71780], [71781, 71966], [71968, 72044], [72045, 72169], [72170, 72439], [72467, 72479], [72486, 72493], [72493, 72503], [72514, 72522], [72522, 72578], [72578, 72720], [72729, 73079], [73080, 73328], [73334, 73726], [73727, 73912], [73913, 74110], [74114, 74291], [74295, 74372], [74373, 74480], [74481, 74565], [74573, 74783], [74794, 74908], [74919, 74988], [74988, 75016], [75024, 75281], [75285, 75877], [75878, 76023], [76037, 76145], [76146, 76237], [76237, 76243], [76245, 76312], [76312, 76323], [76362, 76373], [76374, 76449], [76464, 76626], [76635, 76656], [76670, 76681], [77027, 77060], [77076, 77129], [77134, 77364], [77386, 77408], [77409, 77427], [77429, 77436], [77674, 77801], [77802, 77847], [77852, 77945], [77946, 78135], [78136, 78181], [78182, 78192], [78193, 78320], [78320, 78340], [78341, 78360], [78364, 78454], [78463, 78598], [78603, 79019], [79020, 79138], [79140, 79204], [79204, 79647], [79647, 79867], [79868, 79957]]
        lDepl = [([43, 49], [61355, 61361]), ([61, 66], [69525, 69530]), ([75, 83], [77665, 77673]), ([446, 449], [68755, 68758]), ([1078, 1084], [59982, 59988]), ([1650, 1651], [66741, 66742]), ([1650, 1651], [76996, 76997]), ([1651, 1658], [53213, 53220]), ([1904, 1905], [59374, 59375]), ([1909, 1913], [60010, 60014]), ([1909, 1913], [77072, 77076]), ([2008, 2009], [79957, 79958]), ([2773, 2779], [57202, 57208]), ([2773, 2779], [57274, 57280]), ([2773, 2779], [64609, 64615]), ([2773, 2779], [77503, 77509]), ([3161, 3164], [47839, 47842]), ([3161, 3164], [65749, 65752]), ([3287, 3290], [65447, 65450]), ([3579, 3583], [62892, 62896]), ([3599, 3600], [55799, 55800]), ([3728, 3732], [75281, 75285]), ([3736, 3741], [74914, 74919]), ([3795, 3798], [76942, 76945]), ([3840, 3845], [72462, 72467]), ([3847, 3850], [62897, 62900]), ([5565, 5571], [57202, 57208]), ([5565, 5571], [57274, 57280]), ([5565, 5571], [64609, 64615]), ([5565, 5571], [77503, 77509]), ([7389, 7390], [61104, 61105]), ([7414, 7420], [59401, 59407]), ([7421, 7423], [44584, 44586]), ([7530, 7531], [76373, 76374]), ([8488, 8493], [76032, 76037]), ([9346, 9351], [63168, 63173]), ([9564, 9565], [57646, 57647]), ([9564, 9565], [62464, 62465]), ([11130, 11132], [45687, 45689]), ([11865, 11867], [76451, 76453]), ([12557, 12559], [64189, 64191]), ([12879, 12884], [64554, 64559]), ([12980, 12986], [71195, 71201]), ([13275, 13276], [57284, 57285]), ([13275, 13276], [62565, 62566]), ([13275, 13276], [66573, 66574]), ([13275, 13276], [76705, 76706]), ([13763, 13768], [49075, 49080]), ([14016, 14022], [53451, 53457]), ([15512, 15516], [52675, 52679]), ([16872, 16877], [56399, 56404]), ([16879, 16883], [70790, 70794]), ([18042, 18047], [52702, 52707]), ([18102, 18103], [55799, 55800]), ([18128, 18131], [63809, 63812]), ([18134, 18144], [57699, 57709]), ([18483, 18484], [40476, 40477]), ([18486, 18491], [56404, 56409]), ([18611, 18620], [72448, 72457]), ([19017, 19023], [59982, 59988]), ([19389, 19393], [59559, 59563]), ([20878, 20881], [56352, 56355]), ([21135, 21140], [61475, 61480]), ([21375, 21384], [76626, 76635]), ([21386, 21395], [43141, 43150]), ([21418, 21422], [60010, 60014]), ([21418, 21422], [77072, 77076]), ([21432, 21440], [68778, 68786]), ([21432, 21440], [75016, 75024]), ([21440, 21444], [69508, 69512]), ([21453, 21456], [65226, 65229]), ([21453, 21456], [65497, 65500]), ([21472, 21476], [68786, 68790]), ([21487, 21492], [64841, 64846]), ([21500, 21507], [62580, 62587]), ([21718, 21725], [64434, 64441]), ([21730, 21757], [61225, 61252]), ([21944, 21949], [54989, 54994]), ([22582, 22587], [45146, 45151]), ([22582, 22587], [45823, 45828]), ([22852, 22858], [61140, 61146]), ([23270, 23276], [63109, 63115]), ([23270, 23276], [76821, 76827]), ([23472, 23479], [76979, 76986]), ([23537, 23539], [64189, 64191]), ([23544, 23554], [63179, 63189]), ([23554, 23558], [60072, 60076]), ([23718, 23722], [45456, 45460]), ([23954, 23957], [55014, 55017]), ([23954, 23957], [64894, 64897]), ([24258, 24260], [77019, 77021]), ([24487, 24488], [55799, 55800]), ([24496, 24497], [55799, 55800]), ([24519, 24520], [55799, 55800]), ([24879, 24881], [65104, 65106]), ([25112, 25115], [54622, 54625]), ([25176, 25181], [69525, 69530]), ([25185, 25187], [76691, 76693]), ([25185, 25187], [77446, 77448]), ([25216, 25217], [57646, 57647]), ([25216, 25217], [62464, 62465]), ([25500, 25503], [63392, 63395]), ([25937, 25938], [57646, 57647]), ([25937, 25938], [62464, 62465]), ([26006, 26007], [72442, 72443]), ([26466, 26473], [61212, 61219]), ([26734, 26737], [65780, 65783]), ([26750, 26752], [61805, 61807]), ([26752, 26755], [65780, 65783]), ([27089, 27094], [51622, 51627]), ([30950, 30956], [45460, 45466]), ([30964, 30967], [50951, 50954]), ([31641, 31645], [52810, 52814]), ([32516, 32519], [63389, 63392]), ([32531, 32533], [66523, 66525]), ([32531, 32533], [72460, 72462]), ([32642, 32645], [51619, 51622]), ([32642, 32645], [62566, 62569]), ([32642, 32645], [76848, 76851]), ([32800, 32809], [72448, 72457]), ([33407, 33411], [61808, 61812]), ([34840, 34846], [59401, 59407]), ([35053, 35058], [64841, 64846]), ([36373, 36381], [76344, 76352]), ([36376, 36378], [47896, 47898]), ([37852, 37857], [76659, 76664])]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat
    def jeu_essai_Althusser2(self):
        """freudEtLacan manuscrit1 V1.txt / M1 v2.txt 
        insensible à la casse, aux séparateurs et aux signes diacritiques
        Assertion d'ordre sur occs_deplaces vérifiée / chevauchement dans les déplacements"""
        insertions = [(40503, 40504), (40674, 40675), (40722, 40723), (42551, 42570), (43050, 43058), (43067, 43081), (44158, 44159), (44668, 44674), (45456, 45470), (45672, 45689), (45817, 45845), (47004, 47005), (47310, 47311), (48922, 48937), (49062, 49088), (49745, 49747), (49806, 49807), (50498, 50500), (50796, 50808), (51112, 51138), (51161, 51163), (51344, 51346), (51464, 51465), (51560, 51561), (51583, 51584), (51614, 51644), (51962, 51963), (52332, 52342), (52621, 52623), (52653, 52660), (52669, 52679), (52810, 52824), (52869, 52870), (53030, 53035), (53736, 53746), (54715, 54718), (54968, 55003), (55124, 55133), (55143, 55146), (55635, 55637), (55799, 55801), (55913, 55919), (56514, 56517), (57193, 57214), (57254, 57307), (57428, 57429), (57640, 57646), (57659, 57662), (58553, 58560), (59132, 59134), (59221, 59222), (59374, 59391), (59544, 59563), (59685, 59686), (59982, 59996), (60068, 60093), (60116, 60125), (60286, 60294), (60571, 60574), (60584, 60592), (60724, 60729), (60755, 60774), (60930, 60932), (60980, 60982), (61053, 61069), (61121, 61198), (61206, 61285), (61313, 61331), (61468, 61480), (61578, 61589), (61600, 61618), (61805, 61812), (62185, 62186), (62221, 62222), (62352, 62356), (62424, 62438), (62619, 62644), (62714, 62729), (62887, 62904), (62933, 62937), (63109, 63115), (63168, 63189), (63618, 63621), (63762, 63769), (63809, 63816), (63853, 63855), (64162, 64191), (64306, 64307), (64548, 64571), (64581, 64652), (64814, 64817), (64837, 64848), (64894, 64907), (64913, 64927), (65015, 65018), (65224, 65232), (65437, 65438), (65447, 65456), (65495, 65503), (65692, 65694), (65746, 65783), (66195, 66230), (66523, 66525), (66753, 66765), (66865, 66917), (66924, 66926), (67674, 67677), (68778, 68791), (69312, 69313), (69437, 69438), (69508, 69540), (70077, 70084), (70701, 70720), (70752, 70754), (70787, 70794), (70991, 70999), (71489, 71499), (71595, 71597), (71780, 71781), (72169, 72170), (72439, 72467), (72503, 72514), (74291, 74295), (74480, 74481), (74565, 74573), (74908, 74919), (75877, 75878), (76023, 76037), (76312, 76346), (76352, 76362), (76450, 76466), (76656, 76670), (76683, 77027), (77364, 77386), (77440, 77674), (77848, 77852), (78361, 78364)]
        suppressions =  [(20, 87), (677, 691), (1069, 1070), (1399, 1404), (1698, 1699), (3281, 3290), (5832, 5833), (6067, 6069), (7389, 7390), (7413, 7422), (8492, 8494), (8536, 8537), (9564, 9568), (9683, 9685), (9744, 9746), (10356, 10362), (10416, 10418), (10423, 10426), (10563, 10565), (11021, 11022), (11905, 11919), (12032, 12045), (12405, 12407), (12415, 12416), (12743, 12745), (12907, 12910), (12980, 12989), (13273, 13276), (13359, 13362), (13367, 13388), (14010, 14023), (15241, 15249), (15497, 15505), (15597, 15599), (15764, 15765), (16063, 16064), (16414, 16417), (16872, 16889), (17313, 17314), (17685, 17690), (17757, 17760), (17853, 17864), (17869, 17870), (18039, 18057), (18102, 18103), (18108, 18113), (18457, 18458), (19017, 19023), (19824, 19831), (20403, 20418), (20837, 20838), (20943, 20950), (21068, 21100), (21127, 21140), (21369, 21524), (21667, 21679), (21931, 21933), (21944, 21950), (22536, 22571), (22717, 22730), (22798, 22801), (22954, 22956), (23192, 23198), (23227, 23242), (23270, 23276), (23369, 23370), (23392, 23397), (23484, 23489), (23534, 23563), (23730, 23732), (23944, 23957), (24098, 24118), (24254, 24265), (24487, 24488), (24496, 24498), (24519, 24524), (24879, 24908), (25107, 25127), (25162, 25189), (25492, 25503), (25696, 25700), (25996, 26007), (26048, 26049), (26465, 26472), (26718, 26723), (26732, 26737), (27066, 27072), (27101, 27104), (27850, 27867), (29273, 29274), (29587, 29590), (29651, 29660), (30956, 30973), (31669, 31672), (32516, 32519), (32622, 32659), (35029, 35067), (36373, 36375), (36381, 36403), (37720, 37723), (38532, 38534), (38599, 38600), (39043, 39045)]
        occs_deplaces = [(20, 25), (32, 36), (43, 49), (61, 66), (75, 87), (690, 691), (704, 706), (1650, 1651), (1651, 1658), (1909, 1915), (2773, 2779), (2779, 2785), (3161, 3164), (3579, 3583), (3728, 3732), (3736, 3743), (3789, 3798), (3839, 3843), (5565, 5569), (6055, 6059), (7414, 7422), (9346, 9351), (11130, 11132), (11136, 11138), (12554, 12557), (12879, 12884), (12980, 12983), (13374, 13379), (14010, 14015), (14016, 14023), (14375, 14379), (15431, 15436), (15497, 15500), (16872, 16877), (18042, 18046), (18128, 18131), (18134, 18144), (18483, 18484), (18486, 18493), (19017, 19021), (20410, 20415), (20878, 20881), (20884, 20889), (20916, 20920), (20943, 20948), (20960, 20963), (21089, 21093), (21093, 21098), (21135, 21140), (21375, 21384), (21386, 21394), (21418, 21422), (21424, 21429), (21435, 21436), (21440, 21444), (21472, 21476), (21481, 21486), (21486, 21492), (21500, 21507), (21646, 21659), (21648, 21653), (21718, 21721), (21944, 21948), (22549, 22554), (22582, 22587), (22724, 22730), (22852, 22858), (22913, 22916), (22938, 22939), (23233, 23237), (23472, 23479), (23544, 23554), (23554, 23558), (23718, 23723), (23944, 23948), (24762, 24764), (25107, 25112), (25112, 25115), (25123, 25127), (25162, 25165), (25169, 25176), (25176, 25179), (25182, 25189), (25499, 25503), (25956, 25961), (26467, 26472), (26734, 26737), (26752, 26755), (26939, 26944), (27089, 27094), (27863, 27867), (30958, 30963), (30964, 30967), (31295, 31297), (31641, 31645), (32531, 32538), (33407, 33411), (34840, 34846), (35048, 35052), (35052, 35058), (35062, 35067), (36381, 36394), (42424, 42428), (43141, 43151), (45132, 45139), (45146, 45151), (45456, 45461), (45687, 45689), (45823, 45828), (45837, 45845), (47839, 47842), (49062, 49075), (50796, 50798), (50951, 50954), (51622, 51627), (52654, 52660), (52702, 52706), (52810, 52814), (52815, 52817), (52842, 52845), (53213, 53220), (53446, 53451), (53451, 53457), (54622, 54625), (54987, 54992), (54999, 55003), (56352, 56355), (56399, 56404), (56404, 56411), (57294, 57300), (57699, 57709), (59381, 59385), (59401, 59407), (59407, 59411), (60010, 60016), (60072, 60076), (60770, 60774), (61140, 61146), (61170, 61175), (61182, 61188), (61245, 61252), (61252, 61258), (61265, 61269), (61313, 61315), (61343, 61348), (61355, 61361), (61475, 61480), (61585, 61589), (61585, 61589), (61608, 61611), (61614, 61618), (61808, 61812), (62580, 62587), (62593, 62598), (62892, 62896), (63168, 63173), (63179, 63189), (63391, 63395), (63809, 63812), (64162, 64167), (64162, 64167), (64167, 64172), (64434, 64437), (64554, 64559), (64628, 64634), (64902, 64907), (65749, 65752), (65780, 65783), (66904, 66910), (68781, 68782), (68786, 68791), (69508, 69512), (69525, 69530), (70717, 70720), (70787, 70791), (71195, 71198), (72439, 72442), (72460, 72467), (72480, 72485), (74110, 74114), (74571, 74573), (74782, 74794), (74914, 74919), (75019, 75020), (75281, 75285), (76032, 76037), (76240, 76245), (76319, 76323), (76330, 76335), (76626, 76635), (76656, 76661), (76814, 76821), (76935, 76945), (76979, 76986), (76996, 76997), (77009, 77012), (77376, 77381), (77456, 77460), (77541, 77545), (77554, 77558), (77560, 77561), (77579, 77581), (77599, 77603), (77612, 77616), (77618, 77619), (77637, 77639), (77649, 77654), (77665, 77674), (79865, 79873)]
        tous_remplacements = [(697, 706), (1198, 1199), (1618, 1620), (1645, 1659), (1780, 1781), (1904, 1915), (2223, 2224), (3152, 3164), (3579, 3583), (3599, 3600), (3728, 3743), (3789, 3800), (3839, 3852), (4170, 4171), (4677, 4679), (4732, 4742), (5565, 5575), (5819, 5825), (5909, 5916), (6051, 6060), (6355, 6356), (6400, 6401), (6776, 6777), (7657, 7658), (8475, 8479), (9034, 9042), (9346, 9351), (10135, 10141), (11130, 11151), (11562, 11567), (11865, 11867), (12393, 12398), (12551, 12559), (12851, 12855), (12879, 12884), (13112, 13121), (13763, 13774), (14371, 14379), (15150, 15152), (15339, 15341), (15428, 15436), (15511, 15515), (15660, 15669), (15803, 15805), (16446, 16448), (16838, 16842), (17975, 17990), (18016, 18020), (18125, 18144), (18483, 18493), (18614, 18621), (19386, 19393), (19839, 19856), (20877, 20893), (20910, 20920), (21301, 21308), (21718, 21757), (22852, 22863), (22907, 22944), (23061, 23067), (23073, 23086), (23307, 23311), (23718, 23725), (24762, 24767), (24858, 24866), (25359, 25361), (25948, 25961), (26751, 26755), (26767, 26775), (27635, 27639), (28208, 28212), (28917, 28921), (29303, 29304), (29317, 29320), (29328, 29329), (29337, 29340), (30254, 30255), (30368, 30371), (30833, 30838), (31290, 31297), (31615, 31616), (31638, 31645), (32045, 32047), (32531, 32538), (33158, 33159), (33407, 33411), (34839, 34846), (35358, 35361), (36301, 36306), (36720, 36743), (36797, 36804), (37347, 37349), (37854, 37860), (37995, 38001), (39263, 39270), (39969, 39982), (40475, 40477), (40895, 40897), (40922, 40941), (41063, 41064), (41187, 41196), (41504, 41505), (42424, 42434), (42863, 42868), (42884, 42886), (43014, 43027), (43096, 43102), (43141, 43151), (43472, 43473), (43980, 43981), (44034, 44042), (44875, 44888), (45132, 45139), (45227, 45232), (45367, 45372), (45695, 45698), (45742, 45743), (46146, 46147), (47019, 47020), (47839, 47842), (48395, 48399), (48703, 48709), (49527, 49531), (50511, 50527), (50951, 50954), (51282, 51283), (51820, 51826), (51977, 51987), (52278, 52282), (52306, 52309), (52535, 52551), (53213, 53225), (53833, 53849), (54622, 54625), (54808, 54809), (54896, 54907), (55009, 55013), (55169, 55178), (55311, 55313), (55960, 55962), (56352, 56355), (57540, 57553), (57579, 57585), (57674, 57684), (58031, 58040), (58163, 58169), (58933, 58939), (59399, 59429), (60516, 60527), (60542, 60548), (60909, 60917), (61343, 61392), (62490, 62498), (62542, 62609), (62764, 62767), (62773, 62781), (62999, 63002), (63389, 63399), (64434, 64437), (64528, 64542), (65104, 65106), (65705, 65712), (66547, 66549), (66561, 66574), (67482, 67485), (68042, 68045), (68751, 68758), (69152, 69153), (69166, 69169), (69177, 69178), (69186, 69189), (70134, 70135), (70248, 70252), (70735, 70744), (71195, 71202), (71531, 71532), (71554, 71564), (71966, 71968), (72480, 72485), (73078, 73080), (73328, 73334), (74782, 74794), (75281, 75285), (76240, 76245), (77060, 77076), (77130, 77134), (77945, 77946), (78456, 78463), (78598, 78606), (79865, 79873)]
        lg_texte1 = 39356
        texte_original = self._readFile("Alth1v1.txt") + self._readFile("Alth1v2.txt") 
        blocsCommuns = [(0, 20), (87, 677), (691, 697), (706, 1069), (1070, 1198), (1199, 1225), (1225, 1394), (1394, 1399), (1404, 1446), (1446, 1618), (1620, 1645), (1659, 1698), (1699, 1780), (1781, 1904), (1915, 2223), (2224, 2773), (2785, 3152), (3164, 3281), (3290, 3579), (3583, 3599), (3600, 3606), (3606, 3728), (3743, 3766), (3766, 3774), (3774, 3789), (3800, 3839), (3852, 4170), (4171, 4677), (4679, 4732), (4742, 4857), (4857, 5364), (5364, 5565), (5575, 5819), (5825, 5832), (5833, 5903), (5903, 5909), (5916, 6051), (6060, 6067), (6069, 6148), (6148, 6349), (6349, 6355), (6356, 6400), (6401, 6475), (6475, 6776), (6777, 7389), (7390, 7413), (7422, 7643), (7643, 7657), (7658, 7948), (7948, 8468), (8468, 8475), (8479, 8492), (8494, 8536), (8537, 9034), (9042, 9346), (9351, 9564), (9568, 9683), (9685, 9695), (9695, 9744), (9746, 10130), (10130, 10135), (10141, 10356), (10362, 10416), (10418, 10423), (10426, 10563), (10565, 11021), (11022, 11119), (11119, 11130), (11151, 11420), (11420, 11562), (11567, 11723), (11723, 11746), (11746, 11865), (11867, 11905), (11919, 11941), (11941, 12032), (12045, 12069), (12069, 12164), (12164, 12186), (12186, 12216), (12216, 12393), (12398, 12405), (12407, 12415), (12416, 12537), (12537, 12551), (12559, 12743), (12745, 12851), (12855, 12879), (12884, 12907), (12910, 12980), (12989, 13112), (13121, 13191), (13191, 13221), (13221, 13229), (13229, 13252), (13252, 13273), (13276, 13359), (13362, 13367), (13388, 13400), (13400, 13424), (13424, 13585), (13585, 13763), (13774, 13994), (13994, 14010), (14023, 14285), (14285, 14371), (14379, 15150), (15152, 15241), (15249, 15339), (15341, 15428), (15436, 15497), (15505, 15511), (15515, 15597), (15599, 15628), (15628, 15637), (15637, 15660), (15669, 15764), (15765, 15803), (15805, 16063), (16064, 16128), (16128, 16291), (16291, 16403), (16403, 16414), (16417, 16446), (16448, 16838), (16842, 16872), (16889, 16906), (16906, 17010), (17010, 17313), (17314, 17678), (17678, 17685), (17690, 17711), (17711, 17729), (17729, 17757), (17760, 17853), (17864, 17869), (17870, 17975), (17990, 18016), (18020, 18039), (18057, 18094), (18094, 18102), (18103, 18108), (18113, 18125), (18144, 18159), (18159, 18457), (18458, 18478), (18478, 18483), (18493, 18608), (18608, 18614), (18621, 19007), (19007, 19017), (19023, 19386), (19393, 19587), (19587, 19673), (19673, 19819), (19819, 19824), (19831, 19839), (19856, 19972), (19972, 20094), (20094, 20389), (20389, 20403), (20418, 20470), (20470, 20493), (20493, 20654), (20654, 20837), (20838, 20877), (20893, 20910), (20920, 20943), (20950, 20960), (20963, 21068), (21100, 21127), (21140, 21166), (21166, 21301), (21308, 21321), (21321, 21369), (21524, 21594), (21594, 21646), (21659, 21667), (21679, 21705), (21705, 21710), (21710, 21718), (21757, 21833), (21833, 21931), (21933, 21944), (21950, 22137), (22137, 22511), (22511, 22529), (22529, 22536), (22571, 22582), (22587, 22717), (22730, 22798), (22801, 22852), (22863, 22907), (22944, 22954), (22956, 23026), (23026, 23061), (23067, 23073), (23086, 23181), (23181, 23192), (23198, 23227), (23242, 23270), (23276, 23307), (23311, 23369), (23370, 23392), (23397, 23424), (23424, 23472), (23479, 23484), (23489, 23534), (23563, 23718), (23725, 23730), (23732, 23944), (23957, 24098), (24118, 24158), (24158, 24195), (24195, 24254), (24265, 24482), (24482, 24487), (24488, 24496), (24498, 24514), (24514, 24519), (24524, 24629), (24629, 24634), (24634, 24641), (24641, 24762), (24767, 24858), (24866, 24871), (24871, 24879), (24908, 25070), (25070, 25090), (25090, 25107), (25127, 25151), (25151, 25156), (25156, 25162), (25189, 25273), (25273, 25359), (25361, 25480), (25480, 25492), (25503, 25696), (25700, 25709), (25709, 25748), (25748, 25937), (25937, 25948), (25961, 25996), (26007, 26048), (26049, 26419), (26419, 26465), (26472, 26718), (26723, 26732), (26737, 26751), (26755, 26767), (26775, 26939), (26944, 26960), (26960, 27059), (27059, 27066), (27072, 27089), (27094, 27101), (27104, 27635), (27639, 27644), (27644, 27827), (27827, 27850), (27867, 28208), (28212, 28217), (28217, 28917), (28921, 28942), (28942, 29273), (29274, 29303), (29304, 29317), (29320, 29328), (29329, 29337), (29340, 29463), (29463, 29587), (29590, 29651), (29660, 29668), (29668, 30204), (30204, 30254), (30255, 30368), (30371, 30818), (30818, 30833), (30838, 30847), (30847, 30880), (30880, 30956), (30973, 31094), (31094, 31290), (31297, 31583), (31583, 31615), (31616, 31638), (31645, 31669), (31672, 31679), (31679, 31860), (31860, 32045), (32047, 32247), (32247, 32516), (32519, 32531), (32538, 32557), (32557, 32622), (32659, 33158), (33159, 33407), (33411, 34185), (34185, 34362), (34362, 34546), (34546, 34630), (34630, 34839), (34846, 34960), (34960, 35029), (35067, 35098), (35098, 35358), (35361, 35953), (35953, 36098), (36098, 36301), (36306, 36373), (36375, 36381), (36403, 36478), (36478, 36495), (36495, 36655), (36655, 36676), (36676, 36687), (36687, 36720), (36743, 36797), (36804, 37034), (37034, 37082), (37082, 37254), (37254, 37347), (37349, 37720), (37723, 37762), (37762, 37848), (37848, 37854), (37860, 37995), (38001, 38532), (38534, 38599), (38600, 39043), (39045, 39263), (39270, 39356), (39356, 39372), (39372, 39963), (39963, 39969), (39982, 40348), (40348, 40475), (40477, 40503), (40504, 40674), (40675, 40680), (40680, 40722), (40723, 40895), (40897, 40922), (40941, 40981), (40981, 41063), (41064, 41187), (41196, 41504), (41505, 42056), (42056, 42424), (42434, 42551), (42570, 42863), (42868, 42884), (42886, 42892), (42892, 43014), (43027, 43050), (43058, 43067), (43081, 43096), (43102, 43141), (43151, 43472), (43473, 43980), (43981, 44034), (44042, 44158), (44159, 44668), (44674, 44875), (44888, 45132), (45139, 45146), (45151, 45221), (45221, 45227), (45232, 45367), (45372, 45379), (45379, 45456), (45470, 45672), (45689, 45695), (45698, 45742), (45743, 45817), (45845, 46146), (46147, 46759), (46759, 46782), (46782, 47004), (47005, 47019), (47020, 47310), (47311, 47832), (47832, 47839), (47842, 47856), (47856, 47898), (47898, 48395), (48399, 48703), (48709, 48922), (48937, 49052), (49052, 49062), (49088, 49137), (49137, 49522), (49522, 49527), (49531, 49745), (49747, 49801), (49801, 49806), (49807, 49944), (49944, 50401), (50401, 50498), (50500, 50511), (50527, 50796), (50808, 50951), (50954, 51112), (51138, 51161), (51163, 51282), (51283, 51321), (51321, 51344), (51346, 51439), (51439, 51464), (51465, 51560), (51561, 51583), (51584, 51614), (51644, 51820), (51826, 51833), (51833, 51841), (51841, 51962), (51963, 51977), (51987, 52172), (52172, 52278), (52282, 52306), (52309, 52332), (52342, 52412), (52412, 52535), (52551, 52621), (52623, 52653), (52660, 52669), (52679, 52702), (52706, 52727), (52727, 52810), (52824, 52830), (52830, 52842), (52845, 52869), (52870, 53030), (53035, 53213), (53225, 53446), (53457, 53474), (53474, 53736), (53746, 53833), (53849, 54622), (54625, 54715), (54718, 54808), (54809, 54896), (54907, 54968), (55003, 55009), (55013, 55095), (55095, 55124), (55133, 55143), (55146, 55169), (55178, 55273), (55273, 55311), (55313, 55571), (55571, 55635), (55637, 55799), (55801, 55913), (55919, 55930), (55930, 55960), (55962, 56352), (56355, 56385), (56385, 56399), (56411, 56514), (56517, 56821), (56821, 57186), (57186, 57193), (57214, 57235), (57235, 57254), (57307, 57335), (57335, 57428), (57429, 57434), (57434, 57540), (57553, 57579), (57585, 57604), (57604, 57640), (57646, 57654), (57654, 57659), (57662, 57674), (57684, 57699), (57709, 58006), (58006, 58026), (58026, 58031), (58040, 58157), (58157, 58163), (58169, 58553), (58560, 58570), (58570, 58933), (58939, 59132), (59134, 59221), (59222, 59369), (59369, 59374), (59391, 59399), (59429, 59544), (59563, 59685), (59686, 59982), (59996, 60010), (60016, 60068), (60093, 60116), (60125, 60286), (60294, 60477), (60477, 60516), (60527, 60542), (60548, 60571), (60574, 60584), (60592, 60697), (60697, 60724), (60729, 60755), (60774, 60909), (60917, 60930), (60932, 60980), (60982, 61053), (61069, 61121), (61198, 61206), (61285, 61313), (61331, 61336), (61336, 61343), (61392, 61468), (61480, 61578), (61589, 61600), (61618, 61805), (61812, 62185), (62186, 62204), (62204, 62210), (62210, 62221), (62222, 62352), (62356, 62424), (62438, 62490), (62498, 62542), (62609, 62619), (62644, 62714), (62729, 62764), (62767, 62773), (62781, 62876), (62876, 62887), (62904, 62933), (62937, 62967), (62967, 62999), (63002, 63060), (63060, 63082), (63082, 63109), (63115, 63163), (63163, 63168), (63189, 63234), (63234, 63389), (63399, 63405), (63405, 63618), (63621, 63762), (63769, 63809), (63816, 63853), (63855, 63915), (63915, 64132), (64132, 64137), (64137, 64146), (64146, 64162), (64191, 64196), (64196, 64301), (64301, 64306), (64307, 64314), (64314, 64434), (64437, 64528), (64542, 64548), (64571, 64581), (64652, 64814), (64817, 64837), (64848, 64865), (64865, 64889), (64889, 64894), (64907, 64913), (64927, 65015), (65018, 65104), (65106, 65224), (65232, 65244), (65244, 65437), (65438, 65447), (65456, 65495), (65503, 65692), (65694, 65705), (65712, 65746), (65783, 65824), (65824, 66195), (66230, 66275), (66275, 66523), (66525, 66534), (66534, 66547), (66549, 66561), (66574, 66738), (66738, 66753), (66765, 66865), (66917, 66924), (66926, 66943), (66943, 66950), (66950, 67482), (67485, 67491), (67491, 67674), (67677, 67701), (67701, 68042), (68045, 68050), (68050, 68751), (68758, 68778), (68791, 69123), (69123, 69152), (69153, 69166), (69169, 69177), (69178, 69186), (69189, 69312), (69313, 69437), (69438, 69500), (69500, 69508), (69540, 70077), (70084, 70134), (70135, 70248), (70252, 70701), (70720, 70735), (70744, 70752), (70754, 70787), (70794, 70870), (70870, 70991), (70999, 71195), (71202, 71489), (71499, 71531), (71532, 71554), (71564, 71588), (71588, 71595), (71597, 71780), (71781, 71966), (71968, 72169), (72170, 72439), (72467, 72480), (72485, 72503), (72514, 72579), (72579, 73078), (73080, 73328), (73334, 74110), (74114, 74291), (74295, 74480), (74481, 74565), (74573, 74782), (74794, 74908), (74919, 74988), (74988, 75019), (75020, 75281), (75285, 75877), (75878, 76023), (76037, 76240), (76245, 76312), (76346, 76352), (76362, 76434), (76434, 76450), (76466, 76626), (76635, 76656), (76670, 76683), (77027, 77060), (77076, 77130), (77134, 77364), (77386, 77440), (77674, 77848), (77852, 77945), (77946, 78320), (78320, 78361), (78364, 78450), (78450, 78456), (78463, 78598), (78606, 79139), (79139, 79204), (79204, 79647), (79647, 79865), (79873, 79958)]
        lDepl = [[(20, 25), (64167, 64172)], [(32, 36), (42424, 42428)], [(43, 49), (61355, 61361)], [(61, 66), (69525, 69530)], [(75, 87), (77665, 77674)], [(690, 691), (77560, 77561)], [(690, 691), (77618, 77619)], [(704, 706), (74571, 74573)], [(1650, 1651), (76996, 76997)], [(1651, 1658), (53213, 53220)], [(1909, 1915), (60010, 60016)], [(2773, 2779), (64628, 64634)], [(2779, 2785), (76032, 76037)], [(3161, 3164), (47839, 47842)], [(3161, 3164), (65749, 65752)], [(3579, 3583), (62892, 62896)], [(3728, 3732), (75281, 75285)], [(3736, 3743), (74914, 74919)], [(3789, 3798), (76935, 76945)], [(3839, 3843), (77456, 77460)], [(3839, 3843), (77541, 77545)], [(3839, 3843), (77599, 77603)], [(5565, 5569), (59407, 59411)], [(6055, 6059), (59381, 59385)], [(7414, 7422), (59401, 59407)], [(9346, 9351), (63168, 63173)], [(11130, 11132), (45687, 45689)], [(11136, 11138), (50796, 50798)], [(12554, 12557), (70717, 70720)], [(12879, 12884), (64554, 64559)], [(12980, 12983), (71195, 71198)], [(13374, 13379), (61170, 61175)], [(14010, 14015), (53446, 53451)], [(14016, 14023), (53451, 53457)], [(14375, 14379), (60770, 60774)], [(15431, 15436), (64162, 64167)], [(15431, 15436), (64162, 64167)], [(15431, 15436), (76240, 76245)], [(15497, 15500), (52842, 52845)], [(16872, 16877), (56399, 56404)], [(18042, 18046), (52702, 52706)], [(18128, 18131), (63809, 63812)], [(18134, 18144), (57699, 57709)], [(18483, 18484), (77560, 77561)], [(18483, 18484), (77618, 77619)], [(18486, 18493), (56404, 56411)], [(19017, 19021), (59381, 59385)], [(20410, 20415), (66904, 66910)], [(20878, 20881), (56352, 56355)], [(20884, 20889), (77649, 77654)], [(20916, 20920), (70787, 70791)], [(20943, 20948), (72480, 72485)], [(20960, 20963), (77009, 77012)], [(21089, 21093), (74110, 74114)], [(21093, 21098), (62593, 62598)], [(21135, 21140), (61475, 61480)], [(21375, 21384), (76626, 76635)], [(21386, 21394), (43141, 43151)], [(21418, 21422), (60010, 60016)], [(21424, 21429), (66904, 66910)], [(21435, 21436), (68781, 68782)], [(21435, 21436), (75019, 75020)], [(21440, 21444), (69508, 69512)], [(21472, 21476), (68786, 68791)], [(21481, 21486), (64902, 64907)], [(21481, 21486), (76330, 76335)], [(21486, 21492), (61182, 61188)], [(21500, 21507), (62580, 62587)], [(21646, 21659), (49062, 49075)], [(21648, 21653), (61252, 61258)], [(21718, 21721), (64434, 64437)], [(21944, 21948), (61585, 61589)], [(21944, 21948), (61585, 61589)], [(21944, 21948), (61614, 61618)], [(22549, 22554), (76814, 76821)], [(22582, 22587), (45146, 45151)], [(22582, 22587), (45823, 45828)], [(22724, 22730), (57294, 57300)], [(22852, 22858), (61140, 61146)], [(22913, 22916), (52815, 52817)], [(22938, 22939), (68781, 68782)], [(22938, 22939), (75019, 75020)], [(23233, 23237), (77554, 77558)], [(23233, 23237), (77612, 77616)], [(23472, 23479), (76979, 76986)], [(23544, 23554), (63179, 63189)], [(23554, 23558), (60072, 60076)], [(23718, 23723), (45456, 45461)], [(23944, 23948), (61585, 61589)], [(23944, 23948), (61585, 61589)], [(23944, 23948), (61614, 61618)], [(24762, 24764), (77579, 77581)], [(24762, 24764), (77637, 77639)], [(25107, 25112), (76656, 76661)], [(25112, 25115), (54622, 54625)], [(25123, 25127), (54999, 55003)], [(25123, 25127), (61265, 61269)], [(25123, 25127), (76319, 76323)], [(25123, 25127), (77376, 77381)], [(25162, 25165), (72439, 72442)], [(25169, 25176), (45132, 45139)], [(25169, 25176), (45837, 45845)], [(25169, 25176), (61245, 61252)], [(25169, 25176), (79865, 79873)], [(25176, 25179), (54622, 54625)], [(25182, 25189), (52654, 52660)], [(25499, 25503), (63391, 63395)], [(25956, 25961), (64902, 64907)], [(25956, 25961), (76330, 76335)], [(26467, 26472), (66904, 66910)], [(26734, 26737), (65780, 65783)], [(26752, 26755), (65780, 65783)], [(26939, 26944), (61343, 61348)], [(27089, 27094), (51622, 51627)], [(27863, 27867), (61608, 61611)], [(30958, 30963), (66904, 66910)], [(30964, 30967), (50951, 50954)], [(31295, 31297), (61313, 61315)], [(31641, 31645), (52810, 52814)], [(32531, 32538), (72460, 72467)], [(33407, 33411), (61808, 61812)], [(34840, 34846), (59401, 59407)], [(35048, 35052), (54987, 54992)], [(35052, 35058), (61182, 61188)], [(35062, 35067), (54999, 55003)], [(35062, 35067), (61265, 61269)], [(35062, 35067), (76319, 76323)], [(35062, 35067), (77376, 77381)], [(36381, 36394), (74782, 74794)]]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat        
    def jeu_essai_Althusser3(self):
        """freudEtLacan manuscrit1 V1.txt / M1 v2.txt 
        insensible à la casse, aux séparateurs et aux signes diacritiques
        Pas d'assertion d'ordre sur occs_deplaces"""
        insertions = [(39969, 39982), (40475, 40477), (40503, 40504), (40674, 40675), (40722, 40723), (40895, 40897), (40922, 40941), (41063, 41064), (41187, 41196), (41504, 41505), (42424, 42434), (42551, 42570), (42863, 42868), (42884, 42886), (43014, 43027), (43050, 43058), (43067, 43081), (43096, 43102), (43141, 43151), (43472, 43473), (43980, 43981), (44034, 44042), (44158, 44159), (44668, 44674), (44875, 44888), (45132, 45139), (45146, 45151), (45227, 45232), (45367, 45372), (45456, 45470), (45672, 45689), (45695, 45698), (45742, 45743), (45817, 45845), (46146, 46147), (47004, 47005), (47019, 47020), (47310, 47311), (47839, 47842), (48395, 48399), (48703, 48709), (48922, 48937), (49062, 49088), (49527, 49531), (49745, 49747), (49806, 49807), (50498, 50500), (50511, 50527), (50796, 50808), (50951, 50954), (51112, 51138), (51161, 51163), (51282, 51283), (51344, 51346), (51464, 51465), (51560, 51561), (51583, 51584), (51614, 51644), (51820, 51826), (51962, 51963), (51977, 51987), (52278, 52282), (52306, 52309), (52332, 52342), (52535, 52551), (52621, 52623), (52653, 52660), (52669, 52679), (52702, 52706), (52810, 52824), (52842, 52845), (52869, 52870), (53030, 53035), (53213, 53225), (53446, 53457), (53736, 53746), (53833, 53849), (54622, 54625), (54715, 54718), (54808, 54809), (54896, 54907), (54968, 55003), (55009, 55013), (55124, 55133), (55143, 55146), (55169, 55178), (55311, 55313), (55635, 55637), (55799, 55801), (55913, 55919), (55960, 55962), (56352, 56355), (56399, 56411), (56514, 56517), (57193, 57214), (57254, 57307), (57428, 57429), (57540, 57553), (57579, 57585), (57640, 57646), (57659, 57662), (57674, 57684), (57699, 57709), (58031, 58040), (58163, 58169), (58553, 58560), (58933, 58939), (59132, 59134), (59221, 59222), (59374, 59391), (59399, 59429), (59544, 59563), (59685, 59686), (59982, 59996), (60010, 60016), (60068, 60093), (60116, 60125), (60286, 60294), (60516, 60527), (60542, 60548), (60571, 60574), (60584, 60592), (60724, 60729), (60755, 60774), (60909, 60917), (60930, 60932), (60980, 60982), (61053, 61069), (61121, 61198), (61206, 61285), (61313, 61331), (61343, 61392), (61468, 61480), (61578, 61589), (61600, 61618), (61805, 61812), (62185, 62186), (62221, 62222), (62352, 62356), (62424, 62438), (62490, 62498), (62542, 62609), (62619, 62644), (62714, 62729), (62764, 62767), (62773, 62781), (62887, 62904), (62933, 62937), (62999, 63002), (63109, 63115), (63168, 63189), (63389, 63399), (63618, 63621), (63762, 63769), (63809, 63816), (63853, 63855), (64162, 64191), (64306, 64307), (64434, 64437), (64528, 64542), (64548, 64571), (64581, 64652), (64814, 64817), (64837, 64848), (64894, 64907), (64913, 64927), (65015, 65018), (65104, 65106), (65224, 65232), (65437, 65438), (65447, 65456), (65495, 65503), (65692, 65694), (65705, 65712), (65746, 65783), (66195, 66230), (66523, 66525), (66547, 66549), (66561, 66574), (66753, 66765), (66865, 66917), (66924, 66926), (67482, 67485), (67674, 67677), (68042, 68045), (68751, 68758), (68778, 68791), (69152, 69153), (69166, 69169), (69177, 69178), (69186, 69189), (69312, 69313), (69437, 69438), (69508, 69540), (70077, 70084), (70134, 70135), (70248, 70252), (70701, 70720), (70735, 70744), (70752, 70754), (70787, 70794), (70991, 70999), (71195, 71202), (71489, 71499), (71531, 71532), (71554, 71564), (71595, 71597), (71780, 71781), (71966, 71968), (72169, 72170), (72439, 72467), (72480, 72485), (72503, 72514), (73078, 73080), (73328, 73334), (74110, 74114), (74291, 74295), (74480, 74481), (74565, 74573), (74782, 74794), (74908, 74919), (75019, 75020), (75281, 75285), (75877, 75878), (76023, 76037), (76240, 76245), (76312, 76346), (76352, 76362), (76450, 76466), (76626, 76635), (76656, 76670), (76683, 77027), (77060, 77076), (77130, 77134), (77364, 77386), (77440, 77674), (77848, 77852), (77945, 77946), (78361, 78364), (78456, 78463), (78598, 78606), (79865, 79873)]
        suppressions =  [(20, 87), (677, 691), (697, 706), (1069, 1070), (1198, 1199), (1399, 1404), (1618, 1620), (1645, 1659), (1698, 1699), (1780, 1781), (1904, 1915), (2223, 2224), (2773, 2785), (3152, 3164), (3281, 3290), (3579, 3583), (3599, 3600), (3728, 3743), (3789, 3800), (3839, 3852), (4170, 4171), (4677, 4679), (4732, 4742), (5565, 5575), (5819, 5825), (5832, 5833), (5909, 5916), (6051, 6060), (6067, 6069), (6355, 6356), (6400, 6401), (6776, 6777), (7389, 7390), (7413, 7422), (7657, 7658), (8475, 8479), (8492, 8494), (8536, 8537), (9034, 9042), (9346, 9351), (9564, 9568), (9683, 9685), (9744, 9746), (10135, 10141), (10356, 10362), (10416, 10418), (10423, 10426), (10563, 10565), (11021, 11022), (11130, 11151), (11562, 11567), (11865, 11867), (11905, 11919), (12032, 12045), (12393, 12398), (12405, 12407), (12415, 12416), (12551, 12559), (12743, 12745), (12851, 12855), (12879, 12884), (12907, 12910), (12980, 12989), (13112, 13121), (13273, 13276), (13359, 13362), (13367, 13388), (13763, 13774), (14010, 14023), (14371, 14379), (15150, 15152), (15241, 15249), (15339, 15341), (15428, 15436), (15497, 15505), (15511, 15515), (15597, 15599), (15660, 15669), (15764, 15765), (15803, 15805), (16063, 16064), (16414, 16417), (16446, 16448), (16838, 16842), (16872, 16889), (17313, 17314), (17685, 17690), (17757, 17760), (17853, 17864), (17869, 17870), (17975, 17990), (18016, 18020), (18039, 18057), (18102, 18103), (18108, 18113), (18125, 18144), (18457, 18458), (18483, 18493), (18614, 18621), (19017, 19023), (19386, 19393), (19824, 19831), (19839, 19856), (20403, 20418), (20837, 20838), (20877, 20893), (20910, 20920), (20943, 20950), (20960, 20963), (21068, 21100), (21127, 21140), (21301, 21308), (21369, 21524), (21646, 21659), (21667, 21679), (21718, 21757), (21931, 21933), (21944, 21950), (22536, 22571), (22582, 22587), (22717, 22730), (22798, 22801), (22852, 22863), (22907, 22944), (22954, 22956), (23061, 23067), (23073, 23086), (23192, 23198), (23227, 23242), (23270, 23276), (23307, 23311), (23369, 23370), (23392, 23397), (23472, 23479), (23484, 23489), (23534, 23563), (23718, 23725), (23730, 23732), (23944, 23957), (24098, 24118), (24254, 24265), (24487, 24488), (24496, 24498), (24519, 24524), (24762, 24767), (24858, 24866), (24879, 24908), (25107, 25127), (25162, 25189), (25359, 25361), (25492, 25503), (25696, 25700), (25948, 25961), (25996, 26007), (26048, 26049), (26465, 26472), (26718, 26723), (26732, 26737), (26751, 26755), (26767, 26775), (26939, 26944), (27066, 27072), (27089, 27094), (27101, 27104), (27635, 27639), (27850, 27867), (28208, 28212), (28917, 28921), (29273, 29274), (29303, 29304), (29317, 29320), (29328, 29329), (29337, 29340), (29587, 29590), (29651, 29660), (30254, 30255), (30368, 30371), (30833, 30838), (30956, 30973), (31290, 31297), (31615, 31616), (31638, 31645), (31669, 31672), (32045, 32047), (32516, 32519), (32531, 32538), (32622, 32659), (33158, 33159), (33407, 33411), (34839, 34846), (35029, 35067), (35358, 35361), (36301, 36306), (36373, 36375), (36381, 36403), (36720, 36743), (36797, 36804), (37347, 37349), (37720, 37723), (37854, 37860), (37995, 38001), (38532, 38534), (38599, 38600), (39043, 39045), (39263, 39270)]
        occs_deplaces = [(20, 25), (32, 36), (43, 49), (61, 66), (75, 87), (690, 691), (704, 706), (1650, 1651), (1651, 1658), (1909, 1915), (2773, 2779), (2779, 2785), (3161, 3164), (3579, 3583), (3728, 3732), (3736, 3743), (3789, 3798), (3839, 3843), (5565, 5569), (6055, 6059), (7414, 7422), (9346, 9351), (11130, 11132), (11136, 11138), (12554, 12557), (12879, 12884), (12980, 12983), (13374, 13379), (14010, 14015), (14016, 14023), (14375, 14379), (15431, 15436), (15497, 15500), (16872, 16877), (18042, 18046), (18128, 18131), (18134, 18144), (18483, 18484), (18486, 18493), (19017, 19021), (20410, 20415), (20878, 20881), (20884, 20889), (20916, 20920), (20943, 20948), (20960, 20963), (21089, 21093), (21093, 21098), (21135, 21140), (21375, 21384), (21386, 21394), (21418, 21422), (21424, 21429), (21435, 21436), (21440, 21444), (21472, 21476), (21481, 21486), (21486, 21492), (21500, 21507), (21646, 21659), (21648, 21653), (21718, 21721), (21944, 21948), (22549, 22554), (22582, 22587), (22724, 22730), (22852, 22858), (22913, 22916), (22938, 22939), (23233, 23237), (23472, 23479), (23544, 23554), (23554, 23558), (23718, 23723), (23944, 23948), (24762, 24764), (25107, 25112), (25112, 25115), (25123, 25127), (25162, 25165), (25169, 25176), (25176, 25179), (25182, 25189), (25499, 25503), (25956, 25961), (26467, 26472), (26734, 26737), (26752, 26755), (26939, 26944), (27089, 27094), (27863, 27867), (30958, 30963), (30964, 30967), (31295, 31297), (31641, 31645), (32531, 32538), (33407, 33411), (34840, 34846), (35048, 35052), (35052, 35058), (35062, 35067), (36381, 36394), (42424, 42428), (43141, 43151), (45132, 45139), (45146, 45151), (45456, 45461), (45687, 45689), (45823, 45828), (45837, 45845), (47839, 47842), (49062, 49075), (50796, 50798), (50951, 50954), (51622, 51627), (52654, 52660), (52702, 52706), (52810, 52814), (52815, 52817), (52842, 52845), (53213, 53220), (53446, 53451), (53451, 53457), (54622, 54625), (54987, 54992), (54999, 55003), (56352, 56355), (56399, 56404), (56404, 56411), (57294, 57300), (57699, 57709), (59381, 59385), (59401, 59407), (59407, 59411), (60010, 60016), (60072, 60076), (60770, 60774), (61140, 61146), (61170, 61175), (61182, 61188), (61245, 61252), (61252, 61258), (61265, 61269), (61313, 61315), (61343, 61348), (61355, 61361), (61475, 61480), (61585, 61589), (61585, 61589), (61608, 61611), (61614, 61618), (61808, 61812), (62580, 62587), (62593, 62598), (62892, 62896), (63168, 63173), (63179, 63189), (63391, 63395), (63809, 63812), (64162, 64167), (64162, 64167), (64167, 64172), (64434, 64437), (64554, 64559), (64628, 64634), (64902, 64907), (65749, 65752), (65780, 65783), (66904, 66910), (68781, 68782), (68786, 68791), (69508, 69512), (69525, 69530), (70717, 70720), (70787, 70791), (71195, 71198), (72439, 72442), (72460, 72467), (72480, 72485), (74110, 74114), (74571, 74573), (74782, 74794), (74914, 74919), (75019, 75020), (75281, 75285), (76032, 76037), (76240, 76245), (76319, 76323), (76330, 76335), (76626, 76635), (76656, 76661), (76814, 76821), (76935, 76945), (76979, 76986), (76996, 76997), (77009, 77012), (77376, 77381), (77456, 77460), (77541, 77545), (77554, 77558), (77560, 77561), (77579, 77581), (77599, 77603), (77612, 77616), (77618, 77619), (77637, 77639), (77649, 77654), (77665, 77674), (79865, 79873)]
        tous_remplacements = []
        lg_texte1 = 39356
        texte_original = self._readFile("Alth1v1.txt") + self._readFile("Alth1v2.txt") 
        blocsCommuns = [(0, 20), (87, 677), (691, 697), (706, 1069), (1070, 1198), (1199, 1225), (1225, 1394), (1394, 1399), (1404, 1446), (1446, 1618), (1620, 1645), (1659, 1698), (1699, 1780), (1781, 1904), (1915, 2223), (2224, 2773), (2785, 3152), (3164, 3281), (3290, 3579), (3583, 3599), (3600, 3606), (3606, 3728), (3743, 3766), (3766, 3774), (3774, 3789), (3800, 3839), (3852, 4170), (4171, 4677), (4679, 4732), (4742, 4857), (4857, 5364), (5364, 5565), (5575, 5819), (5825, 5832), (5833, 5903), (5903, 5909), (5916, 6051), (6060, 6067), (6069, 6148), (6148, 6349), (6349, 6355), (6356, 6400), (6401, 6475), (6475, 6776), (6777, 7389), (7390, 7413), (7422, 7643), (7643, 7657), (7658, 7948), (7948, 8468), (8468, 8475), (8479, 8492), (8494, 8536), (8537, 9034), (9042, 9346), (9351, 9564), (9568, 9683), (9685, 9695), (9695, 9744), (9746, 10130), (10130, 10135), (10141, 10356), (10362, 10416), (10418, 10423), (10426, 10563), (10565, 11021), (11022, 11119), (11119, 11130), (11151, 11420), (11420, 11562), (11567, 11723), (11723, 11746), (11746, 11865), (11867, 11905), (11919, 11941), (11941, 12032), (12045, 12069), (12069, 12164), (12164, 12186), (12186, 12216), (12216, 12393), (12398, 12405), (12407, 12415), (12416, 12537), (12537, 12551), (12559, 12743), (12745, 12851), (12855, 12879), (12884, 12907), (12910, 12980), (12989, 13112), (13121, 13191), (13191, 13221), (13221, 13229), (13229, 13252), (13252, 13273), (13276, 13359), (13362, 13367), (13388, 13400), (13400, 13424), (13424, 13585), (13585, 13763), (13774, 13994), (13994, 14010), (14023, 14285), (14285, 14371), (14379, 15150), (15152, 15241), (15249, 15339), (15341, 15428), (15436, 15497), (15505, 15511), (15515, 15597), (15599, 15628), (15628, 15637), (15637, 15660), (15669, 15764), (15765, 15803), (15805, 16063), (16064, 16128), (16128, 16291), (16291, 16403), (16403, 16414), (16417, 16446), (16448, 16838), (16842, 16872), (16889, 16906), (16906, 17010), (17010, 17313), (17314, 17678), (17678, 17685), (17690, 17711), (17711, 17729), (17729, 17757), (17760, 17853), (17864, 17869), (17870, 17975), (17990, 18016), (18020, 18039), (18057, 18094), (18094, 18102), (18103, 18108), (18113, 18125), (18144, 18159), (18159, 18457), (18458, 18478), (18478, 18483), (18493, 18608), (18608, 18614), (18621, 19007), (19007, 19017), (19023, 19386), (19393, 19587), (19587, 19673), (19673, 19819), (19819, 19824), (19831, 19839), (19856, 19972), (19972, 20094), (20094, 20389), (20389, 20403), (20418, 20470), (20470, 20493), (20493, 20654), (20654, 20837), (20838, 20877), (20893, 20910), (20920, 20943), (20950, 20960), (20963, 21068), (21100, 21127), (21140, 21166), (21166, 21301), (21308, 21321), (21321, 21369), (21524, 21594), (21594, 21646), (21659, 21667), (21679, 21705), (21705, 21710), (21710, 21718), (21757, 21833), (21833, 21931), (21933, 21944), (21950, 22137), (22137, 22511), (22511, 22529), (22529, 22536), (22571, 22582), (22587, 22717), (22730, 22798), (22801, 22852), (22863, 22907), (22944, 22954), (22956, 23026), (23026, 23061), (23067, 23073), (23086, 23181), (23181, 23192), (23198, 23227), (23242, 23270), (23276, 23307), (23311, 23369), (23370, 23392), (23397, 23424), (23424, 23472), (23479, 23484), (23489, 23534), (23563, 23718), (23725, 23730), (23732, 23944), (23957, 24098), (24118, 24158), (24158, 24195), (24195, 24254), (24265, 24482), (24482, 24487), (24488, 24496), (24498, 24514), (24514, 24519), (24524, 24629), (24629, 24634), (24634, 24641), (24641, 24762), (24767, 24858), (24866, 24871), (24871, 24879), (24908, 25070), (25070, 25090), (25090, 25107), (25127, 25151), (25151, 25156), (25156, 25162), (25189, 25273), (25273, 25359), (25361, 25480), (25480, 25492), (25503, 25696), (25700, 25709), (25709, 25748), (25748, 25937), (25937, 25948), (25961, 25996), (26007, 26048), (26049, 26419), (26419, 26465), (26472, 26718), (26723, 26732), (26737, 26751), (26755, 26767), (26775, 26939), (26944, 26960), (26960, 27059), (27059, 27066), (27072, 27089), (27094, 27101), (27104, 27635), (27639, 27644), (27644, 27827), (27827, 27850), (27867, 28208), (28212, 28217), (28217, 28917), (28921, 28942), (28942, 29273), (29274, 29303), (29304, 29317), (29320, 29328), (29329, 29337), (29340, 29463), (29463, 29587), (29590, 29651), (29660, 29668), (29668, 30204), (30204, 30254), (30255, 30368), (30371, 30818), (30818, 30833), (30838, 30847), (30847, 30880), (30880, 30956), (30973, 31094), (31094, 31290), (31297, 31583), (31583, 31615), (31616, 31638), (31645, 31669), (31672, 31679), (31679, 31860), (31860, 32045), (32047, 32247), (32247, 32516), (32519, 32531), (32538, 32557), (32557, 32622), (32659, 33158), (33159, 33407), (33411, 34185), (34185, 34362), (34362, 34546), (34546, 34630), (34630, 34839), (34846, 34960), (34960, 35029), (35067, 35098), (35098, 35358), (35361, 35953), (35953, 36098), (36098, 36301), (36306, 36373), (36375, 36381), (36403, 36478), (36478, 36495), (36495, 36655), (36655, 36676), (36676, 36687), (36687, 36720), (36743, 36797), (36804, 37034), (37034, 37082), (37082, 37254), (37254, 37347), (37349, 37720), (37723, 37762), (37762, 37848), (37848, 37854), (37860, 37995), (38001, 38532), (38534, 38599), (38600, 39043), (39045, 39263), (39270, 39356), (39356, 39372), (39372, 39963), (39963, 39969), (39982, 40348), (40348, 40475), (40477, 40503), (40504, 40674), (40675, 40680), (40680, 40722), (40723, 40895), (40897, 40922), (40941, 40981), (40981, 41063), (41064, 41187), (41196, 41504), (41505, 42056), (42056, 42424), (42434, 42551), (42570, 42863), (42868, 42884), (42886, 42892), (42892, 43014), (43027, 43050), (43058, 43067), (43081, 43096), (43102, 43141), (43151, 43472), (43473, 43980), (43981, 44034), (44042, 44158), (44159, 44668), (44674, 44875), (44888, 45132), (45139, 45146), (45151, 45221), (45221, 45227), (45232, 45367), (45372, 45379), (45379, 45456), (45470, 45672), (45689, 45695), (45698, 45742), (45743, 45817), (45845, 46146), (46147, 46759), (46759, 46782), (46782, 47004), (47005, 47019), (47020, 47310), (47311, 47832), (47832, 47839), (47842, 47856), (47856, 47898), (47898, 48395), (48399, 48703), (48709, 48922), (48937, 49052), (49052, 49062), (49088, 49137), (49137, 49522), (49522, 49527), (49531, 49745), (49747, 49801), (49801, 49806), (49807, 49944), (49944, 50401), (50401, 50498), (50500, 50511), (50527, 50796), (50808, 50951), (50954, 51112), (51138, 51161), (51163, 51282), (51283, 51321), (51321, 51344), (51346, 51439), (51439, 51464), (51465, 51560), (51561, 51583), (51584, 51614), (51644, 51820), (51826, 51833), (51833, 51841), (51841, 51962), (51963, 51977), (51987, 52172), (52172, 52278), (52282, 52306), (52309, 52332), (52342, 52412), (52412, 52535), (52551, 52621), (52623, 52653), (52660, 52669), (52679, 52702), (52706, 52727), (52727, 52810), (52824, 52830), (52830, 52842), (52845, 52869), (52870, 53030), (53035, 53213), (53225, 53446), (53457, 53474), (53474, 53736), (53746, 53833), (53849, 54622), (54625, 54715), (54718, 54808), (54809, 54896), (54907, 54968), (55003, 55009), (55013, 55095), (55095, 55124), (55133, 55143), (55146, 55169), (55178, 55273), (55273, 55311), (55313, 55571), (55571, 55635), (55637, 55799), (55801, 55913), (55919, 55930), (55930, 55960), (55962, 56352), (56355, 56385), (56385, 56399), (56411, 56514), (56517, 56821), (56821, 57186), (57186, 57193), (57214, 57235), (57235, 57254), (57307, 57335), (57335, 57428), (57429, 57434), (57434, 57540), (57553, 57579), (57585, 57604), (57604, 57640), (57646, 57654), (57654, 57659), (57662, 57674), (57684, 57699), (57709, 58006), (58006, 58026), (58026, 58031), (58040, 58157), (58157, 58163), (58169, 58553), (58560, 58570), (58570, 58933), (58939, 59132), (59134, 59221), (59222, 59369), (59369, 59374), (59391, 59399), (59429, 59544), (59563, 59685), (59686, 59982), (59996, 60010), (60016, 60068), (60093, 60116), (60125, 60286), (60294, 60477), (60477, 60516), (60527, 60542), (60548, 60571), (60574, 60584), (60592, 60697), (60697, 60724), (60729, 60755), (60774, 60909), (60917, 60930), (60932, 60980), (60982, 61053), (61069, 61121), (61198, 61206), (61285, 61313), (61331, 61336), (61336, 61343), (61392, 61468), (61480, 61578), (61589, 61600), (61618, 61805), (61812, 62185), (62186, 62204), (62204, 62210), (62210, 62221), (62222, 62352), (62356, 62424), (62438, 62490), (62498, 62542), (62609, 62619), (62644, 62714), (62729, 62764), (62767, 62773), (62781, 62876), (62876, 62887), (62904, 62933), (62937, 62967), (62967, 62999), (63002, 63060), (63060, 63082), (63082, 63109), (63115, 63163), (63163, 63168), (63189, 63234), (63234, 63389), (63399, 63405), (63405, 63618), (63621, 63762), (63769, 63809), (63816, 63853), (63855, 63915), (63915, 64132), (64132, 64137), (64137, 64146), (64146, 64162), (64191, 64196), (64196, 64301), (64301, 64306), (64307, 64314), (64314, 64434), (64437, 64528), (64542, 64548), (64571, 64581), (64652, 64814), (64817, 64837), (64848, 64865), (64865, 64889), (64889, 64894), (64907, 64913), (64927, 65015), (65018, 65104), (65106, 65224), (65232, 65244), (65244, 65437), (65438, 65447), (65456, 65495), (65503, 65692), (65694, 65705), (65712, 65746), (65783, 65824), (65824, 66195), (66230, 66275), (66275, 66523), (66525, 66534), (66534, 66547), (66549, 66561), (66574, 66738), (66738, 66753), (66765, 66865), (66917, 66924), (66926, 66943), (66943, 66950), (66950, 67482), (67485, 67491), (67491, 67674), (67677, 67701), (67701, 68042), (68045, 68050), (68050, 68751), (68758, 68778), (68791, 69123), (69123, 69152), (69153, 69166), (69169, 69177), (69178, 69186), (69189, 69312), (69313, 69437), (69438, 69500), (69500, 69508), (69540, 70077), (70084, 70134), (70135, 70248), (70252, 70701), (70720, 70735), (70744, 70752), (70754, 70787), (70794, 70870), (70870, 70991), (70999, 71195), (71202, 71489), (71499, 71531), (71532, 71554), (71564, 71588), (71588, 71595), (71597, 71780), (71781, 71966), (71968, 72169), (72170, 72439), (72467, 72480), (72485, 72503), (72514, 72579), (72579, 73078), (73080, 73328), (73334, 74110), (74114, 74291), (74295, 74480), (74481, 74565), (74573, 74782), (74794, 74908), (74919, 74988), (74988, 75019), (75020, 75281), (75285, 75877), (75878, 76023), (76037, 76240), (76245, 76312), (76346, 76352), (76362, 76434), (76434, 76450), (76466, 76626), (76635, 76656), (76670, 76683), (77027, 77060), (77076, 77130), (77134, 77364), (77386, 77440), (77674, 77848), (77852, 77945), (77946, 78320), (78320, 78361), (78364, 78450), (78450, 78456), (78463, 78598), (78606, 79139), (79139, 79204), (79204, 79647), (79647, 79865), (79873, 79958)]
        lDepl = [[(20, 25), (64167, 64172)], [(32, 36), (42424, 42428)], [(43, 49), (61355, 61361)], [(61, 66), (69525, 69530)], [(75, 87), (77665, 77674)], [(690, 691), (77560, 77561)], [(690, 691), (77618, 77619)], [(704, 706), (74571, 74573)], [(1650, 1651), (76996, 76997)], [(1651, 1658), (53213, 53220)], [(1909, 1915), (60010, 60016)], [(2773, 2779), (64628, 64634)], [(2779, 2785), (76032, 76037)], [(3161, 3164), (47839, 47842)], [(3161, 3164), (65749, 65752)], [(3579, 3583), (62892, 62896)], [(3728, 3732), (75281, 75285)], [(3736, 3743), (74914, 74919)], [(3789, 3798), (76935, 76945)], [(3839, 3843), (77456, 77460)], [(3839, 3843), (77541, 77545)], [(3839, 3843), (77599, 77603)], [(5565, 5569), (59407, 59411)], [(6055, 6059), (59381, 59385)], [(7414, 7422), (59401, 59407)], [(9346, 9351), (63168, 63173)], [(11130, 11132), (45687, 45689)], [(11136, 11138), (50796, 50798)], [(12554, 12557), (70717, 70720)], [(12879, 12884), (64554, 64559)], [(12980, 12983), (71195, 71198)], [(13374, 13379), (61170, 61175)], [(14010, 14015), (53446, 53451)], [(14016, 14023), (53451, 53457)], [(14375, 14379), (60770, 60774)], [(15431, 15436), (64162, 64167)], [(15431, 15436), (64162, 64167)], [(15431, 15436), (76240, 76245)], [(15497, 15500), (52842, 52845)], [(16872, 16877), (56399, 56404)], [(18042, 18046), (52702, 52706)], [(18128, 18131), (63809, 63812)], [(18134, 18144), (57699, 57709)], [(18483, 18484), (77560, 77561)], [(18483, 18484), (77618, 77619)], [(18486, 18493), (56404, 56411)], [(19017, 19021), (59381, 59385)], [(20410, 20415), (66904, 66910)], [(20878, 20881), (56352, 56355)], [(20884, 20889), (77649, 77654)], [(20916, 20920), (70787, 70791)], [(20943, 20948), (72480, 72485)], [(20960, 20963), (77009, 77012)], [(21089, 21093), (74110, 74114)], [(21093, 21098), (62593, 62598)], [(21135, 21140), (61475, 61480)], [(21375, 21384), (76626, 76635)], [(21386, 21394), (43141, 43151)], [(21418, 21422), (60010, 60016)], [(21424, 21429), (66904, 66910)], [(21435, 21436), (68781, 68782)], [(21435, 21436), (75019, 75020)], [(21440, 21444), (69508, 69512)], [(21472, 21476), (68786, 68791)], [(21481, 21486), (64902, 64907)], [(21481, 21486), (76330, 76335)], [(21486, 21492), (61182, 61188)], [(21500, 21507), (62580, 62587)], [(21646, 21659), (49062, 49075)], [(21648, 21653), (61252, 61258)], [(21718, 21721), (64434, 64437)], [(21944, 21948), (61585, 61589)], [(21944, 21948), (61585, 61589)], [(21944, 21948), (61614, 61618)], [(22549, 22554), (76814, 76821)], [(22582, 22587), (45146, 45151)], [(22582, 22587), (45823, 45828)], [(22724, 22730), (57294, 57300)], [(22852, 22858), (61140, 61146)], [(22913, 22916), (52815, 52817)], [(22938, 22939), (68781, 68782)], [(22938, 22939), (75019, 75020)], [(23233, 23237), (77554, 77558)], [(23233, 23237), (77612, 77616)], [(23472, 23479), (76979, 76986)], [(23544, 23554), (63179, 63189)], [(23554, 23558), (60072, 60076)], [(23718, 23723), (45456, 45461)], [(23944, 23948), (61585, 61589)], [(23944, 23948), (61585, 61589)], [(23944, 23948), (61614, 61618)], [(24762, 24764), (77579, 77581)], [(24762, 24764), (77637, 77639)], [(25107, 25112), (76656, 76661)], [(25112, 25115), (54622, 54625)], [(25123, 25127), (54999, 55003)], [(25123, 25127), (61265, 61269)], [(25123, 25127), (76319, 76323)], [(25123, 25127), (77376, 77381)], [(25162, 25165), (72439, 72442)], [(25169, 25176), (45132, 45139)], [(25169, 25176), (45837, 45845)], [(25169, 25176), (61245, 61252)], [(25169, 25176), (79865, 79873)], [(25176, 25179), (54622, 54625)], [(25182, 25189), (52654, 52660)], [(25499, 25503), (63391, 63395)], [(25956, 25961), (64902, 64907)], [(25956, 25961), (76330, 76335)], [(26467, 26472), (66904, 66910)], [(26734, 26737), (65780, 65783)], [(26752, 26755), (65780, 65783)], [(26939, 26944), (61343, 61348)], [(27089, 27094), (51622, 51627)], [(27863, 27867), (61608, 61611)], [(30958, 30963), (66904, 66910)], [(30964, 30967), (50951, 50954)], [(31295, 31297), (61313, 61315)], [(31641, 31645), (52810, 52814)], [(32531, 32538), (72460, 72467)], [(33407, 33411), (61808, 61812)], [(34840, 34846), (59401, 59407)], [(35048, 35052), (54987, 54992)], [(35052, 35058), (61182, 61188)], [(35062, 35067), (54999, 55003)], [(35062, 35067), (61265, 61269)], [(35062, 35067), (76319, 76323)], [(35062, 35067), (77376, 77381)], [(36381, 36394), (74782, 74794)]]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat         
    def jeu_essai_Charroi(self):
        """charroi Debut_A.txt / Debut_E.txt 
        Insensible à tout
        Assertion d'ordre sur occs_deplaces vérifiée"""
        insertions = [[10538, 10541], [10729, 10733], [11001, 11004], [11115, 11122], [11279, 11283], [11351, 11387], [11594, 11595], [11665, 11666], [11816, 11819], [11876, 11877], [12394, 12399], [12456, 12457], [13018, 13019], [13058, 13066], [13301, 13302], [13512, 13515], [13580, 13581], [13637, 13638], [13995, 13996], [14161, 14168], [14354, 14357], [14450, 14456], [14499, 14502], [14555, 14556], [14661, 14668], [14671, 14675], [14898, 14906], [14920, 14922], [15396, 15397], [15564, 15565], [15940, 15946], [16203, 16206], [16376, 16377], [16692, 16700], [17087, 17121], [17130, 17136], [17220, 17221], [17305, 17314], [17438, 17439], [17894, 17895], [17964, 17965], [17975, 17976], [18474, 18476], [18518, 18519], [18682, 18683], [18760, 18762], [18923, 18924], [19084, 19085], [19410, 19412], [19445, 19446], [19473, 19476], [19568, 19571], [19730, 19734], [20028, 20031], [20179, 20182], [20214, 20220], [20298, 20306], [20320, 20323], [20569, 20572], [20864, 20865], [20977, 20983]]
        suppressions =  [[8, 9], [15, 16], [40, 41], [96, 98], [141, 147], [409, 410], [482, 489], [858, 866], [979, 982], [1003, 1007], [1075, 1078], [1116, 1119], [1146, 1152], [1368, 1373], [1387, 1388], [1569, 1578], [1745, 1751], [2084, 2090], [2284, 2290], [2418, 2424], [2539, 2542], [2666, 2669], [2932, 2938], [3025, 3026], [3091, 3092], [3103, 3110], [3380, 3388], [3511, 3514], [3589, 3597], [3690, 3691], [3879, 3880], [4024, 4025], [4275, 4278], [4523, 4524], [5190, 5198], [5257, 5264], [5509, 5510], [5688, 5691], [5719, 5720], [5730, 5731], [6159, 6165], [6410, 6416], [6610, 6611], [6618, 6624], [6722, 6725], [6789, 6790], [6988, 6996], [7470, 7473], [7579, 7580], [7608, 7609], [8025, 8028], [8190, 8193], [8274, 8281], [8384, 8385], [8435, 8436], [8487, 8490], [8499, 8506], [8626, 8627], [8646, 8651], [8792, 8800], [8830, 8831], [8975, 8978], [9005, 9006], [9561, 9562], [9573, 9575], [9637, 9643], [9675, 9678], [9700, 9701], [10073, 10074], [10083, 10084], [10156, 10157], [10381, 10384]]
        occs_deplaces = [[143, 147], [226, 232], [507, 512], [622, 627], [784, 790], [897, 900], [1018, 1019], [1233, 1239], [1273, 1278], [1308, 1311], [1328, 1333], [1496, 1501], [1574, 1578], [1707, 1712], [1908, 1911], [2420, 2424], [2465, 2468], [2688, 2694], [2729, 2734], [2903, 2906], [2946, 2950], [3412, 3420], [3681, 3686], [3742, 3743], [3973, 3981], [4125, 4126], [4185, 4190], [4193, 4199], [4436, 4440], [4450, 4453], [4766, 4770], [4830, 4836], [5190, 5193], [5194, 5198], [5259, 5264], [5465, 5469], [6091, 6095], [6108, 6113], [6160, 6165], [6197, 6200], [6412, 6416], [6633, 6637], [6637, 6641], [6807, 6812], [6812, 6817], [6960, 6962], [6992, 6996], [7059, 7063], [7550, 7556], [8221, 8226], [8229, 8233], [8274, 8279], [8315, 8320], [8320, 8325], [8327, 8331], [8511, 8517], [8583, 8586], [8796, 8800], [9052, 9053], [9074, 9079], [9094, 9099], [9103, 9105], [9259, 9263], [9402, 9405], [9469, 9471], [9661, 9664], [9733, 9741], [9833, 9836], [9841, 9847], [9970, 9974], [10295, 10299], [10334, 10340], [10423, 10433], [10596, 10600], [10610, 10615], [10646, 10650], [10978, 10983], [11301, 11305], [11498, 11501], [11522, 11528], [11633, 11638], [11745, 11749], [11783, 11786], [12069, 12070], [12235, 12239], [12495, 12499], [12577, 12581], [12772, 12776], [12904, 12908], [13058, 13061], [13151, 13156], [13175, 13179], [13334, 13339], [13384, 13389], [13415, 13423], [13431, 13437], [13592, 13597], [13820, 13826], [13894, 13904], [13982, 13985], [14071, 14079], [14470, 14475], [14663, 14668], [14751, 14754], [14902, 14906], [15732, 15734], [15734, 15739], [16160, 16165], [16563, 16569], [16627, 16633], [16696, 16700], [16884, 16888], [17049, 17054], [17068, 17073], [17132, 17136], [17142, 17146], [17246, 17247], [17458, 17460], [17833, 17839], [18028, 18029], [18045, 18049], [18484, 18487], [18487, 18492], [18496, 18500], [18711, 18714], [18805, 18809], [18964, 18967], [18976, 18981], [19120, 19125], [19522, 19525], [19731, 19734], [19873, 19874], [19936, 19938], [20042, 20046], [20056, 20059], [20112, 20118], [20136, 20141], [20152, 20157], [20328, 20332], [20455, 20461], [20778, 20784], [20904, 20912]]
        tous_remplacements = [[31, 34], [48, 49], [108, 113], [121, 124], [379, 391], [521, 526], [540, 541], [683, 686], [712, 718], [807, 812], [897, 902], [939, 940], [968, 975], [1018, 1021], [1035, 1040], [1059, 1060], [1104, 1105], [1160, 1171], [1189, 1195], [1227, 1228], [1233, 1242], [1273, 1281], [1320, 1321], [1328, 1335], [1410, 1415], [1455, 1456], [1496, 1505], [1538, 1543], [1707, 1713], [1816, 1817], [1836, 1840], [1996, 2010], [2039, 2043], [2123, 2125], [2148, 2151], [2181, 2182], [2187, 2193], [2248, 2253], [2455, 2473], [2489, 2494], [2621, 2625], [2639, 2642], [2688, 2695], [2740, 2743], [2830, 2831], [2851, 2867], [3328, 3336], [3348, 3353], [3489, 3501], [3617, 3621], [3696, 3700], [3739, 3743], [3810, 3814], [3992, 4000], [4029, 4033], [4125, 4129], [4205, 4206], [4346, 4348], [4352, 4357], [4382, 4387], [4450, 4454], [4513, 4514], [4549, 4554], [4593, 4603], [4764, 4775], [4830, 4837], [4867, 4868], [4888, 4893], [4908, 4911], [4939, 4942], [5048, 5049], [5107, 5109], [5240, 5244], [5543, 5547], [5664, 5672], [5743, 5747], [5758, 5761], [5801, 5811], [5986, 5987], [6065, 6069], [6108, 6116], [6127, 6131], [6195, 6200], [6231, 6233], [6298, 6302], [6386, 6390], [6557, 6564], [6577, 6588], [6600, 6605], [6656, 6661], [6732, 6735], [6912, 6915], [6926, 6930], [7059, 7065], [7099, 7100], [7197, 7200], [7319, 7324], [7340, 7345], [7425, 7426], [7445, 7449], [7530, 7534], [7675, 7677], [7793, 7796], [7836, 7841], [7901, 7905], [7947, 7950], [7996, 8007], [8052, 8057], [8135, 8137], [8144, 8145], [8221, 8233], [8238, 8240], [8320, 8331], [8511, 8522], [8544, 8550], [8577, 8586], [8811, 8816], [8924, 8928], [9073, 9080], [9103, 9106], [9137, 9139], [9275, 9276], [9306, 9314], [9430, 9434], [9519, 9521], [9526, 9531], [9585, 9588], [9611, 9617], [10126, 10128], [10279, 10283], [10290, 10299], [10334, 10345], [10527, 10532], [10548, 10549], [10610, 10616], [10624, 10629], [10880, 10888], [11013, 11018], [11032, 11034], [11178, 11183], [11209, 11213], [11300, 11305], [11418, 11423], [11460, 11461], [11489, 11494], [11539, 11542], [11556, 11559], [11578, 11579], [11621, 11622], [11674, 11681], [11699, 11707], [11739, 11740], [11745, 11752], [11783, 11789], [11828, 11829], [11836, 11843], [11913, 11918], [11958, 11959], [11999, 12007], [12040, 12043], [12199, 12203], [12304, 12305], [12324, 12326], [12485, 12501], [12530, 12536], [12614, 12615], [12638, 12643], [12673, 12674], [12679, 12683], [12738, 12741], [12939, 12954], [12970, 12973], [13106, 13108], [13122, 13127], [13175, 13180], [13220, 13225], [13313, 13314], [13334, 13348], [13816, 13826], [13838, 13843], [13973, 13985], [14099, 14101], [14177, 14179], [14218, 14220], [14287, 14289], [14467, 14475], [14506, 14508], [14601, 14605], [14681, 14682], [14822, 14826], [14830, 14835], [14860, 14863], [14932, 14938], [14997, 14998], [15032, 15035], [15074, 15083], [15244, 15254], [15309, 15314], [15344, 15345], [15365, 15370], [15385, 15390], [15419, 15424], [15530, 15531], [15590, 15594], [15717, 15719], [16019, 16021], [16138, 16144], [16218, 16220], [16231, 16236], [16276, 16284], [16460, 16461], [16539, 16541], [16582, 16586], [16597, 16599], [16663, 16668], [16707, 16711], [16776, 16778], [16862, 16864], [17029, 17036], [17049, 17056], [17068, 17075], [17155, 17159], [17228, 17233], [17409, 17414], [17425, 17427], [17549, 17556], [17590, 17591], [17688, 17690], [17809, 17814], [17830, 17839], [17920, 17921], [17940, 17943], [18023, 18029], [18166, 18168], [18284, 18287], [18327, 18330], [18390, 18392], [18434, 18436], [18487, 18500], [18543, 18548], [18626, 18628], [18635, 18637], [18711, 18719], [18724, 18726], [18796, 18809], [18986, 18998], [19020, 19026], [19053, 19060], [19277, 19283], [19390, 19396], [19545, 19554], [19575, 19578], [19609, 19610], [19746, 19747], [19777, 19785], [19899, 19901], [19986, 19989], [19994, 19998], [20056, 20061], [20084, 20092], [20614, 20616], [20766, 20768], [20775, 20784], [20819, 20828]]
        lg_texte1 = 10498
        texte_original = self._readFile("Debut_A.txt") + self._readFile("Debut_E.txt")     
        blocsCommuns = [[0, 8], [9, 15], [16, 31], [34, 40], [41, 48], [49, 96], [98, 108], [113, 121], [124, 141], [147, 226], [232, 379], [391, 409], [410, 482], [489, 507], [512, 521], [526, 540], [541, 622], [627, 683], [686, 712], [718, 784], [790, 807], [812, 858], [866, 897], [902, 939], [940, 968], [975, 979], [982, 1003], [1007, 1018], [1021, 1035], [1040, 1059], [1060, 1075], [1078, 1104], [1105, 1116], [1119, 1146], [1152, 1160], [1171, 1189], [1195, 1227], [1228, 1233], [1242, 1273], [1281, 1308], [1311, 1320], [1321, 1328], [1335, 1368], [1373, 1387], [1388, 1410], [1415, 1455], [1456, 1496], [1505, 1538], [1543, 1569], [1578, 1707], [1713, 1745], [1751, 1816], [1817, 1836], [1840, 1901], [1901, 1908], [1911, 1968], [1968, 1996], [2010, 2039], [2043, 2084], [2090, 2123], [2125, 2148], [2151, 2181], [2182, 2187], [2193, 2248], [2253, 2284], [2290, 2418], [2424, 2455], [2473, 2489], [2494, 2539], [2542, 2581], [2581, 2621], [2625, 2639], [2642, 2666], [2669, 2688], [2695, 2729], [2734, 2740], [2743, 2819], [2819, 2830], [2831, 2851], [2867, 2903], [2906, 2932], [2938, 2946], [2950, 3025], [3026, 3091], [3092, 3103], [3110, 3150], [3150, 3328], [3336, 3348], [3353, 3380], [3388, 3412], [3420, 3489], [3501, 3511], [3514, 3581], [3581, 3589], [3597, 3617], [3621, 3681], [3686, 3690], [3691, 3696], [3700, 3739], [3743, 3748], [3748, 3810], [3814, 3879], [3880, 3965], [3965, 3973], [3981, 3992], [4000, 4024], [4025, 4029], [4033, 4080], [4080, 4125], [4129, 4137], [4137, 4185], [4190, 4193], [4199, 4205], [4206, 4275], [4278, 4346], [4348, 4352], [4357, 4382], [4387, 4422], [4422, 4436], [4440, 4450], [4454, 4513], [4514, 4523], [4524, 4549], [4554, 4593], [4603, 4764], [4775, 4830], [4837, 4867], [4868, 4888], [4893, 4908], [4911, 4917], [4917, 4939], [4942, 5048], [5049, 5082], [5082, 5107], [5109, 5190], [5198, 5240], [5244, 5257], [5264, 5465], [5469, 5509], [5510, 5543], [5547, 5664], [5672, 5677], [5677, 5688], [5691, 5719], [5720, 5730], [5731, 5743], [5747, 5758], [5761, 5801], [5811, 5903], [5903, 5986], [5987, 6065], [6069, 6091], [6095, 6108], [6116, 6127], [6131, 6159], [6165, 6195], [6200, 6224], [6224, 6231], [6233, 6298], [6302, 6386], [6390, 6410], [6416, 6557], [6564, 6577], [6588, 6600], [6605, 6610], [6611, 6618], [6624, 6633], [6641, 6647], [6647, 6656], [6661, 6722], [6725, 6732], [6735, 6748], [6748, 6789], [6790, 6807], [6817, 6912], [6915, 6926], [6930, 6941], [6941, 6960], [6962, 6967], [6967, 6988], [6996, 7002], [7002, 7059], [7065, 7099], [7100, 7190], [7190, 7197], [7200, 7319], [7324, 7340], [7345, 7394], [7394, 7400], [7400, 7425], [7426, 7445], [7449, 7470], [7473, 7483], [7483, 7530], [7534, 7539], [7539, 7550], [7556, 7579], [7580, 7608], [7609, 7675], [7677, 7793], [7796, 7836], [7841, 7901], [7905, 7947], [7950, 7988], [7988, 7996], [8007, 8025], [8028, 8052], [8057, 8135], [8137, 8144], [8145, 8190], [8193, 8221], [8233, 8238], [8240, 8274], [8281, 8315], [8331, 8384], [8385, 8435], [8436, 8447], [8447, 8487], [8490, 8499], [8506, 8511], [8522, 8544], [8550, 8577], [8586, 8610], [8610, 8626], [8627, 8646], [8651, 8792], [8800, 8811], [8816, 8830], [8831, 8924], [8928, 8942], [8942, 8975], [8978, 9005], [9006, 9047], [9047, 9052], [9053, 9073], [9080, 9094], [9099, 9103], [9106, 9137], [9139, 9259], [9263, 9275], [9276, 9306], [9314, 9397], [9397, 9402], [9405, 9430], [9434, 9457], [9457, 9469], [9471, 9476], [9476, 9519], [9521, 9526], [9531, 9561], [9562, 9573], [9575, 9585], [9588, 9611], [9617, 9637], [9643, 9661], [9664, 9675], [9678, 9700], [9701, 9733], [9741, 9819], [9819, 9833], [9836, 9841], [9847, 9970], [9974, 10073], [10074, 10083], [10084, 10126], [10128, 10156], [10157, 10279], [10283, 10290], [10299, 10334], [10345, 10381], [10384, 10423], [10433, 10498], [10498, 10506], [10506, 10512], [10512, 10527], [10532, 10538], [10541, 10548], [10549, 10596], [10600, 10610], [10616, 10624], [10629, 10646], [10650, 10729], [10733, 10880], [10888, 10906], [10906, 10978], [10983, 11001], [11004, 11013], [11018, 11032], [11034, 11115], [11122, 11178], [11183, 11209], [11213, 11279], [11283, 11300], [11305, 11351], [11387, 11418], [11423, 11460], [11461, 11489], [11494, 11498], [11501, 11522], [11528, 11539], [11542, 11556], [11559, 11578], [11579, 11594], [11595, 11621], [11622, 11633], [11638, 11665], [11666, 11674], [11681, 11699], [11707, 11739], [11740, 11745], [11752, 11783], [11789, 11816], [11819, 11828], [11829, 11836], [11843, 11876], [11877, 11891], [11891, 11913], [11918, 11958], [11959, 11999], [12007, 12040], [12043, 12069], [12070, 12199], [12203, 12235], [12239, 12304], [12305, 12324], [12326, 12387], [12387, 12394], [12399, 12456], [12457, 12485], [12501, 12530], [12536, 12577], [12581, 12614], [12615, 12638], [12643, 12673], [12674, 12679], [12683, 12738], [12741, 12772], [12776, 12904], [12908, 12939], [12954, 12970], [12973, 13018], [13019, 13058], [13066, 13106], [13108, 13122], [13127, 13151], [13156, 13175], [13180, 13214], [13214, 13220], [13225, 13301], [13302, 13313], [13314, 13334], [13348, 13384], [13389, 13415], [13423, 13431], [13437, 13512], [13515, 13580], [13581, 13592], [13597, 13637], [13638, 13816], [13826, 13838], [13843, 13870], [13870, 13894], [13904, 13973], [13985, 13995], [13996, 14063], [14063, 14071], [14079, 14099], [14101, 14161], [14168, 14172], [14172, 14177], [14179, 14218], [14220, 14225], [14225, 14287], [14289, 14354], [14357, 14442], [14442, 14450], [14456, 14467], [14475, 14499], [14502, 14506], [14508, 14555], [14556, 14601], [14605, 14613], [14613, 14661], [14668, 14671], [14675, 14681], [14682, 14751], [14754, 14822], [14826, 14830], [14835, 14860], [14863, 14898], [14906, 14920], [14922, 14932], [14938, 14997], [14998, 15007], [15007, 15032], [15035, 15074], [15083, 15244], [15254, 15309], [15314, 15344], [15345, 15365], [15370, 15385], [15390, 15396], [15397, 15419], [15424, 15530], [15531, 15564], [15565, 15590], [15594, 15675], [15675, 15717], [15719, 15732], [15739, 15940], [15946, 15986], [15986, 16019], [16021, 16138], [16144, 16149], [16149, 16160], [16165, 16193], [16193, 16203], [16206, 16218], [16220, 16231], [16236, 16276], [16284, 16376], [16377, 16460], [16461, 16539], [16541, 16563], [16569, 16582], [16586, 16597], [16599, 16627], [16633, 16663], [16668, 16692], [16700, 16707], [16711, 16776], [16778, 16862], [16864, 16884], [16888, 17029], [17036, 17049], [17056, 17068], [17075, 17080], [17080, 17087], [17121, 17130], [17136, 17142], [17146, 17155], [17159, 17220], [17221, 17228], [17233, 17246], [17247, 17288], [17288, 17305], [17314, 17409], [17414, 17425], [17427, 17438], [17439, 17458], [17460, 17465], [17465, 17486], [17486, 17492], [17492, 17549], [17556, 17590], [17591, 17681], [17681, 17688], [17690, 17809], [17814, 17830], [17839, 17888], [17888, 17894], [17895, 17920], [17921, 17940], [17943, 17964], [17965, 17975], [17976, 18023], [18029, 18034], [18034, 18045], [18049, 18072], [18072, 18100], [18100, 18166], [18168, 18284], [18287, 18327], [18330, 18390], [18392, 18434], [18436, 18474], [18476, 18484], [18500, 18518], [18519, 18543], [18548, 18626], [18628, 18635], [18637, 18682], [18683, 18711], [18719, 18724], [18726, 18760], [18762, 18796], [18809, 18862], [18862, 18912], [18912, 18923], [18924, 18964], [18967, 18976], [18981, 18986], [18998, 19020], [19026, 19053], [19060, 19084], [19085, 19101], [19101, 19120], [19125, 19266], [19266, 19277], [19283, 19297], [19297, 19390], [19396, 19410], [19412, 19445], [19446, 19473], [19476, 19517], [19517, 19522], [19525, 19545], [19554, 19568], [19571, 19575], [19578, 19609], [19610, 19730], [19734, 19746], [19747, 19777], [19785, 19868], [19868, 19873], [19874, 19899], [19901, 19924], [19924, 19936], [19938, 19943], [19943, 19986], [19989, 19994], [19998, 20028], [20031, 20042], [20046, 20056], [20061, 20084], [20092, 20112], [20118, 20136], [20141, 20152], [20157, 20179], [20182, 20214], [20220, 20298], [20306, 20320], [20323, 20328], [20332, 20455], [20461, 20560], [20560, 20569], [20572, 20614], [20616, 20644], [20644, 20766], [20768, 20775], [20784, 20819], [20828, 20864], [20865, 20904], [20912, 20977]]
        lDepl = [([143, 147], [17132, 17136]), ([226, 232], [11522, 11528]), ([507, 512], [14470, 14475]), ([507, 512], [15734, 15739]), ([507, 512], [16160, 16165]), ([507, 512], [19120, 19125]), ([622, 627], [13592, 13597]), ([784, 790], [11522, 11528]), ([897, 900], [11498, 11501]), ([897, 900], [14751, 14754]), ([897, 900], [20056, 20059]), ([1018, 1019], [12069, 12070]), ([1233, 1239], [13431, 13437]), ([1273, 1278], [13384, 13389]), ([1308, 1311], [19731, 19734]), ([1328, 1333], [13151, 13156]), ([1496, 1501], [17049, 17054]), ([1574, 1578], [18496, 18500]), ([1707, 1712], [10610, 10615]), ([1707, 1712], [17068, 17073]), ([1908, 1911], [13982, 13985]), ([2420, 2424], [17132, 17136]), ([2465, 2468], [19731, 19734]), ([2688, 2694], [16563, 16569]), ([2729, 2734], [13334, 13339]), ([2903, 2906], [11783, 11786]), ([2946, 2950], [11745, 11749]), ([2946, 2950], [12235, 12239]), ([3412, 3420], [20904, 20912]), ([3681, 3686], [18976, 18981]), ([3742, 3743], [18028, 18029]), ([3973, 3981], [14071, 14079]), ([4125, 4126], [17246, 17247]), ([4185, 4190], [13592, 13597]), ([4193, 4199], [13820, 13826]), ([4436, 4440], [10596, 10600]), ([4436, 4440], [20042, 20046]), ([4450, 4453], [18484, 18487]), ([4766, 4770], [11301, 11305]), ([4830, 4836], [20112, 20118]), ([5190, 5193], [13058, 13061]), ([5194, 5198], [14902, 14906]), ([5259, 5264], [14663, 14668]), ([5465, 5469], [12772, 12776]), ([6091, 6095], [13175, 13179]), ([6108, 6113], [11633, 11638]), ([6160, 6165], [14470, 14475]), ([6160, 6165], [15734, 15739]), ([6160, 6165], [16160, 16165]), ([6160, 6165], [19120, 19125]), ([6197, 6200], [18964, 18967]), ([6412, 6416], [17132, 17136]), ([6633, 6637], [10646, 10650]), ([6633, 6637], [12904, 12908]), ([6633, 6637], [16884, 16888]), ([6633, 6637], [18805, 18809]), ([6637, 6641], [17142, 17146]), ([6807, 6812], [17049, 17054]), ([6812, 6817], [18487, 18492]), ([6960, 6962], [19936, 19938]), ([6992, 6996], [16696, 16700]), ([7059, 7063], [12577, 12581]), ([7550, 7556], [20455, 20461]), ([7550, 7556], [20778, 20784]), ([8221, 8226], [20136, 20141]), ([8229, 8233], [10596, 10600]), ([8229, 8233], [20042, 20046]), ([8274, 8279], [17049, 17054]), ([8315, 8320], [17049, 17054]), ([8320, 8325], [18487, 18492]), ([8327, 8331], [17132, 17136]), ([8511, 8517], [16627, 16633]), ([8583, 8586], [19731, 19734]), ([8796, 8800], [18496, 18500]), ([9052, 9053], [19873, 19874]), ([9074, 9079], [10978, 10983]), ([9094, 9099], [20152, 20157]), ([9103, 9105], [15732, 15734]), ([9259, 9263], [12495, 12499]), ([9402, 9405], [19522, 19525]), ([9469, 9471], [17458, 17460]), ([9661, 9664], [18711, 18714]), ([9733, 9741], [13415, 13423]), ([9833, 9836], [19731, 19734]), ([9841, 9847], [20455, 20461]), ([9841, 9847], [20778, 20784]), ([9970, 9974], [18045, 18049]), ([9970, 9974], [20328, 20332]), ([10295, 10299], [18045, 18049]), ([10295, 10299], [20328, 20332]), ([10334, 10340], [17833, 17839]), ([10423, 10433], [13894, 13904])]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat  
        
    def jeu_essai_CB(self):
        """ Claude Bernard
        Insensible à tout / taille bloc pivots 5
        Pas d'assertion d'ordre sur occs_deplaces / inclusion de dep dans d'autres"""
        insertions = [(5090, 5117), (5190, 5193), (5221, 5239), (5272, 5313), (5322, 5417), (5422, 5496), (5504, 5516), (5522, 5623), (5648, 5660), (5667, 5696), (5733, 5736), (5745, 5846), (5851, 5914), (5929, 5932), (5949, 6008), (6091, 6093), (6152, 6161), (6168, 6198), (6273, 6289), (6311, 6312), (6327, 6341), (6351, 6353), (6362, 6461), (6479, 6482), (6536, 6566), (6591, 6878), (6884, 6969), (6984, 6995), (7011, 7040), (7060, 7103), (7128, 7152), (7158, 7193), (7305, 7313), (7393, 7576), (7583, 7672), (7716, 7976), (7981, 8042), (8047, 8195), (8200, 8287), (8311, 8320), (8332, 8339), (8349, 8390), (8407, 8488), (8510, 8563), (8572, 8638), (8643, 8700), (8749, 8982), (8998, 9005), (9094, 9149), (9154, 9173), (9192, 9313), (9339, 9446), (9454, 9469), (9480, 9482), (9508, 9515), (9522, 9540), (9546, 9560), (9674, 9680), (9691, 9886), (9917, 9975), (9984, 10039), (10044, 10244), (10253, 10301), (10347, 10408), (10437, 10544), (10568, 10584), (10592, 10659), (10686, 10689), (10695, 10701), (10744, 10762), (10790, 10800), (10839, 10843), (10913, 10947), (10963, 11001), (11011, 11045), (11092, 11134), (11186, 11328), (11342, 11354), (11372, 11377), (11387, 11390), (11397, 11443), (11446, 11461), (11476, 11479), (11485, 11502), (11523, 11534), (11541, 11565), (11590, 11608), (11713, 11732), (11741, 11749), (11755, 11771), (11776, 11781), (11788, 11802), (11851, 11868), (11874, 11890), (11907, 11920), (11936, 11940), (12015, 12043), (12049, 12052), (12095, 12160), (12175, 12303), (12340, 12341), (12369, 12405), (12431, 12443), (12478, 12501), (12593, 12635), (12643, 12646), (12663, 12768), (12775, 13649)]
        suppressions =  [(19, 123), (261, 292), (297, 329), (338, 342), (348, 392), (417, 484), (529, 539), (548, 566), (571, 587), (613, 614), (630, 652), (699, 701), (863, 997), (1033, 1035), (1044, 1059), (1068, 1101), (1212, 1238), (1253, 1258), (1273, 1282), (1293, 1316), (1325, 1336), (1366, 1381), (1402, 1424), (1501, 1510), (1609, 1632), (1661, 1808), (1813, 1946), (1956, 1971), (1977, 1989), (2007, 2025), (2059, 2120), (2132, 2144), (2208, 2215), (2224, 2232), (2237, 2238), (2362, 2422), (2460, 2473), (2478, 2479), (2529, 2575), (2595, 2621), (2649, 2681), (2688, 2761), (2801, 2919), (2952, 2967), (3017, 3054), (3072, 3074), (3112, 3122), (3131, 3142), (3147, 3179), (3226, 3236), (3256, 3294), (3303, 3333), (3366, 3369), (3444, 3451), (3611, 3624), (3634, 3715), (3762, 3771), (3780, 3824), (3901, 3923), (3941, 3957), (3976, 4039), (4045, 4144), (4172, 4182), (4317, 4318), (4378, 4382), (4399, 4402), (4418, 4431), (4501, 4503), (4558, 4587), (4602, 4609), (4646, 4654), (4712, 4717), (4722, 4805), (4856, 4864), (4920, 4934), (4968, 5011), (5028, 5044), (5050, 5090)]
        occs_deplaces = [(0, 19), (19, 26), (31, 38), (38, 51), (57, 69), (69, 72), (72, 75), (90, 93), (107, 112), (112, 116), (116, 123), (154, 160), (160, 167), (167, 170), (182, 187), (199, 206), (199, 200), (235, 239), (238, 243), (243, 249), (249, 252), (249, 252), (261, 262), (298, 307), (303, 308), (366, 375), (375, 386), (417, 424), (457, 461), (461, 471), (478, 481), (481, 484), (529, 533), (548, 556), (581, 587), (592, 603), (630, 631), (686, 690), (775, 782), (887, 890), (890, 915), (915, 928), (934, 944), (964, 997), (1049, 1054), (1118, 1128), (1305, 1316), (1331, 1334), (1371, 1373), (1376, 1381), (1414, 1424), (1594, 1602), (1624, 1629), (1638, 1639), (1661, 1664), (1695, 1700), (1728, 1734), (1780, 1788), (1813, 1823), (1876, 1880), (1880, 1882), (1895, 1900), (1900, 1911), (1919, 1922), (1925, 1931), (1943, 1946), (1980, 1985), (1994, 2002), (2046, 2051), (2051, 2059), (2059, 2065), (2059, 2064), (2070, 2076), (2083, 2088), (2085, 2089), (2091, 2097), (2105, 2110), (2115, 2120), (2154, 2161), (2183, 2193), (2225, 2232), (2247, 2252), (2312, 2317), (2312, 2326), (2327, 2332), (2332, 2341), (2375, 2380), (2380, 2384), (2382, 2386), (2397, 2405), (2405, 2411), (2411, 2418), (2434, 2438), (2434, 2438), (2438, 2446), (2457, 2461), (2460, 2464), (2535, 2545), (2562, 2566), (2568, 2571), (2583, 2584), (2599, 2621), (2694, 2699), (2708, 2716), (2719, 2722), (2722, 2727), (2739, 2744), (2758, 2761), (2837, 2845), (2849, 2856), (2856, 2859), (2859, 2868), (2868, 2873), (2883, 2886), (2886, 2890), (2890, 2894), (2925, 2935), (2956, 2959), (2963, 2967), (2995, 3002), (3032, 3038), (3038, 3051), (3051, 3054), (3083, 3086), (3086, 3090), (3094, 3099), (3115, 3119), (3115, 3120), (3119, 3122), (3133, 3142), (3134, 3142), (3158, 3166), (3172, 3179), (3173, 3179), (3208, 3212), (3233, 3236), (3261, 3266), (3267, 3270), (3274, 3275), (3279, 3281), (3313, 3317), (3317, 3324), (3324, 3330), (3456, 3458), (3514, 3519), (3590, 3596), (3611, 3613), (3614, 3621), (3621, 3624), (3643, 3647), (3658, 3663), (3660, 3664), (3663, 3668), (3668, 3679), (3692, 3694), (3694, 3705), (3704, 3715), (3820, 3824), (3909, 3921), (3911, 3921), (3976, 3977), (3981, 3987), (3986, 3991), (4045, 4046), (4057, 4063), (4077, 4080), (4122, 4133), (4207, 4211), (4233, 4235), (4251, 4262), (4273, 4279), (4422, 4428), (4509, 4510), (4510, 4515), (4558, 4563), (4576, 4587), (4636, 4639), (4646, 4650), (4652, 4654), (4659, 4666), (4688, 4699), (4741, 4747), (4747, 4764), (4764, 4772), (4772, 4773), (4783, 4788), (4788, 4805), (4856, 4857), (4870, 4880), (4870, 4880), (4880, 4887), (4880, 4884), (4975, 4982), (5001, 5011), (5050, 5060), (5069, 5076), (5106, 5109), (5139, 5158), (5166, 5170), (5178, 5181), (5198, 5205), (5224, 5230), (5226, 5231), (5236, 5239), (5264, 5267), (5276, 5285), (5375, 5380), (5450, 5457), (5450, 5457), (5463, 5468), (5471, 5474), (5504, 5505), (5525, 5535), (5545, 5557), (5557, 5567), (5591, 5616), (5616, 5623), (5616, 5623), (5651, 5660), (5667, 5670), (5686, 5692), (5750, 5774), (5774, 5777), (5784, 5792), (5881, 5894), (5929, 5930), (5938, 5939), (6055, 6059), (6065, 6066), (6104, 6106), (6145, 6152), (6152, 6155), (6183, 6188), (6188, 6198), (6371, 6377), (6407, 6409), (6426, 6429), (6439, 6449), (6541, 6566), (6596, 6601), (6631, 6644), (6651, 6657), (6657, 6662), (6677, 6681), (6692, 6697), (6714, 6718), (6752, 6756), (6807, 6815), (6849, 6855), (6855, 6861), (6930, 6942), (6942, 6948), (6951, 6954), (6955, 6958), (7060, 7062), (7065, 7090), (7090, 7095), (7198, 7208), (7308, 7313), (7404, 7409), (7482, 7487), (7492, 7497), (7539, 7544), (7544, 7555), (7573, 7576), (7625, 7629), (7643, 7647), (7735, 7738), (7739, 7742), (7764, 7767), (7822, 7828), (7852, 7877), (7938, 7945), (8048, 8052), (8052, 8059), (8083, 8092), (8087, 8092), (8094, 8097), (8097, 8101), (8110, 8111), (8136, 8141), (8177, 8181), (8256, 8261), (8271, 8275), (8291, 8296), (8367, 8385), (8385, 8390), (8448, 8454), (8513, 8519), (8551, 8558), (8664, 8668), (8677, 8683), (8705, 8709), (8752, 8760), (8781, 8784), (8789, 8795), (8905, 8910), (8951, 8955), (9034, 9040), (9080, 9084), (9106, 9112), (9112, 9116), (9116, 9121), (9125, 9133), (9133, 9137), (9145, 9149), (9157, 9164), (9196, 9208), (9250, 9255), (9258, 9266), (9279, 9284), (9284, 9288), (9288, 9299), (9310, 9313), (9370, 9376), (9408, 9416), (9421, 9427), (9537, 9540), (9553, 9560), (9553, 9560), (9626, 9633), (9666, 9667), (9724, 9727), (9737, 9742), (9751, 9757), (9827, 9834), (9834, 9839), (9857, 9864), (9874, 9882), (9966, 9972), (9999, 10021), (10021, 10026), (10049, 10057), (10121, 10126), (10153, 10160), (10160, 10165), (10200, 10205), (10212, 10214), (10232, 10237), (10266, 10285), (10384, 10390), (10439, 10446), (10446, 10452), (10614, 10616), (10619, 10625), (10652, 10659), (10688, 10689), (10712, 10714), (10866, 10868), (10897, 10903), (10913, 10917), (10931, 10942), (10986, 10989), (11011, 11013), (11094, 11106), (11109, 11112), (11112, 11124), (11187, 11201), (11241, 11246), (11246, 11256), (11261, 11266), (11266, 11272), (11272, 11305), (11317, 11327), (11317, 11328), (11400, 11409), (11408, 11413), (11434, 11439), (11450, 11461), (11530, 11534), (11664, 11671), (11811, 11816), (11962, 11968), (11969, 11981), (12015, 12023), (12095, 12101), (12140, 12145), (12145, 12154), (12186, 12196), (12214, 12218), (12218, 12222), (12225, 12228), (12243, 12252), (12252, 12266), (12269, 12281), (12281, 12288), (12288, 12291), (12292, 12295), (12329, 12333), (12377, 12385), (12402, 12405), (12418, 12426), (12478, 12488), (12517, 12521), (12517, 12521), (12530, 12533), (12536, 12539), (12544, 12549), (12593, 12600), (12613, 12616), (12618, 12624), (12663, 12675), (12702, 12719), (12743, 12755), (12777, 12781), (12839, 12846), (12846, 12851), (12905, 12912), (12947, 12950), (12955, 12960), (13022, 13028), (13028, 13035), (13047, 13052), (13081, 13085), (13122, 13123), (13161, 13166), (13231, 13235), (13256, 13259), (13325, 13329), (13332, 13337), (13344, 13346), (13493, 13501), (13514, 13518), (13518, 13529), (13600, 13603), (13605, 13612), (13612, 13616)]
        tous_remplacements = [(144, 170), (182, 191), (216, 231), (679, 690), (707, 721), (734, 736), (1006, 1008), (1486, 1488), (1561, 1564), (2029, 2032), (2244, 2252), (2312, 2357), (2438, 2448), (2505, 2525), (2629, 2632), (2983, 3010), (3086, 3099), (3200, 3221), (3340, 3345), (3412, 3414), (3501, 3509), (3533, 3542), (3552, 3555), (3568, 3580), (3847, 3861), (4228, 4235), (4251, 4279), (4330, 4337), (4441, 4467), (4880, 4910), (5139, 5181), (5196, 5214), (5249, 5267), (6035, 6046), (6065, 6088), (6103, 6106), (6298, 6301), (7286, 7288), (7360, 7362), (8291, 8297), (8705, 8720), (9020, 9051), (9068, 9085), (9319, 9335), (9490, 9492), (9640, 9659), (9895, 9904), (10313, 10342), (10551, 10555), (10711, 10714), (10821, 10834), (10857, 10868), (10878, 10881), (10894, 10903), (11167, 11176), (11625, 11633), (11649, 11695), (11806, 11816), (11950, 11981), (12526, 12557)]
        lg_texte1 = 5090
        texte_original = self._readFile("CB_experience.txt") + self._readFile("CB_Texte_de_1857.txt")     
        blocsCommuns = [(123, 144), (170, 179), (179, 182), (191, 199), (206, 216), (231, 235), (252, 261), (292, 297), (329, 338), (342, 348), (392, 417), (484, 492), (492, 507), (507, 510), (510, 529), (539, 548), (566, 571), (587, 592), (603, 613), (614, 620), (620, 630), (652, 679), (690, 699), (701, 707), (721, 724), (724, 734), (736, 742), (742, 775), (782, 788), (788, 863), (997, 1006), (1008, 1018), (1018, 1033), (1035, 1044), (1059, 1068), (1101, 1118), (1128, 1173), (1173, 1181), (1181, 1206), (1206, 1212), (1238, 1253), (1258, 1273), (1282, 1293), (1316, 1325), (1336, 1360), (1360, 1366), (1381, 1387), (1387, 1402), (1424, 1486), (1488, 1501), (1510, 1514), (1514, 1552), (1552, 1561), (1564, 1572), (1572, 1594), (1602, 1609), (1632, 1638), (1639, 1661), (1808, 1813), (1946, 1956), (1971, 1977), (1989, 1994), (2002, 2007), (2025, 2029), (2032, 2046), (2120, 2127), (2127, 2132), (2144, 2154), (2161, 2177), (2177, 2183), (2193, 2208), (2215, 2224), (2232, 2237), (2238, 2244), (2252, 2281), (2281, 2288), (2288, 2297), (2297, 2312), (2357, 2362), (2422, 2434), (2448, 2457), (2473, 2478), (2479, 2499), (2499, 2505), (2525, 2529), (2575, 2583), (2584, 2595), (2621, 2629), (2632, 2649), (2681, 2688), (2761, 2768), (2768, 2801), (2919, 2925), (2935, 2952), (2967, 2976), (2976, 2983), (3010, 3017), (3054, 3061), (3061, 3072), (3074, 3083), (3099, 3112), (3122, 3131), (3142, 3147), (3179, 3188), (3188, 3200), (3221, 3226), (3236, 3256), (3294, 3303), (3333, 3340), (3345, 3358), (3358, 3366), (3369, 3396), (3396, 3402), (3402, 3412), (3414, 3420), (3420, 3437), (3437, 3444), (3451, 3456), (3458, 3480), (3480, 3501), (3509, 3514), (3519, 3533), (3542, 3552), (3555, 3563), (3563, 3568), (3580, 3590), (3596, 3611), (3624, 3634), (3715, 3762), (3771, 3780), (3824, 3847), (3861, 3871), (3871, 3885), (3885, 3901), (3923, 3933), (3933, 3941), (3957, 3960), (3960, 3976), (4039, 4045), (4144, 4165), (4165, 4172), (4182, 4207), (4211, 4223), (4223, 4228), (4235, 4251), (4279, 4297), (4297, 4306), (4306, 4312), (4312, 4317), (4318, 4326), (4326, 4330), (4337, 4372), (4372, 4378), (4382, 4399), (4402, 4418), (4431, 4441), (4467, 4501), (4503, 4509), (4515, 4558), (4587, 4602), (4609, 4636), (4639, 4646), (4654, 4659), (4666, 4688), (4699, 4712), (4717, 4722), (4805, 4840), (4840, 4856), (4864, 4870), (4910, 4920), (4934, 4961), (4961, 4968), (5011, 5028), (5044, 5050), (5117, 5139), (5181, 5190), (5193, 5196), (5214, 5221), (5239, 5249), (5267, 5272), (5313, 5322), (5417, 5422), (5496, 5504), (5516, 5522), (5623, 5648), (5660, 5667), (5696, 5711), (5711, 5714), (5714, 5733), (5736, 5745), (5846, 5851), (5914, 5919), (5919, 5929), (5932, 5938), (5939, 5949), (6008, 6035), (6046, 6055), (6059, 6065), (6088, 6091), (6093, 6103), (6106, 6112), (6112, 6145), (6161, 6168), (6198, 6273), (6289, 6298), (6301, 6311), (6312, 6327), (6341, 6351), (6353, 6362), (6461, 6479), (6482, 6527), (6527, 6536), (6566, 6591), (6878, 6884), (6969, 6984), (6995, 7011), (7040, 7051), (7051, 7060), (7103, 7128), (7152, 7158), (7193, 7198), (7208, 7223), (7223, 7286), (7288, 7301), (7301, 7305), (7313, 7351), (7351, 7360), (7362, 7371), (7371, 7393), (7576, 7583), (7672, 7678), (7678, 7701), (7701, 7706), (7706, 7716), (7976, 7981), (8042, 8047), (8195, 8200), (8287, 8291), (8297, 8311), (8320, 8327), (8327, 8332), (8339, 8349), (8390, 8407), (8488, 8495), (8495, 8510), (8563, 8572), (8638, 8643), (8700, 8705), (8720, 8749), (8982, 8989), (8989, 8998), (9005, 9020), (9051, 9056), (9056, 9068), (9085, 9094), (9149, 9154), (9173, 9192), (9313, 9319), (9335, 9339), (9446, 9454), (9469, 9480), (9482, 9490), (9492, 9508), (9515, 9522), (9540, 9546), (9560, 9593), (9593, 9600), (9600, 9617), (9617, 9626), (9633, 9640), (9659, 9666), (9667, 9674), (9680, 9691), (9886, 9895), (9904, 9917), (9975, 9984), (10039, 10044), (10244, 10253), (10301, 10313), (10342, 10347), (10408, 10429), (10429, 10437), (10544, 10551), (10555, 10568), (10584, 10592), (10659, 10686), (10689, 10695), (10701, 10711), (10714, 10720), (10720, 10737), (10737, 10744), (10762, 10768), (10768, 10790), (10800, 10821), (10834, 10839), (10843, 10857), (10868, 10878), (10881, 10889), (10889, 10894), (10903, 10913), (10947, 10963), (11001, 11011), (11045, 11092), (11134, 11143), (11143, 11167), (11176, 11186), (11328, 11342), (11354, 11372), (11377, 11387), (11390, 11397), (11443, 11446), (11461, 11476), (11479, 11485), (11502, 11523), (11534, 11541), (11565, 11590), (11608, 11620), (11620, 11625), (11633, 11649), (11695, 11713), (11732, 11741), (11749, 11755), (11771, 11776), (11781, 11788), (11802, 11806), (11816, 11851), (11868, 11874), (11890, 11907), (11920, 11936), (11940, 11950), (11981, 12015), (12043, 12049), (12052, 12095), (12160, 12175), (12303, 12329), (12333, 12340), (12341, 12346), (12346, 12369), (12405, 12418), (12426, 12431), (12443, 12478), (12501, 12517), (12521, 12526), (12557, 12567), (12567, 12593), (12635, 12643), (12646, 12663), (12768, 12775)]
        lDepl = [[(0, 19), (5139, 5158)], [(0, 19), (10266, 10285)], [(19, 26), (9827, 9834)], [(19, 26), (10153, 10160)], [(19, 26), (12839, 12846)], [(19, 26), (13605, 13612)], [(31, 38), (7938, 7945)], [(31, 38), (8052, 8059)], [(38, 51), (12252, 12266)], [(57, 69), (6930, 6942)], [(57, 69), (12269, 12281)], [(69, 72), (5178, 5181)], [(72, 75), (6951, 6954)], [(72, 75), (12288, 12291)], [(90, 93), (7573, 7576)], [(90, 93), (12613, 12616)], [(107, 112), (6657, 6662)], [(107, 112), (7822, 7828)], [(112, 116), (6055, 6059)], [(112, 116), (7643, 7647)], [(112, 116), (8177, 8181)], [(112, 116), (8951, 8955)], [(116, 123), (8448, 8454)], [(116, 123), (9857, 9864)], [(116, 123), (9966, 9972)], [(116, 123), (13022, 13028)], [(154, 160), (10384, 10390)], [(160, 167), (5198, 5205)], [(167, 170), (8781, 8784)], [(182, 187), (5463, 5468)], [(199, 206), (5450, 5457)], [(199, 206), (5450, 5457)], [(199, 206), (10652, 10659)], [(199, 200), (10688, 10689)], [(235, 239), (5375, 5380)], [(235, 239), (9080, 9084)], [(238, 243), (7090, 7095)], [(243, 249), (6855, 6861)], [(249, 252), (5264, 5267)], [(249, 252), (5471, 5474)], [(249, 252), (9310, 9313)], [(249, 252), (5264, 5267)], [(249, 252), (5471, 5474)], [(249, 252), (9310, 9313)], [(261, 262), (5504, 5505)], [(298, 307), (12145, 12154)], [(303, 308), (5226, 5231)], [(366, 375), (5276, 5285)], [(366, 375), (11400, 11409)], [(375, 386), (11112, 11124)], [(417, 424), (5616, 5623)], [(417, 424), (5616, 5623)], [(457, 461), (13325, 13329)], [(461, 471), (5525, 5535)], [(461, 471), (5557, 5567)], [(478, 481), (5106, 5109)], [(478, 481), (7764, 7767)], [(481, 484), (6152, 6155)], [(481, 484), (11109, 11112)], [(529, 533), (13081, 13085)], [(548, 556), (6145, 6152)], [(581, 587), (8677, 8683)], [(592, 603), (12663, 12675)], [(630, 631), (5929, 5930)], [(686, 690), (11530, 11534)], [(775, 782), (10897, 10903)], [(887, 890), (9724, 9727)], [(890, 915), (5591, 5616)], [(890, 915), (5750, 5774)], [(890, 915), (6541, 6566)], [(890, 915), (7065, 7090)], [(890, 915), (7852, 7877)], [(915, 928), (6631, 6644)], [(934, 944), (6439, 6449)], [(964, 997), (11272, 11305)], [(1049, 1054), (6692, 6697)], [(1118, 1128), (6188, 6198)], [(1118, 1128), (11246, 11256)], [(1305, 1316), (5545, 5557)], [(1331, 1334), (5236, 5239)], [(1331, 1334), (10986, 10989)], [(1371, 1373), (7060, 7062)], [(1371, 1373), (10212, 10214)], [(1371, 1373), (13344, 13346)], [(1376, 1381), (11530, 11534)], [(1414, 1424), (7544, 7555)], [(1594, 1602), (12377, 12385)], [(1624, 1629), (7404, 7409)], [(1624, 1629), (7492, 7497)], [(1638, 1639), (6065, 6066)], [(1638, 1639), (11011, 11013)], [(1661, 1664), (7735, 7738)], [(1695, 1700), (7482, 7487)], [(1728, 1734), (9250, 9255)], [(1780, 1788), (9125, 9133)], [(1813, 1823), (7198, 7208)], [(1876, 1880), (9834, 9839)], [(1876, 1880), (10160, 10165)], [(1876, 1880), (12846, 12851)], [(1876, 1880), (13612, 13616)], [(1880, 1882), (7060, 7062)], [(1880, 1882), (10212, 10214)], [(1880, 1882), (13344, 13346)], [(1895, 1900), (6657, 6662)], [(1895, 1900), (7822, 7828)], [(1900, 1911), (9196, 9208)], [(1900, 1911), (11969, 11981)], [(1900, 1911), (13518, 13529)], [(1919, 1922), (5774, 5777)], [(1925, 1931), (10619, 10625)], [(1943, 1946), (6955, 6958)], [(1943, 1946), (12292, 12295)], [(1980, 1985), (10121, 10126)], [(1994, 2002), (12418, 12426)], [(2046, 2051), (6596, 6601)], [(2046, 2051), (7539, 7544)], [(2046, 2051), (11261, 11266)], [(2051, 2059), (10049, 10057)], [(2059, 2065), (9751, 9757)], [(2059, 2064), (8087, 8092)], [(2059, 2064), (8291, 8296)], [(2070, 2076), (5686, 5692)], [(2070, 2076), (8789, 8795)], [(2083, 2088), (8256, 8261)], [(2085, 2089), (6677, 6681)], [(2085, 2089), (8664, 8668)], [(2085, 2089), (9145, 9149)], [(2091, 2097), (6849, 6855)], [(2091, 2097), (6942, 6948)], [(2091, 2097), (8513, 8519)], [(2105, 2110), (12955, 12960)], [(2115, 2120), (12777, 12781)], [(2154, 2161), (12905, 12912)], [(2183, 2193), (7198, 7208)], [(2225, 2232), (5784, 5792)], [(2247, 2252), (7308, 7313)], [(2312, 2317), (9116, 9121)], [(2312, 2317), (10021, 10026)], [(2312, 2317), (13047, 13052)], [(2312, 2326), (11187, 11201)], [(2327, 2332), (6596, 6601)], [(2327, 2332), (7539, 7544)], [(2327, 2332), (11261, 11266)], [(2332, 2341), (8083, 8092)], [(2375, 2380), (9106, 9112)], [(2380, 2384), (9284, 9288)], [(2382, 2386), (9112, 9116)], [(2397, 2405), (8752, 8760)], [(2405, 2411), (12095, 12101)], [(2411, 2418), (10439, 10446)], [(2434, 2438), (9133, 9137)], [(2434, 2438), (12329, 12333)], [(2434, 2438), (9133, 9137)], [(2434, 2438), (12329, 12333)], [(2438, 2446), (9125, 9133)], [(2457, 2461), (5375, 5380)], [(2457, 2461), (9080, 9084)], [(2460, 2464), (13231, 13235)], [(2535, 2545), (12186, 12196)], [(2562, 2566), (8271, 8275)], [(2568, 2571), (12536, 12539)], [(2583, 2584), (5504, 5505)], [(2599, 2621), (9999, 10021)], [(2694, 2699), (9553, 9560)], [(2694, 2699), (9553, 9560)], [(2708, 2716), (6807, 6815)], [(2708, 2716), (12243, 12252)], [(2719, 2722), (7739, 7742)], [(2722, 2727), (10232, 10237)], [(2722, 2727), (13161, 13166)], [(2739, 2744), (9553, 9560)], [(2739, 2744), (9553, 9560)], [(2758, 2761), (5264, 5267)], [(2758, 2761), (5471, 5474)], [(2758, 2761), (9310, 9313)], [(2837, 2845), (9258, 9266)], [(2837, 2845), (12015, 12023)], [(2849, 2856), (11664, 11671)], [(2856, 2859), (13600, 13603)], [(2859, 2868), (5651, 5660)], [(2868, 2873), (8385, 8390)], [(2883, 2886), (12530, 12533)], [(2886, 2890), (8048, 8052)], [(2890, 2894), (6714, 6718)], [(2890, 2894), (13514, 13518)], [(2925, 2935), (7198, 7208)], [(2956, 2959), (8094, 8097)], [(2963, 2967), (13081, 13085)], [(2995, 3002), (13028, 13035)], [(3032, 3038), (9034, 9040)], [(3038, 3051), (5881, 5894)], [(3051, 3054), (5264, 5267)], [(3051, 3054), (5471, 5474)], [(3051, 3054), (9310, 9313)], [(3083, 3086), (13600, 13603)], [(3086, 3090), (5166, 5170)], [(3094, 3099), (11530, 11534)], [(3115, 3119), (7625, 7629)], [(3115, 3119), (8136, 8141)], [(3115, 3119), (8905, 8910)], [(3115, 3120), (9737, 9742)], [(3119, 3122), (6152, 6155)], [(3119, 3122), (11109, 11112)], [(3133, 3142), (9874, 9882)], [(3134, 3142), (9626, 9633)], [(3134, 3142), (12593, 12600)], [(3158, 3166), (13493, 13501)], [(3172, 3179), (8448, 8454)], [(3172, 3179), (9857, 9864)], [(3172, 3179), (9966, 9972)], [(3172, 3179), (13022, 13028)], [(3173, 3179), (11811, 11816)], [(3208, 3212), (12214, 12218)], [(3233, 3236), (12947, 12950)], [(3261, 3266), (12544, 12549)], [(3267, 3270), (9537, 9540)], [(3267, 3270), (12402, 12405)], [(3274, 3275), (8110, 8111)], [(3279, 3281), (6104, 6106)], [(3279, 3281), (10712, 10714)], [(3313, 3317), (6426, 6429)], [(3317, 3324), (5450, 5457)], [(3317, 3324), (5450, 5457)], [(3317, 3324), (10652, 10659)], [(3324, 3330), (6855, 6861)], [(3456, 3458), (5938, 5939)], [(3456, 3458), (9666, 9667)], [(3456, 3458), (13122, 13123)], [(3514, 3519), (6752, 6756)], [(3590, 3596), (9370, 9376)], [(3590, 3596), (9421, 9427)], [(3611, 3613), (10866, 10868)], [(3614, 3621), (10897, 10903)], [(3621, 3624), (5667, 5670)], [(3643, 3647), (12218, 12222)], [(3658, 3663), (9279, 9284)], [(3660, 3664), (10913, 10917)], [(3663, 3668), (10200, 10205)], [(3663, 3668), (13332, 13337)], [(3668, 3679), (12743, 12755)], [(3692, 3694), (6407, 6409)], [(3694, 3705), (9288, 9299)], [(3704, 3715), (10931, 10942)], [(3704, 3715), (11317, 11328)], [(3704, 3715), (11450, 11461)], [(3820, 3824), (13256, 13259)], [(3909, 3921), (11094, 11106)], [(3911, 3921), (11317, 11327)], [(3976, 3977), (6065, 6066)], [(3976, 3977), (11011, 11013)], [(3981, 3987), (11266, 11272)], [(3986, 3991), (11408, 11413)], [(4045, 4046), (5929, 5930)], [(4057, 4063), (6371, 6377)], [(4077, 4080), (8097, 8101)], [(4122, 4133), (10931, 10942)], [(4122, 4133), (11317, 11328)], [(4122, 4133), (11450, 11461)], [(4207, 4211), (8705, 8709)], [(4207, 4211), (12517, 12521)], [(4207, 4211), (12517, 12521)], [(4233, 4235), (10614, 10616)], [(4251, 4262), (10931, 10942)], [(4251, 4262), (11317, 11328)], [(4251, 4262), (11450, 11461)], [(4273, 4279), (5224, 5230)], [(4422, 4428), (11962, 11968)], [(4509, 4510), (5938, 5939)], [(4509, 4510), (9666, 9667)], [(4509, 4510), (13122, 13123)], [(4510, 4515), (12140, 12145)], [(4558, 4563), (11434, 11439)], [(4576, 4587), (10931, 10942)], [(4576, 4587), (11317, 11328)], [(4576, 4587), (11450, 11461)], [(4636, 4639), (5264, 5267)], [(4636, 4639), (5471, 5474)], [(4636, 4639), (9310, 9313)], [(4646, 4650), (13231, 13235)], [(4652, 4654), (12225, 12228)], [(4659, 4666), (9626, 9633)], [(4659, 4666), (12593, 12600)], [(4688, 4699), (10931, 10942)], [(4688, 4699), (11317, 11328)], [(4688, 4699), (11450, 11461)], [(4741, 4747), (6651, 6657)], [(4741, 4747), (10446, 10452)], [(4747, 4764), (12702, 12719)], [(4764, 4772), (9408, 9416)], [(4772, 4773), (5938, 5939)], [(4772, 4773), (9666, 9667)], [(4772, 4773), (13122, 13123)], [(4783, 4788), (6183, 6188)], [(4783, 4788), (11241, 11246)], [(4788, 4805), (8367, 8385)], [(4856, 4857), (5938, 5939)], [(4856, 4857), (9666, 9667)], [(4856, 4857), (13122, 13123)], [(4870, 4880), (12478, 12488)], [(4870, 4880), (12478, 12488)], [(4880, 4887), (8551, 8558)], [(4880, 4887), (12281, 12288)], [(4880, 4884), (8705, 8709)], [(4880, 4884), (12517, 12521)], [(4880, 4884), (12517, 12521)], [(4975, 4982), (12618, 12624)], [(5001, 5011), (7198, 7208)], [(5050, 5060), (7198, 7208)], [(5069, 5076), (9157, 9164)]]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat      
    def jeu_essai_Python(self):
        """ Code Python
        Insensible à tout / taille bloc pivots 5
        Pas d'assertion d'ordre sur occs_deplaces"""
        insertions = [(5945, 5968), (6016, 9524), (9558, 9575), (9825, 9924), (9993, 10121), (10155, 10191), (10602, 10664), (10670, 10673), (10684, 10685), (10817, 10846), (10885, 11037), (11143, 11145), (11150, 11170), (11299, 11340), (11453, 11454), (11487, 11581), (11645, 11907), (12004, 12074), (12152, 12190), (12277, 12309), (12465, 12475), (12495, 12537), (12547, 12549), (12556, 12724), (12778, 12910), (12938, 12966), (12973, 12983), (12990, 13028), (13172, 13223), (13335, 13855), (13882, 13886), (13904, 13917), (13945, 14105), (14152, 14154), (14161, 14189), (14219, 14223), (14231, 14278), (14376, 14440), (14466, 14470), (14521, 14670), (14708, 15053), (15143, 15144), (16170, 16218), (16771, 16789), (17046, 17141), (17169, 17868), (17924, 18671), (18699, 18815), (18893, 19155), (19184, 19317), (19423, 19465), (19513, 19514), (19521, 19625), (19630, 19655), (19666, 19725), (19748, 19815), (19822, 19847), (19850, 19917), (19937, 20020), (20025, 20028), (20048, 20111), (20179, 20196), (20219, 26170)]
        suppressions =  [(315, 462), (536, 537), (1069, 1070), (1608, 1632), (1637, 1659), (1678, 1679), (2157, 2167), (2187, 2246), (2264, 2285), (2384, 2389), (2396, 2422), (2429, 2457), (2752, 2773), (2854, 2885), (2923, 2953), (3049, 3073), (5014, 5044), (5480, 5542), (5547, 5575), (5624, 5626), (5633, 5650), (5660, 5682), (5687, 5713), (5730, 5770), (5838, 5850), (5873, 5895)]
        occs_deplaces = [(315, 322), (322, 329), (329, 462), (1678, 1679), (2162, 2167), (2203, 2210), (2210, 2216), (2268, 2285), (2854, 2865), (2865, 2885), (3056, 3073), (5536, 5541), (5554, 5575), (5633, 5639), (5639, 5647), (5647, 5650), (5691, 5695), (5695, 5700), (5707, 5713), (5732, 5751), (5753, 5755), (5838, 5850), (5873, 5895), (6094, 6098), (6112, 6118), (6211, 6233), (7189, 7194), (7500, 7504), (7599, 7716), (7768, 7776), (7902, 7921), (7928, 7936), (8022, 8026), (8198, 8203), (8403, 8408), (8496, 8500), (8757, 8762), (8810, 8815), (12285, 12288), (12600, 12605), (12625, 12626), (12894, 12901), (12973, 12978), (13991, 14003), (14014, 14021), (14073, 14085), (14094, 14101), (17073, 17081), (17427, 17429), (17562, 17564), (18403, 18406), (18488, 18492), (18573, 18578), (18607, 18612), (19304, 19317), (19782, 19789), (20284, 20292), (20685, 20700), (21155, 21175), (21243, 21245), (21485, 21493), (21546, 21551), (21557, 21563), (21807, 21812), (22190, 22195), (22215, 22220), (22275, 22280), (22454, 22456), (22503, 22508), (22523, 22533), (22539, 22544), (22654, 22659), (22840, 22851), (23331, 23334), (23832, 23840), (23946, 23948), (24088, 24096), (24173, 24175), (24312, 24320), (24860, 24866), (25162, 25167), (25190, 25195), (25222, 25227)]
        tous_remplacements = []
        lg_texte1 = 5895
        texte_original = self._readFile("Python_synt_old_2.py") + self._readFile("Python_synthetic.py")     
        blocsCommuns = [(0, 50), (50, 99), (99, 139), (139, 315), (462, 536), (537, 606), (606, 641), (641, 1052), (1052, 1058), (1058, 1069), (1070, 1202), (1202, 1241), (1241, 1347), (1347, 1366), (1366, 1495), (1495, 1608), (1632, 1637), (1659, 1678), (1679, 1742), (1742, 1840), (1840, 1918), (1918, 2005), (2005, 2152), (2152, 2157), (2167, 2187), (2246, 2257), (2257, 2264), (2285, 2339), (2339, 2377), (2377, 2384), (2389, 2396), (2422, 2429), (2457, 2522), (2522, 2595), (2595, 2707), (2707, 2734), (2734, 2752), (2773, 2801), (2801, 2848), (2848, 2854), (2885, 2915), (2915, 2923), (2953, 3049), (3073, 3099), (3099, 3150), (3150, 3188), (3188, 3278), (3278, 4246), (4246, 4757), (4757, 5008), (5008, 5014), (5044, 5084), (5084, 5154), (5154, 5205), (5205, 5290), (5290, 5300), (5300, 5319), (5319, 5425), (5425, 5473), (5473, 5480), (5542, 5547), (5575, 5602), (5602, 5624), (5626, 5633), (5650, 5653), (5653, 5660), (5682, 5687), (5713, 5730), (5770, 5838), (5850, 5873), (5895, 5945), (5968, 6016), (9524, 9558), (9575, 9751), (9751, 9825), (9924, 9993), (10121, 10155), (10191, 10602), (10664, 10670), (10673, 10684), (10685, 10817), (10846, 10885), (11037, 11143), (11145, 11150), (11170, 11299), (11340, 11453), (11454, 11468), (11468, 11487), (11581, 11645), (11907, 12004), (12074, 12152), (12190, 12277), (12309, 12460), (12460, 12465), (12475, 12495), (12537, 12547), (12549, 12556), (12724, 12778), (12910, 12938), (12966, 12973), (12983, 12990), (13028, 13035), (13035, 13099), (13099, 13172), (13223, 13335), (13855, 13882), (13886, 13904), (13917, 13945), (14105, 14152), (14154, 14161), (14189, 14219), (14223, 14231), (14278, 14376), (14440, 14466), (14470, 14521), (14670, 14708), (15053, 15143), (15144, 16170), (16218, 16771), (16789, 17040), (17040, 17046), (17141, 17169), (17868, 17924), (18671, 18699), (18815, 18893), (19155, 19165), (19165, 19184), (19317, 19423), (19465, 19513), (19514, 19521), (19625, 19630), (19655, 19666), (19725, 19748), (19815, 19822), (19847, 19850), (19917, 19937), (20020, 20025), (20028, 20048), (20111, 20179), (20196, 20219)]
        lDepl = [[(315, 322), (6211, 6233)], [(315, 322), (19782, 19789)], [(322, 329), (12894, 12901)], [(329, 462), (7599, 7716)], [(1678, 1679), (12625, 12626)], [(2162, 2167), (12600, 12605)], [(2203, 2210), (14014, 14021)], [(2203, 2210), (14094, 14101)], [(2210, 2216), (12973, 12978)], [(2268, 2285), (7902, 7921)], [(2854, 2865), (13991, 14003)], [(2854, 2865), (14073, 14085)], [(2865, 2885), (12285, 12288)], [(3056, 3073), (7189, 7194)], [(5536, 5541), (24860, 24866)], [(5554, 5575), (19304, 19317)], [(5554, 5575), (22454, 22456)], [(5633, 5639), (6112, 6118)], [(5639, 5647), (17073, 17081)], [(5639, 5647), (20284, 20292)], [(5639, 5647), (21155, 21175)], [(5639, 5647), (21485, 21493)], [(5639, 5647), (23832, 23840)], [(5639, 5647), (24088, 24096)], [(5639, 5647), (24312, 24320)], [(5647, 5650), (23331, 23334)], [(5691, 5695), (6094, 6098)], [(5691, 5695), (7500, 7504)], [(5691, 5695), (8022, 8026)], [(5691, 5695), (8496, 8500)], [(5695, 5700), (7768, 7776)], [(5695, 5700), (7928, 7936)], [(5695, 5700), (8198, 8203)], [(5695, 5700), (8403, 8408)], [(5695, 5700), (8757, 8762)], [(5695, 5700), (8810, 8815)], [(5695, 5700), (18573, 18578)], [(5695, 5700), (18607, 18612)], [(5695, 5700), (21546, 21551)], [(5695, 5700), (21807, 21812)], [(5695, 5700), (22190, 22195)], [(5695, 5700), (22215, 22220)], [(5695, 5700), (22275, 22280)], [(5695, 5700), (22503, 22508)], [(5695, 5700), (22539, 22544)], [(5695, 5700), (22654, 22659)], [(5695, 5700), (25162, 25167)], [(5695, 5700), (25190, 25195)], [(5695, 5700), (25222, 25227)], [(5707, 5713), (21557, 21563)], [(5732, 5751), (20685, 20700)], [(5732, 5751), (22523, 22533)], [(5732, 5751), (22840, 22851)], [(5753, 5755), (17427, 17429)], [(5753, 5755), (17562, 17564)], [(5753, 5755), (21243, 21245)], [(5753, 5755), (23946, 23948)], [(5753, 5755), (24173, 24175)], [(5838, 5850), (18403, 18406)], [(5873, 5895), (18488, 18492)]]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat     
    def jeu_essai_Condorcet(self):
        """ Condorcet
        Insensible à tout / taille bloc pivots 5
        Assertion d'ordre sur occs_deplaces vérifiée"""
        insertions = [(13462, 13463), (13485, 13903), (13908, 15344), (15351, 15542), (15562, 16222), (16228, 16330), (16338, 16552), (16563, 16686), (16693, 16843), (16847, 16929), (16933, 17047), (17055, 17066), (17078, 17161), (17166, 17267), (17272, 17377), (17381, 18302), (18305, 18310), (18315, 18561), (18569, 18655), (18659, 18675), (18680, 18827), (18832, 18857), (18864, 18897), (18907, 19010), (19015, 19046), (19052, 19145), (19158, 19364), (19368, 19431), (19436, 19588), (19595, 19613), (19619, 19730), (19735, 20215), (20221, 20469), (20474, 21382), (21387, 21472), (21478, 21546), (21551, 21694), (21699, 21787), (21792, 22071), (22076, 22115), (22120, 22125), (22131, 22229), (22235, 22999), (23004, 23654), (23662, 23796), (23801, 23849), (23855, 24015), (24025, 24084), (24115, 24129), (24137, 24261), (24266, 24272), (24275, 24281), (24285, 24305), (24313, 24361), (24367, 24388), (24391, 24393), (24407, 24808), (24813, 26218), (26223, 26233), (26242, 26308), (26312, 26401), (26509, 26512), (26531, 26558), (26581, 26599), (26606, 26613), (26709, 26717), (26812, 26816), (27023, 27027), (27079, 27084), (27090, 27096), (27101, 27150), (27154, 27191), (27198, 27281), (27287, 27330), (27336, 27342), (27353, 27355), (27364, 27429), (27437, 27519), (27588, 27593), (27670, 27671), (27791, 27793), (27825, 27832), (27888, 27889), (28062, 28075), (28114, 28120), (28194, 28212), (28222, 28274), (28279, 28295), (28302, 28310), (28315, 28360), (28376, 28384), (28392, 28396), (28402, 28405), (28409, 28413), (28423, 28438), (28444, 28479), (28486, 28487), (28489, 28755), (28764, 28841), (28849, 28881), (28888, 29236), (29242, 29435), (29440, 29514), (29721, 29736), (29743, 29757), (29782, 29828), (29840, 29845), (29851, 29859), (29862, 29867), (29881, 29887), (29938, 29947), (29979, 29980), (30015, 30020), (30034, 30156), (30170, 30180), (30328, 30343), (30373, 30382), (30386, 30393), (30577, 30581), (30619, 30665), (30741, 30752), (30754, 30787), (30792, 30800), (30805, 30841), (30875, 30876), (30891, 30915), (30921, 30925), (30930, 30970), (30997, 31005), (31208, 31218), (31251, 31255), (31294, 31296), (31307, 31326), (31341, 31344), (31350, 31371), (31378, 31410), (31561, 31571), (31660, 31681), (31732, 31738), (31742, 31762), (31799, 31805), (31839, 31847), (31908, 31909), (32017, 32021), (32066, 32077), (32087, 32091), (32098, 32100), (32108, 32120), (32135, 32140), (32156, 32167), (32357, 32379), (32418, 32448), (32456, 32460), (32801, 32836), (32843, 32879), (32895, 32904), (32909, 32914), (32919, 32924), (32925, 32987), (32993, 33008), (33026, 33030), (33041, 33086), (33111, 33114), (33123, 33125), (33140, 33143), (33297, 33300), (33472, 33476), (33548, 33565), (33570, 33579), (33635, 33643), (33659, 33662)]
        suppressions =  [(31, 34), (56, 66), (78, 85), (92, 115), (127, 145), (151, 204), (212, 217), (228, 232), (239, 285), (289, 298), (302, 323), (327, 341), (345, 362), (375, 401), (406, 408), (420, 455), (460, 467), (475, 532), (536, 543), (548, 566), (571, 620), (627, 634), (643, 718), (729, 775), (788, 832), (836, 848), (853, 859), (866, 874), (880, 924), (929, 931), (942, 945), (956, 975), (980, 992), (997, 1052), (1057, 1061), (1066, 1080), (1085, 1099), (1111, 1140), (1146, 1164), (1172, 1176), (1181, 1188), (1193, 1208), (1214, 1404), (1435, 1524), (1532, 1632), (1637, 1650), (1653, 1710), (1714, 1890), (1898, 1985), (1991, 2005), (2008, 2070), (2076, 2092), (2100, 2101), (2106, 2108), (2113, 2119), (2128, 2144), (2148, 2173), (2281, 2284), (2304, 2324), (2347, 2379), (2386, 2391), (2486, 2492), (2587, 2591), (2798, 2802), (2860, 2861), (2866, 2875), (2886, 2898), (2904, 2912), (2938, 2987), (2995, 2998), (3068, 3083), (3281, 3283), (3316, 3325), (3384, 3385), (3596, 3603), (3676, 3693), (3703, 3879), (3884, 4157), (4164, 4497), (4502, 4564), (4573, 5052), (5060, 5296), (5304, 5596), (5602, 5746), (5750, 5896), (5906, 6333), (6339, 6456), (6463, 6568), (6570, 6587), (6596, 6766), (6774, 6775), (6782, 6787), (6797, 6810), (7017, 7025), (7032, 7039), (7065, 7099), (7111, 7118), (7127, 7132), (7146, 7152), (7218, 7229), (7283, 7289), (7304, 7315), (7477, 7500), (7535, 7542), (7625, 7630), (7730, 7734), (7839, 8285), (8291, 8298), (8300, 8310), (8315, 8343), (8348, 8402), (8451, 8501), (8507, 8584), (8589, 8594), (8621, 8624), (8632, 8648), (8843, 8872), (8905, 8907), (8947, 8963), (8969, 8996), (9001, 9040), (9048, 9073), (9080, 9090), (9096, 9100), (9107, 9195), (9200, 9236), (9382, 9390), (9479, 9483), (9488, 9511), (9562, 9571), (9608, 9612), (9646, 9653), (9714, 9715), (9721, 9831), (9931, 9947), (9979, 9981), (9994, 9997), (10007, 10015), (10030, 10039), (10054, 10059), (10072, 10077), (10160, 10162), (10259, 10262), (10270, 10289), (10329, 10334), (10342, 10348), (10693, 10719), (10726, 10757), (10773, 10782), (10792, 10806), (10807, 10923), (10929, 10968), (10986, 11003), (11040, 11043), (11052, 11055), (11223, 11242), (11259, 11268), (11420, 11425), (11497, 11511), (11516, 11522), (11577, 11583), (11599, 11602), (11617, 13434)]
        occs_deplaces = [(81, 85), (106, 115), (158, 164), (164, 168), (171, 173), (185, 194), (251, 258), (260, 268), (275, 278), (294, 298), (313, 318), (327, 333), (346, 352), (354, 362), (424, 438), (438, 443), (475, 495), (501, 506), (508, 510), (510, 524), (643, 655), (655, 665), (670, 680), (680, 687), (687, 692), (696, 700), (757, 763), (763, 775), (788, 796), (798, 802), (808, 817), (836, 842), (842, 847), (882, 893), (893, 899), (899, 904), (906, 916), (916, 923), (956, 965), (1003, 1013), (1024, 1030), (1030, 1036), (1037, 1043), (1043, 1048), (1048, 1052), (1079, 1080), (1135, 1140), (1152, 1160), (1172, 1176), (1200, 1205), (1207, 1208), (1254, 1264), (1269, 1273), (1280, 1289), (1289, 1303), (1309, 1314), (1317, 1346), (1348, 1360), (1360, 1401), (1401, 1404), (1440, 1445), (1445, 1448), (1448, 1459), (1463, 1480), (1483, 1487), (1491, 1496), (1496, 1498), (1507, 1512), (1512, 1518), (1520, 1522), (1547, 1551), (1552, 1556), (1592, 1597), (1598, 1606), (1606, 1613), (1637, 1639), (1639, 1645), (1654, 1659), (1659, 1661), (1665, 1669), (1674, 1676), (1687, 1691), (1693, 1699), (1699, 1703), (1714, 1717), (1724, 1729), (1732, 1735), (1738, 1745), (1750, 1760), (1760, 1836), (1836, 1858), (1879, 1885), (1910, 1917), (1918, 1925), (1925, 1931), (1934, 1940), (1940, 1944), (1944, 1971), (1994, 1999), (2053, 2062), (2128, 2133), (2133, 2139), (2139, 2142), (2165, 2173), (2310, 2324), (2866, 2869), (2872, 2875), (2893, 2898), (2947, 2951), (2965, 2970), (2973, 2977), (3068, 3077), (3320, 3325), (3686, 3689), (3714, 3735), (3735, 3744), (3744, 3748), (3752, 3760), (3795, 3803), (3811, 3819), (3819, 3826), (3826, 3836), (3841, 3853), (3855, 3862), (3862, 3865), (3868, 3879), (3891, 3900), (3908, 3913), (3913, 3922), (3922, 3934), (3945, 3950), (3950, 3952), (3963, 3976), (3997, 4004), (4005, 4011), (4017, 4024), (4040, 4045), (4045, 4049), (4057, 4069), (4072, 4088), (4088, 4095), (4100, 4107), (4114, 4118), (4133, 4138), (4141, 4143), (4143, 4148), (4148, 4157), (4171, 4176), (4176, 4179), (4188, 4194), (4197, 4200), (4204, 4210), (4228, 4240), (4240, 4246), (4246, 4253), (4255, 4258), (4271, 4278), (4281, 4289), (4289, 4293), (4296, 4300), (4304, 4309), (4313, 4317), (4329, 4336), (4342, 4364), (4364, 4368), (4375, 4406), (4409, 4433), (4435, 4459), (4459, 4464), (4528, 4535), (4668, 4685), (4696, 4701), (4701, 4707), (4707, 4711), (4713, 4725), (4725, 4737), (4737, 4748), (4748, 4765), (4765, 4768), (4768, 4821), (4826, 4831), (4834, 4857), (4857, 4862), (4862, 4865), (4865, 4873), (4873, 4879), (4888, 4900), (4917, 4924), (4924, 4928), (4940, 4947), (4947, 4957), (4957, 4962), (4962, 4965), (4965, 4978), (4982, 4989), (5003, 5011), (5011, 5018), (5018, 5022), (5027, 5031), (5034, 5048), (5060, 5062), (5073, 5079), (5090, 5096), (5102, 5107), (5114, 5121), (5127, 5137), (5144, 5146), (5146, 5147), (5147, 5154), (5154, 5159), (5178, 5189), (5189, 5195), (5214, 5224), (5237, 5243), (5243, 5247), (5254, 5266), (5271, 5276), (5290, 5295), (5304, 5307), (5313, 5319), (5328, 5329), (5339, 5348), (5359, 5365), (5368, 5369), (5369, 5373), (5392, 5399), (5406, 5408), (5427, 5441), (5441, 5457), (5457, 5460), (5468, 5471), (5471, 5476), (5476, 5486), (5501, 5507), (5507, 5520), (5531, 5538), (5545, 5551), (5551, 5556), (5565, 5569), (5572, 5582), (5586, 5590), (5595, 5596), (5602, 5612), (5629, 5635), (5665, 5676), (5797, 5871), (5879, 5884), (5892, 5896), (5909, 5911), (5911, 5921), (5921, 5927), (5927, 5973), (5973, 5978), (5983, 6028), (6031, 6034), (6036, 6043), (6043, 6051), (6051, 6077), (6077, 6085), (6085, 6092), (6092, 6097), (6102, 6110), (6110, 6115), (6118, 6120), (6123, 6128), (6128, 6136), (6151, 6159), (6191, 6194), (6194, 6203), (6203, 6207), (6217, 6227), (6231, 6236), (6253, 6279), (6279, 6286), (6286, 6288), (6288, 6293), (6293, 6301), (6303, 6308), (6308, 6315), (6324, 6327), (6327, 6332), (6342, 6348), (6348, 6354), (6363, 6369), (6369, 6380), (6380, 6385), (6390, 6393), (6407, 6414), (6414, 6419), (6420, 6426), (6429, 6440), (6440, 6443), (6443, 6446), (6446, 6456), (6463, 6464), (6466, 6475), (6480, 6491), (6491, 6499), (6502, 6509), (6517, 6521), (6521, 6524), (6524, 6529), (6552, 6556), (6558, 6564), (6570, 6575), (6576, 6580), (6580, 6587), (6602, 6612), (6629, 6635), (6640, 6645), (6645, 6650), (6654, 6664), (6667, 6678), (6685, 6688), (6691, 6706), (6706, 6711), (6723, 6729), (6738, 6740), (6748, 6761), (6761, 6766), (6774, 6775), (6785, 6787), (7017, 7023), (7034, 7039), (7074, 7080), (7080, 7084), (7084, 7088), (7111, 7118), (7218, 7224), (7285, 7289), (7479, 7489), (7489, 7495), (7495, 7500), (7625, 7630), (7730, 7734), (7842, 7846), (7870, 7886), (7889, 7896), (7899, 7909), (7910, 7918), (7918, 7923), (7923, 7928), (7934, 7952), (7973, 7978), (7982, 7985), (7999, 8003), (8003, 8011), (8011, 8028), (8036, 8048), (8048, 8052), (8054, 8057), (8057, 8066), (8069, 8101), (8105, 8125), (8127, 8131), (8133, 8139), (8139, 8144), (8144, 8151), (8154, 8183), (8183, 8187), (8187, 8194), (8209, 8216), (8216, 8221), (8228, 8234), (8238, 8245), (8250, 8255), (8268, 8270), (8270, 8278), (8336, 8340), (8357, 8361), (8364, 8368), (8378, 8381), (8381, 8388), (8391, 8397), (8454, 8460), (8466, 8470), (8470, 8476), (8476, 8487), (8487, 8490), (8493, 8499), (8530, 8536), (8536, 8539), (8548, 8553), (8553, 8560), (8572, 8584), (8632, 8635), (8638, 8646), (8849, 8855), (8867, 8868), (8947, 8951), (8951, 8963), (8977, 8981), (8981, 8986), (8990, 8996), (9011, 9022), (9028, 9031), (9031, 9038), (9038, 9040), (9051, 9060), (9061, 9065), (9083, 9087), (9119, 9122), (9135, 9143), (9155, 9159), (9159, 9166), (9166, 9171), (9171, 9181), (9188, 9195), (9200, 9202), (9205, 9210), (9210, 9219), (9226, 9231), (9382, 9390), (9488, 9500), (9500, 9511), (9562, 9566), (9608, 9612), (9648, 9653), (9721, 9732), (9736, 9745), (9745, 9752), (9755, 9763), (9763, 9770), (9770, 9777), (9777, 9791), (9794, 9807), (9807, 9812), (9932, 9941), (9943, 9947), (10007, 10012), (10034, 10039), (10278, 10289), (10342, 10348), (10693, 10705), (10711, 10718), (10732, 10744), (10744, 10750), (10750, 10757), (10773, 10774), (10774, 10780), (10798, 10803), (10810, 10815), (10815, 10819), (10848, 10853), (10853, 10860), (10863, 10869), (10869, 10874), (10888, 10894), (10894, 10900), (10901, 10903), (10936, 10945), (10950, 10955), (10994, 10999), (10999, 11003), (11040, 11043), (11223, 11228), (11236, 11242), (11497, 11502), (11502, 11511), (11516, 11522), (11577, 11580), (11599, 11602), (11622, 11627), (11628, 11636), (11653, 11659), (11659, 11663), (11667, 11682), (11682, 11698), (11701, 11706), (11706, 11712), (11712, 11727), (11798, 11802), (11804, 11808), (11815, 11819), (11819, 11825), (11828, 11834), (11834, 11838), (11851, 11862), (11862, 11866), (11878, 11887), (11890, 11900), (11902, 11913), (11917, 11922), (11922, 11925), (11940, 11952), (11957, 11961), (11968, 11971), (11977, 11983), (11990, 11995), (11995, 12000), (12011, 12017), (12017, 12020), (12027, 12035), (12037, 12039), (12046, 12051), (12051, 12054), (12073, 12082), (12082, 12088), (12090, 12094), (12097, 12102), (12105, 12112), (12115, 12118), (12118, 12123), (12132, 12135), (12135, 12142), (12142, 12150), (12155, 12159), (12159, 12169), (12169, 12173), (12188, 12192), (12198, 12203), (12203, 12206), (12207, 12211), (12218, 12227), (12240, 12244), (12254, 12258), (12274, 12280), (12281, 12285), (12285, 12293), (12293, 12300), (12308, 12313), (12321, 12328), (12328, 12334), (12334, 12340), (12342, 12347), (12347, 12355), (12355, 12359), (12364, 12368), (12368, 12376), (12379, 12385), (12385, 12396), (12396, 12403), (12405, 12409), (12409, 12413), (12419, 12423), (12423, 12428), (12432, 12437), (12449, 12456), (12478, 12483), (12483, 12494), (12504, 12511), (12511, 12518), (12523, 12526), (12526, 12532), (12534, 12543), (12551, 12558), (12558, 12566), (12577, 12586), (12586, 12592), (12594, 12600), (12609, 12611), (12614, 12616), (12616, 12624), (12629, 12632), (12632, 12636), (12640, 12644), (12649, 12652), (12652, 12656), (12658, 12665), (12668, 12675), (12679, 12685), (12695, 12698), (12698, 12702), (12709, 12711), (12711, 12716), (12716, 12720), (12726, 12730), (12732, 12741), (12749, 12754), (12755, 12759), (12759, 12767), (12774, 12780), (12787, 12790), (12790, 12795), (12799, 12811), (12811, 12815), (12815, 12822), (12822, 12830), (12830, 12835), (12855, 12867), (12867, 12871), (12889, 12896), (12896, 12904), (12904, 12912), (12912, 12917), (12917, 12924), (12926, 12931), (12935, 12942), (12942, 12953), (12953, 12957), (12970, 12972), (12978, 12979), (12987, 12992), (13009, 13012), (13012, 13015), (13029, 13040), (13040, 13047), (13047, 13061), (13061, 13071), (13071, 13075), (13075, 13081), (13081, 13085), (13085, 13091), (13094, 13100), (13100, 13106), (13106, 13112), (13112, 13122), (13130, 13135), (13142, 13151), (13151, 13157), (13157, 13167), (13169, 13171), (13173, 13177), (13185, 13189), (13189, 13201), (13216, 13225), (13232, 13241), (13241, 13250), (13250, 13266), (13271, 13273), (13273, 13282), (13282, 13295), (13299, 13304), (13308, 13314), (13357, 13359), (13373, 13376), (13389, 13395), (13395, 13400), (13504, 13509), (13542, 13547), (13564, 13568), (13571, 13590), (13595, 13601), (13601, 13604), (13606, 13609), (13612, 13617), (13620, 13624), (13631, 13635), (13658, 13666), (13681, 13688), (13699, 13703), (13766, 13775), (13785, 13791), (13796, 13798), (13802, 13813), (13860, 13868), (13874, 13876), (13876, 13883), (13893, 13903), (13950, 13955), (13961, 13966), (14017, 14024), (14037, 14042), (14056, 14063), (14066, 14070), (14070, 14077), (14093, 14098), (14100, 14114), (14139, 14146), (14153, 14157), (14184, 14189), (14198, 14202), (14206, 14209), (14218, 14234), (14234, 14243), (14245, 14250), (14269, 14279), (14283, 14296), (14302, 14307), (14314, 14316), (14333, 14339), (14346, 14350), (14356, 14364), (14371, 14376), (14392, 14398), (14452, 14458), (14465, 14471), (14471, 14478), (14478, 14487), (14496, 14501), (14515, 14519), (14523, 14533), (14544, 14550), (14555, 14571), (14585, 14589), (14595, 14599), (14635, 14645), (14648, 14655), (14655, 14661), (14663, 14674), (14698, 14700), (14702, 14705), (14768, 14772), (14779, 14782), (14782, 14789), (14791, 14794), (14794, 14798), (14803, 14814), (14825, 14836), (14836, 14838), (14847, 14855), (14899, 14908), (14917, 14926), (14926, 14933), (14944, 14948), (14953, 14961), (14983, 14987), (14987, 14993), (15000, 15006), (15019, 15026), (15078, 15087), (15102, 15111), (15111, 15117), (15129, 15134), (15175, 15182), (15225, 15228), (15228, 15232), (15256, 15263), (15278, 15280), (15358, 15364), (15379, 15384), (15408, 15415), (15427, 15431), (15436, 15440), (15448, 15451), (15495, 15501), (15512, 15518), (15562, 15565), (15565, 15572), (15572, 15584), (15648, 15660), (15660, 15673), (15673, 15676), (15682, 15687), (15691, 15710), (15741, 15746), (15753, 15769), (15862, 15877), (15918, 15922), (15989, 15994), (15994, 16001), (16039, 16049), (16049, 16053), (16053, 16057), (16057, 16063), (16063, 16068), (16069, 16074), (16074, 16081), (16152, 16155), (16160, 16168), (16248, 16254), (16272, 16275), (16277, 16279), (16307, 16311), (16320, 16328), (16351, 16361), (16371, 16377), (16409, 16419), (16461, 16463), (16470, 16473), (16487, 16491), (16498, 16500), (16500, 16505), (16523, 16534), (16537, 16544), (16572, 16577), (16580, 16584), (16588, 16599), (16629, 16638), (16670, 16675), (16676, 16681), (16681, 16686), (16695, 16701), (16715, 16730), (16734, 16763), (16772, 16816), (16867, 16872), (16882, 16894), (16908, 16917), (16945, 16949), (16949, 16956), (16977, 16980), (17022, 17030), (17041, 17047), (17055, 17057), (17060, 17064), (17082, 17093), (17124, 17130), (17134, 17139), (17139, 17149), (17149, 17157), (17166, 17172), (17189, 17194), (17195, 17201), (17219, 17225), (17242, 17247), (17255, 17257), (17258, 17260), (17261, 17266), (17272, 17279), (17283, 17291), (17296, 17301), (17336, 17339), (17349, 17360), (17368, 17374), (17407, 17412), (17497, 17500), (17504, 17519), (17555, 17560), (17560, 17592), (17595, 17614), (17615, 17618), (17618, 17622), (17630, 17636), (17646, 17652), (17657, 17664), (17664, 17667), (17667, 17671), (17692, 17699), (17704, 17711), (17722, 17728), (17740, 17769), (17778, 17782), (17782, 17785), (17799, 17808), (17814, 17818), (17824, 17836), (17836, 17843), (17883, 17889), (17894, 17899), (17931, 17934), (17938, 17946), (17966, 17977), (17977, 17983), (18021, 18024), (18026, 18029), (18064, 18079), (18103, 18110), (18110, 18113), (18116, 18122), (18222, 18296), (18296, 18302), (18325, 18371), (18371, 18417), (18434, 18436), (18441, 18467), (18490, 18495), (18519, 18545), (18548, 18554), (18569, 18576), (18585, 18587), (18587, 18591), (18612, 18626), (18680, 18684), (18695, 18712), (18748, 18756), (18784, 18795), (18800, 18807), (18816, 18821), (18821, 18826), (18826, 18827), (18841, 18847), (18866, 18868), (18869, 18874), (18879, 18884), (18888, 18893), (18912, 18920), (18920, 18930), (18930, 18934), (18935, 18944), (18950, 18956), (18960, 18970), (18982, 18989), (18990, 18997), (18997, 19002), (19039, 19044), (19057, 19060), (19060, 19065), (19099, 19104), (19164, 19169), (19169, 19174), (19176, 19183), (19183, 19185), (19185, 19190), (19198, 19202), (19222, 19225), (19260, 19265), (19274, 19277), (19281, 19286), (19286, 19296), (19299, 19303), (19303, 19311), (19329, 19336), (19342, 19347), (19357, 19364), (19385, 19397), (19397, 19400), (19402, 19406), (19415, 19422), (19426, 19431), (19447, 19453), (19489, 19494), (19502, 19506), (19517, 19522), (19522, 19529), (19554, 19562), (19606, 19613), (19644, 19650), (19668, 19672), (19674, 19683), (19686, 19690), (19703, 19707), (19714, 19718), (19718, 19729), (19784, 19795), (19819, 19825), (19836, 19841), (19848, 19853), (19861, 19866), (19880, 19884), (19895, 19898), (19910, 19912), (19922, 19927), (19967, 19973), (19973, 19977), (19977, 19986), (19988, 19999), (20010, 20015), (20025, 20032), (20032, 20037), (20039, 20048), (20060, 20066), (20079, 20082), (20087, 20090), (20104, 20110), (20138, 20150), (20150, 20158), (20163, 20173), (20176, 20178), (20178, 20183), (20188, 20195), (20205, 20210), (20237, 20251), (20255, 20261), (20292, 20298), (20319, 20322), (20323, 20328), (20328, 20333), (20339, 20342), (20347, 20352), (20360, 20365), (20365, 20374), (20390, 20466), (20478, 20502), (20527, 20534), (20534, 20541), (20542, 20549), (20553, 20558), (20558, 20564), (20575, 20579), (20584, 20589), (20610, 20614), (20614, 20620), (20626, 20653), (20658, 20660), (20679, 20685), (20735, 20744), (20749, 20754), (20779, 20803), (21023, 21035), (21044, 21055), (21063, 21068), (21068, 21072), (21072, 21079), (21100, 21105), (21119, 21125), (21125, 21130), (21175, 21187), (21199, 21203), (21215, 21232), (21238, 21244), (21257, 21310), (21323, 21347), (21367, 21371), (21402, 21407), (21410, 21416), (21435, 21438), (21447, 21457), (21478, 21483), (21483, 21488), (21488, 21493), (21493, 21497), (21497, 21508), (21528, 21535), (21572, 21589), (21593, 21597), (21597, 21604), (21619, 21627), (21630, 21634), (21635, 21638), (21641, 21643), (21650, 21655), (21655, 21661), (21663, 21667), (21667, 21672), (21699, 21708), (21710, 21714), (21714, 21727), (21775, 21787), (21811, 21813), (21816, 21830), (21858, 21861), (21875, 21879), (21881, 21891), (21900, 21902), (21930, 21937), (21940, 21947), (21964, 21968), (22081, 22084), (22084, 22092), (22160, 22167), (22170, 22177), (22221, 22229), (22238, 22243), (22243, 22250), (22252, 22258), (22306, 22310), (22310, 22317), (22323, 22332), (22368, 22378), (22378, 22380), (22386, 22391), (22397, 22400), (22400, 22405), (22431, 22439), (22447, 22453), (22468, 22473), (22473, 22479), (22479, 22488), (22488, 22492), (22501, 22510), (22513, 22519), (22519, 22524), (22552, 22558), (22568, 22573), (22587, 22591), (22603, 22624), (22628, 22637), (22681, 22687), (22695, 22701), (22701, 22707), (22710, 22716), (22726, 22733), (22741, 22748), (22778, 22781), (22781, 22785), (22810, 22815), (22828, 22841), (22844, 22852), (22852, 22855), (22855, 22863), (22881, 22886), (22908, 22914), (22939, 22945), (22949, 22961), (22973, 22980), (22988, 22992), (22998, 22999), (23010, 23013), (23020, 23030), (23037, 23049), (23059, 23066), (23072, 23082), (23095, 23101), (23101, 23106), (23106, 23117), (23124, 23132), (23155, 23164), (23195, 23203), (23206, 23213), (23219, 23222), (23226, 23229), (23238, 23244), (23267, 23271), (23285, 23298), (23298, 23306), (23312, 23318), (23321, 23325), (23330, 23337), (23354, 23357), (23363, 23370), (23373, 23377), (23394, 23401), (23401, 23411), (23422, 23438), (23438, 23451), (23461, 23465), (23477, 23500), (23506, 23510), (23510, 23516), (23518, 23548), (23548, 23551), (23551, 23579), (23579, 23590), (23614, 23626), (23668, 23673), (23673, 23678), (23723, 23727), (23739, 23743), (23751, 23756), (23788, 23794), (23816, 23824), (23828, 23833), (23861, 23872), (23875, 23879), (23915, 23919), (23938, 23940), (23940, 23944), (23944, 23954), (23972, 23981), (23981, 23993), (24025, 24038), (24050, 24058), (24075, 24080), (24198, 24203), (24226, 24234), (24234, 24237), (24266, 24270), (24349, 24357), (24437, 24443), (24454, 24467), (24510, 24515), (24515, 24522), (24528, 24534), (24535, 24545), (24574, 24581), (24601, 24607), (24629, 24634), (24634, 24641), (24647, 24656), (24657, 24664), (24687, 24691), (24691, 24704), (24713, 24716), (24716, 24720), (24720, 24725), (24729, 24737), (24767, 24776), (24795, 24799), (24837, 24842), (24850, 24852), (24852, 24857), (24891, 24894), (24947, 24958), (25021, 25026), (25034, 25041), (25045, 25054), (25062, 25064), (25064, 25067), (25077, 25081), (25109, 25114), (25120, 25125), (25145, 25151), (25151, 25167), (25174, 25188), (25188, 25191), (25191, 25197), (25204, 25208), (25254, 25264), (25299, 25308), (25308, 25312), (25355, 25364), (25364, 25369), (25394, 25404), (25410, 25414), (25426, 25432), (25454, 25461), (25464, 25470), (25486, 25504), (25507, 25515), (25524, 25528), (25593, 25600), (25604, 25607), (25607, 25613), (25619, 25629), (25632, 25639), (25650, 25654), (25659, 25662), (25662, 25668), (25702, 25707), (25751, 25753), (25760, 25765), (25774, 25778), (25817, 25826), (25826, 25834), (25846, 25857), (25875, 25879), (25879, 25885), (25916, 25920), (25944, 25946), (25946, 25949), (25949, 25952), (25953, 25957), (25957, 25967), (25972, 25978), (26038, 26042), (26049, 26054), (26066, 26072), (26112, 26114), (26114, 26117), (26120, 26132), (26170, 26173), (26173, 26178), (26275, 26279), (26279, 26282), (26282, 26288), (26288, 26298), (26320, 26324), (26335, 26338), (26338, 26344), (26355, 26363), (26367, 26376), (26376, 26388), (26537, 26543), (26581, 26595), (27079, 27084), (27120, 27125), (27133, 27139), (27143, 27148), (27198, 27200), (27204, 27210), (27213, 27218), (27227, 27231), (27243, 27244), (27258, 27262), (27315, 27319), (27319, 27324), (27364, 27368), (27368, 27377), (27395, 27400), (27410, 27414), (27465, 27470), (27471, 27477), (27483, 27488), (27588, 27590), (27828, 27832), (28231, 28240), (28254, 28265), (28268, 28274), (28321, 28323), (28330, 28338), (28409, 28413), (28429, 28432), (28505, 28513), (28513, 28522), (28536, 28541), (28548, 28552), (28556, 28561), (28595, 28600), (28620, 28621), (28621, 28626), (28644, 28655), (28676, 28684), (28696, 28702), (28704, 28705), (28705, 28717), (28749, 28755), (28782, 28790), (28802, 28805), (28817, 28821), (28827, 28833), (28849, 28852), (28860, 28866), (28869, 28878), (28888, 28895), (28922, 28926), (28926, 28931), (28978, 28983), (29003, 29013), (29036, 29043), (29046, 29051), (29052, 29057), (29079, 29086), (29111, 29119), (29125, 29127), (29127, 29132), (29152, 29153), (29165, 29167), (29205, 29209), (29275, 29282), (29318, 29320), (29358, 29368), (29373, 29377), (29440, 29443), (29443, 29447), (29447, 29452), (29459, 29465), (29471, 29478), (29482, 29486), (29500, 29506), (29746, 29749), (29752, 29754), (29800, 29803), (29803, 29811), (29811, 29817), (29862, 29867), (30016, 30020), (30034, 30045), (30076, 30088), (30092, 30099), (30119, 30123), (30123, 30134), (30141, 30144), (30146, 30150), (30328, 30336), (30390, 30393), (30577, 30581), (30619, 30628), (30636, 30641), (30643, 30653), (30754, 30755), (30757, 30763), (30785, 30787), (30815, 30829), (30896, 30900), (31208, 31211), (31211, 31218), (31307, 31312), (31316, 31324), (31355, 31360), (31363, 31371), (31378, 31388), (31390, 31394), (31396, 31403), (31403, 31410), (31660, 31665), (31732, 31733), (31750, 31759), (31839, 31843), (32156, 32167), (32423, 32428), (32435, 32439), (32456, 32460), (32801, 32813), (32849, 32857), (32930, 32937), (33004, 33008), (33041, 33047), (33054, 33059), (33059, 33065), (33065, 33073), (33076, 33080), (33123, 33125), (33140, 33141), (33472, 33476), (33548, 33556), (33556, 33562), (33635, 33637), (33639, 33643)]
        tous_remplacements = []
        lg_texte1 = 13431
        texte_original = self._readFile("Condorcet_Esimpr1.txt") + self._readFile("Condorcet_Esimpr2.txt")     
        blocsCommuns = [(3, 31), (34, 56), (66, 71), (71, 78), (85, 92), (115, 127), (145, 151), (204, 212), (217, 228), (232, 239), (285, 289), (298, 302), (323, 327), (341, 345), (362, 375), (401, 406), (408, 413), (413, 417), (417, 420), (455, 460), (467, 475), (532, 536), (543, 548), (566, 571), (620, 627), (634, 643), (718, 723), (723, 729), (775, 788), (832, 836), (848, 853), (859, 866), (874, 880), (924, 929), (931, 937), (937, 942), (945, 950), (950, 956), (975, 980), (992, 997), (1052, 1057), (1061, 1066), (1080, 1085), (1099, 1100), (1100, 1105), (1105, 1111), (1140, 1146), (1164, 1172), (1176, 1181), (1188, 1193), (1208, 1214), (1404, 1435), (1524, 1532), (1632, 1637), (1650, 1653), (1710, 1714), (1890, 1898), (1985, 1991), (2005, 2008), (2070, 2076), (2092, 2100), (2101, 2106), (2108, 2113), (2119, 2128), (2144, 2148), (2173, 2281), (2284, 2304), (2324, 2347), (2379, 2386), (2391, 2486), (2492, 2587), (2591, 2798), (2802, 2854), (2854, 2860), (2861, 2866), (2875, 2879), (2879, 2886), (2898, 2904), (2912, 2918), (2918, 2929), (2929, 2938), (2987, 2995), (2998, 3068), (3083, 3160), (3160, 3281), (3283, 3316), (3325, 3384), (3385, 3557), (3557, 3591), (3591, 3596), (3603, 3676), (3693, 3703), (3879, 3884), (4157, 4164), (4497, 4502), (4564, 4573), (5052, 5060), (5296, 5304), (5596, 5602), (5746, 5750), (5896, 5906), (6333, 6339), (6456, 6463), (6568, 6570), (6587, 6596), (6766, 6774), (6775, 6782), (6787, 6792), (6792, 6797), (6810, 7017), (7025, 7032), (7039, 7065), (7099, 7111), (7118, 7124), (7124, 7127), (7132, 7141), (7141, 7146), (7152, 7204), (7204, 7218), (7229, 7248), (7248, 7283), (7289, 7304), (7315, 7329), (7329, 7477), (7500, 7521), (7521, 7531), (7531, 7535), (7542, 7619), (7619, 7625), (7630, 7730), (7734, 7773), (7773, 7839), (8285, 8291), (8298, 8300), (8310, 8315), (8343, 8348), (8402, 8436), (8436, 8451), (8501, 8507), (8584, 8589), (8594, 8621), (8624, 8632), (8648, 8843), (8872, 8905), (8907, 8947), (8963, 8969), (8996, 9001), (9040, 9048), (9073, 9080), (9090, 9096), (9100, 9107), (9195, 9200), (9236, 9382), (9390, 9479), (9483, 9488), (9511, 9558), (9558, 9562), (9571, 9608), (9612, 9646), (9653, 9714), (9715, 9721), (9831, 9931), (9947, 9952), (9952, 9979), (9981, 9994), (9997, 10007), (10015, 10022), (10022, 10030), (10039, 10048), (10048, 10054), (10059, 10072), (10077, 10160), (10162, 10259), (10262, 10270), (10289, 10329), (10334, 10342), (10348, 10693), (10719, 10726), (10757, 10766), (10766, 10773), (10782, 10787), (10787, 10792), (10806, 10807), (10923, 10929), (10968, 10986), (11003, 11015), (11015, 11030), (11030, 11040), (11043, 11052), (11055, 11070), (11070, 11223), (11242, 11259), (11268, 11420), (11425, 11497), (11511, 11516), (11522, 11577), (11583, 11599), (11602, 11617), (13434, 13462), (13463, 13485), (13903, 13908), (15344, 15351), (15542, 15550), (15550, 15562), (16222, 16228), (16330, 16338), (16552, 16563), (16686, 16693), (16843, 16847), (16929, 16933), (17047, 17051), (17051, 17055), (17066, 17078), (17161, 17166), (17267, 17272), (17377, 17381), (18302, 18305), (18310, 18315), (18561, 18569), (18655, 18659), (18675, 18680), (18827, 18832), (18857, 18864), (18897, 18907), (19010, 19015), (19046, 19052), (19145, 19158), (19364, 19368), (19431, 19436), (19588, 19595), (19613, 19619), (19730, 19735), (20215, 20221), (20469, 20474), (21382, 21387), (21472, 21478), (21546, 21551), (21694, 21699), (21787, 21792), (22071, 22076), (22115, 22120), (22125, 22126), (22126, 22131), (22229, 22235), (22999, 23004), (23654, 23662), (23796, 23801), (23849, 23855), (24015, 24025), (24084, 24115), (24129, 24137), (24261, 24266), (24272, 24275), (24281, 24285), (24305, 24313), (24361, 24367), (24388, 24391), (24393, 24399), (24399, 24407), (24808, 24813), (26218, 26223), (26233, 26242), (26308, 26312), (26401, 26509), (26512, 26531), (26558, 26581), (26599, 26606), (26613, 26709), (26717, 26812), (26816, 27023), (27027, 27079), (27084, 27090), (27096, 27101), (27150, 27154), (27191, 27198), (27281, 27287), (27330, 27336), (27342, 27353), (27355, 27364), (27429, 27437), (27519, 27588), (27593, 27670), (27671, 27791), (27793, 27825), (27832, 27888), (27889, 28062), (28075, 28109), (28109, 28114), (28120, 28194), (28212, 28222), (28274, 28279), (28295, 28302), (28310, 28315), (28360, 28369), (28369, 28376), (28384, 28392), (28396, 28402), (28405, 28409), (28413, 28423), (28438, 28444), (28479, 28486), (28487, 28489), (28755, 28764), (28841, 28849), (28881, 28888), (29236, 29242), (29435, 29440), (29514, 29721), (29736, 29743), (29757, 29782), (29828, 29840), (29845, 29851), (29859, 29862), (29867, 29876), (29876, 29881), (29887, 29938), (29947, 29960), (29960, 29979), (29980, 30015), (30020, 30034), (30156, 30170), (30180, 30328), (30343, 30364), (30364, 30373), (30382, 30386), (30393, 30470), (30470, 30476), (30476, 30577), (30581, 30619), (30665, 30735), (30735, 30741), (30752, 30754), (30787, 30792), (30800, 30805), (30841, 30875), (30876, 30891), (30915, 30921), (30925, 30930), (30970, 30997), (31005, 31013), (31013, 31208), (31218, 31251), (31255, 31294), (31296, 31302), (31302, 31307), (31326, 31334), (31334, 31341), (31344, 31350), (31371, 31378), (31410, 31415), (31415, 31561), (31571, 31660), (31681, 31686), (31686, 31732), (31738, 31742), (31762, 31799), (31805, 31839), (31847, 31908), (31909, 31918), (31918, 32017), (32021, 32026), (32026, 32053), (32053, 32066), (32077, 32087), (32091, 32098), (32100, 32108), (32120, 32129), (32129, 32135), (32140, 32156), (32167, 32252), (32252, 32349), (32349, 32357), (32379, 32418), (32448, 32456), (32460, 32801), (32836, 32843), (32879, 32888), (32888, 32895), (32904, 32909), (32914, 32919), (32924, 32925), (32987, 32993), (33008, 33026), (33030, 33041), (33086, 33101), (33101, 33111), (33114, 33123), (33125, 33140), (33143, 33297), (33300, 33318), (33318, 33472), (33476, 33548), (33565, 33570), (33579, 33635), (33643, 33659), (33662, 33699)]
        lDepl = [[(81, 85), (14198, 14202)], [(81, 85), (19714, 19718)], [(106, 115), (28330, 28338)], [(158, 164), (16695, 16701)], [(158, 164), (25145, 25151)], [(164, 168), (32435, 32439)], [(171, 173), (18585, 18587)], [(185, 194), (22501, 22510)], [(251, 258), (15019, 15026)], [(260, 268), (22221, 22229)], [(275, 278), (15225, 15228)], [(294, 298), (21964, 21968)], [(313, 318), (24075, 24080)], [(327, 333), (28696, 28702)], [(346, 352), (20104, 20110)], [(354, 362), (16160, 16168)], [(424, 438), (15660, 15673)], [(438, 443), (15682, 15687)], [(438, 443), (23668, 23673)], [(475, 495), (15691, 15710)], [(501, 506), (16867, 16872)], [(508, 510), (16461, 16463)], [(510, 524), (15753, 15769)], [(643, 655), (15648, 15660)], [(655, 665), (16523, 16534)], [(670, 680), (30643, 30653)], [(680, 687), (16074, 16081)], [(687, 692), (22568, 22573)], [(696, 700), (14346, 14350)], [(696, 700), (16945, 16949)], [(696, 700), (30577, 30581)], [(757, 763), (22473, 22479)], [(763, 775), (16588, 16599)], [(788, 796), (23155, 23164)], [(798, 802), (14346, 14350)], [(798, 802), (16945, 16949)], [(798, 802), (30577, 30581)], [(808, 817), (28231, 28240)], [(836, 842), (23816, 23824)], [(842, 847), (13961, 13966)], [(882, 893), (17966, 17977)], [(893, 899), (25426, 25432)], [(899, 904), (22238, 22243)], [(906, 916), (16351, 16361)], [(916, 923), (16537, 16544)], [(956, 965), (23972, 23981)], [(1003, 1013), (16039, 16049)], [(1024, 1030), (17124, 17130)], [(1024, 1030), (17219, 17225)], [(1030, 1036), (17977, 17983)], [(1037, 1043), (17368, 17374)], [(1043, 1048), (19169, 19174)], [(1048, 1052), (19668, 19672)], [(1079, 1080), (18826, 18827)], [(1079, 1080), (30754, 30755)], [(1079, 1080), (31732, 31733)], [(1135, 1140), (15989, 15994)], [(1152, 1160), (16160, 16168)], [(1172, 1176), (16307, 16311)], [(1200, 1205), (20347, 20352)], [(1200, 1205), (21478, 21483)], [(1200, 1205), (27079, 27084)], [(1207, 1208), (18826, 18827)], [(1207, 1208), (30754, 30755)], [(1207, 1208), (31732, 31733)], [(1254, 1264), (23072, 23082)], [(1269, 1273), (16053, 16057)], [(1280, 1289), (16629, 16638)], [(1280, 1289), (19674, 19683)], [(1289, 1303), (16715, 16730)], [(1309, 1314), (16867, 16872)], [(1317, 1346), (16734, 16763)], [(1348, 1360), (19385, 19397)], [(1360, 1401), (16772, 16816)], [(1401, 1404), (31208, 31211)], [(1440, 1445), (29447, 29452)], [(1445, 1448), (23354, 23357)], [(1448, 1459), (14803, 14814)], [(1463, 1480), (25486, 25504)], [(1483, 1487), (17814, 17818)], [(1491, 1496), (17189, 17194)], [(1496, 1498), (19183, 19185)], [(1496, 1498), (20176, 20178)], [(1496, 1498), (29125, 29127)], [(1507, 1512), (17894, 17899)], [(1507, 1512), (24510, 24515)], [(1512, 1518), (20679, 20685)], [(1512, 1518), (21238, 21244)], [(1512, 1518), (22447, 22453)], [(1512, 1518), (25464, 25470)], [(1520, 1522), (28321, 28323)], [(1547, 1551), (13631, 13635)], [(1547, 1551), (14944, 14948)], [(1547, 1551), (25204, 25208)], [(1547, 1551), (31390, 31394)], [(1552, 1556), (20575, 20579)], [(1592, 1597), (24852, 24857)], [(1598, 1606), (31396, 31403)], [(1606, 1613), (14070, 14077)], [(1637, 1639), (17258, 17260)], [(1637, 1639), (18866, 18868)], [(1639, 1645), (17407, 17412)], [(1639, 1645), (23363, 23370)], [(1654, 1659), (17189, 17194)], [(1659, 1661), (19183, 19185)], [(1659, 1661), (20176, 20178)], [(1659, 1661), (29125, 29127)], [(1665, 1669), (26320, 26324)], [(1665, 1669), (30119, 30123)], [(1674, 1676), (13874, 13876)], [(1687, 1691), (22488, 22492)], [(1693, 1699), (21072, 21079)], [(1699, 1703), (22306, 22310)], [(1714, 1717), (26335, 26338)], [(1724, 1729), (27143, 27148)], [(1732, 1735), (22397, 22400)], [(1738, 1745), (18990, 18997)], [(1750, 1760), (20365, 20374)], [(1760, 1836), (20390, 20466)], [(1836, 1858), (20478, 20502)], [(1879, 1885), (20610, 20614)], [(1910, 1917), (20527, 20534)], [(1918, 1925), (20534, 20541)], [(1925, 1931), (20558, 20564)], [(1934, 1940), (20614, 20620)], [(1940, 1944), (14346, 14350)], [(1940, 1944), (16945, 16949)], [(1940, 1944), (30577, 30581)], [(1944, 1971), (20626, 20653)], [(1994, 1999), (19861, 19866)], [(1994, 1999), (27465, 27470)], [(2053, 2062), (20735, 20744)], [(2128, 2133), (29862, 29867)], [(2133, 2139), (14655, 14661)], [(2133, 2139), (30757, 30763)], [(2139, 2142), (28429, 28432)], [(2165, 2173), (15102, 15111)], [(2310, 2324), (20237, 20251)], [(2310, 2324), (26581, 26595)], [(2866, 2869), (26170, 26173)], [(2872, 2875), (17931, 17934)], [(2893, 2898), (15228, 15232)], [(2947, 2951), (14595, 14599)], [(2947, 2951), (28548, 28552)], [(2947, 2951), (29482, 29486)], [(2965, 2970), (14093, 14098)], [(2965, 2970), (16681, 16686)], [(2973, 2977), (27828, 27832)], [(3068, 3077), (22323, 22332)], [(3320, 3325), (19489, 19494)], [(3320, 3325), (19819, 19825)], [(3320, 3325), (25760, 25765)], [(3686, 3689), (30390, 30393)], [(3714, 3735), (22603, 22624)], [(3735, 3744), (13766, 13775)], [(3744, 3748), (25774, 25778)], [(3752, 3760), (23195, 23203)], [(3795, 3803), (22844, 22852)], [(3811, 3819), (33065, 33073)], [(3819, 3826), (14648, 14655)], [(3826, 3836), (25619, 25629)], [(3841, 3853), (21775, 21787)], [(3855, 3862), (14926, 14933)], [(3862, 3865), (20079, 20082)], [(3868, 3879), (23106, 23117)], [(3891, 3900), (18935, 18944)], [(3908, 3913), (25021, 25026)], [(3913, 3922), (22628, 22637)], [(3922, 3934), (23285, 23298)], [(3945, 3950), (17189, 17194)], [(3950, 3952), (19183, 19185)], [(3950, 3952), (20176, 20178)], [(3950, 3952), (29125, 29127)], [(3963, 3976), (24025, 24038)], [(3997, 4004), (25593, 25600)], [(4005, 4011), (17041, 17047)], [(4017, 4024), (23330, 23337)], [(4040, 4045), (29003, 29013)], [(4045, 4049), (25650, 25654)], [(4045, 4049), (26038, 26042)], [(4057, 4069), (23438, 23451)], [(4072, 4088), (23422, 23438)], [(4088, 4095), (15175, 15182)], [(4088, 4095), (17836, 17843)], [(4088, 4095), (22741, 22748)], [(4088, 4095), (22973, 22980)], [(4100, 4107), (21940, 21947)], [(4114, 4118), (28922, 28926)], [(4133, 4138), (16670, 16675)], [(4141, 4143), (27198, 27200)], [(4143, 4148), (14093, 14098)], [(4143, 4148), (16681, 16686)], [(4148, 4157), (16908, 16917)], [(4171, 4176), (20360, 20365)], [(4176, 4179), (23010, 23013)], [(4188, 4194), (20060, 20066)], [(4197, 4200), (25188, 25191)], [(4197, 4200), (25604, 25607)], [(4197, 4200), (25659, 25662)], [(4204, 4210), (16371, 16377)], [(4228, 4240), (20138, 20150)], [(4240, 4246), (16371, 16377)], [(4246, 4253), (22160, 22167)], [(4246, 4253), (29036, 29043)], [(4255, 4258), (19274, 19277)], [(4271, 4278), (22726, 22733)], [(4281, 4289), (23124, 23132)], [(4289, 4293), (23723, 23727)], [(4296, 4300), (21630, 21634)], [(4296, 4300), (23461, 23465)], [(4304, 4309), (20328, 20333)], [(4304, 4309), (27319, 27324)], [(4313, 4317), (19880, 19884)], [(4329, 4336), (32930, 32937)], [(4342, 4364), (23477, 23500)], [(4364, 4368), (23506, 23510)], [(4375, 4406), (23518, 23548)], [(4409, 4433), (23551, 23579)], [(4435, 4459), (20779, 20803)], [(4459, 4464), (23101, 23106)], [(4528, 4535), (19522, 19529)], [(4668, 4685), (28705, 28717)], [(4696, 4701), (28926, 28931)], [(4701, 4707), (25972, 25978)], [(4707, 4711), (21068, 21072)], [(4713, 4725), (17082, 17093)], [(4725, 4737), (21175, 21187)], [(4737, 4748), (19784, 19795)], [(4748, 4765), (21215, 21232)], [(4765, 4768), (19222, 19225)], [(4765, 4768), (20087, 20090)], [(4765, 4768), (23548, 23551)], [(4765, 4768), (27364, 27368)], [(4768, 4821), (21257, 21310)], [(4826, 4831), (14302, 14307)], [(4834, 4857), (21323, 21347)], [(4857, 4862), (23828, 23833)], [(4857, 4862), (24837, 24842)], [(4862, 4865), (21635, 21638)], [(4865, 4873), (24729, 24737)], [(4873, 4879), (22695, 22701)], [(4888, 4900), (21023, 21035)], [(4888, 4900), (22949, 22961)], [(4917, 4924), (24657, 24664)], [(4924, 4928), (13620, 13624)], [(4940, 4947), (14139, 14146)], [(4947, 4957), (16409, 16419)], [(4957, 4962), (24852, 24857)], [(4962, 4965), (18021, 18024)], [(4965, 4978), (22828, 22841)], [(4982, 4989), (22243, 22250)], [(5003, 5011), (17283, 17291)], [(5011, 5018), (31363, 31371)], [(5018, 5022), (23915, 23919)], [(5018, 5022), (25077, 25081)], [(5027, 5031), (13564, 13568)], [(5034, 5048), (18612, 18626)], [(5060, 5062), (18434, 18436)], [(5060, 5062), (21811, 21813)], [(5060, 5062), (25751, 25753)], [(5060, 5062), (29752, 29754)], [(5073, 5079), (28860, 28866)], [(5090, 5096), (16057, 16063)], [(5102, 5107), (24629, 24634)], [(5114, 5121), (13876, 13883)], [(5127, 5137), (23020, 23030)], [(5144, 5146), (14698, 14700)], [(5144, 5146), (14836, 14838)], [(5146, 5147), (22998, 22999)], [(5146, 5147), (33140, 33141)], [(5147, 5154), (15994, 16001)], [(5154, 5159), (22519, 22524)], [(5178, 5189), (21497, 21508)], [(5189, 5195), (30636, 30641)], [(5214, 5224), (14269, 14279)], [(5237, 5243), (20292, 20298)], [(5243, 5247), (19342, 19347)], [(5243, 5247), (19836, 19841)], [(5254, 5266), (23037, 23049)], [(5271, 5276), (14302, 14307)], [(5290, 5295), (31660, 31665)], [(5304, 5307), (18026, 18029)], [(5313, 5319), (28556, 28561)], [(5313, 5319), (28595, 28600)], [(5328, 5329), (27243, 27244)], [(5328, 5329), (28620, 28621)], [(5339, 5348), (14523, 14533)], [(5359, 5365), (21655, 21661)], [(5368, 5369), (18826, 18827)], [(5368, 5369), (30754, 30755)], [(5368, 5369), (31732, 31733)], [(5369, 5373), (26320, 26324)], [(5369, 5373), (30119, 30123)], [(5392, 5399), (23206, 23213)], [(5406, 5408), (14314, 14316)], [(5427, 5441), (30815, 30829)], [(5441, 5457), (25151, 25167)], [(5457, 5460), (14791, 14794)], [(5457, 5460), (17615, 17618)], [(5457, 5460), (24713, 24716)], [(5468, 5471), (17782, 17785)], [(5471, 5476), (28926, 28931)], [(5476, 5486), (14478, 14487)], [(5476, 5486), (25355, 25364)], [(5476, 5486), (25817, 25826)], [(5501, 5507), (29811, 29817)], [(5507, 5520), (16882, 16894)], [(5531, 5538), (30092, 30099)], [(5545, 5551), (17166, 17172)], [(5551, 5556), (18869, 18874)], [(5565, 5569), (27315, 27319)], [(5572, 5582), (14523, 14533)], [(5586, 5590), (18680, 18684)], [(5595, 5596), (18826, 18827)], [(5595, 5596), (30754, 30755)], [(5595, 5596), (31732, 31733)], [(5602, 5612), (31378, 31388)], [(5629, 5635), (15358, 15364)], [(5665, 5676), (23579, 23590)], [(5665, 5676), (24947, 24958)], [(5797, 5871), (18222, 18296)], [(5879, 5884), (24852, 24857)], [(5892, 5896), (29443, 29447)], [(5909, 5911), (13796, 13798)], [(5911, 5921), (21881, 21891)], [(5921, 5927), (17630, 17636)], [(5927, 5973), (18325, 18371)], [(5973, 5978), (21667, 21672)], [(5983, 6028), (18371, 18417)], [(6031, 6034), (17664, 17667)], [(6031, 6034), (18110, 18113)], [(6036, 6043), (23394, 23401)], [(6043, 6051), (19554, 19562)], [(6051, 6077), (18441, 18467)], [(6077, 6085), (25826, 25834)], [(6085, 6092), (14056, 14063)], [(6092, 6097), (19060, 19065)], [(6102, 6110), (25507, 25515)], [(6102, 6110), (30328, 30336)], [(6110, 6115), (25109, 25114)], [(6118, 6120), (27588, 27590)], [(6118, 6120), (29318, 29320)], [(6123, 6128), (19281, 19286)], [(6123, 6128), (26173, 26178)], [(6128, 6136), (23298, 23306)], [(6151, 6159), (13860, 13868)], [(6191, 6194), (22852, 22855)], [(6191, 6194), (23219, 23222)], [(6194, 6203), (14899, 14908)], [(6203, 6207), (33076, 33080)], [(6217, 6227), (25254, 25264)], [(6217, 6227), (25394, 25404)], [(6231, 6236), (32423, 32428)], [(6253, 6279), (18519, 18545)], [(6279, 6286), (18569, 18576)], [(6286, 6288), (33635, 33637)], [(6288, 6293), (19848, 19853)], [(6293, 6301), (23124, 23132)], [(6303, 6308), (18821, 18826)], [(6308, 6315), (24515, 24522)], [(6324, 6327), (15512, 15518)], [(6327, 6332), (13950, 13955)], [(6342, 6348), (22710, 22716)], [(6342, 6348), (33041, 33047)], [(6348, 6354), (13595, 13601)], [(6348, 6354), (14452, 14458)], [(6348, 6354), (23095, 23101)], [(6348, 6354), (27471, 27477)], [(6348, 6354), (28749, 28755)], [(6348, 6354), (33059, 33065)], [(6363, 6369), (22695, 22701)], [(6369, 6380), (21044, 21055)], [(6380, 6385), (18879, 18884)], [(6380, 6385), (19922, 19927)], [(6380, 6385), (20749, 20754)], [(6380, 6385), (21650, 21655)], [(6390, 6393), (23226, 23229)], [(6390, 6393), (26279, 26282)], [(6407, 6414), (29275, 29282)], [(6414, 6419), (16063, 16068)], [(6414, 6419), (20584, 20589)], [(6414, 6419), (21488, 21493)], [(6414, 6419), (28621, 28626)], [(6414, 6419), (29046, 29051)], [(6420, 6426), (16069, 16074)], [(6420, 6426), (18116, 18122)], [(6420, 6426), (24198, 24203)], [(6420, 6426), (29052, 29057)], [(6429, 6440), (23861, 23872)], [(6440, 6443), (21435, 21438)], [(6443, 6446), (14702, 14705)], [(6443, 6446), (22081, 22084)], [(6443, 6446), (25949, 25952)], [(6446, 6456), (18784, 18795)], [(6463, 6464), (27243, 27244)], [(6463, 6464), (28620, 28621)], [(6466, 6475), (13893, 13903)], [(6480, 6491), (14635, 14645)], [(6491, 6499), (18912, 18920)], [(6491, 6499), (33548, 33556)], [(6502, 6509), (19415, 19422)], [(6502, 6509), (20025, 20032)], [(6517, 6521), (14768, 14772)], [(6521, 6524), (21435, 21438)], [(6524, 6529), (21410, 21416)], [(6552, 6556), (21199, 21203)], [(6558, 6564), (28696, 28702)], [(6570, 6575), (20032, 20037)], [(6576, 6580), (17778, 17782)], [(6576, 6580), (23875, 23879)], [(6580, 6587), (13681, 13688)], [(6602, 6612), (18920, 18930)], [(6629, 6635), (20150, 20158)], [(6640, 6645), (26049, 26054)], [(6645, 6650), (22468, 22473)], [(6654, 6664), (18960, 18970)], [(6667, 6678), (14663, 14674)], [(6685, 6688), (16152, 16155)], [(6685, 6688), (21858, 21861)], [(6691, 6706), (21816, 21830)], [(6706, 6711), (15741, 15746)], [(6723, 6729), (14544, 14550)], [(6738, 6740), (25062, 25064)], [(6738, 6740), (25944, 25946)], [(6738, 6740), (26112, 26114)], [(6748, 6761), (21714, 21727)], [(6748, 6761), (24691, 24704)], [(6761, 6766), (21100, 21105)], [(6774, 6775), (29152, 29153)], [(6785, 6787), (33123, 33125)], [(7017, 7023), (28827, 28833)], [(7034, 7039), (19517, 19522)], [(7074, 7080), (15495, 15501)], [(7080, 7084), (14585, 14589)], [(7084, 7088), (25650, 25654)], [(7084, 7088), (26038, 26042)], [(7111, 7118), (23059, 23066)], [(7218, 7224), (17195, 17201)], [(7285, 7289), (25953, 25957)], [(7479, 7489), (20163, 20173)], [(7489, 7495), (18296, 18302)], [(7495, 7500), (27483, 27488)], [(7495, 7500), (28978, 28983)], [(7625, 7630), (16676, 16681)], [(7730, 7734), (27258, 27262)], [(7842, 7846), (18930, 18934)], [(7870, 7886), (18695, 18712)], [(7889, 7896), (25632, 25639)], [(7899, 7909), (21447, 21457)], [(7910, 7918), (23816, 23824)], [(7918, 7923), (19164, 19169)], [(7918, 7923), (22400, 22405)], [(7923, 7928), (31355, 31360)], [(7934, 7952), (13571, 13590)], [(7973, 7978), (21402, 21407)], [(7982, 7985), (19397, 19400)], [(7999, 8003), (25875, 25879)], [(7999, 8003), (29205, 29209)], [(8003, 8011), (31316, 31324)], [(8011, 8028), (21572, 21589)], [(8036, 8048), (15572, 15584)], [(8048, 8052), (23373, 23377)], [(8054, 8057), (22852, 22855)], [(8054, 8057), (23219, 23222)], [(8057, 8066), (17139, 17149)], [(8069, 8101), (17560, 17592)], [(8105, 8125), (17595, 17614)], [(8127, 8131), (22988, 22992)], [(8133, 8139), (17722, 17728)], [(8139, 8144), (16572, 16577)], [(8144, 8151), (22310, 22317)], [(8154, 8183), (17740, 17769)], [(8183, 8187), (27227, 27231)], [(8187, 8194), (17657, 17664)], [(8187, 8194), (18103, 18110)], [(8209, 8216), (17692, 17699)], [(8216, 8221), (19342, 19347)], [(8216, 8221), (19836, 19841)], [(8228, 8234), (14392, 14398)], [(8228, 8234), (16248, 16254)], [(8228, 8234), (21119, 21125)], [(8228, 8234), (23510, 23516)], [(8228, 8234), (33556, 33562)], [(8238, 8245), (21597, 21604)], [(8250, 8255), (21493, 21497)], [(8268, 8270), (17055, 17057)], [(8268, 8270), (20658, 20660)], [(8268, 8270), (21900, 21902)], [(8268, 8270), (29165, 29167)], [(8270, 8278), (17149, 17157)], [(8336, 8340), (27258, 27262)], [(8357, 8361), (17667, 17671)], [(8364, 8368), (31839, 31843)], [(8378, 8381), (28802, 28805)], [(8381, 8388), (18982, 18989)], [(8391, 8397), (15000, 15006)], [(8454, 8460), (19039, 19044)], [(8466, 8470), (21367, 21371)], [(8466, 8470), (23940, 23944)], [(8470, 8476), (14333, 14339)], [(8470, 8476), (22701, 22707)], [(8476, 8487), (17349, 17360)], [(8487, 8490), (20319, 20322)], [(8493, 8499), (27133, 27139)], [(8530, 8536), (29500, 29506)], [(8536, 8539), (14206, 14209)], [(8548, 8553), (16487, 16491)], [(8548, 8553), (19198, 19202)], [(8553, 8560), (18800, 18807)], [(8572, 8584), (26120, 26132)], [(8632, 8635), (15562, 15565)], [(8638, 8646), (18748, 18756)], [(8849, 8855), (25191, 25197)], [(8849, 8855), (25607, 25613)], [(8849, 8855), (25662, 25668)], [(8867, 8868), (29152, 29153)], [(8947, 8951), (21964, 21968)], [(8951, 8963), (23614, 23626)], [(8951, 8963), (32801, 32813)], [(8977, 8981), (13631, 13635)], [(8977, 8981), (14944, 14948)], [(8977, 8981), (25204, 25208)], [(8977, 8981), (31390, 31394)], [(8981, 8986), (26275, 26279)], [(8990, 8996), (23238, 23244)], [(9011, 9022), (19718, 19729)], [(9028, 9031), (24891, 24894)], [(9031, 9038), (26282, 26288)], [(9038, 9040), (24850, 24852)], [(9038, 9040), (30785, 30787)], [(9051, 9060), (26367, 26376)], [(9061, 9065), (14371, 14376)], [(9061, 9065), (15436, 15440)], [(9061, 9065), (21710, 21714)], [(9061, 9065), (22587, 22591)], [(9061, 9065), (24687, 24691)], [(9061, 9065), (25702, 25707)], [(9083, 9087), (23723, 23727)], [(9119, 9122), (15448, 15451)], [(9135, 9143), (14356, 14364)], [(9155, 9159), (23739, 23743)], [(9159, 9166), (17938, 17946)], [(9166, 9171), (17242, 17247)], [(9166, 9171), (23751, 23756)], [(9171, 9181), (22368, 22378)], [(9188, 9195), (23312, 23318)], [(9200, 9202), (17255, 17257)], [(9205, 9210), (17134, 17139)], [(9210, 9219), (14917, 14926)], [(9210, 9219), (25045, 25054)], [(9210, 9219), (30619, 30628)], [(9210, 9219), (31750, 31759)], [(9226, 9231), (20010, 20015)], [(9382, 9390), (22855, 22863)], [(9488, 9500), (28782, 28790)], [(9500, 9511), (32156, 32167)], [(9562, 9566), (29373, 29377)], [(9608, 9612), (15918, 15922)], [(9608, 9612), (24795, 24799)], [(9648, 9653), (19099, 19104)], [(9648, 9653), (22881, 22886)], [(9721, 9732), (28644, 28655)], [(9736, 9745), (19977, 19986)], [(9745, 9752), (14953, 14961)], [(9745, 9752), (31211, 31218)], [(9755, 9763), (16320, 16328)], [(9763, 9770), (17704, 17711)], [(9770, 9777), (24574, 24581)], [(9777, 9791), (14100, 14114)], [(9794, 9807), (14283, 14296)], [(9807, 9812), (25308, 25312)], [(9932, 9941), (20039, 20048)], [(9943, 9947), (14983, 14987)], [(9943, 9947), (15129, 15134)], [(10007, 10012), (14184, 14189)], [(10034, 10039), (23673, 23678)], [(10278, 10289), (14825, 14836)], [(10342, 10348), (14544, 14550)], [(10693, 10705), (17824, 17836)], [(10711, 10718), (15256, 15263)], [(10732, 10744), (21175, 21187)], [(10744, 10750), (19967, 19973)], [(10750, 10757), (14953, 14961)], [(10750, 10757), (31211, 31218)], [(10773, 10774), (22998, 22999)], [(10773, 10774), (33140, 33141)], [(10774, 10780), (14987, 14993)], [(10798, 10803), (13612, 13617)], [(10810, 10815), (21125, 21130)], [(10810, 10815), (31307, 31312)], [(10815, 10819), (25524, 25528)], [(10848, 10853), (18490, 18495)], [(10853, 10860), (22170, 22177)], [(10863, 10869), (20255, 20261)], [(10869, 10874), (17261, 17266)], [(10888, 10894), (26338, 26344)], [(10894, 10900), (14544, 14550)], [(10901, 10903), (19910, 19912)], [(10936, 10945), (13802, 13813)], [(10950, 10955), (18816, 18821)], [(10994, 10999), (24720, 24725)], [(10999, 11003), (13699, 13703)], [(11040, 11043), (28849, 28852)], [(11040, 11043), (29440, 29443)], [(11223, 11228), (27395, 27400)], [(11236, 11242), (26537, 26543)], [(11497, 11502), (13504, 13509)], [(11502, 11511), (24647, 24656)], [(11516, 11522), (22513, 22519)], [(11577, 11580), (13606, 13609)], [(11599, 11602), (19222, 19225)], [(11599, 11602), (20087, 20090)], [(11599, 11602), (23548, 23551)], [(11599, 11602), (27364, 27368)], [(11622, 11627), (17883, 17889)], [(11628, 11636), (17022, 17030)], [(11653, 11659), (22681, 22687)], [(11659, 11663), (19299, 19303)], [(11659, 11663), (27213, 27218)], [(11659, 11663), (30896, 30900)], [(11667, 11682), (15862, 15877)], [(11667, 11682), (18064, 18079)], [(11682, 11698), (14555, 14571)], [(11701, 11706), (22810, 22815)], [(11706, 11712), (27204, 27210)], [(11712, 11727), (17504, 17519)], [(11798, 11802), (23915, 23919)], [(11798, 11802), (25077, 25081)], [(11804, 11808), (16580, 16584)], [(11804, 11808), (21663, 21667)], [(11815, 11819), (32456, 32460)], [(11819, 11825), (24601, 24607)], [(11828, 11834), (21063, 21068)], [(11834, 11838), (21875, 21879)], [(11834, 11838), (33472, 33476)], [(11851, 11862), (28254, 28265)], [(11862, 11866), (19686, 19690)], [(11878, 11887), (28869, 28878)], [(11890, 11900), (24535, 24545)], [(11902, 11913), (19988, 19999)], [(11917, 11922), (14245, 14250)], [(11922, 11925), (29746, 29749)], [(11940, 11952), (23981, 23993)], [(11957, 11961), (27410, 27414)], [(11968, 11971), (17497, 17500)], [(11977, 11983), (13542, 13547)], [(11990, 11995), (19164, 19169)], [(11990, 11995), (22400, 22405)], [(11995, 12000), (20553, 20558)], [(12011, 12017), (23788, 23794)], [(12017, 12020), (16470, 16473)], [(12027, 12035), (14017, 14024)], [(12027, 12035), (14847, 14855)], [(12027, 12035), (25454, 25461)], [(12037, 12039), (23938, 23940)], [(12046, 12051), (18888, 18893)], [(12051, 12054), (22778, 22781)], [(12073, 12082), (22479, 22488)], [(12073, 12082), (25299, 25308)], [(12082, 12088), (19447, 19453)], [(12082, 12088), (25879, 25885)], [(12090, 12094), (14515, 14519)], [(12090, 12094), (23321, 23325)], [(12097, 12102), (28926, 28931)], [(12105, 12112), (28888, 28895)], [(12115, 12118), (30141, 30144)], [(12118, 12123), (26376, 26388)], [(12132, 12135), (16977, 16980)], [(12132, 12135), (24234, 24237)], [(12132, 12135), (29800, 29803)], [(12135, 12142), (25034, 25041)], [(12142, 12150), (22084, 22092)], [(12155, 12159), (25916, 25920)], [(12159, 12169), (19286, 19296)], [(12169, 12173), (18587, 18591)], [(12188, 12192), (30146, 30150)], [(12198, 12203), (19185, 19190)], [(12198, 12203), (20178, 20183)], [(12198, 12203), (29127, 29132)], [(12203, 12206), (20339, 20342)], [(12207, 12211), (14794, 14798)], [(12207, 12211), (17618, 17622)], [(12207, 12211), (24716, 24720)], [(12218, 12227), (29803, 29811)], [(12240, 12244), (19703, 19707)], [(12254, 12258), (19299, 19303)], [(12254, 12258), (27213, 27218)], [(12254, 12258), (30896, 30900)], [(12274, 12280), (24437, 24443)], [(12274, 12280), (24528, 24534)], [(12281, 12285), (33639, 33643)], [(12285, 12293), (24226, 24234)], [(12293, 12300), (19357, 19364)], [(12293, 12300), (20188, 20195)], [(12308, 12313), (24266, 24270)], [(12321, 12328), (24634, 24641)], [(12328, 12334), (26066, 26072)], [(12334, 12340), (17977, 17983)], [(12342, 12347), (17555, 17560)], [(12347, 12355), (24226, 24234)], [(12355, 12359), (22988, 22992)], [(12364, 12368), (25916, 25920)], [(12368, 12376), (21528, 21535)], [(12379, 12385), (22552, 22558)], [(12385, 12396), (30123, 30134)], [(12396, 12403), (17272, 17279)], [(12405, 12409), (30016, 30020)], [(12409, 12413), (15427, 15431)], [(12419, 12423), (27258, 27262)], [(12423, 12428), (25120, 25125)], [(12432, 12437), (15379, 15384)], [(12432, 12437), (20323, 20328)], [(12432, 12437), (21483, 21488)], [(12432, 12437), (25364, 25369)], [(12449, 12456), (22252, 22258)], [(12478, 12483), (33054, 33059)], [(12483, 12494), (30034, 30045)], [(12504, 12511), (19176, 19183)], [(12511, 12518), (23330, 23337)], [(12523, 12526), (13601, 13604)], [(12526, 12532), (19644, 19650)], [(12534, 12543), (22479, 22488)], [(12534, 12543), (25299, 25308)], [(12551, 12558), (15408, 15415)], [(12558, 12566), (19303, 19311)], [(12577, 12586), (13658, 13666)], [(12586, 12592), (22908, 22914)], [(12594, 12600), (18841, 18847)], [(12609, 12611), (24850, 24852)], [(12609, 12611), (30785, 30787)], [(12614, 12616), (16277, 16279)], [(12616, 12624), (26355, 26363)], [(12629, 12632), (25064, 25067)], [(12629, 12632), (25946, 25949)], [(12629, 12632), (26114, 26117)], [(12632, 12636), (14153, 14157)], [(12632, 12636), (19402, 19406)], [(12632, 12636), (21593, 21597)], [(12632, 12636), (23267, 23271)], [(12640, 12644), (28409, 28413)], [(12640, 12644), (33004, 33008)], [(12649, 12652), (14779, 14782)], [(12652, 12656), (19973, 19977)], [(12652, 12656), (28817, 28821)], [(12658, 12665), (14471, 14478)], [(12668, 12675), (14782, 14789)], [(12668, 12675), (20542, 20549)], [(12668, 12675), (21930, 21937)], [(12668, 12675), (29111, 29119)], [(12679, 12685), (29003, 29013)], [(12695, 12698), (19895, 19898)], [(12698, 12702), (14066, 14070)], [(12709, 12711), (33635, 33637)], [(12711, 12716), (19848, 19853)], [(12716, 12720), (29443, 29447)], [(12726, 12730), (17060, 17064)], [(12732, 12741), (22479, 22488)], [(12732, 12741), (25299, 25308)], [(12749, 12754), (17296, 17301)], [(12755, 12759), (22781, 22785)], [(12759, 12767), (24349, 24357)], [(12759, 12767), (28505, 28513)], [(12774, 12780), (17646, 17652)], [(12787, 12790), (23010, 23013)], [(12790, 12795), (14037, 14042)], [(12799, 12811), (15572, 15584)], [(12811, 12815), (16307, 16311)], [(12815, 12822), (19606, 19613)], [(12815, 12822), (29079, 29086)], [(12822, 12830), (21619, 21627)], [(12822, 12830), (22431, 22439)], [(12822, 12830), (24050, 24058)], [(12822, 12830), (28676, 28684)], [(12830, 12835), (16500, 16505)], [(12830, 12835), (22386, 22391)], [(12855, 12867), (23285, 23298)], [(12867, 12871), (25308, 25312)], [(12889, 12896), (19329, 19336)], [(12896, 12904), (21699, 21708)], [(12904, 12912), (32849, 32857)], [(12912, 12917), (27120, 27125)], [(12917, 12924), (16949, 16956)], [(12926, 12931), (19426, 19431)], [(12935, 12942), (17799, 17808)], [(12942, 12953), (25846, 25857)], [(12953, 12957), (25410, 25414)], [(12970, 12972), (22378, 22380)], [(12978, 12979), (28704, 28705)], [(12987, 12992), (18997, 19002)], [(13009, 13012), (16272, 16275)], [(13012, 13015), (19057, 19060)], [(13029, 13040), (26288, 26298)], [(13040, 13047), (15565, 15572)], [(13047, 13061), (25174, 25188)], [(13061, 13071), (24767, 24776)], [(13061, 13071), (25957, 25967)], [(13071, 13075), (16049, 16053)], [(13075, 13081), (14465, 14471)], [(13081, 13085), (19502, 19506)], [(13085, 13091), (13785, 13791)], [(13094, 13100), (20205, 20210)], [(13100, 13106), (15111, 15117)], [(13106, 13112), (18548, 18554)], [(13106, 13112), (18950, 18956)], [(13106, 13112), (29459, 29465)], [(13106, 13112), (29471, 29478)], [(13112, 13122), (29358, 29368)], [(13130, 13135), (28536, 28541)], [(13142, 13151), (27368, 27377)], [(13151, 13157), (28268, 28274)], [(13157, 13167), (23401, 23411)], [(13157, 13167), (23944, 23954)], [(13169, 13171), (17255, 17257)], [(13173, 13177), (19260, 19265)], [(13185, 13189), (17814, 17818)], [(13189, 13201), (30076, 30088)], [(13216, 13225), (15078, 15087)], [(13232, 13241), (28513, 28522)], [(13241, 13250), (14234, 14243)], [(13250, 13266), (14218, 14234)], [(13271, 13273), (16498, 16500)], [(13271, 13273), (17336, 17339)], [(13273, 13282), (14917, 14926)], [(13273, 13282), (25045, 25054)], [(13273, 13282), (30619, 30628)], [(13273, 13282), (31750, 31759)], [(13282, 13295), (24454, 24467)], [(13299, 13304), (14496, 14501)], [(13308, 13314), (22939, 22945)], [(13357, 13359), (15278, 15280)], [(13357, 13359), (21641, 21643)], [(13373, 13376), (15673, 15676)], [(13389, 13395), (31403, 31410)], [(13395, 13400), (19099, 19104)], [(13395, 13400), (22881, 22886)]]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat         
    def jeu_essai_Homme_1e_4e(self):
        pass
    def jeu_essai_Aragon(self):
        """ Condorcet
        Insensible à tout / taille bloc pivots 5
        Assertion d'ordre sur occs_deplaces vérifiée"""
        insertions = [[1762, 1797],[1808, 1813],[1924, 1925],[2038, 2049],[2244, 2469],[2486, 2487],[2512, 2544],[2577, 2586],[2593, 2600],[2645, 2647],[2774, 2775],[2856, 2892],[2904, 2905],[2925, 2934],[3008, 3009],[3020, 3143],[3190, 3217],[3245, 3246],[3258, 3317],[3333, 3377],[3501, 3502],[3550, 3561],[3604, 3605],[3697, 3704]]
        suppressions =  [[11, 12],[258, 270],[335, 367],[379, 400],[478, 479],[561, 564],[569, 573],[589, 590],[602, 603],[623, 664],[681, 682],[707, 749],[759, 761],[836, 837],[924, 925],[1037, 1157],[1169, 1170],[1356, 1407],[1414, 1485],[1596, 1597],[1710, 1762]]
        occs_deplaces = [[270, 335],[400, 433],[433, 478],[564, 569],[573, 589],[664, 681],[682, 707],[1407, 1414],[1485, 1596],[1597, 1710],[1813, 1924],[1925, 2038],[2179, 2244],[2469, 2486],[2487, 2512],[2544, 2577],[2586, 2593],[2600, 2645],[2816, 2821],[3317, 3333]]
        tous_remplacements = [ [141, 143],[499, 526],[805, 806],[892, 893],[904, 905],[1195, 1199],[1240, 1244],[1303, 1305],[1316, 1317],[2178, 2179],[2795, 2816],[2988, 2989],[3225, 3226],[3237, 3238],[3527, 3529],[3581, 3584],[3644, 3646],[3657, 3658]]
        lg_texte1 = 1762
        texte_original = self._readFile("PN 1er brouillon dernier état.txt") + self._readFile("PN ms déf dernier état.txt")     
        blocsCommuns = [[0, 11],[12, 141],[143, 258],[367, 379],[479, 499],[526, 561],[590, 602],[603, 623],[749, 759],[761, 805],[806, 825],[825, 836],[837, 884],[884, 892],[893, 904],[905, 912],[912, 924],[925, 1037],[1157, 1169],[1170, 1195],[1199, 1220],[1220, 1240],[1244, 1264],[1264, 1303],[1305, 1316],[1317, 1356],[1797, 1808],[2049, 2178],[2647, 2762],[2762, 2774],[2775, 2795],[2821, 2856],[2892, 2904],[2905, 2925],[2934, 2944],[2944, 2988],[2989, 3008],[3009, 3020],[3143, 3190],[3217, 3225],[3226, 3237],[3238, 3245],[3246, 3258],[3377, 3489],[3489, 3501],[3502, 3527],[3529, 3550],[3561, 3581],[3584, 3604],[3605, 3644],[3646, 3657],[3658, 3697]]
        lDepl = [([270, 335], [2179, 2244]),([400, 433], [2544, 2577]),([433, 478], [2600, 2645]),([564, 569], [2816, 2821]),([573, 589], [3317, 3333]),([664, 681], [2469, 2486]),([682, 707], [2487, 2512]),([1407, 1414], [2586, 2593]),([1485, 1596], [1813, 1924]),([1597, 1710], [1925, 2038])]
        resultat = Donnees.resultatAppli.Resultat(insertions, suppressions, occs_deplaces,tous_remplacements, lg_texte1, texte_original,blocsCommuns, lDepl)
        return resultat 
        