# -*- coding: UTF-8 -*-
# Copyright 2003 - 2008: Julien Bourdaillet (julien.bourdaillet@lip6.fr), Jean-Gabriel Ganascia (jean-gabriel.ganascia@lip6.fr)
# This file is part of MEDITE.
#
#    MEDITE is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MEDITE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Here is an implementation of longest common substring in python using
# Ukkonen's algorithm to construct a suffix tree:

import sys, time, random

# kludge: infinity is a very large number
inf = 100000

# Define a class for a node in the suffix tree
class SuffixNode(dict):
    def __init__(self):
        self.suffixLink = None # Suffix link as defined by Ukkonen

    def Print(self,str,ws=""):
        for t in self:
            k,p,s = self[t]
            if p == inf:
                print ("1. Module ukonen, Class SuffixNode, Methode Print, Ligne 36, %s%s" % (ws, str[k:]))                        # Modifiee 26.02.2019, ancienne ligne:   print "%s%s" % (ws, str[k:])
            else:
                print ("2. Module ukonen, Class SuffixNode, Methode Print, Ligne 38, %s%s" % (ws, str[k:p+1]))                     # Modifiee 26.02.2019, ancienne ligne:   print "%s%s" % (ws, str[k:p+1])
                s.Print(str,ws+"|"*(p-k+1))
        

class LCS:
    def __init__(self,str1,str2):
        str = str1 + str2
        inf = len(str)
        self.str = str   #Keep a reference to str to ensure the string is not garbage collected
        self.seed = SuffixNode() #Seed is a dummy node. Suffix link of root points to seed. For any char,there is a link from seed to root
        self.root = SuffixNode() # Root of the suffix tree
        self.root.suffixLink = self.seed
        self.root.depth = 0
        self.deepest = 0,0

        # For each character of str[i], create suffixtree for str[0:i]
        s = self.root; k=0
        for i in range(len(str)):
            self.seed[str[i]] = -2,-2,self.root
            oldr = self.seed
            t = str[i]
            #Traverse the boundary path of the suffix tree for str[0:i-1]
            while True:
                # Decend the suffixtree until state s has a transition for the string str[k:i-1]
                while i>k:
                    kk,pp,ss = s[str[k]]
                    if pp-kk < i-k:
                        k = k + pp-kk+1
                        s = ss
                    else:
                        break
                # Exit this loop if s has a transition for the string str[k:i] (itmeans str[k:i] is repeated);
                # Otherwise, split the state if necessary
                if i>k:
                    tk = str[k]
                    kp,pp,sp = s[tk]
                    if t == str[kp+i-k]:
                        break
                    else: # Split the node
                        r = SuffixNode()
                        j = kp+i-k
                        tj = str[j]
                        r[tj] = j, pp, sp
                        s[str[kp]] = kp,j-1, r
                        r.depth = s.depth + (i-k)
                        sp.depth = r.depth + pp - j + 1
                        if j<len(str1)<i and r.depth>self.deepest[0]:
                            # longueur du plus long segment, sa position de fin
                            self.deepest = r.depth,j-1
                elif t in s:                      # Modifiee 16.04.2019, ancienne ligne:  elif s.has_key(t): 
                    break
                else:
                    r = s
                # Add a transition from r that starts with the letter str[i]
                tmp = SuffixNode()
                r[t] = i,inf,tmp
                # Prepare for next iteration
                oldr.suffixLink = r
                oldr = r
                s = s.suffixLink
            # Last remaining endcase
            oldr.suffixLink = s
        # print ("3. Module ukonen, Class LCS, Methode __init__, Ligne 100, \noldr.suffixLink=", str(oldr.suffixLink))

    def LongestCommonSubstring(self):
        # print ("4. Module ukonen, Class LCS, Methode LongestCommonSubstring, Ligne 103 ") 
        return self.str[self.deepest[1]-self.deepest[0]+1:self.deepest[1]+1]

    def Print(self):
        # print ("5. Module ukonen, Class LCS, Methode Print, Ligne 107 ") 
        self.root.Print(self.str)

class SuffixNodeU(dict):
    def __init__(self, slink=None, depth=0):
        self.suffixLink = slink
        self.depth = depth
        # print ("6. Module ukonen, Class SuffixNodeU, Methode __init__, Ligne 114 ") 
        
class SuffixTrie(object):
    def __init__(self, sequence):
        self.sequence = sequence
        self.length = len(self.sequence)
        self.inf = self.length+1
        self.bottom = SuffixNodeU(depth=-1)
        self.root = SuffixNodeU(self.bottom, depth=0)
        top = self.root
        oldr2 = self.root                              # Ajoutee 20.03.2019. 
        for t in self.sequence:
            # pour chaque char t en position i, construction de ST(sequence[:i])
            self.bottom[t] = self.root #
            r = top
            assert isinstance(r,SuffixNodeU)
            while not (t in r):                 # Modifiee 16.04.2019, ancienne ligne:  while not r.has_key(t):   
                r2 = SuffixNodeU(depth=r.depth+1)
                r[t] = r2
                if not r is top: 
                    oldr2.suffixLink = r2 
                oldr2 = r2
                r = r.suffixLink
            oldr2.suffixLink = r[t]
            top = top[t]
        # print ("7. Module ukonen, Class SuffixTrie, Methode __init__, Ligne 139, \nself.sequence", str(self.sequence))
                
class SuffixTree(object):
    """Construit un suffix tree avec l'algorithme d'Ukkonen"""
    def __init__(self, sequence):
        self.sequence = sequence
        self.length = len(self.sequence)
        self.inf = self.length+1 # infini
        self.bottom = SuffixNodeU()
        self.root = SuffixNodeU(self.bottom)
        s = self.root
        k = 0 ; i = -1
        for t in self.sequence: 
            # pour chaque char t en position i, consturction de ST(sequence[:i])
            i += 1
            # transition t entre bottom en root
            self.bottom[t] = -2,-2,self.root
            # ajout de t en renvoie du end point de ST(sequence[:i])
            s,k = self.update(s,k,i) 
            # le end point n'est pas forc�ment explicite, on le canonize
            s,k = self.canonize(s,k,i)
            # end point + 1 char = active point de ST(sequence[:i+1])
        # print ("8. Module ukonen, Class SuffixTree, Methode __init__, Ligne 161, \nself.sequence", str(self.sequence)) 
            
    def update(self, s,k,i):
        """Ajout du caractère en i """
        # active point = s,k,i-1
        oldroot = self.root
        t = self.sequence[i]
        # active point = end point ? renvoie l'active point explicit�
        end_point,r = self.testAndSplit(s,k,i-1,t) 
        while not end_point:
            r2 = SuffixNodeU()
            r[t] = i,self.inf,r2 # cr�e nouvelle transition
            # cr�e lien suffixe du noeud de l'�tape pr�c�dente
            if not (oldroot is self.root): oldroot.suffixLink = r 
            oldroot = r
            s,k = self.canonize(s.suffixLink,k,i-1) # remonte lien suffixe
            end_point,r = self.testAndSplit(s,k,i-1,t) # end point ?
        if not (oldroot is self.root): oldroot.suffixLink = r
        # print ("9. Module ukonen, Class SuffixTree, Methode update, Ligne 180, \ns=", str(s), ", k=", str(k))
        return s,k # renvoie le end point

    def testAndSplit(self, s,k,p,t):
        """Test si s,k,p est le end point et sinon explicite et renvoir s,k,p"""
        # print ("10. Module ukonen, Class SuffixTree, Methode testAndSplit, Ligne 184 ") 
        if k <= p: # s,k,p implicite
            tk = self.sequence[k]
            k2,p2,s2 = s[tk]
            # le char suivant l'arc s[tk] = t ? => end point
            if t == self.sequence[k2+p-k+1]: return True,s 
            else:# sinon on split l'arc
                r = SuffixNodeU()
                tk2 = self.sequence[k2]
                end = k2+p-k
                s[tk2] = k2, end, r
                tk3 = self.sequence[end+1]
                r[tk3] = end+1, p2, s2
                return False,r
        # s,k,p explicite                
        elif not (t in s): return False,s # pas de tk-transition            # Modifiee 16.04.2019, ancienne ligne:   elif not s.has_key(t): return False,s # pas de tk-transition   
        else: return True,s # tk-transition => end point

    def canonize(self, s,k,p):
        """Si s,k,p implicite, renvoie son plus proche ancètre explicite"""
        # print ("11. Module ukonen, Class SuffixTree, Methode canonize, Ligne 204, \ns=", str(s), ", k=", str(k))
        if p < k: return s,k # s,k,p explicite
        else:
            tk = self.sequence[k]
            k2,p2,s2 = s[tk]
            # si l'arc qui commence en s[tk] est inf�rieur � w, on avance dessus
            while p2-k2 <= p-k:
                k = k+p2-k2+1
                s = s2
                if k <= p: # essaye un nouvel arc
                    tk = self.sequence[k]
                    k2,p2,s2 = s[tk]
            # print ("12. Module ukonen, Class SuffixTree, Methode canonize, Ligne 216, \ns=", str(s), ", k=", str(k))
            return s,k                
                    
            

def test_LCS():
    # print ("13. Module ukonen, Methode test_LCS, Ligne 222 ") 
    alphabet = "GATC"    # Alphabet for the strings
    eos ="$"            #End of string character (should not be in alphabet)
    if len(sys.argv)>1:
        N = int(sys.argv[1])
    else:
        N = 10
           
    N1 = random.randint(N,N)
    N2 = random.randint(N,N)
       
    x = "".join([random.choice(alphabet) for i in range(N1)])
    y = "".join([random.choice(alphabet) for i in range(N2)])
    
    x = x + eos
    y = y + eos
    print (x)                                   # Modifiee 26.02.2019, ancienne ligne:   print x
    print (y)                                   # Modifiee 26.02.2019, ancienne ligne:   print y
    x="""Louis Althusser
    
    " " non lugere, redere, neque re destari sed intelligere " Spinoza "
    
     FREUD ET LACAN
    
    Des amis m'ont, � bon droit, fait le reproche d'avoir parl� de Lacan en trois lignes (I) : d'avoir trop parl� de lui pour ce que j'en disais, et trop peu parl� de lui pour ce que j'en concluais. Ils me demandent quelques mots pour justifier et mon allusion, et son objet. Les voici, - quelques mots, o� il faudrait un livre.Dans l'histoire de la raison occidentale, les naissances font l'objet de tous les soins, pr�vision, pr�cautions, pr�vention, allocations et allu/ocutions, etc.. Le Pr�natal est instu/itutionnel. Quand une jeune science na�t, le cercle de famille est toujours d�j� pr�t pour ill. la jubilation, le bapt�me et les drag�es. Depuis longtemps tout ef/nfant, m�me trouv�, est r�put� fils d'un p�re, et quand c'est un enfant prodige, les p�res se battraient au guichet n'�tait la m�re, et le respect qu'on lui doit. Ainsi, dans notre monde plein, place est pr�vue pour la naissance, place est m�me pr�vue pour la pr�vision de la naissance, "prospec-tivl/e": comme �a, pas de surprise.
    A ma connaissance, dans le cours du XIX�me si�cle, deux ou trois enfants naquirent, qu'on n' ttendait pas : Marx ;Nietzche, Freud. Enfants "naturels", au sens o� la nature offense les m�urs, le bon droit, la morale et le savoir [(I) Cf. Revue de l'Enseignement Philosophique, Juin-Juillet1963, ill. Philosophie l/et Sciences humaines p.7 etNote 14, p.II.] vivre : nature, c'est la r�gle b/viol�e, la fille-m�re, donc l'absence de P�re l�gal. Un enfant sans p�re, la Raison occidentale le lui fait payer cher. Marw, Nietzsche, Freud, durent payer le prix parfois atroce de la survie: prix comp-tabilis� en exclusions, condamnations, injures, mis�res,faim et morts, ou folie. Ke ne parle que d'eux (on pourrait parler d'autres maudits, qui v�curent d/leur arr�t de mort dans la couleur, les sons ou l'�criture - je ne parle que d'eux parce qu'ils furent naissance de sciences, ou de principes de science.
    Que Freud ait pu jouir h/jusqu'� la fin d'un relatif confort personnel, qu'il ait eu l'�me assez ancr�e pour supporter, les interpr�tant, toutes les injures du si�cle, - voil� qui n'est peut-�tre pas sans rapport acec quelques unes des limites de son g�nie. Laissons ce point, qui fait pr�sentement l'objet des �tudes de Lacan. Consid�rons simplement la solitude de Freud, en son temps. Je ne parle pas de la solitude humaine (il eut des ma�tres et des a�/mis, il ne connut pas la faim), je parle de sa solitude th�orique. Car, quand il voulut penser, c'est-�-dire exprimer sous la forme d'un syst�me rigoureux de concepts abr/straits, la d�couverte extraordinaire qu'il retrouvait, chaque jour au rendez-vous de sa pratique, il eut beau se chercher des pr�c�dents th�oriques, des p�res en th�orie, il n'en trouva gu�re. Il dut subir et am�nager la situation th�orique suivante : �tre � lui-m�me son propre p�re, construire de ses mains d'artisan l'espace th�orique o� situer sa d�couverte, tisser avec des fils d'emprunts, pris de droite et de gauche,au jug�, le grand filet � n�uds o� prendre, dans les profondeurs de l'exp�rience aveugle le gongoresque poisson de l'inconscient,que les hommes disent muet, parce qu'il parle m�me quand ils dorment.
    Cela veut dire, pour parler en termes de Kant : Freud d�t penser sa d�couverte et sa pratique dans des concepts import�s, emprunt�s � la physique �nerg�tique, alors dominante, � l'�conomie politique et � la biologie de son temm/ps. Pas d'h�ritage l�gal derri�re lui : sauf un lot de concepts philosophiques (conscience, pr�conscient, inconscient, etc...) peut-�tre plus encombrants que f�conds ; aucun fonds l�gu�s par quelque anc�tre que ce soit : pour tout anc�tres, des artistes : Sophocle, Shakespeare, Goethe, des moralistes etc.. ill. /Th�oriquement, Freud a mont� seul son entreprise : produisant ses concepts propres, ses concepts "domestiques" sous la production des concepts import�s,d�rob�s plutot, car, - n�gligence du Code Napol�on, qui n'a pas encore rang� dans les biens meublew/s les concepts philosophiques (il n'y a pas de brevet d'invention en philosophie !), on peut encore emprunter sans artorisation, c'est-�-dire sans sanction ,/(Code P�nal) un concept th�orique � un tiers.
    C'est ainsi que nous re��mes Freud. Longues suites de tr/extes, profonds, parfois clairs, parfois obscurs, souvent �nigme/atiques et contradictoires, probl�matiques, arm�s de concepts dont beaucoup nous semblent, � premi�re vue, p�rim�s, non ad�quats � leur contenu. Car nous ne doutons point,aujourd'hui, de l'existence de ce contenu : la pratique analytique elle-m�me.R�sumons donc cet objet qu'est pour nous Freud : I- Une pratique (la cure analytique). 2- Une technique (technique de la cure), qui donne lieu � une exposition abstraite,d'aspect th�orique. 3- Une th�orie, qui est en rapport avecla pratique et la technique. Cet ensemble organique pratique(I), technique (2), th�orique (3), nous rappelle la structure de toute s/discipline scientifique. Formellement, ce que Freud nous donne poss�de donc bien la structure d'une sciencr/e. Formellement : car les difficult�s de la terminologie conceptuelle de Freud, la disproportion parfois sensible entre ses concepts et leur contenu, nous conduit � poser la question :dans cet ensemble organique pratique-technique-th�orique, avons-nous affaire � une ensemble vraiment stabilis�, vraiment fix� au niveau scientifique ? Autrement dit, est-ce que la th�orie y est vraiment th�orie, au sens scientifique ? N'y est-elle pas souvent la simple transposition th�orique de la technique ? Et la technique la simple transposition m�thodologique de la pratique (la cure) ? De l� l'id�e, tr�s couramment admise, que sous ses dehors th�oriques (dus � une pr�tention respectable, mais vaine, chez Freud m�me ), la psycha/lyse demeure une pratique, simple pratique donnant parfois des r�sultats, mais pas toujoj/urs ; simple pratique prolong�e en technique (r�gles de la m�thode analytique) mais sans th�orie, du moins sans vraie th�orie : ce qu'elle donne comme th�oru/ie, n'�tant que les concepts techniques, donc aveul/g/g/les, o� elle r�fl�chit  sa pratique ; simple pratique sans th�orie... peut etre alors tout bonnement magie? qui r�ussirait comme toj/ute magien/, par l'eg/ffet de son prestige, et de ses prestiges, mis au service d'un besoin social,alors sa seule raison, sa vraie raison. Levy-Strauss aurait fait la th�orie de cette magie, de cette pratique sociale, en d�signant dans le chaman l'anc�tre de Freud. Pratique grosse d'une th�orie en partie silencieuse ? Pratique fi�re ou honteuse de n'�tre que la magie sociale des temps modernes ? Qu'est donc la psychanalyse ?
    - I - Le premier mot de Lacan est pour dire : Freud a fond� une science, au sens rigoureus du terme. Une science nouvelle, qui est la science d'un objet nouveau : l'inconscient. D�claration rigoureuse. Si la psychanalyse est bien une science, car elle est la science d'un objet propre, elle est aussi une science selon la structure de toute science : poss�dant une th�orie et une technique (m�thode) qui permettent la connaissance et la transformation de son objet dans une pratique sp�cifique. Comme en toute science authentique, la pratique n'est pas l'absolu de la science, mais un moment th�oriquement subordonn� ; le moment o� la th�orie devenue m�thode (technique) entre en contact th�orique (con-naissance) ou pratique (action : la cure) avec son objet propre (l'inconscient). Si cette th�se est exacte, la pratique analytique (la cure). qui absorbe toute l'attention des interpr�tes et des philosophes avides de l'intimit� du couple confidentiel o�l'aveu malade dt le secret professs/ionnel m�dical �changent les promesses sacr�es de l'intersubjectivit�, ne d�tient pas les secrets de la psychanalyse : elle d�tient seulement une partie de sa r�alit�, celle qui existe dans la pratique. Elle ne d�tient pas ses secrets th�oriques. Si cette th�se est exate, la technique, m�thode, ne d�tient pas non plus, sinon comme toute m�thode, c'est-�-dire par d�l�gation, non de la pratique, mais de la th�orie, les secrets de la psychanalyse. Seule, la th�orie les d�tient, comme en toute science. En cent lieux de son �uvre, Freud s'est dit th�oricien.En cent lieux de son �uvre, Freud, inlassablement, a compar� la psychanalyse, sous le rapport de la scientificit�, � la science physique issue de Galil�e. En cent lieux de son oeuvre, Freud a r�p�t� que la pratique (la cure) et la techni-que q/analytiques (la m�thode analytique) n'�taient authenti-quew/s que parce que fond�es sur une th�orie scientifiw/que ? En cent lieux, Freud a redit qu'une pratique et une technique, m�me f�condes, ne pouvaient m�riter le nom de scientifiques,que si une th�orie scientifique leur en donnait, non par simple d�claration, mais par fondation rigoureuse, le droit. Le premier mot de Lacan est de prendre ce mot � la lettre. Et d'en tirer la cons�quence : de revenir � Freud pour chercher, discerner et cerner en lui la th�orie dont tout le reste, tant technq/(ill.(/ie que pratique, est issu en droit. Revenir � Freud. Pourquoi ce nouveau retour aux sources ? Lacan ne revient pas � Freud comme Husserl � Galil�e ou � Thal�s, pour saisir une naissance � sa naissance,- c'est-�-dire pour r�aliser ce pr�jug� philosophique religieux de la puret�, qui, comme toute eau jaillisant au jour, n'est pure qu'� l'instant m�me, au pur instant, de sa naissance, au pur passage de la non-science � la science. Pour lui, ce passage n'est pas pur, il est encore impur : la puret� vient apr�s ce passage, non dans le passage encore "vaseux" (l'inviv/sible vase de son pass�, suspendue dans l'eau naissante, qui feint la trab/nsparence. Retour � Freud veut dire : retour � la th�orie bk/ien �tablie, bien fix�e, bien assise dans Freud m�me, � la th�orie m�re, m�rement r�fl�chie, m�rement �tay�e, m�rement v�rifi�e, � la th�orie assez avanc�e et install�e dans la vie (y compris la vie pratique) pour y avoir construit sa demeure, produit sa m�thode, et engendr� sa pratique. Le retour � Freud n'est pas un retour � la naissance de Freud :mais un retour � sa maturit�. La jeunc/esse de Freud, ce passage �mouvant de la non-encore-science � la science, peut nous int�resser, certes, mais � t/un tout autre titre : au titre d'un exemple d'arch�ologie d'une science,- ou comme indice n�gatif de non-(ill.(/turit�, alors pour bien dater la maturit� m�me. La jeus/nesse r/d'une science est son �ge m�r : avant cet �ge, elle est vieille, ayant l'�ge des pr�jug�s dont elle vit comme un enfant les pr�jug�s, dont/c l'�ge, de ses parents. Qu'une th�orie jeune, donc m�re, puisse retomber en enfance, c'est-�-dire dans les pr�jug�s de ses a�n�s et de leur descendance : toute l'histoire de la psychanalyse le prouve.C'est l� le sens profond du retour � Freud, proclam� par Lacan. Nous avons � revenir � Freud, pour revenir � la maturit� de la th�orie freudienne, non pas � son enfance, mais � son �h/ge m�r, qui est sa vraie jeunesse,- nous avons � revenir � Freud par de l� l'infantilisme th�orique, la retomb�e en enfance, o� la psychanalyse d'aujourd'hui savoure les avantages de son r�visionnisme. Cette retomb�e en enfance porte un nom, que les ph�nom�nologues comprendront d'embl�e : psychologisme.- ou un autre nom que des marxistes entendront : pragmatisme. Toute l'histoire moderne de la psychanalyse illustre le jugement de Lacan. La raison occidentale (raison juridique, morale et politique autant que scientifique) ne consentit en effet, apr�s des ann�es de m�connaissance, m�pris et injures - moyens d'q/ailleurs toujours disponibles le cas �ch�ant - � conclure un pacte de coexistence pacifique avec la psychanalyse, que sous la condition absolue de l'annexer � ses propres mythes : la psychologie, qu'elle soit behaviouriste (Dalbiez)ou ph�nom�nologique (Merleau-Ponty) ou existentialiste (Sartre) ; ou la biologie-neurologie, plus ou moins jaksonienne = jaksonienne (Ey) ; ou la "sociologie" de type "culturaliste" ou "anthropologique" (dominant aux USA : Kardiner Malinowski  M. Mead etc). A cette mythfication de la psychanalyse, discipline reconnue officiellement, au prix dd/"alliances-cop/mpromis scell�s avec des lign�es imaginaires d'adoption, un tr�s grand nombre de psychanalystes souscrivirent, trop heureux de sortir enfin de leur ghetto th�orique, d'�tre "reconnus" comme membres de plein droit de la Grande Famile (sic( de la p/Psycholo-gie, de la Neurologie, de la Psychi�trie, de la M�decine, de la Sociologie, de l'Antr/hropologie, de la Philosophie, - ttop heureux de donner � leur succ�s pratique le label de cette reconnaissance "th�orique" qui leur conf�rait enfin, apr�s des d�cades d'injures et d'ostracisme, droit de cit� dans le bon monde : celui de la s/Science, de la M�decine et de la Philosophie. Ils n'avaient pas pris garde au tour suspect de cet accord, pensant que le monde se rendaient � leurs raisons,-quand il se rendaient, sous les honneurs, aux raisons de ce monde, - pr�f�rant en somme ses honneurs � ses injures. Par l�, ils oubliaient qu'une science n'est telle que si elle peut, de plein droit, pr�tendre posss�der un objet propre, - qui soit le sien, et ne soit que le sien, - et non la portion congrue d'un objet conc�d� par une autre science, un de ses "aspects", un de ses restes, qu'on peut toujours accommoder dans les cuisines � sa mani�re, une foi repu le ma�tre de maison. De fait, si tout de la psychanalysese r�duit au "conditionnement" behaviouriste a/ou pavlovien de la petite enfance ; s'il se r�duit � une dialectique des stades de maturation m/neuro-bio-motrice, co�ncidant grossomodo avec les stades d�crits par Freud sous la terminologie de l'oral, de l'anal et du g�nital, de la latence et de la pubert� ; s'il se r�duit au conflit entre l'instinct de plaisir (libido, plus ou moins biologique ou plus ou moins"vitale") et le principe de r�alit� (normes et obligations de la r�alit� ext�rieure : la soci�t�) ; s'il se r�duit enfin � l'exp�rience originaire de la lutte des consciences hegellienne, du pour-autrui ph�nom�nologique, ou de la "b�ance" de l'Etre heideggerien ; si toute la psychanalyse n'est que cet art d'accommoder les restes de la neurologie, de la biologie, de la psychologie, de l'anthropologie et de la philosophie, que lui revient-il alors pour objet, qui la distingue vraiment de ces disciplines et fasse d'elle une science majeure? C'est l� qu'intervient, une fois encore, Lacan : pour d�fendre, contre toutes ces "r�ductions" et d�viations qui dominent aujourd'hui les interpr�tations th�oriques de l'analyse, son irr�ductibilit�, qui n'est que l'irr�ductibililit� de son objet. Qu'il faille � cette d�fense une lucidit� et fermet� hors du commun, aptes � repousser tous les assauts de l'hoi/spitalit� d�vorante des disciplines �num�r�es, nul n'o/en peut douter, qui aura, une fois dans sa vie, mesur� le besoin de s�curit� (th�orique, moral, social, �conomique), c'ew/st-�-dire l'inqui�tude des corporations (dont le statut esy/t indissolublement scientifique-professionnel-juridique-�conomique)menac�es dans leur �quilibre et confort par l'appariy/tion d'une science singuli�re, qui force chacun � s'iterroger, non seulement sur sa discipline, mais sur ses raisons d'y croire,c'est-�-dire de ne pas douter par l'apparition d'une science qui, pour peu qu'on y croie, risque de bouleverser l/es fronti�res existantes, donc le statu quo des disciplines m�dicales, donc l'int�grit� de leur territoire intellectuel, social, �conomique et moral. De l�, le tour passinonn� du langage de Lacan, qui ne peut vivre et survivre qu'� l'�tat d'alerte et de pr�/�vention : langage d'un homme assi�g�. condamn� par la force d�mesur�e des structures concern�es, donc des corporations et des hommes, � devancer leurs coups, � fein-dre au moins de les rendre avant de les avoir re�us, d�courageant ainsi l'adversaire de l'�craser sous les siens. De l� aussi, ce recours, souvent paradoxal, � la caution de philosophies du tout au tout �trang�res � son entreprise scientifique (Hegel, Heidegger), comme � autant de t�moins d'intimi-dation, jet�s au visage des uns pour les tenir en respect ;comme autant de t�moins d'une objectivit� possible, alo/li�e naturelle de sa pens�e, pour rassurer ou enseigner les autres. Que ce recours n'ait pas �t� sans �quivoque, c'est certain. Mais que ce recours ait �t� quasi indispensable poj/ur soutenir un discours adress� du dedans au seuls m�de-v/cins il faudrait ignorer tant pa pauvret� conceptuelle des �tudes m�c/dicales en g�n�ral, que le profond besoin de th�orie des meilleurs m�decins, surtour/t des jeunes, pour le condamner sans appel. Et puisque j'en suis � son langage, qui fait pour les uns tout le prestige de Lacan ("Gongora de la psychanalyse", "Grand dragon", grand officiant d'un culte �sot�rique o� le geste, le mutisme et la componction, peuvent compo-ser aussi bien le rituel d'une ferveur et d'une communication, - que d'une fascination tr�s "parisienne"), - et pour les autres (savants ou philosophes au premier rang. son "artifice", sa singularit�. et sa difficult�, il faut qu'on voie qu'il n'est pas sans rapport avec les conditions de son exercice p�dagogique : ayant � enseigner la th�orie de l'inconscient � des m�decins, anak/lystes ou analys�s, Lacan devait leur donner, dans la rh�torique de sa parole, l'�quivallent mim� du langage de l'inconscient, qui est, comme chacun sait, en son essence ultime, "witz", calembour, m�taphore rat�e ou r�ussie : l'�quivallent de l'exp�rience v�d/cue das/ns leur pratique, qu'elle soit de/'ah/nalyste ou d'analys�. Mais un langg/age p�dagogiw/que n'est jamais li� au destin de son public ni marqu� par lui. que si le p�dagogue se m�conna�t p�dago-gue. Un p�dagogue qui se conna�t p�dagogue, d�s qu'il cesse d'enseigner un tel public, se t/retrouve libre en son langage, -comme peut l'�tre Lacan, qu'un philosophe, pour peu qu'il se en donne la peine, peut aujourd'hui, au prix de quelques cl�s proprement lire � livre ouvert. Il suffit, pour ce faire, de reconna�tre que Lacan donne enfin � la pens�e et � l'exp�rience de Freud, les concepts scientifique (qu'elles exigent ; qu'il les leur donne � la fois en d�finissant son objet, et les "lois" de cet objet.
    II. Quel est l'objet de la psychanl/alyse ? - Ce � quoi la technique analytique a affaire dans la pratique analytique de la cure, c'est-�-dire : non pas la cure m�me, non pas cette situation pr�tendu/�ment duelle o� la premi�re ph�nom�nologie ou morale venues trouve � satisfaire son attente, - mais les effets, prolong�s dans l'adulte survivant, de l'effrayante aventure qui, de la naissance � la liquidation de l'Oedipe, transforme/a e/un petit animal engendr� par un homme et une femme, en petit enfant humain. Ce devenir-humain du petit �tre biologique issu de la parturition humaine : voil�, en son lieu, l'objet de la psychanalyse. Que ce petit �tre biologique survive, et au lieu de survivre enfant des boiw/s dec/venu petit de loups ou d'ours(on en montrait dans les cours princi�res du XVIII�me s.), si/urvive comme enfant humain (ayant �chapp� � toutes les morts de l'enfance, dont combien sont des morts humaines, morts sanctionnant l'�chec du devenir-humain), telle est l'�preuve effrayante que tous les hommes, adultes, ont surmont�e : ils sont, � jamais amn�siques, les t�moins, et bien souvent les victimes de cette victoire, pp/ortant au plus sourd, c'est � dire au plus bavard, d'eux-m�mes, les blessures, infirmit�s et courbatures de ce combat pour la vie ou la mort humaines. Certains, la plupart, en sont sortis � peu pr�s indemnes, -ou du moins tiennent, � haute voi, � bien le faire savoir - ;beaucoup de ces anciens combattants en restent marqu�s pourla vie ; certains mourront, un peu plus tard, de leur combat,les vieilles blessures soudain rouvertes dans l'explosion psychotique, dans la folie, ou le destin d'une n�vrose d'angoisse, L'Humanit� n'inscrit que ses morts officiels sur les M�moriaux de ses g/Guerres : ceux qui ont su mourir � temps. hommes, dans des guerres humaines, o� ne se d�chirent et sacrifient que des loups et dieux humains. La psychanalyse, o/enses c/seule/s survivants, s'occupe d'une autre guerre, atroce,de la seule Go/uu/erre sans M�moire ni M�moriaux, que l'Humanit� feint de n'avoir jamais livr�e, celle qu'elle pense avoir toujours gagn�e d'avance, tout simplement parce qu'elle n'est que de lui avoir surv�cu, de vivre et s'enfanter dans la Culu/tu_re humaine : Guerre qui, � chaque instant, se livre en chacun de ses rejetons, qui ont, jet�s, de/�jet�s, rejet�s,chacun pour soi, dans la solitude et la mort, � parcourir la L/longue Marche forc�e qui de larves mamif�res, fait des enfants humains. Qu'en cet objet le biologiste ne trouve pas son cm/ompte : certes, cette histoire n'est pas biologique ! puisqu'ele/le est toute enti�re domin�e, d�s le d�part, par la contrainte for-c�e de l'Ordre Humain, que chaque m�re impose, dans l' "amour" maternel  m�me, par son rythme alimentaire et son dressage, au petit animal humain. Que l'histoire, la "o/sociologie", ou l'anthropologie n'y trouvent pas leur compte, que la "psychologie" s'y perde, rien d'�tonnant : puisque les premi�res ont affaire � la soci�t�, c'est-�-dire � ce qui n'est plus le petit animal, - qui ne devient humain que d'avoir franchi cet espace infini qui s�pare la vie de l'homme, la/e biologie de l'historique, la "nati/ure" de la "culture" et la derni�re a affaire � je ne sais quelle "nature humaine", qu'elle suppose toute faite, quand elle est justement contrainte de na�tre ou ne pas �tre ; Que la "psychologie" s'y perde rien d'�trange puisqu'elle pense avoir affaire � quelque "nature" ou "non nature" humaine, c'est � dire culturelle, alors que le na�tre ou n'�tre pas de l'humain( ou non) est l'objet de la psychanalyseque la "philosophie" y perde ses rep�res et repaires, certes ! puisque s/ces origines lui d�robent les seules Origines auxquelles elle rende hommage de son �tre : d/Dieu, la c/Conscience, la c/Culture. On soup�onnera que l'objet de la psychanalyse puisse �tre sp�cifique, et que la modalit� de sa mati�re, comme la sp�cificit� de ses "m�canismes" (pour reprendre un mot de Freud), d'un tout autre ordre que la mati�re ou les "m�canismes" dont le biologiste, le neurologue, l'anthropologue, le sociologi/ue, le psychologue et le philosophe ont a conna�tre. Il suffit  de reconna�tre cette sp�cificit�, dont/c la distinction d'objet qui la fonde, pour reconna�tre � la psychanalyse un droit entier � la sp�cificit� de ses concepts, ajust�s � la sp�cificit�de son objet.
    III. Lacan, qui le proclame � maintes reprises ne contesterait pas que, sans le surgissement d'une nouvelle science : la linguistique, son entreprise de th�orisation e�t �t� impossible. Ainsi en va t il en histoire des sciences, o� une science ne le devient souvent que par le d�tour d'autres sciences,non seulement de sciences existantes � son bapt�me, mais aussi de telle science nouvelle, parfois plus jeune qu'elle. L'opaque jet�e sur la th�orie freudienne par le mod�le de la physique �nerg�tiste de Maxwell, se trouve aujourd'hui dissip�e par les lumi�res que la linguistique structurale jette sur son obk/jet. Freud avait d�j� dit que tout tenait au langage ; et que le discours de l'inconscient �tait lui m�me structur� comme un langage. Freud dans son grand premier ouvrage,l/a Science des R�ves, qui n'est pas anecdotique ou superc/fi-ciel comme on le croit trop souvent, mais fondamental, en avait �tudi� les "m�canismes" oo/u "loise/", qui se r�duisent � deux : le d�placement, et la condensation, et � leur variantes propres. Lacan y reconnut deux figures essentielles d�sign�es par la Linguistique : la m�tonymie et la m�taphore. Par l�, le lapsus, l'acte manqu�, le mot d'esprit les syptomes, devenaient, comme les �l�ments du r�ve m�me :des Signifiants, inscrits dans la cha�ne d'un discours inconscient, doublant en silence, c'est-�-dire � voix assoudissante, dans k/la m�connaissance du "refoulement", la cha�ne du disx/cours verbal du sujet humain. Par l�, nous �tions introduit au paradoxe, formellement fal/milier � la Linguistique, d'un du/iscours dou(ill.(l(ill.(/e mais un, inconscient et verbal, n'ayant pour champ qu'un champ unique, sans nul qu-del� qu'en lui-m�me : le champ de la "Cha�ne signifiante" elle m�me. Par l�, toutes les acquisitions de Saussure et de la Linguistique id/ssue de lui, entraii/ent de plein droit dans l'intelligence des processus tant du discours de l'inconscient, que du discours verbal du sujet, et de leurs rapports, c'est-�-dire de leurs non-rapport. Par l�, toutes les interpr�tations philosophiques-d/i�/d�alistes de l'inconscient comme seconde conscience, de l'inconscient comme mauvaise-foi (Sartre�/), de l'inconscient comme survivance canc�reuse d'une structure inactuelle, d'un non-sens (Merleau-Ponty), toutes les interpr�tations de l'inconscient comme "�a" biologico-social (Jung)�taient rejet�es, r�cus�es, gomm�es. Restait � d�finir (je suis contraint au pire sch�matisme, mais comment y �chapper en quelques lignes ?) le sens de ce primat de la structure formelle du langage, de ses "m�canismes" dans la pratique de l'interpr�tation analytique, en fonction m�me de ce que l'analyse interpr�te : son objet,c'est-�-dire les effets actuels, chez les survivants, de l'"hominisation" forc�e. J'en demande pardon � Teilhard qui avait, souw/s ce terme, des vues autrement teligieuses)/( du petit animal humain. Il ne suffit pas, pour r�pondre � cette question, d'invoquer simplement le primat du langage. Qui est le seul objet et moyen de la pratique analytique. Tout ce qui advient dans la cure se joue bien dans le langage, et par le langage (y compris le silence, ses rythmes, ses c/scansions). Mais il fallait montrer pourqui/oi et comment le r�le exclusif du langage dans la cure, � la fois mati�re premi�re de la pratique analytique, et moyen de production de ses effets (le passage, comme dit Lacan, d'une "parole vide" � une "parole pleine") est fond� non seulement dans la seule pratique analytique, mais aussi dans son objet, qui fonde en derni�re insstance et cette pratique et sa technique : donc, puisque science il y a, dans la de son objet. C'est l�, sans nul doute, la partie la plus profonde et la plus personnelle de l'�uvre de Lacan. Ce passage de l'exs/istence (� la limite purement)biologique � l'existence humaine (enfant d'homme), Lacan a montr�, avec clart� et rigueur qu'il s'op�rait sous la Loi de l'Ordre [verso folio 16] et que cette loi de l'Ordre (humain)se confondait dans son essence formelle avec l'ordre du Langage. Qu'entendre par cette formule, au premier abord �nigmatique ? D'abord que le tout de ce passage ne peut s'apprr/�hender que sous les esp�ces d'un langage r�current, que d�sign� par l/c/e langage soit de l'adulte, soit de l'enfant en �tat de cure, d�sign�, assign�, localis�, sous la loi du langage, en quoi se fixe et donne tout ordre humain, donc tout r�le humain. Ensuite que, dans cette assignation m�me par le langage, transpara�t la pr�sence actuelle, perp�tu�e, de l'efficace absolue de l'Ordre dans le passage m�me.Indiquons le en quelques mots tr�s brefs, et marquons � cet effet les deux grands moments de ce passage. 1/ Le moment de la relation duelle, pr�-oedip�enne, o� l'enfant n'ayant affaire qu'� un alter-ego, la m�re, qui scande sa vie de sa pr�sence (da !) et de son absence (v/fort !) vit cette relation duelle sur le mode de la fascination imaginaire de l'ego, �tant lui-m�me cet autre, tel autre, tout autre,tous les autres de l'identification narcissique primaire, sans pouvoir jamais prendre vis-�-vis de l'autre ni de soi la distance objectivante du Tiers ; 2/ Le moment de l'Oedipe,o� une structure ternaire surgit sur le fond de la structure duelle, quand le Tiers se m�le en intrus � la satisfaction imaginaire de la fascination duelle, en bouleverse l'�conomie, rompt les fascinations de l'imaginaire, et introduit l'enfant � ce que Lacan appelle l'Ordre Symbolique, celui du langage objectivant, qui lui permettra enfin de dire : je, tu, il ou elle, qui permettra donc au petit �tre de se situer comme enfant humain dans un monde de tiers adultes. Deux f/grands moments donc : 1/ celui de l'Imaginaire(pr�-oedipien) 2/ celui du Su/ymbolique (l'Oedipe r�solu),ou, pour parler un langage diff�rent, celui de l'objectivit� reconnue dans son usage (symbolique), mais non encore connue (la connaissance de l'objectivit� relevant d'un tout autre "�ge", et aussi d'une tout autre pratique.)Et voici le point capital, dont nous devons la prise conceptuelle � Lacan seul : ces deux moments sont domin�s, gouvern�s, et marqu�s par une unique Loi, celle du Symbolique. Le moment de l'Imaginaire lui-m�me, qu'on vient de pr�senter, quelques lignes plus haut, pour la clart�, comme pr�c�dant le symbolique, comme distinct de lui, - donc k/le premier moment o� l'enfant vit sa relation il/mm�diate avec un �tre humain (m�re) sans la reconna�tre pratiquement, comme une relation symbolique, c'est-�-dire comme la relation d'un petit enfant humain avec une m�re humaine), est marqu� et structur� en sa dialectique par la dialectique m�me de o/P/Ordre Symbolique, c'est-�-dire de l'Ordre humain, de la Norme humaine, (les normes des t/rythmes temporels de l'alimentation,de l'hygi�r/ne, des comportements, des attitudes con/cr�tes de la reconnaissance, d'acceptation, de refus, de oui et de non � l'enfant, n'�tant que la meni/ue monnaie, les modalit�s empiriques de cet Ordre constituant, Ordre de la Loi et di/uDroit d'assignation attributoire ou exclusoire) sous la forme m�me de l'Ordre du Signifiant. c'est-�-dire sous la forme d'un o/Ordre formellement identiqi/ue � l'ordre du Langage.L� o� une certaine lecture de Freud ne voyait que l'enfance heureuse et sans lois, le paradis de la "perversit� polymorphe", une sorte d'�tat de nature scand�e seulement par des stades d'allure biologique, attach�s � la primaut� fonctionnelle de telle partie du corps humain, lieux de besoins"vitaux" (oral, anal, g�nital), Lacan lit, con�oit et montre l'efficad/ce de l'Ordre, de la Loi, qui guette d�s avant sa nas/issance tout petit d'homme � na�tre, et s'empare de lui d�s son premier cri, pour lui assigner sa place et son r�le, donc sa destination forc�s ( Toutes les �tapes franchies par le petit d'homme ne sont, sous le r�gne de la Loi, du code d'assignation et de communication (donc de non-communication) humains : ces satisfactions portent en elles la marque ind�l�bile et cn/onstituante de la Loi, de la pr�tention de la Loi humaine, qui,comme toute loi, n'est "ignor�e" de personne, surr/tout de ses ignorants, mais peut �tre tourn�e ou viol�e par chacun, surtout par ses purs fid�les. C'est pourquoi, toute r�duction des traumatismes enfantins au seul d�compte des "frustrations" biologiques, est fauss�e dans son principe, puisque la Loi qui fait, comme L/Loi, abstraction de tout contenu, n'existe et n'agit comme Loi que dans cette abstraction, et que le petit d'homme subit et re�oit cette r�gle humaine d'airain avec son premier souffle. Par l� commence, a toujours d�j� commenc�, m�me sans aucun p�re vivant, ce qui est la pr�sence du P�re(qui est Loi), donc de l'Ordre du Signifiant humain, c'est-�-dire de la Loi de culture : ce discours, condition absoo/lue de tout discours, ce discours pr�sent de haut, c'est-�-dire absent dans l'ab�me, en tout discours verbal, le discours de cet Ordre, ce discours de l'Autre, du grand Tiers, qui est cet Ordre m�me : le dio/ss/cours de l'inconscient. Par l� nous est donn�e une prise, conceptuelle, sur l'inconscient, qui est, en chaque �tre humain le Lieu absolu o� son discours cherche son propre lieu, son topos, cherche, rate, et parfois trouve son propre lieu, de l'ancre propre de son lieu, dans l'imposition,l'imposture, la complicit� et la d�n�gation de ses propres fascinations imaginaires.Qu'en l'Oee/dipe l'enfant devienne enfant humain, en mettnt � l'�preuve du Symbolique ses fantasmes imaginaires, et fini/ns/isse, si tout "va", par devenir et s'as/ccepter ce qu'il est : petit gar�p/on ou petite fille parmi des adultes, ayant des droits d'enfant dans ce monde d'adultes, et poss�dant, comme tout enfant,le plein droit de devenir un jour "comme papa", c'est-�-dire un �tre humain masculin, ayant une femme (et non plus seulementune m�re), ou "comme maman", c'est-�-dire un �tre humain f�minin ayant un �poux (et non seulement un p�re), - cela n'est que le terme de la Longue Marche forc�e vers l'enfance humaine. Qu'en cet ultime drame, tout se joue dans la mati�re d'un langage form� pr�c�demment et tout entier centr� ordonne  autour du phallus, : insigne du P�re, insigne du Droit, insigne de la Loi, imagophantasmatique de tout Droit, - voil� qui peut para�tre �tonnant ou arbitraire, - mais tous les psychanalystes l'attesc/tent pour un fait d'exp�rience. La derni�re �tape de l'Oedipe, la "castration", peut en donner l'id�e. Lorsque le petit gar�on vit et r�sout la situation tragique et b�n�fique de la castration, il accepte de ne pas avoir le m�me Droit (phallus) que son p�re, en particulier de ne pas avoir le Droit du P�re sur sa m�re, qui se r�v�le alors sous l'intol�rable statut du double emploi, m�re pour le petit gar�on,(femme pour le p�re ; mais assumant de ne pas avoir le m�me Droit que son p�re, il y gagne l'assurance d'avoir un jour, plus tard, quand il sera devenu adulte, le Droit qui lui est alors refus�, faute de "moyens". Il n'a qu'un petit Dri/oit, qui devient grand s'il sait lui-m�me deveb/nir grand, ayant bien "mang� sa soupe". Lorsque de son c�t� la petite fille vit et assume la situation tragique et b�n�fique de la castration, elle accepte de ne pas avoir le m�me Droit que sa m�re, elle accepte donc doublement de ne pas avoir le m�me Droit (phallus) que son p�re, puisque sa m�re ne l'a pas (pas de phallus), bien que femme, parce que femme, et elle accepte en m�me temps de n'avoir pas le m�me Droit que sa m�re, c'est-�-dire de n'�tre pas encore une femme, comme l'est sa m�re. Mais elle y gagne en revanche son petit Droit :celui de petite fille, et les promesses d'un grand Droit, Droit entier de femme, quand eo/lle sera devenue adulte, si elle si/at/it grandir, en acceptant la l/Loi de l'Ordre humain, c'est-�-dire en soumettant, au besoin pour la tourner, - en ne mangeant pas "bien" sa soupe. Dans tous les cas, soit le moment de la fascination duelle de l'Imaginaire (1/), soit le moment (Oedipe)de la reconnaissance v�cue de l'insertion dans l'Ordre symbolique (2 :),toute la dialectique du passage est marquo/�e en son essence derni�re par le sceau de l'Ordre humain, du Sybolique, dont la Linguistique nous donne les lois formelles, c'est-�-dire le concept La th�orie psychanalytique peut nous donner ainsi ce qui fait de toute science, non une pure sp�culation, mais une science : la d�finition de l'essence formelle de son objet, condition de possibilit� de toute action pratique, technique, sur ses objets concrets eux-m�mes. Par l�, la th�orie psychanalytique �chappe aux antinomies classiques formul�es par exemple par Politzz/er, lorsque cet auteur, conscience aig�e de la contradiction, exigeant de la psychanalyse quelle f�t une science du "concret", vraie "psychologie concr�te", lui reprochait ses abstractions : l'inconscient, le complexe d'�dipe, le comm/plexe de castration, etc. Comment, disait Politzer, la psychanalyse peut-elle pr�tendre � �tre, si elle persiste dans es abstractions, qui ne sont que le "concret" ali�n� dans une psychologie abstraite et m�taphysique ? Comment rejoindre le "concret" � partid/r de telles abstractions, de l'abstrait ? En v�rit�, nulle science ne peut se passer de/'abstractions, m�me lorsqu'elle n'a, dans sa "pratique" (qui n'est pas la pratique th�orique de cette science, mais la pratique de son application concr�te) affaire qu'� d/ces variations singuli�res et uniques que sont les "drames" individuels. Telles que Lacan les pense dans Freud, - et Lacan ne pense rien d'autre que les concepts de Freud m�me, en leur donnantla forme de notre scientificit�, la seule scientificit� qui soit, - les "abstractions" de la psychanalyse sont f/bien les authentiqi/ues concepts de leur objet, dans la mesure o�, en tant que concepts de leur objet, elles cong/tiennent en elles l'indice: la mesure et le fondement de la n�cessit� de leur abstraction, c'est-�-dire la mesure m�me de leyr rapport au "concret", donc leur propre rapport au concret de leur pratique(application) appliqu�e (la cure)     L'�dipe n'est donc pas un "sens" cach�, � qui ne manque que la conscience, - l'�dipe n'est pas une struv/cture enfouie dans le pass�, qu'il est toujours possible de restructurer ou d�passer en en "r�activant le sens" ; l'Oedu/ipe est la structure dramatique th��trale  [verso folio 22] imp�/os�e par l'Ordre humain(le n�tre) � tout candidat, involontaire et forc� � l'humanit�, une strcture contenant en elle-m�me non seulement la possibilit�, mais la n�cessit� des variations concr�tes dans lesquelles elle existe, pour tout individu qui peut parvenir � son seuil, le vivre et lui survivre. La psychanalyse, das/ns sa pratique (la cure). travaille sur les effets [verso folio 22] concrets de s/ces variations, c'est-�-dire sur la modalit� de la nodalit� sp�cifique et absolument singuli�re dans laquelle le passage de l'Od/edipe a �t� abord�, franchi, partiellement rat� ou �lud� par tel ou tel individu. Ces variations peuvent p�tre pens�es et connues dans leur essence m�me � partir de le structure de l'invariant �dipe, pour la raison, pr�cis�m�/ent, qui fait que t�/out ce passage a �t� marquo/�, d�s son pr�alable de la fascination, dans ses formes les plus aberrantes, comme dans ses formes les plus "b/normales", par la Loi de cette structure, ultime forme de l'acc�s au Symbolique sous la Loi du m�me du Symbolique.Il faut �tre Sartre [en marge, folio 24], c'est-�-dire moral au point d'�tre pure conscience, pour croire que la mort d'un p�re puisse se confondre avec la mort de la Loi, qu'on puise s'�pargner l'�dipe, et courir le monde "sans surmoi". la "petite mer-vee": l'inconscient, dont sa "conscience" (de la mauvaise foi), �videmment, pr�sec/rve Sartre, �crit pourtant, sous son nez, ces trois mots en quatre vof/cables : la petite-m�re-veille, Sartre le dit tout seul : il �tait une petite-m�re-veille, ce merveilleux petit enfant, qui savait veiller sur soi comme s'il e�t �t� sa propre m�re, en l'absence m�me de sa m�re, qui lui �tait une �gale, sa s�ur, pour la jubilation les embrassades et la compassion. Sa m�re, traqua/�e, pouvait le laisser le c�ur l�ger : il veillait lui-m�me sur soi,/.Associons... Une merveilleuse-petite-p/merveille : une m�re-veil-leuse-petite-m�re-veille. M�re en veilleuse, puisque l'enfant veilla/ait sur soi. Sartre le dit tout seul : il ne fallut pas moins d'un P�re patent� (un "�minent psychanalyste") pour le bien assurer qu'il n'avait pas de surmoi. Comme le disait nagu�re un autre Ma�tre : "si tout est rose, rien n'est rose", - ce qui peut se dire aussi : "si le rose n'est pas, tout est rose.". Par quoi s'�claire sans doute que Sartre n'ait ci/ure d'�crire sa m/Morale, puisqu'il n'a cru qu'elle et n'�crit qu'elle. Les Rois sont toujours nus. Gr�ce � Lacan, nous saurons sans appel que, si tout le reste est litt�rature, et d'abord cette litt�rature sur la psychanalyse, la psychanlyse, d�cid�mment, m�rite un autre respect : celui qu'on accorde ou refuse � une science. 
    """
    y="""L. Althusser 
    
    Freud et Lacan
    Note liminaire 
    Disons-le sans d�g/tour : qui veut aujourd'hui tout simplement comprendre la d�couverte r�volutionnaire de Freud, non seulement reconna�tre son existence, mais aussi conna�tre son sens, doit traverser, au prix de grands efforts critiques et th�oriques, l'immense espace de pr�jug�s id�ologiques qui nous s�pare de Freud. Car non seulement la d�couverte de Freud � �t�, comme on va le voir, r�duite � des disciplines qui lui sont, en leur essence, �trang�res (biologie, psychologie, sociologie, philosophie) ; non seulement de nopp/mbreux psychanalystes (notamment dans l'�cole am�ricaine) se sont faits les complices de ce r�visionnisme ; mais, qui plus est, ce r�visionnisme a lui m�me objectivement servi la prodigieuse exploitation id�ologique dont la psychanalyse a �t� l'objet et la victime. Ce n'est pas sans raison que nagu�re (en 1948) des marxistes fran�ais d�nonc�rent dans cette exploitation une "id�ologie r�actionnaire", servant d'argument dans la lutte id�ologique contre le marxisme, et de moyen pratique d'intimidation et de mystification des consciences. Mais on peut bien dire aujourd'hui que ces m�mes marxistes furent, � leur mani�re, directement ou indirectement, les premi�res victimes de l'id�ologie qu'ils d�non�aient : puisqu'ils la confondirent avec la d�couverte r�volutionnaire de Freud, acceptant ainsi dans le fait les positions de l'adversaire, subissans/t ses propres conditions, et reconnaissant tout bonnement dans l'image qu'il leur imposait la pr�tendue r�alit� de la psychanalyse. Toute l'histoire pass�e des rapports entre le marxisme et la psychanalu/yse repose, pour l'essentiel, sur cette confusion et cette imposture. Qu'il f�t particuli�rement malais� d'y �chapper, nous le comprenons d'abord par la fonction de cette id�ologie : les id�es "dominantes" ayant, en l'esp�ce, jou� � la perfection leur r�le de "domination", s'imposant � leur insu aux esprits m�mes qui voulaient les combattre. Mais nous le comprenons aussi par l'existence du r�visionnisme psychanalyt/tique qui rendit possible cette exploitation : la chute dans l'id�ologie commen�a en effet par la chute de la psychanalyse dans le biologisme, le psychologisme et le sociologisme. Que ce r�visionnisme ait pu s'autoriser de l'�quivoque de certains concepts de Freud, qui fut contraint, comme tout inventeur, de penser sa d�couverte dans des concepts th�oriques existants, donc constitu�s � d'autres fins, nous pouvons aussi le comprendre (Marx ne fut-il pas lui aussi contraint de penser sa d�couverte dans des certains concepts hegeliens ?). Il n'est rien en cela qui puisse surprendre un esprit un peu averti de l'histoire des sciences, et particuli�rement de l'histoire des d�couvertes inaugurant des sciences nouvelles, - et soucieux de cerner l'irr�ductible d'une d�couverte et de son objet dans les concepts qui l'exprim�rent � sa naissance, et qui, rendus inactuels par le progr�s des connaissances, peuvent ult�rieurement la masquer. Faire retour � Freud impose donc aujourd'hui 1/ non seulement qu'on r�cuse comme une grossi�re mystification la couche id�ologique de l'exploitation r�actionnaire 2/ mais encore qu'on �vite de tomber dans les �quivoques, plus subtiles, et soutenues par les prestiges de quelques disciplines plus ou moins scientifiques, du r�visionnisme psychanalytique 3/ et qu'on se consacre enfin � un travail s�rieux et critique historico-th�orique pour id/dentifier et d�finir, dans les concepts que Freud dut employer, le v�ritable rapport �pist�mologique existant entre ces concepts et le contenu qu'ils pensaient. Sans ce triple travail de critique id�ologique (1,2) et d'�lucidation �pist�mologique (3), inaugur� en France par Lacan, la d�couverte de Freud restera, en sa sp�cificit�, hors de notre port�e. Et, ce qui est autrement grave, nous prendrons pour Freud justement ce qu'on a mis � notre port�e, que nous voulions le refuser (l'exploitation id�ologique r�actionnaire), ou que, plus ou moins inconsid�r�ment, nous y souscrivions (les diff�rentes formes du r�visionnisme bio-psycho-sociologique). Dans les deux cas nous demeurerions prisonniers, � des niveaux diff�rents, des cat�gories explicites ou implicites de l'exploitation et du r�visionnisme id�ologiques. Les marxistes, qui savent d'exp�rience quelles d�formations furent impos�es par ses adversaires � la pens�e de Marx, peuvent comprendre que Freud ait pu subir le m�me destin, et quelle doit �tre l'importance th�oriqur/e d'un authentique "retour � Freud". Il (sic( voudront bien admettre qu'un aussi bref article, qui se propose d'aborder un probl�me de cette importance, s'il ne veut pas le trahir, doive se borner � l'essentiel : situer l'objet de la psychanalyse, pour en donner une premi�re d�finition, dans les concepts qui permettent la localisation, pr�alable indispensable � l'�lucidation de cet objet. Il (sic( voudront bien admettre en cons�quence qu'on fasse intervenir ces concepts dans leur forme rigoureuse, comme le fait toute discipline scientifique, sans les affadir en un commentaire de vulgarisation approximative, ni sans entreprendre de les d�velopper vraiment en une analyse qui exigerait un tout autre espace. L'�tude s�rieuse de Freud et de Lacan, que chacun peut entreprendre, donnera seule ma mesure exacte de ces cp/oncepts, et permettra de d�finir les probl�mes en suspens dans une r�flexion th�orique d�j� riche de r�sultats et de promesses. 
    L.A. 
    Des amis m'ont, � bon droit, fait le reproche d'avoir parl� de Lacan en trois lignes (1) : d'avoir trop parl� de lui pour ce que j'en disais, et trop peu parl� de lui pour ce que j'en concluais. Ils me demandent quelques mots pour justifier et mon allusion, et son objet. Les voici, -quelques mots, o� il faudrait un livre. Dans l'histoire de la Raison Occidentale, les naissances font l'objet de tous les soins, pr�vision, pr�cautions, pr�vention, allocations et allocutions, etc... Le Pr�natal est institutionnel. Quand une jeune science na�t, le cercle de famille est toujours d�j� pr�t pour l'�tonnement, la jubilation, le bapt�me et les drag�es. Depuis longtemps tout enfant, m�me trouv�, est r�put� fils d'un p�re, et quand c'est un enfant prodige, les p�res se battraient au guichet n'�tait la m�re, et le respect qu'on lui doit. Ainsi, dans notre monde plein, place est pr�vue pour la naissance, place est m�me pr�vue pour la pr�vision de la naissance, "prospective" : ainsi, pas de surprise. A ma connaissance, dans le cours du XIX�me si�cle, deux ou trois enfants naquirent, qu'on n'attendait pas : Marx, Nietzsche, Freud. Enfants "naturels", au sens o� la nature offense les m�urs, le bon droit, la morale et le savoir-vivre : nature, c'est la r�gle viol�e, la fille-m�re, donc l'�bscence (sic( de P�re l�gal. Un enfant sans p�re, la Raison Occidentale le lui fait payer cher. Marx, Nietzsche, Freud, -durent acquitter la note, parfois atroce, de la survie : prix comptabilis� en exclusions, condamnations, injures, mis�res, faim et morts, ou folie. Je ne parle que d'eux (on pourrait parler d'autres maudits, qui v�curent leur arr�t de mort dans la couleur, les sons ou le po�me) - je ne parle que d'eux parce qu'ils furent naissance de sciences, ou de principes de critique et de science. Que Freud ait connu la pauvret�, la calomnie, et la pers�cution, qu'il ait eu l'�me assez ancr�e pour supporter, les interpr�tant, toutes les injures du si�cle, - voil� qui n'est peut-�tre pas sans rapport avec quelques unes des limites et des impasses de son g�nie. Laissons ce point, dont l'examen est sans doute pr�matur�. Consid�rons simplement la solitude de Freud, en son temps. Je ne parle pas de la solitude humaine (il eut des ma�tres et des amis, bien qu'il connut la faim), je parle de sa solitude th�orique. Car, quand il voulut penser, c'est-�-dire exprimer sous la forme d'un syst�me rigoureux de concepts abstraits, la d�couverte extraordinaire qu'il retrouvait, chaque jour au rendez-vous de sa pratique, il eut beau se chercher des pr�c�dents th�oriques, des p�res en th�orie, il n'en trouva gu�re. Il d�t [sic] subir et am�nager la situation th�orique suivante : �tre � lui-m�me son propre p�re, construire de ses mains d'artisan l'espace th�orique o� situer sa d�couverte, tisser avec des fils d'emprunts, pris de droite et de gauche, au jug�, le grand filet � n�uds o� prendre, dans les profondeurs de l'exp�rience aveugle le redout/<nd>ant poisson de l'inconscient, que les hommes disent muet, parce qu'il parle m�me quand ils dorment. Cela veut dire, pour s'exprimer dans des termes de Kant : Freud dut penser sa d�couverte et sa pratique dans des concepts "import�s", emprunt�s � la physique �nerg�tique, alors dominante, � l'�conomie politique et � la biologie de son temps. Pas d'h�ritage l�gal derri�re lui : sauf un lot de concepts philosophiques (conscience, pr�conscient, inconscient, etc...) peut-�tre plus encombrants que f�conds ; aucun fond l�gu� par quelque anc�tre que ce soit : pour tout anc�tres des aetistes (sic( : Sophocle, Shakespeare, Moli�re, Goethe, - des sentences, etc... Th�oriquement, Freud a mont� seul son affaire : produisant ses concepts propres, ses concepts "domestiques", sous la production des concepts import�s, emprunt�s � l'�tat des sciences existantes, et, il faut bien le dire dans l'horizon du monde id�ologique o� baign�rent ces concepts. [verso folio 5] C'est ainsi que nous re��mes Freud. Longue suite de textes, profonds, parfois clairs, parfois obscurs, souvent �nigmatiques et contradictoires, probl�matiques, arm�s de concepts dont beaucoup nous semblent, � premi�re vue, p�rim�s, non ad�quats � leur contenu. Car nous ne doutons point, aujourd'hui, de l'existence de ce contenu : la pratique analytique elle-m�me, son effet. R�sumons donc cet objet qu'est pour nous Freud : 1- Une pratique (la cure analytique). 2( Une technique (m�thode de la cure), qui donne lieu � une exposition abstraite, d'aspect th�orique. 3( Une th�orie qui est en rapport avec la pratique et la technique. Cet ensemble organique pratique (1), technique (2), th�orique (3), nous rappelle la structure de toute discipline scientifique. Formellement, ce que Freud nous donne poss�de donc bien la structure d'une science. Formellement : car les difficult�s de la terminologie conceptuelle de Freud, la disproportion parfois sensible entre ses concepts et leur contenu, nous conduit � poser la question : dans cet ensemble organique pratique-technique-th�orique, avons-nous affaire � un ensemble vraiment stabilis�, vraiment fix� au niveau scientifique ? Autrement dit, la th�orie y est-elle vraiment th�orie, au sens scientifique ? N'y est-elle pas souvent la simple transposition m�thodologique de la pratique (la cure) ? De l� l'id�e, tr�s couramment admise, que sous ses dehors th�oriques (dus � une pr�tention respectable, mais vaine, chez Freud m�me), la psychanalyse demeurerait une simple pratique donnant parfois des r�sultats, mais pas toujours ; simple pratique prolong�e en technique (r�gles de la m�thode analytique) mais sans th�orie, du moins sans vraie th�orie : ce qu'elle d�clare th�orie, n'�tant que les concepts techniques aveugles, o� elle r�fl�chit les r�gles de sa pratique ; simple pratique sans th�orie... peut-�tre alors tout bonnement magie ? qui r�ussirait comme toute magie, par l'effet de son prestige, et de ses prestiges, mis au service d'un besoin ou demande sociaux, alors sa seule raison, sa vraie raison. L�vi Strauss aurait fait la th�orie de cette magie, de cette pratique sociale que serait la psychanalyse, en d�signant dans le chaman l'anc�tre de Freud. Pratique grosse d'une th�orie en partie silencieuse ? Pratique fi�re ou honteuse de n'�tre que la magie sociale des temps modernes ? Qu'est donc la psychanalyse ? 
    - I - Le premier mot de Lacan est pour dire : dans son principe Freud a fond� une science, au sens rigoureux du terme. Une science nouvelle, qui est la science d'un objet nouveau : l'inconscient. D�claration rigoureuse. Si la psychanalyse est bien une science, car elle est la science d'un objet propre, elle est aussi une science selon la structure de toute science : poss�dant une th�orie et une technique (m�thode) qui permettent la connaissance et la/a transformation de son objet dans une pratique sp�cifique. Comme en toute science authentique, la pratique n'est pas l'absolu de la science, mais un moment th�oriquement subordonn� ; le moment o� la th�orie devenue m�thode (technique) entre en contact th�orique (connaissance) ou pratique (la cure) avec son objet propre (l'inconscient). Si cette th�se est exacte, la pratique analytique (la cure) qui absorbe toute l'attention des interpr�tes et des philosophes avides de l'intimit� du couple confidentiel o� l'aveu malade et le secret professionnel m�dical �changent les promesses sacr�es de "l'intersubjectivit�", ne d�tient pas les secrets de la psychanalyse : elle d�tient seulement une partie de sa r�alit�, celle qui existe dans la pratique. Elle ne d�tient pas ses secrets th�oriques. Si cette th�se est exacte, la technique, m�thode, ne d�tient pas non plus, sinon comme toute m�thode, c'est-�-dire par d�l�gation, non de la pratique, mais de la th�orie, les secrets de la psychanalyse. Seule, la th�orie les d�tient, comme en toute discipline scientifique. En cent lieux de son �uvre, Freud s'est dit th�oricien. En cent lieux de son �uvre, Freud, inlassablement, a compar� la psychanalyse, sous le rapport de la scientificit�, � la science physique issue de Galil�e. En cent lieux de son �uvre, Freud a r�p�t� que la pratique (la cure) et la technique analytique (la m�thode analytique) n'�taient authentiques/s que parce que fond�es sur une th�orie scientifique. En cent lieux, Freud a redit qu'une pratique et une technique, m�me f�condes, ne pouvaient m�riter le nom de scientifiques, que si une th�orie leur en donnait, non par simple d�claration, mais par fondation rigoureuse, le droit. Le premier mot de Lacan est de prendre ce mot � la lettre. Et d'en tirer la cons�quence : de revenir � Freud pour chercher, discerner et cerner en lui la th�orie dont tout le reste, tant technique que pratique, est issu en droit. Revenir � Freud. Pourquoi ce nouveau retour aux sources ? Lacan ne revient pas � Freud comme Husserl � Galil�e ou � Thal�s, pour saisir une naissance � sa naissance, - c'est-�-dire pour r�aliser ce pr�jug� philosophique religieux de la puret�, qui, comme toute eau jaillissant au jour, n'est pure qu'� l'instant m�me, au pur instant, de sa naissance, au pur passage de la non-science � la science. Pour lui, ce passage n'est pas pur, il est encore impur : la puret� vient apr�s ce passage, elle n'est pas dans le passage encore "vaseux" (l'invisible vase de son pass�, suspendue dans l'eau naissante, qui feint la transparence, c'est-�-dire l'innocence). Retour � Freud veut dire : retour � la th�orie bien �tablie, bien fix�e, bien assise dans Freud m�me, � la th�orie m�re, m�rement r�fl�chie, m�rement | �tay�e, m�rement v�rifi�e, � la th�orie assez avanc�e et install�e dans la vie ( y compris la vie pratique) pour y avoir construit sa demeure, produit sa m�thode, et engendr� sa pratique. Le retour � Freud n'est pas un retour � la naissance de Freud : mais un retour � sa maturit�. La jeunesse de Freud, ce passage �mouvant de la non-encore-science, � la science, peut nous int�resser, certes, mais � un tout autre titre : au titre d'un exemple d'arch�ologie d'une science, - ou comme indice n�gatif de non-maturit�, alors pour bien dater la maturit� m�me et son av�nement. La jeunesse d'une science est son �ge m�r : avant cet �ge, elle est vieille, ayant l'�ge des pr�jug�s dont elle vit comme un enfant les pr�jug�s, donc l'�ge de ses parents. Qu'une th�orie jeune, donc m�re, puisse retomber en enfance, c'est-�-dire dans les pr�jug�s de ses a�n�s et de leur descendance : toute l'histoire de la psychanalyse le prouve. C'est l� le sens profond du retour � Freud, proclam� par Lacan. Nous avons � revenir � Freud, pour revenir � l/la maturit� de la th�orie freudienne, non pas � son enfance, mais � son �ge m�r, qui est sa vraie jeunesse, - nous avons � revenir � Freud par del� l'infantilisme th�orique, la retomb�e en enfance, o� la psychanalyse d'aujourd'hui savoure les avantages de ses abandons. Cette retomb�e en enfance porte un nom, que les ph�nom�nologues comprendront d'embl�e : psychologisme, - ou un autre nom que les marxistes entendront d'embl�e : pragmatisme. Toute l'histoire moderne de la psychanalyse illustre le jugement de Lacan. La Raison Occidentale (raison juridique, religieuse, morale et politique autant que scientifique) ne consentit en effet, apr�s des ann�es de m�connaissance, m�pris et injures -moyens d'ailleurs toujours disponibles le cas �ch'ant (sic( - � conclure un pacte de coexistence pacifique avec la psychanalyse, que sous la condition absolue de l'annexer � ses propres sciences ou � ses propres mythes : � la psychologie, qu'elle soit behaviouriste (Dalbiez) ou ph�nom�nologique (Merleau-Ponty) ou existentialiste (Sartre) ; � la bio-neurologie, plus ou moins jaksonienne (Ey) ; � la "sociologie" de type "culturaliste" ou "anthropologique" (dominant aux USA : Kardiner, M. Mead, etc). A cette mythfication (sic( de la psychanalyse, discipline reconnue officiellement, aux prix d'alliances-compromis scell�s avec des lign�es imaginaires d'adoption mais de tr�s r�els pouvoirs, un tr�s grand nombre de psychanalystes souscrivirent, trop heureux de sortir enfin de leur ghetto th�orique, d'�tre "reconnus" comme membres de plein droit de la Grande Famille de la Psychologie, de la Neurologie, de la Psychi�trie (sic(, de la M�decine, de la Sociologie, de l'Anthropologie, de la Philosophie, - trop heureux de coller sur leur succ�s pratique le label de cette reconnaissance "th�orique" qui leur conf�rait enfin, apr�s des d�cades d'injures et d'exil, droit de cit� dans le bon monde : celui de la Science, de la M�decine, de la Philosophie. Ils n'avaient pas pris garde au tour suspect de cet accord, croyant que le monde se rendait � leurs raisons, - quand eux-m�mes se rendaient, sous les honneurs, aux raisons de ce monde, - pr�f�rant ses honneurs � ses injures. Par l�, ils oubliaient qu'une science n'est telle que si elle peut, de plein droit, pr�tendre � la propri�t� d'un objet propre, - qui soit le sien, et ne soit que le sien, - et non � la portion congrue d'un objet pr�t�, conc�d�, abandonn� par une autre science, � l'un de ses "aspects", de ses restes, qu'on peut toujours accommoder dans les cuisines � sa mani�re, une fois le Patron repu. De fait, si le tout de la psychanalyse se r�duit au "conditionnement" behaviouriste ou pavlovien de la petite enfance ; s'il se r�duit � une dialectique des stades d�crits par Freud sous la terminologie de l'oral, de l'anal et du g�nital, de la latence et de la pubert� ; s'il se r�duit enfin � l'exp�rience originaire de la lutte h�lg�glienne, du pour-autrui ph�nom�nologique, ou de la "b�ance" de l'Etre heideggerien ; si toute la psychanalyse n'est que cet art d'accommoder les restes de la neurologie, de la biologie, de la psychologie, de l'anthropologie et de la philosophie, que lui revient-il alors en propre pour objet, qui la distingue vraiment de ces disciplines et fasse d'elle une science de plein droit ? (2) C'est l� qu'intervient, une fois encore, Lacan : pour d�fendre, contre toutes ces "r�ductions" et d�viations qui dominent aujourd'hui les interpr�tations th�oriques de l'analyse, son irr�ductibilit�, qui n'est que l'irr�ductibilit� de son objet. Qu'il faille, � cette d�fense, une lucidit� et fermet� hors du commun, aptes � repousser tous les assauts de l'hospitalit� d�vorante des disciplines �num�r�es, nul n'en peut douter, qui aura, une fois dans sa vie, mesur� le besoin de s�curit� (th�orique, moral, social, �conomique), c'est-�-dire l'inqui�tude des corporations (dont le statut est indisolublement (sic( scientifique-professionnel-juridique-�conomique) menac�es dans leur �quilibre et confort par l'apparition d'une discipline singuli�re, qui force chacun � s'interroger, non seulement sur sa discipline, mais sur ses raisons d'y croire, c'est-�-dire d'en douter par l'apparition d'une science qui, pour peu qu'on y croie, risque de bouleverser des fronti�res existantes, donc le statu quo des disciplines m�dicales, psychologiques et autres donc l'int�grit� de territoires intellectuels, sociaux �conomiques et moraux De l�, la passion contenue, la contention passionn�e du langage de Lacan, qui ne peut vivre et survivre qu'� l'�tat d'alerte et de pr�vention : langage d'un homme d'avance assi�g�, et condamn� par la force �crasante des structures des corporations et des hommes menac�s, � devancer leurs coups, � feindre au moins de les rendre avant de les avoir re�us, d�courageant ainsi l'adversaire de l'�craser sous les siens. De l� aussi, ce recours, souvent paradoxal, � la caution de philosophies du tout au tout �trang�res � son entreprise scientifique [soulign� ray� en bleu] (Hegel, Heidegger), comme � autant de t�moins d'intimidation, jet�s au visage des uns pour les tenir en respect ; comme � autant de t�moins d'une objectivit� possible, alli�e naturelle de sa pens�e, pour rassurer ou enseigner les autres. Que ce recours n'ait pas �t� sans �quivoques, c'est certain. Mais que ce recours ait �t� quasi indispensable pour soutenir un discours adress� du dedans aux seuls m�decins - il faudrait ignorer, tant la pauvret� conceptuelle des �tudes m�dicales en g�n�ral, que le profond besoin de th�orie des meilleurs m�decins, surtout des jeunes pour le condamner sans appel. Et puisque j'en suis � son langage, qui fait pour les uns tout le prestige de Lacan ("Gongora de la psychanalyse", "Grand dragon", grand officiant d'un culte �sot�rique o� le geste, le mutisme et la componction, peuvent composer le rituel aussi bien d'une communication fervente, - que d'une fascination tr�s "parisienne"), - et pour les autres (savants ou philosophes au premier rang) son "artifice", sa singularit� et sa difficult�, il faut qu'on voie qu'il n'est pas sans rapport avec les conditions de son exercice p�dagogique : ayant � enseigner la th�orie de l'inconscient � des m�decins, analystes ou analys�s, Lacan dut leur donner, dans la rh�torique de sa parole, l'�quivalent mim� du langage de l'inconscient, qui est, comme chacun sait, en son essence ultime, "witz", calembour, m�taphore rat�e ou r�ussie : l'�quivalent de l'exp�rience v�cue dans leur pratique, qu'elle soit d'analyste ou d'analys�. Mais un langage p�dagogique n'est jamais li� au destin de son public ni marqu� par lui, que si d'aventure le p�dagogue se m�conna�t p�dagogue, donc th�oricien. Un p�dagogue qui se conna�t, p�dagogue, qui donc se sait th�oricien de ce qu'il enseigne, d�s qu'il cesse d'enseigner ce public, peut se retrouver libre en son langage, - comme peut l'�tre Lacan qu'un philosophe, s'il s'en donne vraiment la peine, peut aujourd'hui, au prix de quelques cl�s, proprement lire � livre ouvert. Il suffit, � cette fin, de reconna�tre que Lacan conf�re enfin � la pens�e de Freud, les concepts scientifiques qu'elle exige ; qu'il les lui conf�re en d�finissant � la fois son objet, et les "lois" de cet objet. 
    - II - Quel est l'objet de la psychanalyse ? - Ce � quoi la technique analytique a affaire dans la pratique de la cure, c'est-�-dire : non pas la cure m�me , non pas cette situation pr�tend�ment duelle ou la premi�re ph�nom�nologie ou morale venue trouve � satisfaire son besoin, - mais les "effets" prolong�s dans l'adulte survivant, de l'extraordinaire aventure qui, de la naissance � la liquidation de l'�dipe, transforme un petit animal engendr� par un homme et une femme, en petit enfant humain. Ce devenir-humain [soulign� en bleu] du petit �tre biologique issu de la parturition humaine : voil�, en son lieu, l'objet de la psychanalyse qui porte le simple nom de l'inconscient. Que ce petit �tre biologique survive, et au lieu de survivre enfant des bois devenu petit de loup [ill.] ou d'ours (on en montrait dans les cours princi�res du XVIII�me si�cle), survive enfant humain (ayant �chapp� � toutes les morts de l'enfance, dont combien sont des morts humaines, morts sanctionnant l'�chec du devenir-humain), telle est l'�preuve que tous les hommes, adultes, ont surmont�e : ils sont, � jamais amn�siques, les t�moins, et bien souvent les victimes de cette victoire, portant au plus sourd, c'est-�-dire au plus criant d'eux-m�mes, les blessures, infirmit�s et courbatures de ce combat pour la vie ou la mort humaines. Certains, la plupart, en sont sortis � peu pr�s indemnes, - ou du moins tiennent, � haute voix, � bien le faire savoir - ; beaucoup de ces anciens combattants en restent marqu�s pour la vie ; certains mourront, un peu plus tard, de leur combat, les vieilles blessures soudain rouvertes dans l'explosion psychotique, dans la folie, l'ultime compulsion d'une r�action th�rapeutique n�gative. | | d'autres, les plus nombreux, le plus "normalement" du monde, sous le d�guisement d'une d�faillance " organique " [verso folio 12] . L'Humanit� n'inscrit que ses morts officiels sur les M�moriaux de ses Guerres : ceux qui ont su mourir � temps, c'est-�-dire tard, hommes, dans des guerres humaines, o� ne se d�chirent et sacrifient que des loups et des dieux humains. La psychanalyse, en ses seuls survivants, s'occupe d'une autre lutte, atroce, de la seule Guerre sans M�moire ni M�moriaux, que l'Humanit� feint de n'avoir jamais livr�e, celle qu'elle pense avoir toujours gagn�e d'avance, tout simplement parce qu'elle n'est que de lui avoir surv�cu, de vivre et s'enfanter comme Culture dans la Culture humaine : Guerre qui, � chaque instant, se livre en chacun de ses projets, jets, rejets, ses rejetons, qui ont, projet�s, jet�s, dejet�s, rejet�s, chacun pour soi, dans la solitude et contre la mort, � parcourir la Longue Marche forc�e, qui de larves mammif�res, fait des enfants humains, sujets. Qu'en cet objet le biologiste ne trouve pas son compte : certes, cette histoire n'est pas biologique ! puisqu'elle est toute enti�re domin�e, d�s le d�part, par la contrainte forc�e de l'Ordre Humain, que chaque m�re grave, sous "amour" ou haine maternelles, d�s son rythme alimentaire et dressage dans le petit animal humain. Que l'histoire, la "sociologie", ou l'anthropologie n'y trouvent pas leur compte, rien d'�tonnant : puisqu'elles ont affaire � la soci�t� donc � la Culture, c'est-�-dire � ce qui n'est plus le petit animal, - qui ne devient humain que d'avoir franchi cet espace infini qui s�pare la vie de l'humain, la biologique de l'historique, la "nature" de la "culture". Que la psychologie s'y perde, rien d'�trange ! puisqu'elle pense avoir, en son "objet", affaire � quelque "nature", ou "non-nature" humaine, � la gen�se de cet existant identifi� et enregistr� sous les Contr�les de la Culture (de l'humain), - quand l'objet de la psychanalyse est la question pr�alable absolue, le na�tre ou n'�tre pas, l'ab�me al�atoire de l'humain m�me en chaque rejeton d'homme. Que la "philosophie" y perde ses rep�res et repaires, certes ! puisque ces singuli�res origines lui d�robent les seules Origines auxquelles elle rende hommage de son �tre : Dieu, la Raison, la Conscience, l'Histoire et la Culture. On soup�onnera que l'objet de la psychanalyse puisse �tre sp�cifique, et que la modalit� de sa mati�re, comme la sp�cificit� de ses "m�canismes" (pour reprendre un mot de Freud), soient d'un tout autre ordre que la mati�re ou les "m�canismes" dont le biologiste, le neurologue, l'anthropologue, le sociologue, le psychologue et le philosophe ont � conna�tre. Il suffit de reconna�tre cette sp�cificit�, donc la distinction d'objet qui la fonde, pour reconna�tre � la psychanalyse un droit radical � la sp�cificit� de ses concepts, ajust�s � la sp�cificit� de son objet : l'inconscient.
    - III - Lacan ne contesterait pas que, sans le surgissement d'une nouvelle science : la linguistique, son entreprise de th�orisation e�t �t� impossible. Ainsi va l'histoire des sciences, ou une science ne le devient souvent que par le recours et le d�tour d'autres sciences, non seulement de sciences existantes � son bapt�me, mais aussi de telle science nouvelle tard venue, qui a besoin de temps pour na�tre. L'opaque provisoire de l'ombre port�e sur la th�orie freudienne par le mod�le de la physique �nerg�tiste de Helmoltz et Maxwell, se trouve aujourd'hui lev�e par le jour que la linguistique structurale jette sur son objet | | en permettant son acc�s. [verso folio 13] Freud avait d�j� dit que tout tenait au langage ; Lacan le pr�cise : "l/<L>e discours de l'inconscient est structur� comme un langage". Dans son grand premier ouvrage la Science des R�ves, qui n'est pas anecdotique ou superficiel comme on le croit souvent, mais fondamental, Freud en avait �tudi� les "m�canismes", ou "lois", r�duisant leurs variantes � deux : le d�placement et la condensation, Lacan y reconnut deux figures essentielles d�sign�es par la Linguistique : la m�tonymie et la m�taphore. Par l�, le lapsus, l'acte manqu�, le mot d'esprit et le sympt�me, devenaient comme les �l�ments du r�ve m�me : des Signifiants, inscrits dans la cha�ne d'un discours inconscient, doublant en silence, c'est-�-dire � voix assourdissante, dans la m�connaissance du "refoulement", la cha�ne du discours verbal du sujet humain. Par l�, nous �tions introduit au paradoxe, formellement familier � la Linguistique, d'un discours double et un, inconscient et verbal, n'ayant pour double champ qu'un champ unique, sans nul au-del� qu'en lui-m�me : le champ de la "Cha�ne signifiante". Par l�, les plus importantes des acquisitions de Saussure et de la Linguistique issue de lui, entraient de plein droit dans l'intelligence du processus tant du discours de l'inconscient, que du discours verbal du sujet, et de leur rapport, c'est-�-dire de leur non-rapport identique � leur rapport. Par l�, toutes les interpr�tations philosophiques-id�alistes de l'inconscient comme seconde conscience, de l'inconscient comme mauvaise-foi (Sartre), de l'inconscient comme survivance canc�reuse d'une structure inactuelle ou non-sens (Merleau-Ponty), toutes les interpr�tations de l'inconscient comme "�a" biologico-arch�typique (Jung) devenaient ce qu'elles �taient : non pas le degr� z�ro de la th�orie, mais des "th�ories" nulles, des malentendus id�ologiques. Restait � d�finir (je suis contraint au pire sch�matisme, mais comment y �chapper en quelques lignes ?) le sens de ce primat de la structure formelle du langage, et de ses "m�canismes", rencontr�s dans la pratique de l'interpr�tation analytique, en fonction m�me du fondement de cette pratique : son objet, c'est-�-dire les "effets" actuels, chez les survivants, de l'"hominisation" forc�e du petit animal humain. Il ne suffit pas, pour r�pondre � cette question, d'invoquer simplement le primat de fait du langage, qui est le seul objet et moyen de la pratique analytique. Tout ce qui advient dans la cure se joue bien dans le langage, et par le langage (y compris le silence, ses rythmes, ses scansions). Mais il faut montrer en droit pourquoi et comment le r�le de fait du langage dans la cure, � la fois mati�re premi�re de la pratique analytique, et moyen de production de ses effets (le passage, comme dit Lacan, d'une "parole vide" � une "parole pleine") n'est fond� en fait dans la seule pratique analytique, que parce qu'il est fond� en droit dans son objet, qui fonde en derni�re instance et cette pratique et sa technique : donc, puisque science il y a, dans la th�orie de son objet. C'est l�, sans nul doute, la partie la plus profonde et la plus personnelle de l'�uvre de Lacan : sa d�couverte. Ce passage de l'existence (� la limite purement) biologique, � l'existence humaine (enfant d'homme), Lacan a montr�, avec clart� et rigueur qu'il s'op�rait sous la Loi de l'Ordre que j'appellerai Loi de Culture, et que cette Loi de l'Ordre se confondait dans son essence formelle avec l'ordre du Langage. Qu'entendre par cette formule, au premier abord �nigmatique ? D'abord que le tout de ce passage ne peut s'appr�hender que sous les esp�ces d'un langage r�current, que d�sign� par ce langage, de l'adulte ou de l'enfant en situation de cure, d�sign�, assign�, localis�, sous la loi du Langage, en quoi se fixe et donne tout ordre humain, donc tout r�le humain. Ensuite que, dans cette assignation par le Langage de la cure, transpara�t la pr�sence actuelle, perp�tu�e, de l'efficace absolue de l'Ordre dans le passage m�me, de la Loi de Culture dans le devenir-humain. Pour l'indiquer en quelques mots tr�s brefs, marquons � cet effet les deux grands moments de ce passage. 1) Le moment de la relation duelle, pr�-oedipienne, o� l'enfant n'ayant affaire qu'� un alter-ego, la m�re, qui scande sa vie de sa pr�sence (da !) et de son absence (fort !) (3) vit cette relation duelle sous le mode de la fascination imaginaire de l'ego, �tant lui-m�me cet autre, tel autre, tout autre, tous les autres de l'identification narcissique primaire, sans pouvoir jamais prendre vis-�-vis de l'autre ni de soi la distance objectivante du tiers ; 2) Le moment de l'�dipe, o� une structure ternaire surgit sur le fond de la structure duelle, quand le p�re se m�le en intrus � la satisfaction imaginaire de la fascination duelle, en bouleverse l'�conomie, en rompt les fascinations, et introduit l'enfant � ce que Lacan appelle l'Ordre Symbolique, celui du langage objectivant, qui lui permettra enfin de dire : je, tu, il ou elle, qui permettra donc au petit �tre de se situer comme enfant humain dans un monde de tiers adultes. Deux grands moments donc : 1) celui de l'Imaginaire (pr�-oedipien) 2) celui du Symbolique (l'�dipe r�solu), ou, pour parler ici un langage diff�rent, celui de l'objectivit� reconnue dans son usage (symbolique), mais non encore connue (la connaissance de l'objectivit� relevant d'un tout autre "�ge", et aussi d'une tout autre pratique). Et voici le point capital, dont nous devons la prise conceptuelle � Lacan seul [soulign� ray� en bleu] : ces deux moments sont domin�s, gouvern�s, et marqu�s par une unique Loi, celle du Symbolique. Le moment de l'Imaginaire lui-m�me, qu'on vient de pr�senter, quelques lignes plus haut, pour �tre clair, comme pr�c�dant le symbolique, comme distinct de lui, - donc le premier moment o� l'enfant vit sa relation imm�diate avec un �tre humain (m�re) sans la reconna�tre-pratiquement comme la relation symbolique qu'elle est (c'est-�-dire comme la relation d'un petit enfant humain avec une m�re humaine), - est marqu� et structur� en sa dialectique par la dialectique m�me de l'Ordre Symbolique, c'est-�-dire de l'Ordre humain, de la Norme humaine, (les normes des rythmes temporels de l'alimentation, de l'hygi�ne, des comportements, des attitudes concr�tes de la reconnaissance, l'acceptation, le refus, le oui ou le non � l'enfant, n'�tant que la menue monnaie, les modalit�s empiriques de cet Ordre constituant, Ordre de la Loi et du Droit d'assignation attributoire ou exclusoire) sous la forme m�me de l'Ordre du Signifiant, c'est-�-dire sous la forme d'un Ordre formellement identique � l'ordre du langage. (4) L� o� une lecture superficielle ou orient�e de Freud ne voyait que l'enfance heureuse et sans lois, le paradis de la "perversit� polymorphe", une sorte d'�tat de nature scand�e seulement par des stades d'allure biologique, attach�s � la primaut� fonctionnelle de telle partie du corps humain, lieux de besoins "vitaux" (oral, anal, g�nital) (5), Lacan lit, con�oit et montre l'efficace de l'Ordre, de la Loi, qui guette d�s avant sa naissance tout petit d'homme � na�tre, et s'empare de lui d�s son premier cri, pour lui assigner sa place et son r�le, donc sa destination forc�e. Toutes les �tapes franchies par le petit d'homme le sont, sous le r�gne de la Loi, du code d'assignation et de communication (et de non-communication) humains, ses "satifactions" portent en elles la marque ind�l�bile et constituante de la Loi, de la pr�tention de la Loi humaine, qui comme toute loi, n'est "ignor�e" de personne, surtout de ses ignorants, mais peut �tre tourn�e ou viol�e par chacun, surtout par ses purs fid�les. C'est pourquoi, toute r�duction des traumatismes enfantins au seul d�compte des "frusturations biologiques, est fauss�e dans son principe, puisque la Loi qui les concerne fait, comme Loi, abstraction de tous les contenus, n'existe et n'agit comme Loi que par et dans cette abstraction, et que le petit d'homme subit et re�oit cette R�gle avec son premier souffle (6). Par l� commence, a  toujours d�j� commenc� m�me sans aucun p�re vivant, ce qui est la pr�sence en acte du P�re (qui est Loi), donc de l'Ordre du Signifiant humain, c'est-�-dire de la Loi de Culture : ce discours, condition absolue de tout discours, ce discours pr�sent de haut, c'est-�-dire absent en son ab�me, en tout discours verbal, le discours de cet Ordre, ce discours de l'Autre, de grand Tiers, qui est cet ordre m�me : le discours de l'inconscient. Par l� nous est donn�e une prise, conceptuelle, sur l'inconscient, qui est, en chaque �tre humain le Lieu absolu o� son discours singulier cherche son propre lieu, son tomos, cherche, rate, et, le ratant, trouve son propre lieu, l'ancre propre de son lieu, dans l'imposition, l'imposture, la complicit� et la d�n�gation de ses propres fascinations imaginaires. Qu'en l'�dipe l'enfant devienne enfant humain, en mettant � l'�preuve du Symbolique ses fantasmes imaginaires, et finisse si tout "va", par devenir et s'accepter ce qu'il est : petit gar�on ou petite fille parmi des adultes, ayant ses droits d'enfant dans ce monde d'adultes, et poss�dant, comme tout enfant, le plein droit de devenir un jour "comme papa", c'est-�-dire un �tre humain masculin, ayant une femme (et non plus seulement une m�re), ou "comme maman", c'est-�-dire un �tre humain f�minin ayant un �poux (et non seulement un p�re), - cela n'est que le terme de la Longue Marche forc�e vers l'enfance humaine. Qu'en cet ultime drame, tout se joue dans la mati�re d'un langage form� pr�c�demment, qui,  dans l'�dipe, se centre tout entier et s'ordonne [ray� en bleu] autour du Signifiant phallus : insigne du P�re, insigne du Droit, insigne de la Loi, image | phantasmatique de tout Droit, - voil� qui peut para�tre �tonnant ou arbitraire, - mais tous les psychanalystes l'attestent pour un fait d'exp�rience.`La derni�re �tape de l'�dipe, la "castration", peut en donner l'id�e. Lorsque le petit gar�on vit et r�sout la situation tragique et b�n�fique de la castration, il accepte de ne pas avoir le m�me Droit (phallus) que son p�re, en particulier de ne pas avoir le Droit du P�re sur sa m�re, qui se r�v�le alors dot�e de l'intol�rable statut du double emploi, m�re pour le petit gar�on, femme pour le p�re ; mais assumant de ne pas avoir le m�me Droit que son p�re, il y gagne l'assurance d'avoir un jour, plus tard, quand il sera devenu adulte, le Droit qui lui est alors refus�, faute de "moyens". Il n'a qu'un petit Droit qui deviendra grand s'il sait lui-m�me devenir grand, ayant bien "mang� sa soupe". Lorsque de son c�t� la petite fille vit et assume la situation tragique et b�n�fique de la castration, elle accepte de ne pas avoir le m�me Droit que sa m�re, elle accepte donc doublement de ne pas avoir le m�me Droit (phallus), que son p�re, puisque sa m�re ne l'a pas (pas de phallus), bien que femme, parce que femme, et elle accepte en m�me temps de n'avoir pas le m�me Droit que sa m�re, c'est-�-dire de n'�tre pas encore une femme, comme l'est sa m�re. Mais elle y gagne en revanche son petit Droit : celui de petite fille, et les promesses d'un grand Droit, Droit entier de femme, quand elle sera devenue adulte, si elle sait grandir, en acceptant la Loi de l'Ordre humain, c'est-�-dire en s'y soumettant, au besoin pour la tourner, - en ne mangeant pas "bien" sa soupe. Dans tous les cas, soit le moment de la fascination duelle de l'Imaginaire (1), soit le moment (�dipe) de la reconnaissance v�cue de l'insertion dans l'ordre symbolique (2), toute la dialectique du passage est marqu�e en son essence derni�re par le sceau de l'Ordre humain, du Symbolique, dont la Linguistique nous donne les lois formelles, c'est-�-dire le concept formel. La th�orie psychanalytique peut nous donner ainsi ce qui fait de toute science, non une pure sp�culation, mais une science : la d�finition de l'essence formelle de son objet, condition de possibilit� de toute application pratique, technique, sur ses objets concrets eux-m�mes. Par l�, la th�orie psychanalytique �chappe aux antinomies id�alistes classiques formul�es par exemple par Politzer, lorsque cet auteur, exigeant de la psychanalyse. | | (dont il fut le premier, en France, � saisir la port�e th�orique r�volutionnaire) [verso folio 18] qu'elle f�t une science du concret", vraie "psychologie concr�te", lui reprochait ses abstractions : l'inconscient, le complexe d'�dipe, le complexe de castration, etc. Comment, disait Politzer, la psychanalyse peut-elle pr�tendre � �tre la science du concret, qu'elle veut et peut �tre, si elle persiste dans des abstractions, qui ne sont que le "concret" ali�n� dans une psychologie abstraite et m�taphysique ? Comment rejoindre le "concret" � partir de telles abstractions, de l'abstrait ? En v�rit�, nulle science ne peut se passer d'abstractions, m�me lorsqu'elle n'a, dans sa "pratique" (qui n'est pas la pratique th�orique de cette science, mais la pratique de son application [soulign� en bleu] concr�te [soulign� ray� en bleu]) affaire qu'� ces variations singuli�res et uniques que sont les "drames" individuels. Telles que Lacan les pense dans Freud, - et Lacan ne pense rien d'autre que les concepts de Freud m�me, en leur donnant la forme de notre scientificit�, la seule scientificit� qui soit, - les "abstractions" de la psychanalyse sont bien les authentiques concepts scientifiques de leur objet, dans la mesure o�, en tant que concepts de leur objet, elles contiennent en elles l'indice, la mesure et le fondement de la n�cessit� de leur abstraction, c'est-�-dire la mesure m�me de leur rapport au "concret", donc leur propre rapport au concret de leur application, commun�ment appel�e pratique analytique (la cure). L'�dipe n'est donc pas un "sens" cach�, � qui ne manque que la conscience ou la parole, - l'�dipe n'est pas une structure enfouie dans le pass�, qu'il est toujours possible de restructurer ou d�passer en en "r�activant le sens" ; l'�dipe est la structure dramatique, la "machine th��trale" (1/7) impos�e par la Loi de Culture � tout candidat, involontaire et forc� � l'humanit�, une structure contenant en elle-m�me non seulement la possibilit�, mais la n�cessit� des variations concr�tes dans lesquelles elle existe, pour tout individu qui peut parvenir � son seuil, le vivre et lui survivre. La psychanalyse, dans son application, dite sa pratique, (la cure) travaille sur les "effets" (2/8) concrets de ces variations, c'est-�-dire sur la modalit� de la nodalit� sp�cifique et absolument singuli�re dans laquelle le passage � l'�dipe a �t�, est abord�, franchi, partiellement rat� ou �lud� par tel ou tel individu. Ces variations peuvent �tre pens�es et connues dans leur essence m�me, � partir de la structure de l'invariant �dipe, pour la raison, pr�cis�ment, qui fait que tout ce passage a �t� marqu�, d�s son pr�alable de la fascination, dans ses formes les plus "aberrantes" comme dans ses formes les plus "normales", par la Loi de cette structure, ultime forme de l'acc�s au Symbolique sous la Loi m�me du Symbolique. Je sais que ces br�ves indications non seulement para�tront, mais sont sommaires et sch�matiques, et que nombre de notions, ici invoqu�es ou avanc�es, exigeraient de longs d�veloppements pour �tre justifi�es et fond�es. M�me �clair�es dans leur fondement, et dans les rapports qu'elles entretiennent avec l'ensemble des notions qui les soutiennent, m�me rapport�es � la lettre des analyses de Freud, elles posent � leur tour des probl�mes, non seulement des probl�mes de forp/mulation, de d�finition et d'�claircissement des concepts pr�sent�s, mais des probl�mes r�els, ouverts, produits par le d�veloppement de l'effort de th�orisation dont il vient d'�tre question. Par exemple : comment penser rigoureusement le rapport entre la structure formelle du langage, condition de possibilit� absolue de l'existence et de l'intelligence de l'inconscient d'une part, les structures concr�tes de la parent� d'autre part, et enfin les structures id�ologiques dans lesquelles sont v�cues les fonctions distinctes (paternit�, maternit�, enfance) implq/ii/qu�es dans les structures concr�tes de la parent� ? Peut-on concevoir que la variation (historique) de ces derni�res structures (parent�, id�ologie) puisse affecter sensiblement sur tel ou tel aspect des instances isol�es par Freud ? Autre probl�me : dans quelle mesure la d�couverte de Freud, pens�e dans sa rationalit�, peut-elle, par la simple d�finition de son objet, et de son lieu, retenc/tir sur les disciplines dont elle se distingue, (par exemple la psychologie, la psychosociologie, la sociologie) et provoquer en elles des questions sur le statut (parfois probl�matique) de leur objet ? Derni�re question, enfin, parmi tant d'autres : quels sont les rapports existant entre la th�orie analytique et 1/ ses conditions d'apparition historique d'une part 2/ ses conditions sociale[sic] d'application d'autre part ? Qui donc �tait Freud pour avoir pu � la fois fonder la th�orie analytique, et inaugurer, comme Analyste N�1, analyste auto-analys�, P�re originaire, la longue filiation des praticiens qui se r�clament de lui ? Qui donc sont les psychanalystes, pour accepter � la fois la th�orie freudienne, la tradition didactique freudienne, et les conditions �conomiques et sociales (d�finies par le statut social d'une corporation �troitement rattach�e � la corporation m�dicale) dans lesquelles ils exercent ? Dans quelle mesure les origines historiques de la psychanalyse et les conditions sociales e/dt/e son exercice retentissent-elles sur la th�orie, et la pratique analytique ? Dans quelle mesure le silence des psychanalystes sur ces probl�mes, le refoulement dont ces probl�mes sont tacitement frapp�s dans le monde analytique affectent-ils et la th�orie et la technique analytiques ? L'�ternelle question de la "fin de l'analyse" n'est-elle pas directement en rapport avec ce refoulement, c'est-�-dire avec la non-pens�e de ces probl�mes ? Autant de probl�mes r�els vraiment ouverts, qui constitue [sic] d'orez/s et d�j� des champs de recherche. Il n'est pas impossible que certaines notions ne sortent, dans un proche avenir, transform�es de cette �preuve. Cette �preuve, c'est en d�finitive, celle � laquelle Freud a soumis en son domaine, une certaine image traditionnelle, juridique, morale et philosophique, en d�rinitive [sic] id�ologique de l'"homme", du "sujet" humain. Ce n'est pas en vain que Freud a paf/rfois compar� sa d�couverte � la r�volution copernicienne. Depuis Copernic nous savons que la terre n'est plus le centre de l'univers. Depus/is Marx, nous savons que le sujet humain, l'ego �conomique, politique ou philosophique n'est plus le centre de l'histoire, - nous savons m�me, contre Hegel, que l'histoire n'a pas de "centre", mais poss�de une structure qui n'a de centre que dans la m�connaissance id�ologique. Freud nous d�couvre � son tour que le sujet r�el, l'individu dans sa constitution singuli�re, n'a pas la structure d'un ego, centr� sur le moi, la conscience, ou l'"existence", - que ce soit l'existence du pour-soi, du corps-propre, ou du "comportement", - que le sujet humain est d�centr�, constitu� par une structure qui n'a de centre que dans la m�connaissance imaginaire du "moi"./, c'est-�-dire dans les formations id�ologiques o� il se "reconna�t". Par l�, nous est sans doute ouvere/te une des voies par lesquelles nous parviendrons peut-�tre un jour � une meilleure intelligence de cette structure de la m�connaissance qui int�resse au premier chef toute recherche sur l'id�ologie. 
    Notes de l'article : Freud et Lacan. 
    Note 1. (p. 5) Cf. Revue de l'Enseignement philosophique, juin-juillet 1963, Philosophie et Sciences Humaines, p. 7, et p. 11, note 14 : "Marx a fond� sa th�orie sur le rejet du mythe de l'"homo oeconomicus", Freud a fond� sa th�orie sur le rejeu/t du mythe de l'"homo psychologicus". Lacan a vu et compris la rupture lib�ratrice de Freud. Il l'a comprise dans le sens plein du terme, la prenant au mot de sa rigueur, et la for�ant � produire, sans tr�ve ni concessions, ses propres cons�quences. Il peut, comme chacun, errer dans le d�tail, voire dans le choix de ses rep�res philosophiques : on lui doit l'essentiel."
    Note 2. (p. 10) Les tentations les plus mena�antes sont repr�sent�es par la philosophie (qui r�duit volontiers toute psychanalyse � l'exp�rience duelle de la cure, et y trouve de quoi "v�rifier" les th�mes de l'intersubjectivit� ph�nom�nologique, de l'existence-projet, ou plus g�n�ralement du personnalisme.) ; par la psychologie, qui annexe, comme autant d'attributs d'un "sujet" qui, manifestement, ne lui pose aucun probl�me, la plupart des cat�gories de la psychanalyse ; par la sociologie enfin, qui, venant au secours de la psychologie, fournit dr [sic] quoi donner au "principe de r�alit�" son contenu objectif (les imp�ratifs sociaux et familiaux) que le "sujet" n'a plus qu'� "int�rioriser" pour �tre arm� d'un "surmoi" et des cat�gories correspondantes. Soumise ainsi � la psychologie ou � la sociologie, la psychanalyse se r�duit � une technique de r�adaptation "�motionnelle" ou "affective", � une r��ducation de la fonction relationnelle, qui n'ont �videmment rien � voir avec son objet r�el, - mais qui r�pondent malheureusement � une forte demande, et qui plus est, tr�s orient�e, dans le monde contemporain. C'est par ce biais que la psychanalyse est devenue un objet de consommation courante dans la culture, c'est-�-dire dans l'id�ologie moderne.
    Note 3. (p. 16) Ce sont l� les deux expressions de langue allemande, rendues c�l�bres par Freud, - par lesquelles un petit enfant qu'il observait, sanctionnait l'apparition et la disparition de sa m�re, dans la manipulation d'un objet quelconque qui la "figurait".
    Note 4. (p. 17) Formellement. Car la Loi de Culture, dont le langage est la forme et l'acc�s premiers, ne s'�puise pas dans le langage : elle a pour contenu les structures de la parent� r�elles, et les formations id�ologiques d�termin�es dans lesquelles les personnages inscrits en ces structures vivent leur fonction. Il ne suffit pas de savoir que la famille occidentale est patriarchale et exogamique (structure de la parent�), - il faut aussi �lucider les formations id�ologiques qui gouvernent la paternit�, la maternit�, et l'enfance : qu'est-ce que "�tre p�re", "�tre m�re", "�tre enfant" ? dans notre monde pr�sent ? Sur ces formations id�ologiques sp�cifiques, tout un travail de recherche reste encore � accomplir.
    Note 5. (p. 17) Une certaine neuro-biologie et une certaine psychologie ont �t� fort aise de d�couvrir dans Freud une th�orie des "stades" qu'elles ont sans h�siter directement et exhaustivement traduite dans une th�orie de la "maturation stadiale", soit neuro-biologique, soit bio-neuro-psychologique, - en accordant m�caniquement � la maturation neuro-biologique le r�le d'une "essence" dont les "stades" freudiens ne seraient que le pur et simple "ph�nom�ne". Perspective qui n'est qu'une r��dition du vieux parall�lisme m�caniste.
    Note 6. (p. 18) On risquerait de m�conna�tre la port�e th�orique de cette condition formelle, si on lui opposait l'apparence biologique des concepts (libido, affects, pulsions, d�sir) Dans lesquels Freud pense le "contenu" de l'inconscient. Ainsi lorsqu'il dit que le r�ve est le "plein-du-d�sir" (Wunscherf�llung). C'est dans le m�me sens que Lacan dit que le but de la cure est de faire acc�der l'homme au "langage de son d�sir" inconscient. C'est pourtant � partir de cette condition formelle que ces concepts (apparemment biologiques) prennent leur sens authentique, que ce sens peut �tre assign� et pens�, et qu'une technique de la cure peut �tre d�finie et appliqu�e. Le d�sir, cat�gorie fondamentale de l'inconscient, n'est intelligible dans sa sp�cificit�, que comme le sens singulier du discours de l'inconscient du sujet humain : le sens qui surgit dans le "jeu" et par le "jeu" de la cha�ne signifiante dont le discours de l'inconscient est compos�. Comme tel, le "d�sir" est marqu� par la structure qui commande le devenir-humain. Comme t l [sic], le d�sir se distingue radicalement du "besoin" organique d'essence biologique. Entre le besoin oraganique et le d�sir inconscient, il n'existe pas de continuit� d'essence, - pas plus qu'il n'existe de continuit� d'essence entre l'existence biologique de l'homme et son existence historique. Le d�sir est d�termin� dans son �tre �quivoque (son "manque-�-�tre", dit Lacan) par la structure de l'Ordre qui lui impose sa marque, et le voue � une existence sans lieu, l'existence du refoulement, � ses ressources comme � ses d�ceptions. On n'acc�de pas � la r�alit� sp�cifique du d�sir en partant du besoin organique, pas plus qu'on n'acc�de � la r�alit� sp�cifique de l'existence historique en partant de l'existence biologique de l'"homme". Au contraire : de m�me que ce sont les cat�gories de l'histoire qui permettent de d�finir la sp�cificit� de l'existence historique de l'homme, y compris des d�terminations apparemment purement biologiques comme ses "besoins", ou les ph�nom�nes d�mographiques, en distinguant son existence historique d'une existence purement biologique, - de m�me ce sont les cat�gories essentielles de l'inconscient qui permettent d'appr�hender et de d�finir les sens m�me du d�sir, en le distinguant des r�alit�s biologiques qui le supportent, (exact ment [sic] comme l'existence biologique supporte l'existence historique) mais sans le constituer.
    Note 7 (p. 20) Expression de Lacan ("machine"), reprenant Freud ("ein anderes Schauspiel... Schauplatz"). De Politzer, qui parle de "drame", � Freud et Lacan, qui parlent de th��tre, sc�ne, mise en sc�ne, machinerie, genre th��tral, metteur en sc�ne, etc... il y a toute la distance du spectateur, qui se prend pour le th��tre, au th��tre m�me.
    Note 8 (p. 20) Si on entend ce terme d'"f/effet" dans le contexte d'une th�orie classique de la causalit�, on concevra par lui la pr�sence actuelle de la cause dans son effet (cf.Spinoza).
    Note 9. (p. 21) Cf. "Les Mots" (Gallimard 1964). Sartre, enfant, �tait pour son entourage "une petite merveille". Il se reconnaissait, enfant, dans ce "mot" primitif, peut-�tre le mot de tous ses Mots. 
    """
    a = time.time()
    S = LCS(x,y)
    b = time.time()
    print ('LCS ',b-a)
    #S.Print()
    #print S.LongestCommonSubstring()
    #print "abcd"[1:10] 
    #ST = SuffixTree(x+y)
    a = time.time()
    ST = SuffixTree(x+y)
    b = time.time()
    print ('ukokul',b-a)
    e="""r = {0:'tt', 2:'yy'}
    s = {0:'tt', 1:'yy'}
    t = {0:'tt', 1:'yy', 2:s}
    if s == r: print ("egal")
    print (t,repr(t),t.__str__(),str(t))"""
    
if __name__ == '__main__':
    test_LCS()